Ext.define('collaboration.Api', {singleton:true, deleteTag:function(id) {
  return collaboration.Support.newRequest('tags?' + Ext.Object.toQueryString({artefactId:id}), undefined, undefined, 'DELETE');
}, createTag:function(params) {
  return collaboration.Support.newRequest('tags?' + Ext.Object.toQueryString(params), undefined, undefined, 'POST');
}, updateTag:function(params) {
  return collaboration.Support.newRequest('tags?' + Ext.Object.toQueryString(params), undefined, undefined, 'PUT');
}, getTags:function() {
  return collaboration.Support.newRequest('tags', undefined, undefined, 'GET');
}, actions:{tasks:{transformMsg:function(text, taskObj) {
  var oldText = taskObj.msg;
  var newMsg = text + '\n\n---\n' + 'FROM: ' + taskObj.creator + '\n' + 'TO: ' + taskObj.to + '\n' + oldText;
  return newMsg;
}, resolve:function(id, text, taskObj) {
  taskObj.msg = collaboration.Api.actions.tasks.transformMsg(text, taskObj);
  taskObj.state = collaboration.assets.Artefacts.collabTypes.tasks.actions.resolve.state;
  return collaboration.Api.modifyArtefactByType('tasks', id, taskObj);
}, finish:function(id, text, taskObj) {
  taskObj.msg = collaboration.Api.actions.tasks.transformMsg(text, taskObj);
  taskObj.state = collaboration.assets.Artefacts.collabTypes.tasks.actions.finish.state;
  return collaboration.Api.modifyArtefactByType('tasks', id, taskObj);
}, handback:function(id, text, taskObj) {
  taskObj.msg = collaboration.Api.actions.tasks.transformMsg(text, taskObj);
  taskObj.to = taskObj.creator;
  taskObj.state = collaboration.assets.Artefacts.collabTypes.tasks.actions.handback.state;
  return collaboration.Api.modifyArtefactByType('tasks', id, taskObj);
}, take:function(id, taskObj) {
  if (taskObj) {
    taskObj.state = collaboration.assets.Artefacts.collabTypes.tasks.actions.take.state;
  }
  return collaboration.Api.modifyArtefactByType('tasks', id, taskObj);
}}, messages:{ack:function(id, text, taskObj) {
  taskObj.flags.acknowledged = true;
  return collaboration.Api.modifyArtefactByType('messages', id, taskObj, 'acknowledge', text);
}}}, module:{instance:{getReferenceTypes:function(instance) {
  var url = collaboration.Module.getReferenceUrl(instance, null, null, true, null, true);
  return collaboration.Api.getGetRequestPromise(url);
}}}, streams:{channels:{getAll:function() {
  return collaboration.Support.globalController.streamStore.channels;
}}, topics:{getAll:function() {
  return collaboration.Support.globalController.streamStore.topics;
}, getByChannelId:function(channelId) {
  return collaboration.Support.globalController.streamStore.topics[channelId];
}, updateTopicByChannelId:function(channelId, topicArray) {
  collaboration.Support.globalController.streamStore.topics[channelId] = topicArray;
}, refresh:function(moduleId, channelId) {
  return new Ext.Promise(function(fulfill, reject) {
    collaboration.Api.user.streams.getTopics(moduleId).then(function(topicArray) {
      collaboration.Api.streams.topics.updateTopicByChannelId(channelId, topicArray);
      fulfill();
    }, function(reason) {
      collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_TOPIC_UPDATE_ERROR') + moduleId + ' : ' + channelId);
      reject();
    });
  });
}}}, user:{streams:{getChannels:function() {
  return new Ext.Promise(function(fulfill, reject) {
    var collabInstances = BpcCommon.Api.getModuleInstances('collaboration');
    var restrictedAccess = collaboration.Support.getSetting('collaboration', 'module_restrictInstanceAccess');
    var instanceModelArray = collabInstances.getData().getRange();
    collabInstances = [];
    Ext.Array.each(instanceModelArray, function(instanceModel) {
      var moduleId = instanceModel.get('moduleId');
      if (restrictedAccess && !BpcCommon.Api.hasUserRight('useModuleInstance_' + moduleId)) {
        collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_WARN_NO_ACCESS_PRE') + moduleId + collaboration.Support.TR('COLLABORATION_WARN_NO_ACCESS_SUF'));
      } else {
        collabInstances.push(instanceModel);
      }
    });
    var channelsByModuleIdPromises = [];
    var helper = [];
    Ext.Array.each(collabInstances, function(item) {
      if (collaboration.Support.getSetting(item.id, 'instance_channelUi') !== 'inbox') {
        helper.push(item.getData());
        channelsByModuleIdPromises.push(collaboration.Api.getChannels(item.id));
      }
    });
    Ext.Promise.all(channelsByModuleIdPromises).then(function(value) {
      var channelArr = [];
      Ext.Array.each(value, function(response, index, array) {
        var moduleId = helper[index].moduleId;
        var channelUI = collaboration.Support.getSetting(moduleId, 'instance_channelUi');
        if (response && response.total > 0) {
          var channels = response.items;
          var color = '#' + (Math.random() * 16777215 << 0).toString(16);
          if (channelUI === 'blog' || channelUI === 'journal') {
            if (response.total > 1) {
              Ext.log('Collaboration config can be wrong');
            }
            var channel = channels[channels.length - 1];
            if (channel['color']) {
              color = channel['color'];
            }
            channelArr.push(Ext.applyIf({channelId:channel['artefactId'], color:color, instanceType:channelUI, instanceName:helper[index]['instanceName']}, channel));
          } else {
            if (channelUI === 'forum') {
              Ext.Array.each(channels, function(channel) {
                var isChannelActive = channel['active'];
                if (channel['color']) {
                  color = channel['color'];
                }
                if (Ext.isDefined(isChannelActive) && isChannelActive === true) {
                  channelArr.push(Ext.applyIf({channelId:channel['artefactId'], color:color, instanceType:channelUI, instanceName:helper[index]['instanceName']}, channel));
                }
              });
            } else {
              collaboration.Support.notify('channelUI type not in range');
            }
          }
        }
      });
      fulfill(channelArr);
    });
  });
}, getTopics:function(moduleId) {
  return new Ext.Promise(function(fulfill, reject) {
    collaboration.Api.getTopics(moduleId).then(function(response) {
      var topicsArr = [];
      if (response !== null && response.items) {
        if (response && response.total > 0) {
          Ext.Array.each(response.items, function(topic) {
            var isTopicActive = topic['active'];
            if (Ext.isDefined(isTopicActive) && isTopicActive === true) {
              topicsArr.push(Ext.applyIf({id:topic['artefactId'], topicId:topic['artefactId']}, topic));
            }
          });
          fulfill(topicsArr);
        } else {
          fulfill([]);
        }
      } else {
        fulfill([]);
      }
    });
  });
}, getForumTopics:function(moduleId, channelId) {
  return new Ext.Promise(function(fulfill, reject) {
    var topicsArr = [];
    var url = collaboration.Module.getBaseUrl() + moduleId + '/topics?channelId\x3d' + channelId;
    collaboration.Api.getGetRequestPromise(url).then(function(response) {
      if (response !== null && response.items) {
        if (response.total > 0) {
          Ext.Array.each(response.items, function(topic) {
            var isTopicActive = topic['active'];
            if (Ext.isDefined(isTopicActive) && isTopicActive === true) {
              topicsArr.push(Ext.applyIf({id:topic['artefactId'], topicId:topic['artefactId']}, topic));
            } else {
              fulfill([]);
            }
          });
          fulfill(topicsArr);
        } else {
          collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_FORUMTOPICS'));
          fulfill([]);
        }
      } else {
        collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_FORUMTOPICS'));
        fulfill([]);
      }
    }, function(reason) {
      fulfill([]);
    });
  });
}}}, getArtefactTypes:function() {
  return collaboration.Support.getSupportedTypes(true);
}, getDefaultInbox:function() {
  return collaboration.Module.getDefaultCollabInstance();
}, showActivityFlowWindow:function(referenceId, instanceId, moduleId, title, referenceType) {
  title = title === undefined ? '' : title;
  var streamWindow = Ext.create('collaboration.components.window.Window', {title:collaboration.Support.TR('FEED') + ': ' + title, iconCls:collaboration.assets.Icons.commentsIcon, height:document.body.clientHeight * 0.6, width:document.body.clientWidth * 0.7, titleMin:title, layout:'fit', items:[{xtype:'colFeed', referenceId:referenceId, instanceId:instanceId, referenceType:referenceType}]});
  streamWindow.show();
  return streamWindow;
}, getEditorWindow:function(record, collabType, moduleId, iconCls, cfg) {
  var win = Ext.create({xtype:'collabEditorWindow', iconCls:iconCls, collabType:collabType, items:[Ext.apply({xtype:'collabEditorPanel', moduleId:moduleId, collabType:collabType, record:Ext.isDefined(record) ? record : null}, cfg || {})]});
  win.show();
  return win;
}, applyGlobalReference:function(referenceId) {
  Ext.GlobalEvents.fireEvent('collabSetReference', referenceId);
}, getGlobalDefaultInstance:function() {
  var defaultInstance;
  var instancesStore = BpcCommon.Api.getFlatModuleInstanceStore('collaboration');
  instancesStore.each(function(instance) {
    if (instance.get('isDefaultInstance')) {
      defaultInstance = instance.get('id');
    }
  });
  return defaultInstance;
}, configCorrect:function() {
  var defaultInstance = this.getGlobalDefaultInstance();
  return !!defaultInstance;
}, getOverview:function(artefactType, query, startDate, endDate) {
  var url = collaboration.Module.getBaseUrl() + 'artefacts?action\x3doverview';
  if (Ext.isDefined(artefactType)) {
    url = collaboration.Module.getBaseUrl() + 'artefacts?type\x3dtasks\x26action\x3daggregate\x26actionValue\x3dstate';
  }
  if (query) {
    url += '?query\x3d' + query;
  }
  if (startDate || endDate) {
    if (!query) {
      url += '?';
    } else {
      url += '\x26';
    }
    url += 'startDate\x3d' + startDate + '\x26endDate\x3d' + endDate;
  }
  return this.getGetRequestPromise(url);
}, getBaseModule:function(instanceId) {
  return BpcCommon.Api.getBaseModuleIdById(instanceId);
}, openReference:function(r) {
  this.jumpToReference(r.get('src_instanceId'), r.get('reference'));
}, jumpToReference:function(instanceId, referenceId) {
  var payload;
  var moduleId = this.getBaseModule(instanceId);
  if (moduleId === 'monitor' && referenceId && instanceId) {
    payload = {storeState:{filters:[{property:collaboration.Support.getSetting(instanceId, 'column_id'), value:referenceId}]}};
  }
  if (moduleId === 'vam' && referenceId) {
    payload = {assets:[referenceId]};
  }
  if (payload) {
    BpcCommon.Api.showModule(instanceId, false, payload);
  }
}, getArtefactsByModuleAndInstanceId:function(instanceId) {
  var url = collaboration.Module.getReferenceUrl(instanceId, null, null, true);
  return this.getGetRequestPromise(url);
}, getArtefactsByModuleAndInstanceIdAndRefernceId:function(instanceId, referenceId) {
  var url = collaboration.Module.getReferenceUrl(instanceId, referenceId, null, true, true);
  return this.getGetRequestPromise(url);
}, getReferenceTypes:function(instanceId) {
  var moduleId = this.getBaseModule(instanceId);
  Ext.log({msg:'getReferenceTypes', dump:{mid:moduleId, iid:instanceId}});
  return this.getGetRequestPromise(collaboration.Module.getBaseUrl() + 'module/' + moduleId + '/' + instanceId + '/referenceTypes');
}, getUsersSave:function() {
  return BpcCommon.Api.getUsers();
}, getUserAndGroups:function(usersList) {
  var userAndGroups = [];
  userAndGroups.push(usersList.length ? usersList : this.getUsersSave());
  userAndGroups.push(BpcCommon.Api.getRoles());
  userAndGroups.push(BpcCommon.Api.getUserOrganisations());
  return Ext.Promise.all(userAndGroups);
}, getTopics:function(instanceId) {
  return this.getGetRequestPromise(collaboration.Module.getBaseUrl() + instanceId + '/topics');
}, getChannels:function(instanceId) {
  return this.getGetRequestPromise(collaboration.Module.getBaseUrl() + instanceId + '/channels');
}, createTopic:function(instanceId, topicName, payload) {
  payload = payload || {};
  payload.topic = topicName;
  return this.getCreateRequestPromise(collaboration.Module.getBaseUrl() + instanceId + '/topics', payload);
}, createChannel:function(instanceId, channelName, instanceTypeOrPayload) {
  var url = collaboration.Module.getBaseUrl() + instanceId + '/channels', params;
  if (Ext.isString(instanceTypeOrPayload)) {
    params = {instanceType:instanceTypeOrPayload, channel:channelName};
  } else {
    params = Ext.apply({channel:channelName}, instanceTypeOrPayload);
  }
  return this.getCreateRequestPromise(url, params);
}, createArtefactUrlFromType:function(type) {
  return collaboration.Module.getBaseUrl() + 'artefacts' + '?type\x3d' + type;
}, createArtefact:function(url, payload) {
  return this.getCreateRequestPromise(url, payload);
}, modifyArtefactByType:function(type, itemId, payload, action, actionText) {
  var params = {type:null};
  if (type === 'comments') {
    params.type = type;
  }
  if (type === 'messages') {
    params.type = type;
    if (action) {
      params.action = action;
    }
    if (actionText) {
      params.actionText = actionText;
    }
  }
  if (type === 'tasks') {
    params.type = type;
    if (action) {
      params.action = action;
    }
  }
  return this.getChangeRequestPromiseWithData(collaboration.Module.getBaseUrl() + 'artefacts/' + itemId + '?' + Ext.Object.toQueryString(params), payload);
}, cloneArtefact:function(id) {
  return this.getChangeRequestPromise(collaboration.Module.getBaseUrl() + 'artefacts/' + id + '?action\x3dclone');
}, modifyTopic:function(instanceId, topicId, payload) {
  var url = collaboration.Module.getBaseUrl() + instanceId + '/topics/' + topicId;
  return this.getChangeRequestPromiseWithData(url, payload);
}, modifyChannel:function(instanceId, channelId, payload) {
  var url = collaboration.Module.getBaseUrl() + instanceId + '/channels/' + channelId;
  return this.getChangeRequestPromiseWithData(url, payload);
}, assignArtefact:function(itemId, loginName, type) {
  var url = collaboration.Module.getBaseUrl() + 'artefacts/' + itemId + '?type\x3d' + type + '\x26action\x3dassign\x26actionValue\x3d' + loginName;
  return this.getChangeRequestPromise(encodeURI(url));
}, deleteTopic:function(topicId) {
  var url = this.getCollabBaseUrl() + topicId;
  return this.getDeleteRequestPromise(url, null, {isTopic:true});
}, deleteChannel:function(channelId) {
  var url = this.getCollabBaseUrl() + channelId;
  return this.getDeleteRequestPromise(url, null, {isChannel:true});
}, deleteArtefacts:function(arrayWithIds) {
  var url = collaboration.Module.getBaseUrl() + 'bulkDelete' + '/bulk';
  return this.getDeleteRequestPromise(url, JSON.stringify(arrayWithIds));
}, getCollabBaseUrl:function(noDefaultInstanceInPath) {
  return collaboration.Module.getBaseUrl() + !noDefaultInstanceInPath ? collaboration.Module.getDefaultCollabInstance() + '/' : '';
}, getGetRequestPromise:function(url) {
  return new Ext.Promise(function(resolve, reject) {
    Ext.Ajax.request({url:url, method:'GET', success:function(response) {
      resolve(Ext.decode(response.responseText, true));
    }, failure:function(response) {
      reject(response.status);
    }});
  });
}, getChangeRequestPromiseWithData:function(url, data) {
  return new Ext.Promise(function(resolve, reject) {
    Ext.Ajax.request({url:url, method:'PUT', jsonData:data, success:function(response) {
      resolve(Ext.decode(response.responseText, true));
    }, failure:function(response) {
      reject(Ext.decode(response.responseText, true));
    }});
  });
}, getChangeRequestPromise:function(url) {
  return new Ext.Promise(function(resolve, reject) {
    Ext.Ajax.request({url:url, method:'PUT', success:function(response) {
      resolve(response.status);
    }, failure:function(response) {
      reject(response.status);
    }});
  });
}, getCreateRequestPromise:function(url, payload) {
  return new Ext.Promise(function(resolve, reject) {
    Ext.Ajax.request({url:url, method:'POST', jsonData:payload, success:function(response) {
      resolve(Ext.decode(response.responseText, true));
    }, failure:function(response) {
      reject(response.status);
    }});
  });
}, getDeleteRequestPromise:function(url, payload, params) {
  if (!Ext.isDefined(payload)) {
    payload = null;
  }
  if (!Ext.isDefined(params)) {
    params = null;
  } else {
    if (Ext.isObject(params)) {
      url += '?';
      Ext.Object.each(params, function(k, v, obj) {
        url += k + '\x3d' + v + '\x26';
      });
    }
  }
  return new Ext.Promise(function(resolve, reject) {
    Ext.Ajax.request({url:url, method:'DELETE', jsonData:payload, headers:{'Content-Type':'application/json'}, success:function(response) {
      resolve(response.status);
    }, failure:function(response) {
      reject(response.status);
    }});
  });
}, getTopicById:function(topicId) {
  return collaboration.Support.globalController.getTopicById(topicId);
}, getChannelById:function(channelId) {
  return collaboration.Support.globalController.getChannelById(channelId);
}});
Ext.define('collaboration.service.References', {singleton:true, supportedModules:['monitor', 'vam'], getRequestConfig:{monitor:function(moduleId, instanceId, reference) {
  return {method:'GET', url:collaboration.Module.getBasePath(moduleId) + instanceId + '/' + reference};
}, vam:function(moduleId, instanceId, reference) {
  var xmlString = '\x3capi\x3e\x3corganizationId\x3e' + collaboration.Support.getSetting(instanceId, 'organizationId') + '\x3c/organizationId\x3e' + '\x3crequests\x3e\x3crequest\x3e\x3cmodule\x3eam\x3c/module\x3e\x3ccontext\x3easset\x3c/context\x3e\x3cmethod\x3equery\x3c/method\x3e\x3cparameters\x3e\x3cfilters\x3e\x3cfilter\x3e\x3ckey\x3eid\x3c/key\x3e\x3coperator\x3e\x3d\x3c/operator\x3e\x3cvalue\x3e' + reference + '\x3c/value\x3e\x3c/filter\x3e\x3c/filters\x3e\x3c/parameters\x3e\x3c/request\x3e\x3c/requests\x3e\x3c/api\x3e';
  return {method:'POST', url:BpcCommon.Api.getBackendUrl(collaboration.Support.getSetting(instanceId, 'proxyId')) + collaboration.Support.getSetting(instanceId, 'targetUrl'), xmlData:(new DOMParser).parseFromString(xmlString, 'text/xml')};
}}, loadData:function(requestConfig) {
  return Ext.Ajax.request(requestConfig);
}});
Ext.define('collaboration.Support', {singleton:true, requires:['collaboration.service.*'], globalController:undefined, uiTypes:[{type:'inbox', viewName:'Inbox'}, {type:'blog', viewName:'Blog'}, {type:'journal', viewName:'Journal'}, {type:'forum', viewName:'Forum'}], artefactTypes:['comments', 'tasks', 'messages', 'activities', 'webhooks'], artefactTypeConfig:{comments:{}, tasks:{}, messages:{}}, icons:{settings:'fa-cogs', admin:'fa-tools'}, iconPrefix:'x-fal', iconSuffix:'fa-', fallbackStateColor:'#42A5F5', 
getBpcIconCls:function(fontAwesomeName) {
  if (Ext.String.startsWith(fontAwesomeName, this.iconPrefix)) {
    return fontAwesomeName;
  }
  if (!Ext.String.startsWith(fontAwesomeName, this.iconSuffix)) {
    fontAwesomeName = this.iconSuffix + fontAwesomeName;
  }
  return this.iconPrefix + ' ' + fontAwesomeName.trim();
}, loadReference:function(moduleId, instanceId, reference) {
  var r = collaboration.service.References;
  if (r.supportedModules.indexOf(moduleId) !== -1) {
    return r.loadData(r.getRequestConfig[moduleId](moduleId, instanceId, reference));
  }
}, getSupportedTypes:function(asStringArray) {
  var typeDefinitions = BpcCommon.Api.getSetting('collaboration', 'module_supportedArtefactTypes') || [];
  return typeDefinitions.filter(function(item) {
    return item.enabled;
  }).map(function(item) {
    return asStringArray ? item.id : item;
  });
}, days:{0:'SUNDAY', 1:'MONDAY', 2:'TUESDAY', 3:'WEDNESDAY', 4:'THURSDAY', 5:'FRIDAY', 6:'SATURDAY'}, defaultInputDateFormat:'C', defaultDisplayDateFormat:'d.m.Y H:i:s', baseUrl:'/cxf/bpc-collaboration/collab/', dateFormatter:function(utcString, output) {
  return Ext.Date.format(Ext.Date.parse(utcString, this.defaultInputDateFormat), output || this.defaultDisplayDateFormat);
}, newRequest:function(url, params, data, method) {
  var $jscomp$this = this;
  params = params === undefined ? {} : params;
  data = data === undefined ? null : data;
  method = method === undefined ? 'GET' : method;
  return new Promise(function(resolve, reject) {
    Ext.Ajax.request({url:$jscomp$this.baseUrl + url, params:params, method:method, jsonData:data, headers:{'Content-Type':'application/json'}, success:function(response) {
      resolve(response.responseText ? Ext.decode(response.responseText, true) : undefined);
    }, failure:function(response) {
      reject(response);
    }});
  });
}, virtimo:['#46C5BE', '#B5E0E5', '#E6DEDA', '#B2AEB0', '#FF935D', '#F23C4D', '#F7BE57', '#BCB936', '#86CDD4', '#338891', '#FF6E25', '#B80C1C', '#A09A9D', '#D1C3BB', '#B2AEB0'], copyToClipBoard:function(text, extCmp) {
  function copyToClp(stringWithNewLines) {
    try {
      var myFluffyTextarea = document.createElement('textarea');
      myFluffyTextarea.innerHTML = stringWithNewLines;
      var parentElement = document.getElementById(extCmp.getId());
      parentElement.appendChild(myFluffyTextarea);
      myFluffyTextarea.select();
      document.execCommand('copy');
      parentElement.removeChild(myFluffyTextarea);
      return true;
    } catch (e) {
      return false;
    }
  }
  return copyToClp(text);
}, getResponseObject:function(responseObj) {
  responseObj = responseObj || {};
  if ('responseText' in responseObj && responseObj.responseText) {
    return Ext.decode(responseObj.responseText, true);
  } else {
    if ('responseJson' in responseObj && responseObj.responseJson) {
      return responseObj.responseJson;
    }
  }
}, promiseDialog:function() {
  return new Promise(function(resolve) {
    Ext.Msg.confirm(collaboration.Support.TR('DELETE'), collaboration.Support.TR('DELETE_ENTRY_WARNING_Q'), function(btnText) {
      resolve(btnText === 'yes');
    });
  });
}, getTagBoxStyle:function(inActive, color) {
  var bgColor = color || this.getRandomVirtimoColors()[0], textColor = 'bisque', borderColor = '#000';
  if (inActive) {
    bgColor = '#eee';
    textColor = 'darkgrey';
    borderColor = 'darkgrey';
  }
  return 'background-color:' + bgColor + ';color:' + textColor + ';font-size:14px;border:1px; border-style: solid; border-color: ' + borderColor + '; padding-left:0.15em; padding-right: 0.15em;';
}, getRandomVirtimoColors:function() {
  var shuffle = function shuffleArray(array) {
    for (var i = array.length - 1; i > 0; i--) {
      var j = Math.floor(Math.random() * (i + 1));
      var temp = array[i];
      array[i] = array[j];
      array[j] = temp;
    }
    return array;
  };
  return shuffle(['#46C5BE', '#B5E0E5', '#E6DEDA', '#B2AEB0', '#FF935D', '#F23C4D', '#F7BE57', '#BCB936', '#86CDD4', '#338891', '#FF6E25', '#B80C1C', '#A09A9D', '#D1C3BB', '#B2AEB0']);
}, showSteamWindow:function(channelId, topicId, moduleId, topicRecord) {
  Ext.create('collaboration.components.window.Window', {title:collaboration.Support.TR('COLLABORATION_BLOGENTRY'), height:document.body.clientHeight * 0.9, width:document.body.clientWidth * 0.9, minWidth:275, layout:{type:'hbox', align:'stretch'}, items:[{xtype:'collabTopic', channelId:channelId, moduleId:moduleId, showNewButton:topicRecord.get('active') === true, showCloseButton:true, topicId:topicId, header:false, topicRecord:topicRecord, flex:1}, collaboration.Module.getEditorPlaceHolderObj()], 
  titleMax:collaboration.Support.TR('BLOG'), titleMin:collaboration.Support.TR('BLOG'), iconCls:'x-fal fa-comments', listeners:{show:function(win, eOpts) {
    Ext.Function.defer(function() {
      var title = win.down('collabtopic').getViewModel().get('topicTitle');
      if (title) {
        win.titleMin = collaboration.Support.TR('BLOG') + ' - ' + title;
      }
    }, 1000);
  }}}).show();
}, createHtmlTooltipFromObjectNotEncoded:function(o, hideNull) {
  var htmlString = '\x3ctable\x3e';
  try {
    var hideNullValues = !(Ext.isDefined(hideNull) && !hideNull);
    Ext.iterate(o, function(key, value) {
      if (key !== 'id') {
        var hide = value === null && hideNullValues;
        if (!hide) {
          if (value && (Ext.isArray(value) || Ext.isObject(value))) {
            value = JSON.stringify(value, undefined, 2);
          }
          htmlString += '\x3ctr\x3e';
          htmlString += '\x3ctd\x3e\x3cb\x3e' + key + ':\x3c/b\x3e\x3c/td\x3e';
          htmlString += '\x3ctd\x3e' + value + '\x3c/td\x3e';
          htmlString += '\x3c/tr\x3e';
        }
      }
    });
    htmlString += '\x3c/table\x3e';
  } catch (e) {
    htmlString = 'Fehler beim Verarbeiten des Records.';
  }
  return htmlString;
}, TR:function(key, otherModuleString) {
  return this.getTranslation(key, otherModuleString);
}, getTranslation:function(key, otherModuleString) {
  var starter = 'COLLABORATION';
  if (Ext.isDefined(otherModuleString)) {
    starter = otherModuleString.toUpperCase();
  }
  starter += '_';
  if (!Ext.String.startsWith(key, starter)) {
    key = starter + key;
  }
  key = key.toUpperCase();
  if (this.trCache[key]) {
    return this.trCache[key];
  }
  var translation = BpcCommon.Api.getTranslation(key);
  if (key === translation) {
  }
  if (!this.trCache[key]) {
    this.trCache[key] = translation;
  }
  return translation;
}, getStatesAsArray:function(allStates) {
  var statesCfg = collaboration.Support.getSetting(collaboration.Module.getDefaultCollabInstance(), 'default_taskStates') || {};
  return Object.keys(statesCfg).filter(function(k) {
    return allStates ? true : !statesCfg[k].isDisabled;
  }).map(function(k) {
    var state = statesCfg[k];
    state.id = k;
    state.state = k;
    state.translated = collaboration.Support.TR(state.languageKey);
    return state;
  });
}, getSetting:function(instanceId, settingName) {
  if (this.settingCache[instanceId] && this.settingCache[instanceId][settingName]) {
    return this.settingCache[instanceId][settingName];
  }
  var setting = BpcCommon.Api.getSetting(instanceId, settingName);
  if (this.settingCache[instanceId]) {
    this.settingCache[instanceId][settingName] = setting;
  } else {
    this.settingCache[instanceId] = {settingName:setting};
  }
  return setting;
}, trCache:{}, settingCache:{}, capitalizeFirstLetter:function(string) {
  return string.charAt(0).toUpperCase() + string.slice(1);
}, canDelete:function(collabType) {
  var can = true;
  if (Ext.isDefined(collabType)) {
    var session = BpcCommon.Api.getUserSession();
    var right = session.get('rights');
    if (right.indexOf('collaboration_cannotDeleteArtefacts') !== -1 || right.indexOf('collaboration_cannotDelete' + collaboration.Support.capitalizeFirstLetter(collabType)) !== -1) {
      can = false;
    }
  }
  return can;
}, renderState:function(stateValue, instanceId, asHtml) {
  instanceId = instanceId || collaboration.Module.getDefaultCollabInstance();
  var stateCfg = BpcCommon.Api.getSetting(instanceId, 'default_taskStates') || {};
  var matches = Object.keys(stateCfg).map(function(k) {
    return {iconCls:collaboration.Support.getBpcIconCls(stateCfg[k].iconCls || 'fa-bug'), text:collaboration.Support.TR(stateCfg[k].languageKey || ''), stateValue:k, color:stateCfg[k].color || collaboration.Support.fallbackStateColor};
  }).filter(function(i) {
    return i.stateValue === stateValue;
  });
  return matches.length === 1 ? asHtml ? collaboration.Module.getIconClsSpan(matches[0].iconCls, matches[0].color) + '' + matches[0].text : matches[0] : stateValue;
}, getStateValue:function(instanceId, state, key) {
  instanceId = instanceId || collaboration.Module.getDefaultCollabInstance();
  var stateCfg = BpcCommon.Api.getSetting(instanceId, 'default_taskStates') || {};
  var singleStateCfg = stateCfg[state] || {};
  return singleStateCfg[key];
}, notify:function(textOrConfig) {
  var originalNotification = textOrConfig;
  if (Ext.isString(textOrConfig)) {
    textOrConfig = {type:'INFO', text:originalNotification};
  } else {
    if (Ext.isObject(textOrConfig)) {
      if (!textOrConfig.hasOwnProperty('type')) {
        textOrConfig.type = 'INFO';
      }
    } else {
      textOrConfig = {type:'INFO', text:'Oups... Something went wrong during processing the notification.'};
    }
  }
  BpcCommon.Api.showNotification(textOrConfig);
}, getStreamInstancesByType:function(ArrayOrString) {
  var matchingInstances = [];
  if (Ext.isString(ArrayOrString)) {
    ArrayOrString = [ArrayOrString];
  }
  var collabInstanceModelStore = BpcCommon.Api.getModuleInstances('collaboration');
  var collabInstances = collabInstanceModelStore.getData();
  collabInstances.each(function(instance) {
    var moduleId = instance.get('moduleId');
    if (Ext.isDefined(moduleId)) {
      if (collaboration.Support.getSetting(moduleId, 'instance_channelUi') !== 'inbox') {
        var channelUI = collaboration.Support.getSetting(moduleId, 'instance_channelUi');
        if (Ext.Array.indexOf(ArrayOrString, channelUI) !== -1) {
          matchingInstances.push({moduleId:moduleId, instanceName:instance.get('instanceName')});
        }
      }
    }
  });
  return matchingInstances;
}, getBpcModuleInstances:function(ArrayOrString) {
  var matchingInstances = [];
  ArrayOrString = Ext.isString(ArrayOrString) ? [ArrayOrString] : ArrayOrString;
  Ext.Array.each(ArrayOrString, function(moduleName) {
    var instanceModelStore = BpcCommon.Api.getModuleInstances(moduleName);
    var instances = instanceModelStore.getData();
    instances.each(function(instance) {
      var moduleId = instance.get('moduleId');
      if (Ext.isDefined(moduleId)) {
        matchingInstances.push({instanceName:instance.get('instanceName'), instanceId:moduleId, combined:{moduleId:moduleName, instanceId:moduleId}});
      }
    });
  });
  return matchingInstances;
}, initBlogJournal:function(moduleId, channelPrefix) {
  return new Ext.Promise(function(resolve, reject) {
    collaboration.Api.getChannels(moduleId).then(function(response) {
      if (response !== null && response.items) {
        if (response.total === 0) {
          collaboration.Api.createChannel(moduleId, channelPrefix + moduleId, channelPrefix).then(function(obj) {
            if (obj !== null && obj.id) {
              resolve(obj.id);
            }
          }, function() {
            reject('Es trat ein Fehler auf bei der Erstellung des Channels.');
          });
        } else {
          var channels = response.items;
          var channel = channels[channels.length - 1];
          resolve(channel['artefactId']);
        }
      } else {
        reject('Es trat ein Fehler auf.');
      }
    }, function() {
      reject('Es trat ein Fehler auf bei der Abfrage der Channels.');
    });
  });
}, mapStateToConfig:function(state) {
  var stateSetting = collaboration.Support.getSetting(collaboration.Module.getDefaultCollabInstance(), 'default_taskStates');
  if (stateSetting && Ext.isObject(stateSetting) && stateSetting.hasOwnProperty(state)) {
    var stateConfig = {};
    stateConfig.text = stateSetting[state]['defaultText'];
    stateConfig.iconCls = stateSetting[state]['iconCls'];
    if (state === 'closed') {
      stateConfig.color = '#55AA55';
    } else {
      if (state === 'open') {
        stateConfig.color = '#D49A6A';
      } else {
        if (state === 'in_progress') {
          stateConfig.color = '#407F7F';
        } else {
          stateConfig.color = '#42A5F5';
        }
      }
    }
    return stateConfig;
  }
}, colorCodeImage:function(v) {
  if (Ext.isEmpty(v)) {
    return '\x3cspan style\x3d"text-decoration: none; display: inline-block; vertical-align: top; margin: 2px 2px 1px 2px; ' + 'border-radius: 7px; box-shadow: 0px 0px 3px 0px #d0d0d0; height: 12px; width: 12px; ' + '"\x3e\x26nbsp;\x3c/span\x3e';
  }
  var c = [parseInt(v.substring(0, 2), 16), parseInt(v.substring(2, 4), 16), parseInt(v.substring(4, 6), 16)];
  var min = '', max = '';
  for (var i = 0; i < 3; i++) {
    min += Ext.String.leftPad(Math.max(c[i] - 48, 0).toString(16), 2, '0');
    max += Ext.String.leftPad(Math.min(c[i] + 16, 255).toString(16), 2, '0');
  }
  return '\x3cspan style\x3d"text-decoration: none; display: inline-block; vertical-align: top; margin: 1px 2px 0px 2px; ' + 'border-radius: 7px; box-shadow: 0px 0px 3px 0px #d0d0d0; height: 14px; width: 14px; ' + 'background: #' + v + '; ' + 'background: linear-gradient(to top right, #' + min + ', #' + max + '); ' + '"\x3e\x26nbsp;\x3c/span\x3e';
}, getSpanForIconCls:function(iconCls, color, size, marginRight, marginLeft) {
  marginRight = marginRight || '3px';
  marginLeft = marginLeft || undefined;
  size = size || '14px';
  color = color || '#eee';
  return '\x3cspan class\x3d"' + collaboration.Support.getBpcIconCls(iconCls) + '" style\x3d"color:' + color + '; font-size: ' + size + '; margin-right: ' + marginRight + ';' + (marginLeft ? 'margin-left:' + marginLeft + ';' : '') + '"\x3e\x3c/span\x3e';
}, getMetaByType:function(type) {
  var viewConfig = BpcCommon.Api.getSetting(collaboration.Module.getDefaultCollabInstance(), 'viewConfig') || {};
  var $jscomp$destructuring$var0 = viewConfig;
  var meta = $jscomp$destructuring$var0.meta;
  meta = meta || {};
  return meta[type] || {};
}});
Ext.define('collaboration.assets.Icons', {singleton:true, remove:'x-fal fa-remove', retweet:'x-fal fa-retweet', comment:'x-fal fa-comment', barchart:'x-fal fa-chart-bar', user:'x-fal fa-user', clock:'x-fal fa-clock', bookmark:'x-fal fa-bookmark', angleRight:'x-fal fa-angle-right', angleDoubleLeft:'x-fal fa-angle-double-left', angleDoubleRight:'x-fal fa-angle-double-right', bars:'x-fal fa-bars', fire:'x-fal fa-fire', cog:'x-fal fa-cog', image:'x-fal fa-image', clone:'x-fal fa-clone', refresh:'x-fal fa-sync', 
plus:'x-fal fa-plus', edit:'x-fal fa-edit', submit:'x-fal fa-submit', check:'x-fal fa-check', externalLink:'x-fal fa-external-link', mailReply:'x-fal fa-mail-reply', info:'x-fal fa-info', paperclip:'x-fal fa-paperclip', users:'x-fal fa-users', filter:'x-fal fa-filter', bug:'x-fal fa-bug', trash:'x-fal fa-trash', circle:'x-fal fa-circle', times:'x-fal fa-times', chevronCircleRight:'x-fal fa-chevron-circle-right', chain:'x-fal fa-link', arrowDown:'x-fal fa-arrow-down', envelope:'x-fal fa-envelope', 
history:'x-fal fa-history', picture:'x-fal fa-picture', upload:'x-fal fa-upload', save:'x-fal fa-save', comments:'x-fal fa-comments', download:'x-fal fa-download', bell:'x-fal fa-bell', calendar:'x-fal fa-calendar', search:'x-fal fa-search', reply:'x-fal fa-reply', tags:'x-fal fa-tags', flash:'x-fal fa-flash', toggleOn:'x-fal fa-toggle-on', undo:'x-fal fa-undo', eye:'x-fal fa-eye', minusSquare:'x-fal fa-minus-square', checkSquare:'x-fal fa-minus-square', messageIcon:'x-fal fa-envelope', newMessageIcon:'x-fal fa-plus', 
newCollabArtefactIcon:'x-fal fa-plus', collabOverviewIcon:'x-fal fa-list-ul', messageOverviewIcon:'x-fal fa-list-ul', colorIcon:'x-fal fa-paint-roller', userIcon:'x-fal fa-user fa-fw', userColor:'#55AA55', groupIcon:'x-fal fa-users fa-fw', groupColor:'#d12525', roleIcon:'x-fal fa-users fa-fw', roleColor:'orange', webhookIcon:'x-fal fa-bullhorn', messagesIcon:'x-fal fa-envelope', tasksIcon:'x-fal fa-check-circle iconNotification3', commentsIcon:'x-fal fa-comments', activitiesIcon:'x-fal fa-clock'});
Ext.define('collaboration.controller.CollaborationController', {extend:'Ext.app.Controller', requires:['collaboration.Support'], listen:{global:{monitor_contextMenu_created:'addCollabFeatures', newTopic:'loadStreamData', topicUpdated:'loadStreamData', topicDeleted:'loadStreamData', newChannel:'loadStreamData', channelUpdated:'loadStreamData', channelDeleted:'loadStreamData'}}, init:function() {
  this.loadStreamData();
  collaboration.Support.globalController = this;
}, getTopicById:function(topicId) {
  var returnVal = null;
  var found = false;
  Ext.iterate(this.streamStore.topics, function(key, val) {
    Ext.Array.each(val, function(topic, index, val) {
      if (topic['topicId'] === topicId) {
        returnVal = topic;
        found = true;
        return false;
      }
    });
    if (found) {
      return false;
    }
  });
  return returnVal;
}, getChannelById:function(channelId) {
  var returnVal = null;
  Ext.Array.each(this.streamStore.channels, function(channel, index, channels) {
    if (channel['channelId'] === channelId) {
      returnVal = channel;
      return false;
    }
  });
  return returnVal;
}, streamStore:{channels:[], topics:{}, helperStore:{}}, loadStreamData:function() {
  Ext.log({msg:this.self.getName() + ': loadStreamData', level:'info'});
  var me = this;
  return new Ext.Promise(function(fulfill, reject) {
    collaboration.Api.user.streams.getChannels().then(function(successChannelArray) {
      me.streamStore.channels = successChannelArray;
      var topicPromises = [];
      var helper = [];
      if (Ext.isArray(successChannelArray) && successChannelArray.length > 0) {
        Ext.Array.each(successChannelArray, function(channel, item, array) {
          if (channel.instanceType !== 'forum') {
            topicPromises.push(collaboration.Api.user.streams.getTopics(channel.instanceId));
          } else {
            topicPromises.push(collaboration.Api.user.streams.getForumTopics(channel.instanceId, channel.channelId));
          }
          helper.push(channel.channelId);
        });
      }
      me.loadTopicDataNew(topicPromises).then(function(successTopicArray) {
        var topicStore = {};
        Ext.Array.each(successTopicArray, function(item, index, array) {
          if (item.length > 0) {
            Ext.Array.each(item, function(topicObj) {
              if (!topicObj.channelId) {
                topicObj.channelId = helper[index];
              }
            });
          }
          topicStore[helper[index]] = item;
        });
        me.streamStore.topics = topicStore;
        fulfill();
      }, function(reason) {
        collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_TOPICS_LOADED_ERROR'));
      });
    });
  });
}, loadTopicDataNew:function(topicPromises) {
  Ext.log({msg:this.self.getName() + ': loadTopicDataNew', level:'info'});
  return Ext.Promise.all(topicPromises);
}, addCollabFeatures:function(menu) {
  if (menu && menu.moduleId) {
    var holderArr = menu.query('#collabMenuHolder');
    if (holderArr.length) {
      holderArr[0].menu = Ext.create('collaboration.components.contextmenu.CollabMenu', {moduleId:menu.moduleId, src_instanceId:menu.moduleId, src_moduleId:BpcCommon.Api.getBaseModuleIdById(menu.moduleId), src_record:menu.record ? menu.record : undefined});
      menu.getViewModel().set('collabEnabled', true);
    }
  }
}});
Ext.define('collaboration.Module', {singleton:true, requires:['collaboration.Api', 'collaboration.controller.CollaborationController'], user:null, users:null, moduleId:undefined, lastReferenceId:null, constructor:function() {
  BpcCommon.Api.onReady(this.bootstrapCollab.bind(this));
}, bootstrapCollab:function() {
  try {
    this.setTranslationOverwrites();
    this.user = BpcCommon.Api.getUserName();
    this.loadUsers();
    Ext.getApplication().getController('collaboration.controller.CollaborationController');
  } catch (e) {
  }
}, setTranslationOverwrites:function() {
  var obj = BpcCommon.Api.getSetting('collaboration', 'customLanguageKeys'), lang = BpcCommon.Api.getSetting('_core', 'language') || 'de';
  Object.keys(obj).map(function(key) {
    var value = obj[key];
    collaboration.Support.trCache[key] = Ext.isObject(value) && value.de && value.en ? value[lang] : value;
    collaboration.Support.notify('LANGKEY_' + key + 'HAS_BEEN_OVERWRITTEN');
  });
}, createQuery:function(queryName) {
  var query = {};
  query[queryName] = [];
  return query;
}, createBoolQuery:function(mustNot, should, must) {
  var boolQuery = {'bool':{'must_not':[], 'should':[], 'must':[]}};
  if (Ext.isDefined(must) && Ext.isObject(must)) {
    boolQuery.bool.must.push(must);
  }
  return boolQuery;
}, createTermQuery:function(field, value) {
  var term = {};
  term[field] = [value];
  return {'terms':term};
}, createBaseFilterWithTermQuery:function(name, field, value) {
  var q = this.createQuery(name);
  q[name].push(this.createBoolQuery(undefined, undefined, this.createTermQuery(field, value.toLowerCase())));
  return q;
}, getAssetForType:function(type, asset) {
  var colTypeAsset = null;
  var collabTypes = collaboration.assets.Artefacts.collabTypes;
  if (collabTypes[type] && collabTypes[type][asset] && Ext.isDefined(collabTypes[type][asset])) {
    colTypeAsset = collabTypes[type][asset];
  }
  return colTypeAsset;
}, getCollabInboxTitle:function(specifier, totalCount) {
  return specifier + ' (' + totalCount + ')';
}, usersMap:{}, loadUsers:function() {
  var usersList = BpcCommon.Api.getSetting('collaboration', 'custom_userlist') || [];
  collaboration.Api.getUserAndGroups(usersList).then(function(array) {
    var $jscomp$this = this;
    var userOrgRoleList = [];
    if (array && Ext.isArray(array)) {
      array.map(function(arr, i) {
        if (i === 0) {
          var users = $jscomp$this.createCollabUserList(arr);
          var tempUsers = Ext.clone(users);
          tempUsers.map(function(u) {
            if (u.userName) {
              var name = u.userName;
              delete u.userName;
              $jscomp$this.usersMap[name] = u;
            }
          });
          return users;
        } else {
          if (i === 1) {
            return $jscomp$this.createCollabRolesList(arr);
          } else {
            if (i === 2) {
              return $jscomp$this.createCollabOrgList(arr);
            }
          }
        }
      }).map(function(arr) {
        userOrgRoleList = userOrgRoleList.concat(arr);
      });
    }
    if (!userOrgRoleList.length) {
      collaboration.Support.notify(collaboration.Support.TR('USERSORGROLE_LIST_EMPTY_WARNING'));
    }
    this.users = userOrgRoleList;
  }.bind(this));
}, createCollabOrgList:function(array) {
  array = array || [];
  return array.map(function(orgName) {
    return {userName:orgName, type:'org'};
  });
}, createCollabRolesList:function(array) {
  array = array || [];
  return array.map(function(item) {
    if (Ext.isString(item)) {
      return {userName:item, type:'role'};
    } else {
      if (item.id) {
        delete item.id;
      }
      if (item.name) {
        item.userName = item.name;
      }
      item.type = 'role';
      return item;
    }
  });
}, createCollabUserList:function(array) {
  if (Ext.isArray(array)) {
    Ext.Array.each(array, function(userEntry) {
      if (userEntry.id) {
        delete userEntry.id;
      }
      userEntry.type = 'user';
    });
    array.push({userName:collaboration.assets.Artefacts.allSpecifier, firstName:collaboration.Support.TR('ALL'), lastName:collaboration.Support.TR('USER'), type:'group'});
    return array;
  }
  return [];
}, getCollabColumn:function(moduleId, text) {
  var cfg = BpcCommon.Api.getSetting(moduleId, 'module_collab_descriptor') || {};
  return {xtype:'collabWidgetColumn', dataIndex:'CollabData', moduleId:moduleId, text:cfg.columnText || text || 'Collaboration'.toUpperCase(), headerIconCls:cfg.columnHeaderIconCls || collaboration.Support.getBpcIconCls('fa-comments-alt')};
}, getEditorPlaceHolderObj:function() {
  return {xtype:'container', itemId:'editorPlaceHolder', width:820, items:[], hidden:true};
}, getDefaultCollabInstance:function() {
  var legacyDefaultCollabId = collaboration.Support.getSetting('collaboration', 'default_collabInstance');
  if (legacyDefaultCollabId) {
    return legacyDefaultCollabId;
  } else {
    var instancesStore = BpcCommon.Api.getModuleInstances('collaboration');
    if (instancesStore) {
      var instanceIds = [];
      instancesStore.each(function(r) {
        var moduleId = r.get('moduleId');
        if (moduleId && BpcCommon.Api.getSetting(moduleId, 'instance_channelUi') === 'inbox') {
          var headerEnabled = BpcCommon.Api.getSetting(moduleId, 'isDefaultInstance');
          if (headerEnabled) {
            instanceIds.push(moduleId);
          }
        }
      });
      return instanceIds.length ? instanceIds[0] : undefined;
    }
  }
}, getBaseUrl:function() {
  return '/cxf/bpc-collaboration/collab/';
}, takeItem:function(itemId) {
  var url = collaboration.Module.getBaseUrl() + 'artefacts/' + itemId + '?action\x3dtake\x26type\x3dtasks';
  Ext.Ajax.request({method:'PUT', url:url}).then(function(response, opts) {
    collaboration.Support.notify(itemId + ' ' + collaboration.Support.TR('COLLABORATION_ASSIGN_SUCCESS') + '.');
  }, function(response, opts) {
    collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ASSIGN_ERROR') + ' ' + itemId);
  });
}, createUrl:function(moduleId, type) {
  return collaboration.Module.getBaseUrl() + moduleId + '/' + type;
}, getColumnHeaderHtmlString:function(icon, color, text) {
  return '\x3cspan class\x3d"' + icon + '" style\x3d"' + color + '"\x3e\x3c/span\x3e\x26nbsp;' + text;
}, getIconClsSpan:function(iconCls, color) {
  return '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + '; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
}, getToColumn:function() {
  return this.getUserColumn('\x3cspan class\x3d"x-fa fa-user" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + ' ' + collaboration.Support.TR('TO'), 'to');
}, getUserColumn:function(text, dataIndex) {
  return {text:text, dataIndex:dataIndex, renderer:function(value, meta) {
    var displayName = '';
    if (Ext.isDefined(value)) {
      displayName = value;
      var tt = value;
      if (value === collaboration.assets.Artefacts.allSpecifier) {
        displayName = collaboration.assets.Artefacts.allSpecifierText;
        meta.tdStyle = collaboration.assets.Artefacts.allSpecifierGridCellStype;
      }
      if (value in collaboration.Module.usersMap) {
        var userObject = collaboration.Module.usersMap[value];
        displayName = collaboration.Module.getRenderedUsername(value);
        tt = collaboration.Support.createHtmlTooltipFromObjectNotEncoded(userObject, false);
        if ('type' in userObject && Ext.isDefined(userObject.type) && userObject.type !== null) {
          var color = collaboration.assets.Artefacts.userColor;
          var iconCls = collaboration.assets.Artefacts.userIcon;
          if (userObject.type === 'group') {
            color = collaboration.assets.Artefacts.groupColor;
            iconCls = collaboration.assets.Artefacts.groupIcon;
          }
          if (userObject.type === 'role') {
            color = collaboration.assets.Artefacts.roleColor;
            iconCls = collaboration.assets.Artefacts.roleIcon;
          }
          if (userObject.type === 'org') {
            color = collaboration.assets.Artefacts.roleColor;
            iconCls = collaboration.assets.Artefacts.roleIcon;
          }
          displayName = collaboration.Module.getIconClsSpan(iconCls, color) + displayName;
        }
      }
      meta.tdAttr = 'data-qtip\x3d"' + tt + '"';
    }
    return displayName;
  }};
}, getRenderedUsername:function(userName) {
  var $jscomp$destructuring$var1 = collaboration.Module.usersMap[userName] || {};
  var firstName = $jscomp$destructuring$var1.firstName;
  var lastName = $jscomp$destructuring$var1.lastName;
  if (firstName && lastName && firstName.length > 0 && lastName.length > 0) {
    return firstName + ' ' + lastName + ' (' + userName + ')';
  }
  return userName;
}, webhookColumns:function() {
  return [{dataIndex:'created', hidden:true}, {dataIndex:'hook', hidden:true}, {dataIndex:'statusCode', text:collaboration.Support.TR('STATE'), renderer:function(v, record) {
    var icon = null;
    var value = v;
    var iconCls = collaboration.assets.Icons.checkSquare;
    var color = 'green';
    if (value !== 200) {
      color = 'red';
      iconCls = collaboration.assets.Icons.minusSquare;
    }
    icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + ' ; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
    return icon + ' ' + value;
  }}, {dataIndex:'artefactId', text:collaboration.Support.TR('COLLABORATION_ARTEFACT_ID')}, {dataIndex:'artefactType', text:collaboration.Support.TR('TYPE'), renderer:function(v, record) {
    var icon = null;
    var value = v;
    var iconCls = collaboration.assets.Icons.bug;
    var color = '#eee';
    if (Ext.isDefined(v)) {
      iconCls = collaboration.Module.getAssetForType(v, 'iconCls');
      value = collaboration.Module.getAssetForType(v, 'singleText');
    }
    icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + ' ; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
    return icon + ' ' + value;
  }}, {dataIndex:'artefactHeader', text:collaboration.Support.TR('ARTEFACT') + ' ' + collaboration.Support.TR('HEADER')}, {dataIndex:'request', hidden:true}, {dataIndex:'response', hidden:true}];
}, monitor:'/bpc-monitor/monitor/', core:'/bpc-core/', getCollabIntegrationDescriptor:function(moduleId) {
  return collaboration.Support.getSetting(moduleId, 'module_collab_descriptor');
}, getBasePath:function(baseModuleId) {
  if (this[baseModuleId]) {
    return '/cxf' + this[baseModuleId];
  }
}, capitalize:function(str) {
  return str.replace(/\w\S*/g, function(txt) {
    return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();
  });
}, formatDate:function(date) {
  try {
    var now = new Date;
    var dateString = '';
    var timeDay = collaboration.Support.TR('HOURS');
    if (date !== null) {
      var fromDate = parseInt(date.getTime() / 1000), toDate = parseInt(now.getTime() / 1000), timeDiff = (toDate - fromDate) / 3600;
      timeDiff = Math.round(timeDiff);
      if (timeDiff === 1) {
        timeDay = collaboration.Support.TR('HOUR');
      } else {
        if (timeDiff > 24) {
          timeDiff = timeDiff / 24;
          timeDiff = Math.round(timeDiff);
          timeDay = collaboration.Support.TR('DAYS');
        }
      }
      dateString = collaboration.Support.TR('COLLABORATION_VOR') + ' ' + timeDiff + ' ' + timeDay + ' ' + collaboration.Support.TR('COLLABORATION_AGO');
      if (timeDiff === 0) {
        dateString = collaboration.Support.TR('NOW');
      }
      return dateString;
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR formatDate', dump:e, stack:true, level:'error'});
  }
}, cleanRecord:function(record) {
  if (record) {
    if (record['sort']) {
      delete record['sort'];
    }
    if (record['overdue']) {
      delete record['overdue'];
    }
    Ext.iterate(record, function(k, v) {
      if (v === '') {
        delete record[k];
      }
      if (Ext.String.startsWith(k, '_')) {
        delete record[k];
      }
    });
    return record;
  }
}, xml2json:function(xml) {
  var me = this;
  try {
    var obj = {};
    if (xml.children.length > 0) {
      for (var i = 0; i < xml.children.length; i++) {
        var item = xml.children.item(i);
        var nodeName = item.nodeName;
        if (typeof obj[nodeName] == 'undefined') {
          obj[nodeName] = me.xml2json(item);
        } else {
          if (typeof obj[nodeName].push == 'undefined') {
            var old = obj[nodeName];
            obj[nodeName] = [];
            obj[nodeName].push(old);
          }
          obj[nodeName].push(me.xml2json(item));
        }
      }
    } else {
      obj = xml.textContent;
    }
    return obj;
  } catch (e) {
    Ext.log({dump:e});
  }
}, vamSettingValue:function(store, key, defaultValue) {
  var index = store.findExact('name', key);
  return index >= 0 ? store.getAt(index).data.value : defaultValue;
}, loadMonitorReferenceData:function(rec) {
  var baseModule, monitorModuleId, processId, data;
  if (rec && rec.get('src_instanceId') && rec.get('reference')) {
    monitorModuleId = rec.get('src_instanceId');
    baseModule = collaboration.Api.getBaseModule(monitorModuleId);
    processId = rec.get('reference');
    if (baseModule === 'monitor' && processId) {
      var url = collaboration.Module.getBasePath(baseModule) + monitorModuleId + '/' + processId;
      Ext.Ajax.request({method:'GET', url:url, success:function(response, opts) {
        var obj = Ext.decode(response.responseText, true);
        if (obj && Ext.isObject(obj) && obj.totalCount) {
          data = obj.data[0];
        }
      }, failure:function(response, opts) {
        data = undefined;
      }});
    }
  }
  return data;
}, loadVamReferenceData:function(rec) {
  var me = this;
  var instanceId = rec.get('src_instanceId');
  if (instanceId) {
    var xmlString = '\x3capi\x3e\x3corganizationId\x3e' + collaboration.Support.getSetting(instanceId, 'organizationId') + '\x3c/organizationId\x3e' + '\x3crequests\x3e\x3crequest\x3e\x3cmodule\x3eam\x3c/module\x3e\x3ccontext\x3easset\x3c/context\x3e\x3cmethod\x3equery\x3c/method\x3e\x3cparameters\x3e\x3cfilters\x3e\x3cfilter\x3e\x3ckey\x3eid\x3c/key\x3e\x3coperator\x3e\x3d\x3c/operator\x3e\x3cvalue\x3e' + rec.get('reference') + '\x3c/value\x3e\x3c/filter\x3e\x3c/filters\x3e\x3c/parameters\x3e\x3c/request\x3e\x3c/requests\x3e\x3c/api\x3e';
    var data;
    Ext.Ajax.request({method:'POST', url:BpcCommon.Api.getBackendUrl(collaboration.Support.getSetting(instanceId, 'proxyId')) + collaboration.Support.getSetting(instanceId, 'targetUrl'), xmlData:(new DOMParser).parseFromString(xmlString, 'text/xml'), success:function(response, opts) {
      var respObj = me.xml2json(response.responseXML);
      if (respObj && Ext.isObject(respObj) && respObj.response.overallResult === 'true') {
        data = respObj.response.responses.response.body.assets.asset;
      }
    }, failure:function(response, opts) {
    }});
    return data;
  }
}, getReferenceUrl:function(src_instanceId, referenceId, referenceType, module, getOverview, getReferenceTypes) {
  var url = collaboration.Module.getBaseUrl();
  var src_moduleId = collaboration.Api.getBaseModule(src_instanceId);
  if (module) {
    url += 'module/';
  } else {
    url += 'dummyId/';
  }
  if (src_moduleId && src_instanceId) {
    url += src_moduleId + '/' + src_instanceId;
  }
  if (getReferenceTypes) {
    url += '/referenceTypes';
  }
  if (referenceId) {
    url += '/' + referenceId;
  }
  if (getOverview) {
    url += '/overview';
  }
  var hasIntegrationReference = false;
  if (src_moduleId === 'monitor' || src_moduleId === 'vam') {
    var ref = collaboration.Module.getIntegrationReference(src_instanceId);
    if (ref) {
      url += '?integrationReference\x3d' + ref;
      hasIntegrationReference = true;
    }
  }
  if (referenceType) {
    url += (hasIntegrationReference ? '\x26' : '?') + 'referenceType\x3d' + referenceType;
  }
  return url;
}, getIntegrationReference:function(instanceId) {
  var collabDescriptor = collaboration.Support.getSetting(instanceId, 'module_collab_descriptor');
  return collabDescriptor && 'integrationReference' in collabDescriptor && collabDescriptor.integrationReference ? collabDescriptor.integrationReference : undefined;
}});
Ext.define('collaboration.assets.Artefacts', {singleton:true, requires:['collaboration.assets.Icons'], collabTypes:{'empty':{iconCls:'x-fal fa-comment-slash'}, 'total':{text:collaboration.Support.TR('TOTAL')}, 'tasks':{iconCls:collaboration.assets.Icons.tasksIcon, text:collaboration.Support.TR('COLLABORATION_TYPE_TASK_PLURAL'), singleText:collaboration.Support.TR('COLLABORATION_TYPE_TASK_SINGLE'), plural:collaboration.Support.TR('COLLABORATION_TYPE_TASK_PLURAL'), actionText:'Task', model:'collaboration.model.CollabData', 
'new':collaboration.Support.TR('COLLABORATION_TYPE_TASK_NEW'), create:collaboration.Support.TR('COLLABORATION_TYPE_TASK_NEW_CREATE'), chartColor:'#7E57C2', actions:{resolve:{iconCls:collaboration.assets.Icons.toggleOn, style:'color:grey;cursor:pointer;', text:'Erledigen', state:'resolved', fn:function(id, artefact) {
  Ext.Msg.prompt('Aufgabe erledigen', 'Nachricht', function(btn, text) {
    if (btn === 'ok') {
      collaboration.Api.actions.tasks.resolve(id, text, artefact);
    }
  });
}}, finish:{iconCls:collaboration.assets.Icons.toggleOn, style:'color:grey;cursor:pointer;', text:'Abschließen', state:'closed', fn:function(id, artefact) {
  Ext.Msg.prompt('Aufgabe abschließen', 'Abschlussnachricht', function(btn, text) {
    if (btn === 'ok') {
      collaboration.Api.actions.tasks.finish(id, text, artefact);
    }
  });
}}, handback:{iconCls:collaboration.assets.Icons.undo, style:'color:grey;cursor:pointer;', text:'Zurückgeben', state:'open', fn:function(id, artefact) {
  Ext.Msg.prompt('Aufgabe zurückgeben', 'Nachricht', function(btn, text) {
    if (btn === 'ok') {
      collaboration.Api.actions.tasks.handback(id, text, artefact);
    }
  });
}}, take:{iconCls:collaboration.assets.Icons.undo, style:'color:grey;cursor:pointer;', text:'Übernehmen', state:'in_progress'}}}, 'messages':{iconCls:collaboration.assets.Icons.messageIcon, text:collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_SINGLE'), singleText:collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_SINGLE'), plural:collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_PLURAL'), actionText:'Message', model:'collaboration.model.CollabData', 'new':collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_NEW'), 
create:collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_NEW_CREATE'), chartColor:'#29B6F6', actions:{ack:{fn:function(id, artefact) {
  Ext.Msg.prompt('Nachricht quittieren', 'Quittiermeldung', function(btn, text) {
    if (btn === 'ok') {
      collaboration.Api.actions.messages.ack(id, text, artefact);
    }
  });
}}}}, 'comments':{iconCls:collaboration.assets.Icons.commentsIcon, text:collaboration.Support.TR('COLLABORATION_TYPE_COMMENT_SINGLE'), singleText:collaboration.Support.TR('COLLABORATION_TYPE_COMMENT_SINGLE'), plural:collaboration.Support.TR('COLLABORATION_TYPE_COMMENT_PLURAL'), actionText:'Comment', model:'collaboration.model.CollabData', 'new':collaboration.Support.TR('COLLABORATION_TYPE_COMMENT_NEW'), create:collaboration.Support.TR('COLLABORATION_TYPE_COMMENT_NEW_CREATE'), chartColor:'#66BB6A'}, 
'webhooks':{iconCls:collaboration.assets.Icons.webhookIcon, singleText:'Webhook', model:'collaboration.model.CollabWebhooks', url:collaboration.Module.getBaseUrl() + 'artefacts?type\x3dwebhooks'}}, forumContextAufklappen:'x-fal fa-angle-double-down', forumContextZuklappen:'x-fal fa-angle-double-up', replyDefaultText:collaboration.Support.TR('COLLABORATION_ANSWER'), replyOnAnswerText:collaboration.Support.TR('COLLABORATION_SEND'), replyOnAnswerIcon:'x-fal fa-envelope', replyOnCancelIcon:'x-fal fa-times', 
collabIcon:'x-fal fa-commenting-o iconNotification3', collabText:collaboration.Support.TR('COLLABORATION_COLLABORATION'), topicStateActive:collaboration.Support.TR('COLLABORATION_ACTIVE'), topicStateInactive:collaboration.Support.TR('COLLABORATION_INACTIVE'), topicStateArchieved:collaboration.Support.TR('COLLABORATION_ARCHIVED'), greenColor:'#4CAF50', webhookWidgetName:collaboration.Support.TR('WEBHOOKS_NAME'), webhookWidgetDescription:collaboration.Support.TR('WEBHOOKS_DESC'), webhookWidgetTitle:collaboration.Support.TR('WEBHOOKS_TITLE'), 
inbox_messages_inbox:collaboration.Support.TR('INBOX'), inbox_messages_drafts:collaboration.Support.TR('DRAFTS'), inbox_messages_outbox:collaboration.Support.TR('OUTBOX'), userIcon:'x-fal fa-user fa-fw', userColor:'#55AA55', groupIcon:'x-fal fa-users fa-fw', groupColor:'#d12525', roleIcon:'x-fal fa-users fa-fw', roleColor:'orange', webhookIcon:'x-fal fa-bullhorn', messagesIcon:'x-fal fa-envelope', messageIcon:'x-fal fa-envelope', newMessageIcon:'x-fal fa-plus', newCollabArtefactIcon:'x-fal fa-plus', 
collabOverviewIcon:'x-fal fa-list-ul', messageOverviewIcon:'x-fal fa-list-ul', messageOverviewText:collaboration.Support.TR('COLLABORATION_OVERVIEW'), tasksIcon:'x-fal fa-check-circle iconNotification3', tasksText:collaboration.Support.TR('COLLABORATION_TYPE_TASK_PLURAL'), tasksTooltip:collaboration.Support.TR('COLLABORATION_TYPE_TASK_PLURAL'), messagesText:collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_PLURAL'), messageText:collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_PLURAL'), messageTooltip:collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_PLURAL'), 
newMessageText:collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_NEW'), activitiesText:collaboration.Support.TR('ACTIVITYSTREAM'), activitiesIcon:'x-fal fa-clock', defaultFullTextSearch:collaboration.Support.TR('FULLTEXTSEARCH'), moduleName:'module_name', watchIcon:'x-fal fa-eye', takeIcon:'x-fal fa-user', meSpecifierText:collaboration.Support.TR('ME'), allSpecifier:'${ALL}', allSpecifierText:collaboration.Support.TR('COLLABORATION_ALL_USERS'), allSpecifierGridCellStype:'background-color:#FFF59D;color:#FF8A65;', 
editIcon:'x-fal fa-pencil', createIcon:'x-fal fa-plus-square', refreshIcon:'x-fal fa-refresh', deleteIcon:'x-fal fa-trash', streamRefreshTooltip:collaboration.Support.TR('STREAM_UPDATE'), streamEditTooltip:collaboration.Support.TR('STREAM_EDIT'), streamEditorIconCls:'x-fal fa-edit', streamEditorWindowTitle:collaboration.Support.TR('COLLABORATION_STREAM_EDITOR'), topicActiveIcon:'x-fal fa-check-circle', topicInactiveIcon:'x-fal fa-times-circle', creatorIcon:'x-fal fa-user', activeFromIcon:'x-fal fa-calendar-check', 
activeToIcon:'x-fal fa-calendar-times', teamIcon:'x-fal fa-users', archiveIcon:'x-fal fa-archive', betreffIcon:'x-fal fa-bookmark', forumChannelIcon:'x-fal fa-hashtag', columns:{created:{iconCls:collaboration.assets.Icons.clock}, creator:{iconCls:collaboration.assets.Icons.users}, header:{iconCls:collaboration.assets.Icons.bookmark}, msg:{iconCls:collaboration.assets.Icons.comment}, reference:{iconCls:collaboration.assets.Icons.info}, meta:{iconCls:collaboration.assets.Icons.paperclip}}});
Ext.define('collaboration.widgets.BaseWidget', {extend:'Ext.panel.Panel', alias:'widget.collabBaseWidget', mixins:['BpcCommon.mixin.Widget'], layout:'card', title:false, isTypeSupported:function() {
  var types = collaboration.Support.getSupportedTypes().map(function(i) {
    return i.id;
  });
  return types.indexOf(this.collabType) !== -1;
}, addUnsupportedTypeCmp:function() {
  this.removeAll();
  this.add({xtype:'container', html:'not supported'});
}, addSettingsPanel:function() {
  this.settingPanel = this.add({xtype:'bpcSettingPanel', settings:this.settingStore, buttons:[{text:collaboration.Support.TR('OK'), handler:this.configureWidget.bind(this)}, {text:BpcCommon.Api.getTranslation('COLLABORATION_STANDARDVALUES'), handler:this.prepareWidgetConfiguration.bind(this, true)}]});
}, isSupported:true, initComponent:function() {
  this.defaults = {moduleId:collaboration.Support.getSetting('collaboration', 'default_collabInstance')};
  this.isSupported = this.isTypeSupported();
  try {
    if (this.isSupported) {
      this.initWidgetSettings();
    }
  } catch (e) {
  }
  this.callParent(arguments);
  if (this.isSupported) {
    try {
      this.addSettingsPanel();
    } catch (e$0) {
    }
    this.initWidget();
    return;
  }
  this.addUnsupportedTypeCmp();
}, initWidget:function() {
  if (this.settingPanel && this.settingPanel.isComponent && this.settingPanel.isVisible(true)) {
  } else {
    this.contentPanel = this.loadWidgetCmp();
    this.getLayout().setActiveItem(this.contentPanel);
  }
}, refreshWidget:function() {
  try {
    if (this.isSupported) {
      this.contentPanel.store.load();
    } else {
      BpcCommon.Api.showNotification('REFRESH_NOT_POSSIBLE');
    }
  } catch (e) {
    BpcCommon.Api.showNotification('WIDGET_REFRESH_ERROR');
  }
}, getFilterId:function() {
  var widgetConfig = this.getWidgetConfiguration();
  return widgetConfig.filterId && widgetConfig.filterId.length ? widgetConfig.filterId : undefined;
}, updateDbWrapperWithTotal:function(prefix, totalCount) {
  var wrapper = this.up('dbWidgetWrapper');
  if (wrapper) {
    prefix = prefix || this.widgetConfiguration.title || collaboration.Module.getAssetForType(this.collabType, 'plural');
    wrapper.setTitle(prefix + ' (' + totalCount + ')');
  }
}, configureWidget:function() {
  this.getLayout().setActiveItem(this.settingPanel.isVisible(true) ? this.contentPanel : this.settingPanel);
}, loadWidgetCmp:function() {
  var types = {comments:{collabType:'comments', childCmp:'collabStreamGrid', disableSelection:true}, messages:{collabType:'messages', plugins:[{ptype:'rowexpander', pluginId:'rowexpander', rowBodyTpl:new Ext.XTemplate('\x3cp\x3e\x3cspan class\x3d"fa-stack fa-lg"\x3e' + '\x3cspan class\x3d"fa fa-square fa-stack-2x" style\x3d"color: darkseagreen"\x3e\x3c/span\x3e' + '\x3cspan class\x3d"fa x-fal fa-envelope fa-stack-1x"\x3e\x3c/span\x3e' + '\x3c/span\x3e{msg:htmlEncode2}\x3c/p\x3e')}]}, tasks:{collabType:'tasks'}};
  var cmpConfig = Ext.apply({}, types[this.collabType], {xtype:'collabGenericGrid', instanceMode:'widget', header:false, widgetInstance:this, moduleId:collaboration.Module.getDefaultCollabInstance(), filterId:this.getFilterId(), hideSearchBar:this.widgetConfiguration.hideSearchBar, hideBottomBar:this.widgetConfiguration.hideBottomBar, hideGridHeader:this.widgetConfiguration.hideGridHeader, widgetGridActions:this.widgetConfiguration.actions, widgetColumns:this.widgetConfiguration.columns, groupField:this.widgetConfiguration.groupField, 
  groupShowCount:this.widgetConfiguration.groupShowCount, groupCollapseNotToday:this.widgetConfiguration.groupCollapseNotToday, duedateFormat:this.widgetConfiguration.duedateFormat, sortColumn:this.widgetConfiguration.sortColumn, applyFocus:this.widgetConfiguration.focus});
  return this.add(cmpConfig);
}});
Ext.define('collaboration.widgets.Tasks', {extend:'collaboration.widgets.BaseWidget', alias:'widget.collabTasksWidget', statics:{WIDGET_NAME:BpcCommon.Api.getTranslation('COLLABORATION_TYPE_TASK_PLURAL'), WIDGET_DESCRIPTION:BpcCommon.Api.getTranslation('COLLABORATION_WIDGET_TASK_DESC') + '.', WIDGET_ICON_CLS:collaboration.assets.Artefacts.tasksIcon}, widgetConfigSettingMap:{filterId:{settingType:'collabFilterSelector'}, moduleId:{settingType:'moduleInstance'}, hideSearchBar:{settingType:'boolean'}, 
actions:{settingType:'json'}, hideBottomBar:{settingType:'boolean'}, hideGridHeader:{settingType:'boolean'}, focus:{settingType:'boolean'}, columns:{settingType:'json'}, duedateFormat:{settingType:'text'}, sortColumn:{settingType:'text'}}, config:{widgetConfiguration:{filterId:null, hideSearchBar:false, hideBottomBar:false, hideGridHeader:false, actions:null, columns:null, focus:false, duedateFormat:'d.m.Y H:i', sortColumn:'duedate'}}, collabType:'tasks'});
Ext.define('collaboration.widgets.Messages', {extend:'collaboration.widgets.BaseWidget', statics:{WIDGET_NAME:BpcCommon.Api.getTranslation('COLLABORATION_TYPE_MESSAGE_PLURAL'), WIDGET_DESCRIPTION:BpcCommon.Api.getTranslation('COLLABORATION_WIDGET_MESSAGE_DESC') + '.', WIDGET_ICON_CLS:collaboration.assets.Artefacts.messageIcon}, widgetConfigSettingMap:{filterId:{settingType:'collabFilterSelector'}, groupField:{settingType:'text'}, groupShowCount:{settingType:'boolean'}, groupCollapseNotToday:{settingType:'boolean'}, 
moduleId:{settingType:'moduleInstance'}, hideSearchBar:{settingType:'boolean'}, actions:{settingType:'json'}, hideBottomBar:{settingType:'boolean'}, hideGridHeader:{settingType:'boolean'}, focus:{settingType:'boolean'}, columns:{settingType:'json'}}, collabType:'messages'});
Ext.define('collaboration.widgets.Comments', {extend:'collaboration.widgets.BaseWidget', statics:{WIDGET_NAME:BpcCommon.Api.getTranslation('COLLABORATION_TYPE_COMMENT_PLURAL'), WIDGET_DESCRIPTION:BpcCommon.Api.getTranslation('COLLABORATION_WIDGET_COMMENT_DESC') + '.', WIDGET_ICON_CLS:collaboration.assets.Icons.commentsIcon}, widgetConfigSettingMap:{filterId:{settingType:'text'}}, collabType:'comments'});
Ext.define('collaboration.widgets.BaseStreamWidget', {extend:'collaboration.widgets.BaseWidget', alias:'widget.collabBaseStreamWidget', mixins:['BpcCommon.mixin.Widget'], layout:'card', activeItem:0, items:[], title:false, listeners:{afterrender:function() {
  this.setDbWrapperIconCls();
}}, height:500, moduleId:undefined, widgetConfigSettingMap:{moduleId:{settingType:'blogJournalSelector'}, topicId:{settingType:'text'}}, config:{widgetConfiguration:{topicId:'null'}}, wrongConfigPanel:{xtype:'container', padding:'10 10 10 10', items:[{xtype:'box', html:collaboration.Support.TR('COLLABORATION_CONFIG_INCOMPLETE')}, {xtype:'button', text:collaboration.Support.TR('COLLABORATION_CONFIG_SHOW'), iconCls:collaboration.assets.Icons.cog, handler:function() {
  var widget = this.up('collabBaseStreamWidget');
  widget.getLayout().setActiveItem(widget.settingPanel);
}}]}, initComponent:function() {
  Ext.log(this.self.getName() + ': initComponent');
  var me = this;
  try {
    this.initWidgetSettings();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent booting WidgetSettings', dump:e, stack:true, level:'error'});
  }
  this.callParent();
  this.initWidget();
  try {
    this.settingPanel = this.add({xtype:'bpcSettingPanel', settings:this.settingStore, buttons:[{text:collaboration.Support.TR('OK'), handler:this.configureWidget.bind(this)}, {text:collaboration.Support.TR('COLLABORATION_STANDARDVALUES'), handler:this.prepareWidgetConfiguration.bind(this, true)}]});
  } catch (e$1) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent booting Widget', dump:e$1, stack:true, level:'error'});
  }
  var config = this.getWidgetConfiguration();
  if (config.moduleId && config.moduleId !== 'null') {
    var moduleName = collaboration.Support.getSetting(config.moduleId, 'module_name');
    if (moduleName) {
      Ext.Function.defer(function() {
        me.up('dbWidgetWrapper').setTitle(moduleName);
      }, 500);
    }
  }
}, getWrapperReference:function() {
  var plugin = this.down('collabBlog');
  if (plugin && plugin.isComponent) {
    return plugin;
  } else {
    return null;
  }
}, updateTopicValue:function(topicId) {
  this.settingPanel.down('[fieldLabel\x3dtopicId]').setValue(topicId);
  var dbInstance = this.up('dbMain');
  if (dbInstance.isComponent) {
    dbInstance.getController().saveState();
  }
}, initWidget:function() {
  try {
    var config = this.getWidgetConfiguration();
    var cmp = null;
    if (config.topicId && config.topicId !== 'null') {
      cmp = {xtype:'panel', layout:'fit', items:[{xtype:'collabTopic', moduleId:this.moduleId, topicId:config.topicId, instanceMode:'widget', header:false, listeners:{afterrender:function(cmp) {
        Ext.defer(function() {
          var title = cmp.getViewModel().get('topicTitle');
          var initTitle = cmp.up('dbWidgetWrapper').getTitle();
          cmp.up('dbWidgetWrapper').setTitle(initTitle + ' - ' + title);
        }, 1000);
      }}}], bbar:[{xtype:'tbfill'}, {text:'Zurück', iconCls:collaboration.assets.Icons.angleDoubleLeft, disabled:!BpcCommon.Api.hasUserRole('bpcadmin'), handler:function(btn) {
        var cmp = btn.up('collabBlogWidget');
        cmp.updateTopicValue('');
      }}]};
    } else {
      if (Ext.isDefined(this.moduleId)) {
        cmp = {xtype:'widgetGridWrapper', parentWidgetCmp:'collabBlogWidget', childCmp:'collabBlog', gridCmp:'collabTopicList', moduleId:this.moduleId, listeners:{afterrender:function(cmp) {
          var instanceName = collaboration.Support.getSetting(cmp.moduleId, 'module_name');
          cmp.up('dbWidgetWrapper').setTitle(instanceName);
        }}};
      }
    }
    this.wrapper = this.add(cmp);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR bootstrapWidget', dump:e, stack:true, level:'error'});
  }
}, refreshWidget:function() {
  Ext.log(this.self.getName() + ': refreshWidget');
  try {
    this.down('grid').getStore().load();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR refreshWidget', dump:e, stack:true, level:'error'});
  }
}, configureWidget:function() {
  Ext.log(this.self.getName() + ': configureWidget');
  var card = this.settingPanel;
  if (card && card.isComponent && card.isVisible(true)) {
    card = this.wrapper;
  }
  this.getLayout().setActiveItem(card);
}, updateDbWrapperWithTotal:function(prefix, totalCount) {
  var wrapper = this.up('dbWidgetWrapper');
  if (wrapper && wrapper.isComponent) {
    wrapper.setTitle(prefix + ' (' + totalCount + ')');
  }
}, setDbWrapperIconCls:function() {
  Ext.log(this.self.getName() + ': customizeDashboardWrapper');
  try {
    var wrapper = this.up('dbWidgetWrapper');
    var grid = wrapper.down('grid');
    if (grid && grid.isComponent) {
      var iconCls = null;
      if (grid.collabType) {
        if (grid.collabType === 'comments') {
          iconCls = collaboration.assets.Icons.commentsIcon;
        }
        if (grid.collabType === 'tasks') {
          iconCls = collaboration.assets.Artefacts.tasksIcon;
        }
        if (grid.collabType === 'messages') {
          iconCls = collaboration.assets.Artefacts.messageIcon;
        }
      } else {
        iconCls = collaboration.assets.Artefacts.activitiesIcon;
      }
      if (iconCls !== null) {
        wrapper.setIconCls(iconCls);
      }
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR customizeDashboardWrapper', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('collaboration.widgets.Blog', {extend:'collaboration.widgets.BaseStreamWidget', alias:'widget.collabBlogWidget', statics:{WIDGET_NAME:collaboration.Support.TR('BLOG'), WIDGET_DESCRIPTION:collaboration.Support.TR('COLLABORATION_WIDGET_BLOG_DESC') + '.', WIDGET_ICON_CLS:collaboration.assets.Artefacts.topicActiveIcon}});
Ext.define('collaboration.widgets.admin.Webhooks', {extend:'collaboration.widgets.BaseWidget', alias:'widget.collabWebhooksWidget', statics:{WIDGET_NAME:collaboration.assets.Artefacts.webhookWidgetName, WIDGET_DESCRIPTION:collaboration.assets.Artefacts.webhookWidgetDescription, WIDGET_ICON_CLS:collaboration.assets.Artefacts.webhookIcon}, items:[{xtype:'collabGenericGrid', collabType:'webhooks', title:collaboration.assets.Artefacts.webhookWidgetTitle, header:false, instanceMode:'widget', updateParentCmpFn:function(totalCount) {
  this.up('collabWebhooksWidget').updateDbWrapperWithTotal(collaboration.assets.Artefacts.webhookWidgetTitle, totalCount);
}}]});
Ext.define('collaboration.widgets.Forum', {extend:'collaboration.widgets.BaseWidget', alias:'widget.collabForumWidget', statics:{WIDGET_NAME:collaboration.Support.TR('FORUM'), WIDGET_DESCRIPTION:collaboration.Support.TR('COLLABORATION_WIDGET_FORUM_DESC') + '.', WIDGET_ICON_CLS:collaboration.assets.Artefacts.collabIcon}, moduleId:undefined, widgetConfigSettingMap:{moduleId:{settingType:'forumSelector'}, topicId:{settingType:'text'}}, config:{widgetConfiguration:{topicId:'null'}}, initComponent:function() {
  try {
    this.initWidgetSettings();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent booting WidgetSettings', dump:e, stack:true, level:'error'});
  }
  this.callParent();
  try {
    this.addSettingsPanel();
  } catch (e$2) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent booting Widget', dump:e$2, stack:true, level:'error'});
  }
  this.initWidget();
}, initWidget:function() {
  var config = this.getWidgetConfiguration();
  if (config.moduleId && config.moduleId !== 'null') {
    if (config.topicId && config.topicId !== 'null') {
      this.getLayout().setActiveItem(this.add(this.loadForumTopic(config.moduleId, config.topicId)));
    } else {
      this.getLayout().setActiveItem(this.loadForum(config.moduleId));
      var forumName = collaboration.Support.getSetting(config.moduleId, 'module_name');
      if (forumName) {
        Ext.Function.defer(function() {
          this.up('dbWidgetWrapper').setTitle(forumName);
        }.bind(this), 500);
      }
    }
    return;
  }
  this.getLayout().setActiveItem(this.settingPanel);
}, loadForumTopic:function(moduleId, topicId) {
  try {
    if (moduleId) {
      var topicMeta = collaboration.Api.getTopicById(topicId) || {};
      return this.add({xtype:'panel', layout:'fit', items:[{xtype:'collabTopic', moduleId:moduleId, topicId:topicId, channelId:topicMeta['channelId'], instanceMode:'widget', holderCmp:this, header:false, listeners:{afterrender:function(cmp) {
        Ext.defer(function() {
          var topicMeta = collaboration.Api.getTopicById(topicId);
          if (!topicMeta) {
            return;
          }
          var title = topicMeta['topic'] || 'EMPTY';
          var initTitle = collaboration.Support.getSetting(moduleId, 'module_name') || '';
          var channelMeta = collaboration.Api.getChannelById(topicMeta['channelId']) || '';
          var channelTitle = '\x3cspan style\x3d"color:' + (channelMeta['color'] || 'EMPTY') + '"\x3e # \x3c/span\x3e' + (channelMeta['channel'] || 'EMPTY');
          cmp.up('dbWidgetWrapper').setTitle(initTitle + ' ( ' + channelTitle + ' - ' + title + ' )');
        }, 1000);
      }}}], bbar:[{xtype:'tbfill'}, {text:collaboration.Support.TR('BACK'), iconCls:collaboration.assets.Icons.angleDoubleLeft, handler:function(btn) {
        var cmp = btn.up('collabForumWidget');
        cmp.updateTopicValue('');
      }}]});
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR bootstrapWidget', dump:e, stack:true, level:'error'});
  }
}, loadForum:function(moduleId) {
  try {
    if (moduleId) {
      return this.add({xtype:'widgetGridWrapper', parentWidgetCmp:'collabForumWidget', childCmp:'collabForumWidgetGrid', moduleId:moduleId});
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR bootstrapWidget', dump:e, stack:true, level:'error'});
  }
}, updateTopicValue:function(topicId) {
  this.settingPanel.down('[fieldLabel\x3dtopicId]').setValue(topicId);
  var dbInstance = this.up('dbMain');
  if (dbInstance.isComponent) {
    dbInstance.getController().saveState();
  }
}, configureWidget:function() {
  var card = this.settingPanel;
  var forumName;
  if (card && card.isComponent && card.isVisible(true)) {
    var forumSelector = card.down('combo');
    var topicIdField = card.down('[fieldLabel\x3dtopicId]');
    if (forumSelector) {
      forumName = forumSelector.lastMutatedValue;
      var moduleId = forumSelector.getValue();
      this.moduleId = moduleId;
      if (topicIdField && topicIdField.getValue() !== '' && topicIdField.getValue() !== 'null') {
        var topicId = topicIdField.getValue();
        var topicMeta = collaboration.Api.getTopicById(topicId);
        if (Ext.isDefined(topicId) && topicMeta && Ext.isObject(topicMeta)) {
          card = this.loadForumTopic(moduleId, topicId);
        } else {
          card = this.loadForum(moduleId);
        }
      } else {
        card = this.loadForum(moduleId);
      }
    }
  }
  this.getLayout().setActiveItem(card);
  if (forumName) {
    this.up('dbWidgetWrapper').setTitle(forumName);
  }
}});
Ext.define('collaboration.widgets.InboxOverview', {extend:'collaboration.widgets.BaseWidget', alias:'widget.collabWidgetInboxOverview', statics:{WIDGET_NAME:collaboration.Support.TR('INBOX_EN') + ' - ' + collaboration.Support.TR('OVERVIEW'), WIDGET_DESCRIPTION:BpcCommon.Api.getTranslation('COLLABORATION_WIDGET_INBOX_DESC') + '.', WIDGET_ICON_CLS:'x-fal fa-chart-bar'}, widgetConfigSettingMap:{iconOnlyInTab:{settingType:'bool', value:false}}, initComponent:function() {
  this.initWidgetSettings();
  this.callParent(arguments);
  try {
    this.addSettingsPanel();
  } catch (e) {
  }
  this.initWidget();
}, initWidget:function() {
  this.getLayout().setActiveItem(this.loadWidgetComponents());
}, loadWidgetComponents:function() {
  var $jscomp$this = this;
  var overviewConfigBase = {xtype:'tabpanel', tabPosition:'left', tabRotation:0, defaults:{moduleId:collaboration.Module.getDefaultCollabInstance(), instanceMode:'widget', iconAlign:'top', textAlign:'center', tabConfig:{width:140}}, items:[{xtype:'collabChart', title:collaboration.Support.TR('OVERVIEW'), iconCls:'x-fal fa-chart-bar'}, {xtype:'collabGenericGrid', collabType:'comments'}, {xtype:'collabGenericGrid', collabType:'messages'}, {xtype:'collabGenericGrid', collabType:'tasks'}], bbar:[{xtype:'collabGridInput', 
  flex:3, inTabpanel:true, collabType:this.collabType, gridReference:'collabGenericGrid'}, {xtype:'collabDateRangeField', inTabpanel:true, gridReference:'collabGenericGrid'}], reloadData:Ext.emptyFn()};
  var events = ['newComment', 'newMessage', 'newTask', 'itemDeleted', 'messageUpdated', 'taskUpdated', 'itemUpdated'];
  this.bpcAutolisteners = events.map(function(event) {
    var cfg = {destroyable:true};
    cfg[event] = $jscomp$this.refreshWidget.bind($jscomp$this);
    return Ext.on(cfg);
  });
  this.wrapper = this.add(overviewConfigBase);
  return this.wrapper;
}, destroyAutoListeners:function() {
  this.bpcAutolisteners.map(function(obj) {
    return obj.destroy();
  });
  this.bpcAutolisteners = [];
}, refreshWidget:function() {
  this.wrapper.reloadData();
}, configureWidget:function() {
  this.getLayout().setActiveItem(this.settingPanel && this.settingPanel.isComponent && this.settingPanel.isVisible(true) ? this.wrapper : this.settingPanel);
}});
Ext.define('collaboration.BpcInterface', {extend:'BpcCommon.BpcInterface', requires:['collaboration.Support', 'collaboration.assets.Icons', 'collaboration.assets.Artefacts', 'collaboration.widgets.Tasks', 'collaboration.widgets.Messages', 'collaboration.widgets.Comments', 'collaboration.widgets.Blog', 'collaboration.widgets.admin.Webhooks', 'collaboration.widgets.Forum', 'collaboration.widgets.InboxOverview'], config:{moduleConfigurationComponents:['collaborationConfigMain'], requiredModules:[], 
widgets:['collaboration.widgets.Tasks', 'collaboration.widgets.Messages', 'collaboration.widgets.Comments', 'collaboration.widgets.admin.Webhooks', 'collaboration.widgets.Blog', 'collaboration.widgets.Forum', 'collaboration.widgets.InboxOverview'], baseModuleView:'collaboration.view.module.Collaboration', version:'@version@', build:'@build@'}});
Ext.define('collaboration.components.base.ActivityCard', {extend:'Ext.container.Container', alias:'widget.collabActivityCard', layout:{type:'vbox', align:'stretch'}, initComponent:function() {
  this.activityBoy = Ext.create({xtype:'box', padding:'2 0 2 0'});
  this.timebox = Ext.create({xtype:'box', padding:'2 0 2 0', tooltipValue:'', listeners:{afterrender:function(cmp) {
    var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, listeners:{beforeshow:function(tt) {
      tt.update(collaboration.Support.TR('CREATED') + ':' + tt.targetCard.tooltipValue);
    }}});
  }}, style:{fontSize:'12px', whiteSpace:'nowrap', overflow:'hidden', textOverflow:'ellipsis'}});
  this.items = [this.activityBoy, this.timebox];
  this.callParent(arguments);
}, refreshData:function(record) {
  if (record) {
    this.activityBoy.update('\x3cspan style\x3d"color:blue !important; font-weight: bold; margin-right: 2px;"\x3e' + this.getActivityIcon(record.get('artefactType')) + '\x3c/span\x3e' + this.getActivityTranslation(record.get('header')) + '\x3cspan style\x3d"margin-left: 2px; color:blue;"\x3e' + (record.get('creator') || ' -1 ') + '\x3c/span\x3e');
    var created = record.get('created');
    this.timebox.tooltipValue = Ext.Date.format(created, 'd.m.Y H:i:s');
    this.timebox.update('\x3cspan class\x3d"' + collaboration.Support.getBpcIconCls('fa-clock') + '"\x3e\x3c/span\x3e\x26nbsp;\x26nbsp;' + collaboration.Module.formatDate(created));
  }
}, getActivityIcon:function(artefact) {
  if (artefact === 'comments' || artefact === 'tasks' || artefact === 'messages') {
    return '\x3cspan class\x3d"x-fa ' + collaboration.Module.getAssetForType(artefact, 'iconCls') + '"\x3e\x3c/span\x3e\x26nbsp;\x26nbsp;' + collaboration.Module.getAssetForType(artefact, 'singleText');
  }
  if (artefact === 'topic') {
    return '\x3cspan class\x3d"x-fal ' + 'fa-quote-right' + '"\x3e\x3c/span\x3e\x26nbsp;\x26nbsp;' + collaboration.Support.TR('TOPIC');
  }
  if (artefact === 'channel') {
    return '\x3cspan class\x3d"x-fal ' + 'fa-comment-o' + '"\x3e\x3c/span\x3e\x26nbsp;\x26nbsp;' + collaboration.Support.TR('CHANNEL');
  }
  if (artefact === 'improve - bulkDelete') {
    return '\x3cspan class\x3d"' + collaboration.assets.Icons.trash + '"\x3e\x3c/span\x3e\x26nbsp;\x26nbsp;' + collaboration.Support.TR('ARTEFACTS');
  }
  if (artefact === 'improve') {
    return ' IMPROVE ';
  }
  return ' -1 ';
}, getActivityTranslation:function(activityName) {
  var activityMap = {create:collaboration.Support.TR('WAS_CREATED_BY'), assign:collaboration.Support.TR('WAS_DELEGATED_BY'), improve:collaboration.Support.TR('IMPROVE'), topicCreated:collaboration.Support.TR('WAS_CREATED_BY'), channelCreated:collaboration.Support.TR('WAS_CREATED_BY'), upload:collaboration.Support.TR('WAS_UPLOADED_BY'), 'delete':collaboration.Support.TR('WAS_DELETED_BY')};
  return activityMap[activityName] ? activityMap[activityName] : collaboration.Support.TR('NOT_FOUND');
}});
Ext.define('collaboration.components.chart.Chart', {extend:'Ext.panel.Panel', alias:'widget.collabChart', listeners:{'afterrender':function(cmp) {
  Ext.Function.defer(function() {
    cmp.initChart();
  }, 50);
}}, initChart:function() {
  this.setLoading(BpcCommon.Api.getTranslation('COLLABORATION_INBOX_OVERVIEW_WILL_BE_LOADED'));
  this.loadOverData().then(function(data) {
    this.createChartConfig(data);
  }.bind(this), function(error) {
    collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ERROR_WHILE_INIT_INBOX_OVERVIEW'));
    this.setLoading(false);
  }.bind(this));
}, chart:undefined, createChart:function(chartConfig) {
  this.chart = AmCharts.makeChart(this.id + '-body', chartConfig);
}, createChartConfig:function(data) {
  var cfg = {'type':'serial', 'theme':'light', 'dataProvider':data || [], 'valueAxes':[{'minimum':0, 'axisAlpha':0, 'dashLength':4, 'position':'left'}], 'startDuration':1, 'graphs':[{'balloonText':"\x3cspan style\x3d'font-size:13px;'\x3e[[category]]: \x3cb\x3e[[value]]\x3c/b\x3e\x3c/span\x3e", 'labelText':'[[value]]', 'colorField':'color', 'cornerRadiusTop':8, 'customBulletField':'bullet', 'fillAlphas':0.8, 'lineAlpha':0, 'type':'column', 'valueField':'points'}], 'marginTop':0, 'marginRight':0, 'marginLeft':0, 
  'marginBottom':0, 'autoMargins':false, 'categoryField':'name', 'categoryAxis':{'axisAlpha':0, 'gridAlpha':0, 'inside':true, 'tickLength':0}};
  var me = this;
  cfg.listeners = [{'event':'clickGraphItem', 'method':function(event) {
    var collabTypeDePlural = event.item.category, collabType = null;
    if (collabTypeDePlural.toLowerCase() === 'kommentare') {
      collabType = 'comments';
    } else {
      if (collabTypeDePlural.toLowerCase() === 'aufgaben') {
        collabType = 'tasks';
      } else {
        if (collabTypeDePlural.toLowerCase() === 'nachrichten') {
          collabType = 'messages';
        } else {
          var states = collaboration.Support.getSetting(collaboration.Module.getDefaultCollabInstance(), 'default_taskStates');
          var found = null;
          Ext.iterate(states, function(state, obj) {
            Ext.iterate(obj, function(key, value) {
              if (key === 'defaultText') {
                if (value === collabTypeDePlural) {
                  found = state;
                  return false;
                }
              }
            });
          });
          if (found !== null) {
            var tabpanel = me.up('tabpanel');
            var grids = tabpanel.query('widgetGridWrapper');
            if (grids !== null && grids.length > 0) {
              Ext.Array.each(grids, function(grid) {
                if ('filterId' in grid) {
                  if (grid.filterId === 'tasksWidget_' + found) {
                    tabpanel.setActiveTab(grid);
                    return false;
                  }
                }
              });
            }
          }
          return;
        }
      }
    }
    if (collabType !== null) {
      var tabpanel = me.up('tabpanel');
      var grids = tabpanel.query('collabGenericGrid');
      if (grids !== null && grids.length > 0) {
        Ext.Array.each(grids, function(grid) {
          if ('collabType' in grid) {
            if (grid.collabType === collabType) {
              tabpanel.setActiveTab(grid);
              return false;
            }
          }
        });
      }
    }
  }}];
  this.createChart(cfg);
  this.setLoading(false);
}, loadOverData:function(query, startDate, endDate) {
  var me = this;
  return new Ext.Promise(function(resolve, reject) {
    if (collaboration.Api.configCorrect() === true) {
      if (me.collabType && Ext.isDefined(me.collabType) && me.collabType === 'tasks') {
        collaboration.Api.getOverview(me.collabType, query, startDate, endDate).then(function(successJson) {
          if (Ext.isObject(successJson) && 'values' in successJson) {
            var data = [];
            Ext.Array.each(successJson.values, function(item) {
              var state = item.value;
              var c = collaboration.Support.fallbackStateColor;
              var states = collaboration.Support.getSetting(collaboration.Module.getDefaultCollabInstance(), 'default_taskStates');
              if (state in states) {
                if ('defaultText' in states[state]) {
                  state = states[state].defaultText;
                }
              }
              var obj = {name:state, points:item.count, color:c};
              data.push(obj);
            });
            resolve(data);
          } else {
            reject(null);
          }
        }, function(reason) {
          reject(null);
        });
      } else {
        collaboration.Api.getOverview(undefined, query, startDate, endDate).then(function(successJson) {
          if (Ext.isObject(successJson)) {
            var data = [];
            var graphDisplayOrder = ['comments', 'messages', 'tasks'];
            Ext.each(graphDisplayOrder, function(val) {
              Ext.Object.each(successJson, function(k, v) {
                k = k.replace('Count', '');
                var c = collaboration.Module.getAssetForType(k, 'chartColor');
                var text = collaboration.Module.getAssetForType(k, 'plural');
                var obj = {name:text, points:v, color:c};
                if (val === k) {
                  data.push(obj);
                }
              });
            });
            resolve(data);
          } else {
            reject(null);
          }
        }, function(reason) {
          reject(null);
        });
      }
    } else {
      reject(null);
    }
  });
}, iconOnly:false, initComponent:function() {
  if (this.iconOnly) {
    this.title = null;
  }
  this.callParent(arguments);
}, refreshChart:function() {
  this.setLoading(BpcCommon.Api.getTranslation('COLLABORATION_INBOX_OVERVIEW_WILL_BE_UPDATED'));
  this.loadOverData().then(function(data) {
    this.chart.dataProvider = data;
    this.chart.validateData();
    this.setLoading(false);
  }.bind(this), function(error) {
    collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ERROR_WHILE_UPDATING_INBOX_OVERVIEW'));
    this.setLoading(false);
  }.bind(this));
}});
Ext.define('collaboration.components.base.CollabActivityGrid', {extend:'Ext.grid.Panel', alias:'widget.collabActivityGrid', requires:[], sortableColumns:false, iconCls:collaboration.assets.Artefacts.activitiesIcon, title:collaboration.assets.Artefacts.activitiesText, forceFit:true, collabType:'activities', initComponent:function() {
  try {
    this.columns = [{text:collaboration.Support.TR('CREATED'), dataIndex:'created', xtype:'datecolumn', format:'d.m.Y', flex:1}, {text:collaboration.Support.TR('CREATOR'), dataIndex:'creator', flex:1}, {text:collaboration.Support.TR('HEADER_ACTION'), dataIndex:'header', flex:2}, {text:collaboration.Support.TR('MSG_ITEM'), dataIndex:'msg', flex:3}, {text:collaboration.Support.TR('REFERENCE'), dataIndex:'reference', flex:1}];
    this.store = this.createStore(this.moduleId);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent', dump:e, stack:true, level:'error'});
  }
  this.callParent(arguments);
}, createStore:function() {
  return Ext.create('Ext.data.BufferedStore', {model:'collaboration.model.CollabActivityData', leadingBufferZone:50, trailingBufferZone:50, pageSize:300, proxy:{type:'ajax', url:collaboration.Module.getBaseUrl() + 'artefacts?type\x3dactivities', reader:{type:'json', rootProperty:'items', totalProperty:'total'}}, autoLoad:true});
}});
Ext.define('collaboration.components.base.CollabGrid', {extend:'Ext.grid.Panel', alias:'widget.collabGenericGrid', controller:'collabGenericGridController', viewModel:{type:'collabGridModel'}, requires:[], iconCls:collaboration.assets.Artefacts.tasksIcon, forceFit:true, instanceMode:'module', collabType:undefined, collabMethod:undefined, updateGridTypeWrapper:true, contextMenuDisabled:false, header:false, iconOnly:false, collabTabMode:false, filterId:undefined, duedateFormat:'d.m.Y H:i', prepareGridForWidget:function() {
  var $jscomp$this = this;
  var plugins = this.getPlugins();
  this.plugins = plugins || [];
  this.selModel = {mode:'SINGLE'};
  var activeColumns = ['creator', 'header', 'created'].concat(this.collabType === 'messages' ? ['to'] : []).concat(this.collabType === 'tasks' ? ['lastupdate', 'duedate', 'overdue', 'state'] : []).concat(['bpcactions']);
  this.columns.push({xtype:'collabWidgetActionColumn', dataIndex:'bpcactions', text:this.getColumnHeaderHtmlString(collaboration.assets.Icons.edit, 'color:#4CAF50', collaboration.Support.TR('ACTIONS'))});
  this.columns.map(function(col) {
    col.hidden = activeColumns.indexOf(col.dataIndex) === -1;
    if ($jscomp$this.widgetColumns) {
      col.hidden = $jscomp$this.widgetColumns.indexOf(col.dataIndex) === -1;
    }
    return col;
  });
  if (this.widgetColumns) {
    this.columns.sort(function(a, b) {
      var indexA = $jscomp$this.widgetColumns.indexOf(a.dataIndex);
      var indexB = $jscomp$this.widgetColumns.indexOf(b.dataIndex);
      return indexA > indexB ? 1 : -1;
    });
  }
  if (this.collabType === 'webhooks') {
    this.columns = collaboration.Module.webhookColumns();
  }
  if (this.hideGridHeader) {
    this.hideHeaders = true;
  }
  this.tbar = [{xtype:'collabGridInput', hidden:this.hideSearchBar === true, _grid:this, flex:1, collabType:this.collabType}];
}, prepareGridForModule:function() {
  this.selModel = {mode:'MULTI'};
  if (Ext.isDefined(this.collabType) && this.collabType !== 'activities' && this.instanceMode === 'module') {
    this.dockedItems = this.setupToolbar();
  }
}, filter:undefined, customFilterEnabled:true, initComponent:function() {
  var $jscomp$this = this;
  try {
    if (this.filter) {
      this.filterId = this.filter;
      this.customFilterEnabled = false;
    }
    var skipDefaults = this.isCustomView || false;
    if (skipDefaults === false) {
      var asset = collaboration.Module.getAssetForType(this.collabType, 'iconCls');
      if (asset) {
        this.iconCls = asset;
      }
      if (this.collabType && Ext.isString(this.collabType)) {
        var title = collaboration.Module.getAssetForType(this.collabType, 'plural');
        if (title) {
          this.title = Ext.isDefined(this.iconOnly) && this.iconOnly === true ? null : title;
        }
      }
    }
    if (this.collabTabMode === true) {
      var tooltip = collaboration.Module.getAssetForType(this.collabType, 'plural');
      if (tooltip !== null) {
        if (this.filterId && this.filterId !== '') {
          tooltip += ' - Filter: ' + this.filterId;
        }
        if ('tabConfig' in this) {
          this.tabConfig.tooltip = tooltip;
        } else {
          this.tabConfig = {tooltip:tooltip};
        }
      }
    }
    this.store = this.createStore(this.moduleId, this.collabType, this.collabMethod);
    if (this.store.groupingEnabled) {
      var groupShowCount = this.groupShowCount;
      this.features = [{ftype:'grouping', groupHeaderTpl:Ext.create('Ext.XTemplate', '\x3cdiv\x3e\x3cspan style\x3d"display: inline-block;width:30%;"\x3e{groupValue:this.formatDay}\x3c/span\x3e\x3cspan style\x3d"display: inline-block;width:55%;text-align: right;"\x3e{children:this.formatTitle}\x3c/span\x3e\x3cspan style\x3d"display: inline-block;width:15%;text-align: right;"\x3e{groupValue:this.formatDate}\x3c/span\x3e\x3c/div\x3e', {formatDate:function(name) {
        return Ext.Date.format(name, 'd.m.Y');
      }, formatTitle:function(children) {
        children = children === undefined ? [] : children;
        return groupShowCount ? ' ( ' + children.length + ' ) ' : '';
      }, formatDay:function(name) {
        return collaboration.Support.TR(collaboration.Support.days[String(name.getDay())]);
      }})}];
    }
    this.columns = this.getDefaultGridColumns(this.collabType);
    if (this.instanceMode === 'module') {
      this.prepareGridForModule();
    }
    if (this.instanceMode === 'widget') {
      this.prepareGridForWidget();
      this.updateGridTypeWrapper = false;
    }
    if (this.collabType === 'comments') {
      Ext.Array.each(this.columns, function(column) {
        if (column.dataIndex === 'header') {
          column.hidden = true;
        }
        if (column.dataIndex === 'creator') {
          column.hidden = true;
        }
        if (column.dataIndex === 'msg') {
          column.text = '\x3cspan class\x3d"' + collaboration.assets.Icons.commentsIcon + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + collaboration.Support.TR('COLLABORATION_TYPE_COMMENT_PLURAL');
        }
      });
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent', dump:e, stack:true, level:'error'});
  }
  this.defaultEmptyText = collaboration.Support.TR('NO_DATA_' + (this.collabType || '').toUpperCase());
  if (!Ext.isDefined(this.viewConfig)) {
    this.viewConfig = {emptyText:this.getEmptyTextDiv(this.defaultEmptyText)};
  }
  this.callParent(arguments);
  if (Ext.isDefined(this.collabType)) {
    this.getViewModel().set('collabDataType', this.collabType);
  }
  collaboration.Api.getTags().then(function(r) {
    r = r || {};
    $jscomp$this.tags = r.items || [];
  })['finally'](function() {
    $jscomp$this.store.load({callback:function(r, op, success) {
      if ($jscomp$this.instanceMode === 'widget' && $jscomp$this.sortColumn) {
        $jscomp$this.store.sort($jscomp$this.sortColumn);
      }
    }});
  });
}, scrollToBottom:function() {
  var $jscomp$this = this;
  try {
    Ext.defer(function() {
      var v = $jscomp$this.getView();
      if (v) {
        v.scrollBy(0, 999999);
      }
    }, 500);
  } catch (e) {
  }
}, getEmptyTextDiv:function(text, iconCls) {
  iconCls = iconCls || collaboration.Module.getAssetForType(this.collabType, 'iconCls');
  var span = collaboration.Support.getSpanForIconCls(iconCls, '#121212', '30px');
  return '\x3cdiv style\x3d"width: 100%;height: 90%; justify-content: center;display: flex;"\x3e\x3cdiv style\x3d"display: flex; flex-direction:column;justify-content:center;align-items:center;width: 80%;"\x3e\x3cdiv\x3e' + span + '\x3c/div\x3e\x3cdiv\x3e' + text + '\x3c/div\x3e\x3c/div\x3e\x3c/div\x3e';
}, tags:[], getDefaultGridColumns:function(type, tags) {
  var $jscomp$this = this;
  tags = tags === undefined ? [] : tags;
  var moduleFeatures = BpcCommon.Api.getSetting('collaboration', 'module_features') || [];
  moduleFeatures = moduleFeatures.filter(function(feature) {
    return feature.enabled;
  });
  var hideChannelAndTopic = moduleFeatures.length === 0;
  var obj = collaboration.Support.getMetaByType(type);
  var metaCols = Object.keys(obj).map(function(dataIndex) {
    var isDate = obj[dataIndex].type && obj[dataIndex].type === 'date';
    var format = obj[dataIndex].format || 'd.m.Y';
    var key = dataIndex;
    return {dataIndex:dataIndex, xtype:'gridcolumn', text:obj[dataIndex].label ? obj[dataIndex].label : dataIndex, renderer:function(v, m, r) {
      var meta = r.get('meta') || {};
      var value = meta[key] || '';
      if (isDate && value.length) {
        return Ext.Date.format(Ext.Date.parse(value), format);
      } else {
        return value;
      }
    }};
  });
  if (this.tags && this.tags.length > 0 && tags.length === 0) {
    tags = this.tags;
  }
  var columns = [{text:this.getColumnHeaderHtmlString(collaboration.assets.Icons.eye, 'color:#4CAF50', collaboration.Support.TR('CREATED')), dataIndex:'created', xtype:'datecolumn', width:150, format:this.store.groupingEnabled ? 'H:i' : 'd.m.Y H:i:s'}, Ext.apply({width:120}, collaboration.Module.getUserColumn('\x3cspan class\x3d"x-fal fa-users" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + collaboration.Support.TR('FROM'), 'creator')), {text:this.getColumnHeaderHtmlString(collaboration.assets.Icons.bookmark, 
  'color:#4CAF50', collaboration.Support.TR('HEADER')), dataIndex:'header', flex:2, renderer:function(v, m, r) {
    var s = r.get('state'), style;
    if (s) {
      var styleProp = collaboration.Support.getStateValue($jscomp$this.moduleId, s, 'style');
      if (styleProp && styleProp.length) {
        style = styleProp;
      }
    }
    return style ? '\x3cspan style\x3d"' + style + '"\x3e' + v + '\x3c/span\x3e' : v;
  }}, {text:this.getColumnHeaderHtmlString(collaboration.assets.Icons.comment, 'color:#4CAF50', collaboration.Support.TR('TYPE_MESSAGE_SINGLE')), dataIndex:'msg', flex:3, formatter:'htmlEncode2'}].concat(metaCols).concat([{text:this.getColumnHeaderHtmlString('x-fal fa-tags', 'color:#4CAF50', collaboration.Support.TR('TAGS')), dataIndex:'tags', flex:1, hidden:true, formatter:'htmlEncode2'}, {dataIndex:'tagIds', text:this.getColumnHeaderHtmlString('x-fal fa-tags', 'color:#4CAF50', collaboration.Support.TR('TAGS')), 
  renderer:function(value, meta, record) {
    value = value || [];
    if (Ext.isString(value)) {
      value = [value];
    }
    var tagSpans = value.map(function(id, i) {
      id = id || '';
      var match = $jscomp$this.tags.filter(function(i) {
        return i.artefactId === id.trim();
      });
      return '\x3cdiv style\x3d"' + collaboration.Support.getTagBoxStyle(false, match.length ? match[0].color : undefined) + (i === 0 ? 'margin-left: auto;' : '') + ' margin-bottom: 0.1em;"\x3e ' + (match.length ? match[0].name : id) + '\x3c/div\x3e';
    });
    return tagSpans.join('');
  }}, {text:this.getColumnHeaderHtmlString(collaboration.assets.Icons.comment, 'color:#4CAF50', collaboration.Support.TR('CHANNEL')), dataIndex:'channelId', hidden:hideChannelAndTopic, flex:3, renderer:function(value, meta) {
    var channelName;
    if (value) {
      var channels = collaboration.Api.streams.channels.getAll();
      Ext.Array.each(channels, function(channel) {
        if ('channel' in channel && Ext.isDefined(channel['channel']) && value === channel['channelId']) {
          channelName = channel.channel;
          if (channel.instanceType === 'blog') {
            channelName = channel.instanceName;
          }
          return false;
        }
      });
      meta.tdAttr = 'data-qtip\x3d"' + Ext.util.Format.htmlDoubleEncode(channelName || '') + '"';
    }
    return Ext.util.Format.htmlEncode2(channelName || '');
  }}, {text:this.getColumnHeaderHtmlString(collaboration.assets.Icons.comment, 'color:#4CAF50', collaboration.Support.TR('COMMENT')), dataIndex:'topicId', hidden:hideChannelAndTopic, flex:3, renderer:function(value, meta) {
    var topicName;
    if (value) {
      var topicsObject = collaboration.Api.streams.topics.getAll();
      Ext.iterate(topicsObject, function(channelId, arrayOfTopics) {
        Ext.Array.each(arrayOfTopics, function(topicObj) {
          if ('topic' in topicObj && Ext.isDefined(topicObj['topic']) && value === topicObj['topicId']) {
            topicName = topicObj.topic;
            return false;
          }
        });
      });
      meta.tdAttr = 'data-qtip\x3d"' + Ext.util.Format.htmlDoubleEncode(topicName || '') + '"';
    }
    return Ext.util.Format.htmlEncode2(topicName || '');
  }}, {text:this.getColumnHeaderHtmlString(collaboration.assets.Icons.comment, 'color:#4CAF50', collaboration.Support.TR('REFERENCE')), dataIndex:'reference', flex:2, renderer:function(value, meta, rec, rowIndex, colIndex, store, view) {
    if (value) {
      var refType = 'Standard';
      var supportedModules = ['monitor', 'vam'];
      var baseModuleId = rec.get('src_moduleId');
      if (baseModuleId && supportedModules.indexOf(baseModuleId) !== -1) {
        var moduleId = rec.get('src_instanceId');
        if (moduleId) {
          var idKey = collaboration.Support.getSetting(moduleId, 'column_id');
          if (idKey) {
            idKey = collaboration.Module.capitalize(idKey);
            var modName = collaboration.Support.getSetting(moduleId, 'module_name');
            idKey = collaboration.Support.TR('REFERENCE') + ': ' + idKey;
            meta.tdAttr = 'data-qtip\x3d"' + idKey + ' ' + Ext.util.Format.htmlDoubleEncode(value) + '\x3cbr\x3e' + 'Modul: ' + modName + '\x3cbr\x3e' + 'Referenztyp: ' + refType + '"';
          }
        }
      }
    }
    var referenceName = rec.get('referenceName');
    if (referenceName && referenceName.length) {
      value = referenceName;
    }
    return Ext.util.Format.htmlEncode2(value);
  }}]);
  if (type === 'messages') {
    columns.push(collaboration.Module.getToColumn());
  }
  if (type === 'tasks') {
    columns.push({text:this.getColumnHeaderHtmlString(collaboration.assets.Icons.calendar, 'color:#4CAF50', collaboration.Support.TR('UNTIL')), dataIndex:'duedate', xtype:'datecolumn', width:this.duedateFormat.length * 20, renderer:function(value, meta) {
      if (value) {
        value = Ext.Date.format(value, $jscomp$this.duedateFormat);
        meta.tdAttr = 'data-qtip\x3d"' + value + '"';
      }
      return value;
    }});
    columns.push({dataIndex:'overdue', width:40, minWidth:40, renderer:function(value, metaData) {
      return collaboration.Support.colorCodeImage(value === true ? 'F06292' : '81C784');
    }});
    columns.push({text:this.getColumnHeaderHtmlString(collaboration.assets.Icons.calendar, 'color:#4CAF50', collaboration.Support.TR('STATE')), dataIndex:'state', renderer:function(v, record) {
      return collaboration.Support.renderState(v, undefined, true);
    }});
    columns.push(collaboration.Module.getToColumn());
  }
  columns.push({xtype:'actioncolumn', width:30, items:[{iconCls:collaboration.assets.Icons.externalLink, tooltip:collaboration.Support.TR('COLLABORATION_JUMP_TP_MODULE'), isActionDisabled:function(tableView, rowIndex, colIndex, item, record) {
    return !record.get('reference');
  }, handler:function(tableView, rowIndex, colIndex, item, e, record, row) {
    var instanceId = record.get('src_instanceId');
    var recordId = record.get('reference');
    if (instanceId && recordId) {
      collaboration.Api.jumpToReference(instanceId, recordId);
    } else {
      BpcCommon.Api.showNotification('Jump not possible');
    }
  }}]});
  Ext.Array.each(columns, function(col) {
    if ('groupable' in col) {
      col.groupable = false;
    } else {
      col.groupable = false;
    }
  });
  return columns;
}, getColumnHeaderHtmlString:function(icon, color, text) {
  return '\x3cspan class\x3d"' + icon + '" style\x3d"' + color + '"\x3e\x3c/span\x3e\x26nbsp;' + text;
}, createStore:function(instanceId, collabType, collabMethod, autoLoad) {
  autoLoad = autoLoad === undefined ? false : autoLoad;
  var url = collaboration.Module.getBaseUrl() + instanceId + '/' + collabType;
  if (collaboration.Support.artefactTypes.indexOf(collabType) !== -1) {
    url = collaboration.Module.getBaseUrl() + 'artefacts?type\x3d' + collabType + (Ext.isDefined(collabMethod) ? '\x26folder\x3d' + collabMethod : '');
  }
  var storeConfig = {model:collaboration.Module.getAssetForType(collabType, 'model'), leadingBufferZone:50, trailingBufferZone:50, pageSize:collaboration.Support.artefactTypes.indexOf(collabType) === -1 && this.instanceMode === 'widget' ? 30 : 100, proxy:{type:'ajax', url:url, reader:{type:'json', rootProperty:'items', totalProperty:'total'}, extraParams:this.filterId && this.filterId.length ? {filterId:this.filterId} : undefined, listeners:{exception:this.onLoadException.bind(this)}}, autoLoad:autoLoad, 
  listeners:{beforesort:this.onBeforeSort.bind(this), load:this.onLoad.bind(this)}};
  if (this.instanceMode === 'widget' && this.groupField && this.groupField.length) {
    storeConfig.groupField = this.groupField && this.groupField === 'created' ? 'groupCreated' : this.groupField;
    storeConfig.groupingEnabled = true;
    storeConfig.remoteSort = true;
    storeConfig.sorters = [{property:'created', direction:'ASC'}];
  }
  return Ext.create(this.instanceMode === 'widget' ? 'Ext.data.Store' : 'Ext.data.BufferedStore', storeConfig);
}, errorText:'Achtung - Fehler bei der Abfrage der Daten - Die Operation kann nicht ausgeführt werden.', onLoad:function(s, records, success) {
  if (success) {
    this.setEmptyText();
  } else {
    this.setEmptyText(this.errorText);
  }
  if (s.groupingEnabled && this.groupCollapseNotToday) {
    var v = this.getView();
    if (v) {
      var groups = v.features[1].getCache();
      var d = new Date;
      d.setHours(0, 0, 0, 0);
      var groupName = d.toString();
      Object.keys(groups).map(function(name) {
        if (name !== groupName) {
          v.features[1].collapse(name);
        } else {
          v.features[1].expand(name, true);
        }
      });
    }
  }
  if (success && this.applyFocus) {
    this.scrollToBottom();
  }
}, setEmptyText:function(text) {
  text = text || this.getEmptyTextDiv(this.defaultEmptyText);
  this.getView().setEmptyText(text);
}, onLoadException:function(proxy, response, operation, eOpts) {
  if (response) {
    this.setEmptyText(this.errorText);
  }
}, getMetaByType:function(type) {
  var viewConfig = BpcCommon.Api.getSetting(this.moduleId, 'viewConfig') || {};
  var $jscomp$destructuring$var2 = viewConfig;
  var meta = $jscomp$destructuring$var2.meta;
  type = type || this.collabType;
  meta = meta || {};
  return meta[type] || {};
}, onBeforeSort:function(s, sorters) {
  Ext.log({dump:sorters});
  var metaForType = this.getMetaByType();
  var customMeta = Object.keys(metaForType);
  var sortNotPossible = ['msg', 'to'];
  if (sorters && sorters.length) {
    sorters = sorters.map(function(sorter) {
      var prop = sorter.getProperty();
      if (customMeta.indexOf(prop) !== -1) {
        sorter.setProperty('meta.' + prop);
      } else {
        if (sortNotPossible.indexOf(prop) !== -1) {
          BpcCommon.Api.showNotification('SORT NOT POSSIBLE: ' + prop);
          return undefined;
        }
      }
      return sorter;
    }).filter(function(s) {
      return Ext.isDefined(s);
    });
  }
}, setupToolbar:function() {
  var toolbar = {xtype:'toolbar', overflowHandler:'scroller', padding:'5 5 5 5', dock:'top', defaults:{scale:'medium', margin:'3 3 3 3'}, items:[{iconCls:collaboration.assets.Icons.plus, tooltip:collaboration.Support.TR('CREATE'), handler:'openCollabEditor', disabled:this.collabMethod === 'sent' || this.collabMethod === 'closed' || this.collabType === 'comments' && !this.collabMethod}, {iconCls:collaboration.assets.Icons.reply, tooltip:collaboration.Support.TR('REPLY'), handler:'openCollabAnswer', 
  hidden:this.collabType !== 'tasks', bind:{disabled:'{activeSelection}'}}, {iconCls:collaboration.assets.Icons.trash, tooltip:collaboration.Support.TR('DELETE'), handler:'deleteItem', disabled:!collaboration.Support.canDelete(this.collabType) && !(this.collabMethod === 'sent' || this.collabMethod === 'closed' || this.collabType === 'comments' && !this.collabMethod)}, {iconCls:collaboration.assets.Icons.clone, tooltip:collaboration.Support.TR('DUPLICATE'), handler:'cloneArtefact', bind:{disabled:'{activeSelection}'}}, 
  {xtype:'tbfill'}, {xtype:'collabbasefiltercombo', listeners:{change:'applyBaseFilter'}, hidden:!this.customFilterEnabled || this.collabMethod === 'sent' || this.collabMethod === 'closed' || this.collabType === 'comments' && !this.collabMethod}, {xtype:'collabGridInput', collabType:this.collabType, _grid:this, flex:3}, {xtype:'collabDateRangeField', gridReference:'collabGenericGrid'}, {iconCls:collaboration.assets.Icons.refresh, tooltip:collaboration.Support.TR('REFRESH'), handler:'loadStore'}]};
  return [toolbar];
}, lastTotalCount:null, updateParentComponent:function(totalCount) {
  this.lastTotalCount = totalCount;
  if (this.instanceMode === 'widget') {
    this.widgetInstance.updateDbWrapperWithTotal(undefined, totalCount);
  } else {
    if (this.initialConfig && this.initialConfig.title) {
      this.setTitle(collaboration.Module.getCollabInboxTitle(this.initialConfig.title, totalCount));
    } else {
      if ('iconOnly' in this && Ext.isDefined(this.iconOnly) && this.iconOnly === true) {
        this.setTitle(collaboration.Module.getCollabInboxTitle('', totalCount));
      } else {
        if (this.instanceMode === 'widget') {
          this.setTitle(collaboration.Module.getCollabInboxTitle(collaboration.Module.getAssetForType(this.collabType, 'plural'), totalCount));
        }
      }
    }
  }
  if (this.instanceMode === 'module' && this.updateGridTypeWrapper) {
    var wrapper = this.up('collabTypewrapper');
    if (wrapper) {
      wrapper.updateTypeWrapperTitle(totalCount);
    }
  }
}, updateParentCmpFn:null});
Ext.define('collaboration.view.grid.CollabGridController', {extend:'Ext.app.ViewController', alias:'controller.collabGenericGridController', requires:[], listen:{global:{commentUpdated:'onNewData', messageUpdated:'onNewData', taskUpdated:'onNewData', artefact:'onArtefactEvent', acknowledged:'onAcknowledged'}}, control:{'collabGenericGrid':{rowdblclick:'onRowDblClick', selectionchange:'onSelect', cellcontextmenu:'onContextMenu', select:'onSingleSelect'}}, _view:undefined, _store:undefined, init:function() {
  this._view = this.getView();
  this._store = this._view.getStore();
  this._store.on('load', this.onLoad.bind(this));
}, onSingleSelect:function(rowModel, record, index, eOpts) {
  var me = this;
  me.getViewModel().set('selectedRecord', null);
  me.getViewModel().set('selectedRecord', record.getData());
  me.getViewModel().set('itemSelected', true);
}, onContextMenu:function(tableView, td, cellIndex, record, tr, rowIndex, e, eOpts) {
  e.preventDefault();
  var g = this.getView();
  if (g && !g.contextMenuDisabled && ['messages', 'tasks'].indexOf(g.collabType) !== -1) {
    g.setSelection(record);
    Ext.create({xtype:'collabGridContextMenu', selectedRecord:record, gridReference:g, collabType:g.collabType}).showAt(e.getXY());
  }
}, applyBaseFilter:function(cmp, newVal, oldVal, eOpts) {
  try {
    var me = this;
    var view = me.getView();
    var store = view.getStore();
    if (newVal && newVal.length > 0) {
      cmp.getViewModel().set('valueSelected', true);
    } else {
      cmp.getViewModel().set('valueSelected', false);
    }
    var p = store.getProxy();
    var params = p.getExtraParams() || {};
    params.filterId = newVal;
    params.instanceId = this.getView().moduleId;
    p.setExtraParams(params);
    store.load();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR applyBaseFilter', dump:e, stack:true, level:'error'});
  }
}, takeItem:function(btn) {
  try {
    var record = this.getViewModel().get('lastRecord');
    collaboration.Module.takeItem(record.get('artefactId'));
    Ext.Function.defer(function() {
      btn.up('collabGenericGrid').getView().getStore().load();
    }, 100);
  } catch (e) {
  }
}, onNewData:function(msgPayload) {
  try {
    this._view.setLoading(true);
    Ext.Function.defer(function() {
      this.loadStore();
    }.bind(this), 400);
  } catch (e) {
  }
}, onArtefactEvent:function(eventPayload) {
  try {
    if (eventPayload.properties.type === this._view.collabType) {
      Ext.log({dump:eventPayload});
      this.loadStore();
    }
  } catch (e) {
  }
}, onAcknowledged:function(data) {
  var v = this.getView();
  var $jscomp$destructuring$var3 = data || {};
  var properties = $jscomp$destructuring$var3.properties;
  var $jscomp$destructuring$var4 = properties || {};
  var artefactId = $jscomp$destructuring$var4.artefactId;
  if (artefactId && v.instanceMode && v.instanceMode === 'widget' && v.collabType === 'messages') {
    if (v.store.find('artefactId', artefactId) !== -1) {
      this.loadStore();
    }
  }
}, onRowDblClick:function(row, record, tr, rowIndex, e, eOpts) {
  var collabType = this.getView().collabType;
  var match = Ext.Array.indexOf(['webhooks'], collabType);
  if (this.getView().collabType === 'comments' && !this.getView().collabMethod) {
    return;
  }
  if (match === -1) {
    this.openCollabEditor(record);
  } else {
    if (collabType === 'webhooks') {
      var win = Ext.createByAlias('widget.collabGenericWindow', {iconCls:collaboration.Module.getAssetForType(collabType, 'iconCls'), title:collaboration.Module.getAssetForType(collabType, 'singleText'), items:[{xtype:'collabWebhookViewer', recordData:record.getData()}]});
      win.show();
    }
  }
}, openCollabEditor:function(record) {
  try {
    if (record.isComponent || !Ext.isDefined(record)) {
      record = null;
    }
    var view = this.getView();
    collaboration.Api.getEditorWindow(record, view.collabType, view.moduleId, view.iconCls);
  } catch (e) {
  }
}, openCollabAnswer:function(record) {
  try {
    var view = this.getView();
    var records = view.getSelection();
    if (records.length > 0) {
      record = records[0];
    }
    if (record.isComponent || !Ext.isDefined(record)) {
      record = null;
    }
    var sourceRecord = Ext.clone(record);
    var sourceModule = Ext.clone(record.get('src_moduleId'));
    var sourceInstance = Ext.clone(record.get('src_instanceId'));
    if (sourceRecord && sourceModule && sourceInstance) {
      record = null;
      collaboration.Api.getEditorWindow(record, view.collabType, view.moduleId, undefined, {sourceRecord:sourceRecord, sourceModule:sourceModule, sourceInstance:sourceInstance, isAnswer:true});
    } else {
      if (sourceRecord.get('topicId')) {
        record = null;
        var topicMeta = collaboration.Api.getTopicById(sourceRecord.get('topicId'));
        if (topicMeta) {
          sourceInstance = topicMeta['instanceId'];
          collaboration.Api.getEditorWindow(record, view.collabType, view.moduleId, undefined, {sourceRecord:sourceRecord, sourceInstance:sourceInstance, isAnswer:true});
        }
      }
    }
  } catch (e) {
  }
}, jumpToRecord:function() {
  try {
    var records = this.getView().getSelection();
    if (records && records.length > 0) {
      var record = records[0];
      var instanceId = record.get('src_instanceId');
      var referenceId = record.get('reference');
      if (Ext.isDefined(instanceId) && Ext.isDefined(referenceId)) {
        collaboration.Api.jumpToReference(instanceId, referenceId);
      } else {
        collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_MODULEJUMP_NOT_POSSIBLE'));
      }
    }
  } catch (e) {
  }
}, deleteItem:function() {
  var me = this, view = this.getView(), records = view.getSelection();
  Ext.Msg.confirm(collaboration.Support.TR('DELETE'), collaboration.Support.TR('DELETE_ENTRY_WARNING_Q'), function(btnText) {
    if (btnText === 'yes') {
      try {
        if (records.length) {
          var recordIdList = records.map(function(r) {
            return r.get('artefactId');
          });
          if (recordIdList.length) {
            view.setLoading(true);
            collaboration.Api.deleteArtefacts(recordIdList).then(function(success) {
              Ext.Function.defer(function() {
                me.loadStore();
              }, 1000);
            }, function(error) {
            });
          }
        }
      } catch (e) {
      }
    }
  });
}, runAction:function(item) {
  var vm = this.getViewModel();
  var lastRecord = vm.get('lastRecord');
  if (lastRecord !== null) {
    collaboration.assets.Artefacts.collabTypes[item.actionType].actions[item.actionName].fn(lastRecord.get('artefactId'), lastRecord.data);
  }
}, onSelect:function(selModel, record, index, eOpts) {
  try {
    var records = this.getView().getSelection();
    if (records) {
      this.getViewModel().set('selectionCount', records.length);
      this.getViewModel().set('lastRecord', records[0]);
      this.getViewModel().set('selectedRecords', records);
      this.getViewModel().notify();
    }
  } catch (e) {
  }
}, loadStore:function() {
  this.getView().getStore().load();
}, onLoad:function(store, records, successful, operation, eOpts) {
  try {
    var view = this.getView();
    view.setLoading(false);
    var totalCount = store.getTotalCount();
    if (view.instanceMode && Ext.isDefined(view.instanceMode) && view.instanceMode === 'widget') {
      totalCount = store.getCount();
    }
    this.getViewModel().set('totalCount', totalCount);
    view.updateParentComponent(totalCount);
    if (view && view.instanceMode && view.instanceMode === 'widget') {
      this.updateDbWrapperTitle(store);
    }
  } catch (e) {
  }
}, cloneArtefact:function(btn) {
  var view = this.getView();
  var selection = view.getSelection();
  if (selection.length > 0) {
    collaboration.Api.cloneArtefact(selection[0].get('artefactId')).then(function(value) {
      this.loadStore();
    }.bind(this), function(reason) {
      collaboration.Support.notify(collaboration.Support.TR('ERROR_WHILE_CLONING_ARTEFACT'));
    });
  }
}, updateDbWrapperTitle:function(store) {
  try {
    var wrapper = this.getView().up('dbWidgetWrapper');
    var title = '';
    if (wrapper) {
      if (wrapper.config && wrapper.config.configuration && wrapper.config.configuration.title && Ext.isString(wrapper.config.configuration.title)) {
        title += wrapper.config.configuration.title;
        var totalCount = store.getCount();
        title += ' (' + totalCount + ')';
      } else {
        title = wrapper.getTitle();
      }
      wrapper.setTitle(title);
    }
  } catch (e) {
  }
}});
Ext.define('collaboration.components.base.CollabGridModel', {extend:'Ext.app.ViewModel', alias:'viewmodel.collabGridModel', data:{selectionCount:0, lastRecord:null, selectedRecord:null, totalCount:0, collabDataType:null, itemSelected:false}, formulas:{hideFinishTask:function(get) {
  var hidden = true;
  var dataType = get('collabDataType');
  if (dataType !== null && dataType === 'tasks') {
    var record = get('lastRecord');
    var records = get('selectedRecords');
    var allCreatedByUser = true;
    var allTasksStateResolved = true;
    Ext.each(records, function(record) {
      var creator = record.get('creator');
      var state = record.get('state');
      if (creator !== collaboration.Module.user) {
        allCreatedByUser = false;
      }
      if (state !== 'resolved') {
        allTasksStateResolved = false;
      }
    });
    if (!allCreatedByUser || !allTasksStateResolved) {
      return hidden;
    }
    if (record !== null) {
      var state = record.get('state');
      var creator = record.get('creator');
      if (state && state !== null && state === 'resolved' && creator === collaboration.Module.user) {
        hidden = false;
      }
    }
  }
  return hidden;
}, hideHandbackTask:function(get) {
  var hidden = true;
  var dataType = get('collabDataType');
  if (dataType !== null && dataType === 'tasks') {
    hidden = false;
    var enabled = collaboration.Support.getSetting(collaboration.Module.getDefaultCollabInstance(), 'instance_quickActionHandbackEnabled');
    hidden = !enabled;
  }
  return hidden;
}, hideTakeTask:function(get) {
  var hidden = true;
  var dataType = get('collabDataType');
  if (dataType && dataType !== null && dataType === 'tasks') {
    hidden = false;
  }
  return hidden;
}, activeSelection:function(get) {
  var is = true;
  if (get('selectionCount') > 0) {
    is = false;
  }
  Ext.log({dump:this.getView(), msg:'view inside viewmodel..'});
  return is;
}, artefactWithReference:function(get) {
  var is = false;
  if (get('lastRecord') !== null && get('selectionCount') === 1) {
    var record = get('lastRecord');
    var reference = record.get('reference');
    if (Ext.isDefined(reference) && reference !== null) {
      is = true;
    }
  }
  return is;
}, isArtefactSentToAll:function(get) {
  var returnVal = false;
  if (get('lastRecord') !== null) {
    var record = get('lastRecord');
    if (record.get('to').indexOf(collaboration.assets.Artefacts.allSpecifier) > -1) {
      returnVal = true;
    }
  }
  return returnVal;
}, previewElement:function(get) {
  var elem = null, dataUri = null;
  var rec = get('selectedRecord');
  if (rec !== null) {
    if (rec.base64string && rec.contentType) {
      dataUri = 'data:' + rec.contentType + ';base64,' + rec.base64string;
    }
    elem = dataUri;
  }
  return elem;
}}});
Ext.define('collaboration.components.base.CollabWidgetCommentsWrapper', {extend:'Ext.container.Container', alias:'widget.collabWidgetCommentsWrapper', collabType:undefined, moduleId:undefined, layout:'border', minHeight:150, height:150, items:[{xtype:'collabStreamCardCollab', region:'center'}, {xtype:'container', itemId:'eastWidgetContainer', layout:'fit', region:'east', hidden:true, scrollable:true, width:'300'}]});
Ext.define('collaboration.components.core.toolbar.FloatingPanel', {extend:'Ext.panel.Panel', alias:'widget.floatingpanel', itemId:'floatingPanel', alwaysOnTop:true, floating:true, scrollable:true, shadow:false, layout:'fit'});
Ext.define('collaboration.components.base.StreamCardCollab', {extend:'Ext.panel.Panel', alias:['widget.collabStreamCardCollab', 'widget.collabStreamCardComment'], controller:'collabStreamCardCollabController', viewModel:{type:'collabStreamCardCollab'}, layout:'fit', header:false, style:'background-color: white;', tbar:[{xtype:'container', layout:'hbox', defaults:{vmHolderComponent:'collabStreamCardCollab'}, items:[{xtype:'collabUserBox'}, {xtype:'collabModuleBox', bind:{hidden:'{!referenceAvailable}'}}, 
{xtype:'collabReferenceBox', bind:{hidden:'{!referenceAvailable}'}}, {xtype:'collabJumpBox', bind:{hidden:'{!referenceAvailable}'}}]}], items:[{xtype:'collabGenericBox', bind:{html:'\x3cp\x3e\x3cspan style\x3d"font-style:italic;"\x3e{msg}\x3c/span\x3e\x3c/p\x3e'}}], bbar:[{xtype:'toolbar', layout:'hbox', defaults:{vmHolderComponent:'collabStreamCardCollab', flex:0}, items:[{xtype:'collabTimebox'}, {xtype:'collabEditBox', bind:{hidden:'{hideEdit}'}}, {xtype:'collabReplyBox'}, {xtype:'tbfill', flex:1}, 
{xtype:'blogInfoBox'}]}], getObjectPreset:function() {
  var obj = {};
  var vm = this.getViewModel().getData();
  if (vm.msg && Ext.isDefined(vm.msg)) {
    obj.msgBefore = vm.msg;
  }
  if (vm.topicId && Ext.isDefined(vm.topicId)) {
    obj.topicId = vm.topicId;
  }
  if (vm.moduleId && Ext.isDefined(vm.moduleId)) {
    obj.moduleId = vm.moduleId;
  }
  if (vm.collabType && Ext.isDefined(vm.collabType)) {
    obj.collabType = vm.collabType;
  }
  if (vm.channelId && Ext.isDefined(vm.channelId)) {
    obj.channelId = vm.channelId;
  }
  if (vm.src_moduleId && vm.src_instanceId && vm.reference) {
    if (Ext.isDefined(vm.src_moduleId) && vm.src_moduleId !== null) {
      obj.src_moduleId = vm.src_moduleId;
    }
    if (Ext.isDefined(vm.src_instanceId) && vm.src_instanceId !== null) {
      obj.src_instanceId = vm.src_instanceId;
    }
    if (Ext.isDefined(vm.reference) && vm.reference !== null) {
      obj.reference = vm.reference;
    }
    if (Ext.isDefined(vm.referenceType) && vm.referenceType !== null) {
      obj.referenceType = vm.referenceType;
    }
  }
  return obj;
}});
Ext.define('collaboration.components.base.StreamCardCollabController', {extend:'Ext.app.ViewController', alias:'controller.collabStreamCardCollabController', onJumpBoxClick:function() {
  var vm = this.getViewModel();
  var instanceId = vm.get('src_instanceId');
  var referenceId = vm.get('reference');
  if (Ext.isDefined(instanceId) && Ext.isDefined(referenceId)) {
    collaboration.Api.jumpToReference(instanceId, referenceId);
  } else {
    collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_MODULEJUMP_NOT_POSSIBLE'));
  }
}, onEditBoxClick:function() {
  var vmData = this.getViewModel().getData();
  var wrapper = this.getView().up('collabWidgetCommentsWrapper').down('#eastWidgetContainer');
  var visible = wrapper.isVisible(true);
  this.getView().recData.collabType = 'comments';
  this.getView().recData.artefactId = vmData.artefactId;
  if (visible) {
    var cmp = wrapper.child();
    if (cmp && !cmp['itemid']) {
      wrapper.hide();
    } else {
      wrapper.removeAll();
      wrapper.add([{xtype:'blogUpdatepanel', vmHolderComponent:'colArtefactCard', record:this.getView().recData}]);
    }
  } else {
    wrapper.removeAll();
    wrapper.add([{xtype:'blogUpdatepanel', vmHolderComponent:'collabWidgetCommentsWrapper', record:this.getView().recData}]);
    wrapper.show();
  }
}, onReplyBoxClick:function() {
  var wrapper = this.getView().up('collabWidgetCommentsWrapper').down('#eastWidgetContainer');
  var visible = wrapper.isVisible(true);
  if (visible) {
    var cmp = wrapper.child();
    if (cmp && cmp['itemid'] === 'replyPanel') {
      wrapper.hide();
    } else {
      wrapper.removeAll();
      wrapper.add([{xtype:'replypanel', itemid:'replyPanel', vmHolderComponent:'colArtefactCard'}]);
    }
  } else {
    wrapper.removeAll();
    wrapper.add([{xtype:'replypanel', itemid:'replyPanel', vmHolderComponent:'collabWidgetCommentsWrapper'}]);
    wrapper.show();
  }
}, onAttachmentBoxClick:function() {
  var vm = this.getViewModel();
  var obj = vm.get('meta');
}, onReferenceBoxOver:function(e, htmlEl, eOpts) {
  var floatingPanel = Ext.create({xtype:'floatingpanel', itemId:'processInfoFloatingPanel', maxWidth:850, width:550, items:[{xtype:'collabProcessInfo', parentCmp:this.getView(), recData:this.getView().recData}], listeners:{afterrender:function(panel, eOpts) {
    var pos = e.getXY();
    pos[0] += 30;
    pos[1] += 10;
    panel.showAt(pos);
  }}});
  floatingPanel.show();
}, onReferenceBoxOut:function(e, htmlEl, eOpts) {
  var floatingPanel = Ext.ComponentQuery.query('#processInfoFloatingPanel')[0];
  if (floatingPanel) {
    floatingPanel.destroy();
  }
}, tryLoadingReferenceData:function() {
  var vm = this.getViewModel();
  var baseModuleId = vm.get('src_moduleId');
  if (baseModuleId === 'monitor') {
    var baseUrl = collaboration.Module.getBasePath(baseModuleId);
    var moduleId = vm.get('src_instanceId');
    var refId = vm.get('reference');
    if (refId) {
      Ext.Ajax.request({method:'GET', url:baseUrl + moduleId + '/' + refId}).then(function(response, opts) {
        var obj = Ext.decode(response.responseText, true);
        var referenceSource = null;
        if (obj !== null && Ext.isArray(obj) && obj.length > 0) {
          referenceSource = obj.data[0];
        }
        vm.set('referenceSource', referenceSource);
      }, function(response, opts) {
        Ext.log({dump:response, msg:response.status});
      });
    }
  }
}});
Ext.define('collaboration.components.base.StreamCardCollabModel', {extend:'Ext.app.ViewModel', alias:'viewmodel.collabStreamCardCollab', data:{reference:null, src_instanceId:null, src_moduleId:null, meta:null, creator:null, referenceSource:null, referenceType:null, topicId:null, channelId:null}, formulas:{formattedDateString:function(get) {
  var created = get('created'), now = new Date, fromDate = parseInt(created.getTime() / 1000), toDate = parseInt(now.getTime() / 1000), timeDiff = (toDate - fromDate) / 3600, timeDiff = Math.round(timeDiff), dateString = 'vor ' + timeDiff + ' Stunden';
  if (timeDiff === 0) {
    dateString = 'gerade eben';
  }
  if (timeDiff === 1) {
    dateString = 'vor ' + timeDiff + ' Stunde';
  }
  if (timeDiff > 24) {
    timeDiff = timeDiff / 24;
    timeDiff = Math.round(timeDiff);
    dateString = 'vor ' + timeDiff + ' Tagen';
  }
  dateString = '\x3ci\x3e' + dateString + '\x3c/i\x3e';
  dateString = '\x3cspan class\x3d"x-fal fa-clock-o" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + dateString;
  return dateString;
}, formattedUserString:function(get) {
  var formattedUserString = get('creator');
  if (formattedUserString !== null) {
    formattedUserString = '\x3cspan class\x3d"x-fal fa-user" style\x3d"color:blue"\x3e\x3c/span\x3e\x26nbsp;\x26nbsp;' + collaboration.Module.getRenderedUsername(formattedUserString);
    if (get('referenceAvailable') === true) {
      formattedUserString += ' in: ';
    }
  } else {
    formattedUserString = '';
  }
  return formattedUserString;
}, hideEdit:function(get) {
  var hide = true;
  var creator = get('creator');
  if (creator !== null && creator === collaboration.Module.user) {
    hide = false;
  }
  return hide;
}, formattedModule:function(get) {
  var formattedModule = '';
  var moduleId = get('src_instanceId');
  if (moduleId !== null) {
    var moduleName = collaboration.Support.getSetting(moduleId, 'module_name');
    var moduleIcon = collaboration.Support.getSetting(moduleId, 'module_iconCls');
    formattedModule = ' \x3cspan class\x3d"' + moduleIcon + '" style\x3d"color:orange"\x3e\x3c/span\x3e\x26nbsp;' + moduleName;
  }
  return formattedModule;
}, formattedDate:function(get) {
  var created = get('created');
  return Ext.Date.format(created, 'd.m.Y H:i:s');
}, formattedReference:function(get) {
  var string = '';
  var reference = get('reference');
  var referenceType = get('referenceType');
  if (Ext.isDefined(reference) && reference !== null) {
    var moduleId = get('src_instanceId');
    if (moduleId && Ext.isDefined(moduleId)) {
      var idKey = collaboration.Support.getSetting(moduleId, 'column_id');
      if (idKey && Ext.isDefined(idKey)) {
        idKey = collaboration.Module.capitalize(idKey);
        if (Ext.isDefined(referenceType) && referenceType !== null) {
          reference += ' : ' + referenceType;
        }
        string = ' \x3cspan class\x3d"x-fal fa-info" style\x3d"color:forestgreen"\x3e\x3c/span\x3e\x26nbsp;\x26nbsp;' + idKey + ' ' + reference;
      }
    }
  }
  return string;
}, formattedJumpReference:function(get) {
  return ' \x3cspan class\x3d"x-fal fa-external-link" style\x3d"color:forestgreen;cursor: pointer;"\x3e\x3c/span\x3e';
}, formattedEdit:function(get) {
  return '\x3cspan class\x3d"x-fal fa-edit" style\x3d"color:grey;cursor: pointer;"\x3e Bearbeiten\x3c/span\x3e';
}, formattedReply:function(get) {
  return '\x3cspan class\x3d"x-fal fa-mail-reply" style\x3d"color:grey;cursor: pointer;"\x3e Antworten\x3c/span\x3e';
}, formattedAttachments:function(get) {
  var attString = '';
  return attString;
}, referenceAvailable:function(get) {
  var refDefined = false;
  var reference = get('reference');
  if (Ext.isDefined(reference) && reference !== null) {
    refDefined = true;
  }
  return refDefined;
}, refDefined:function(get) {
  var refDefined = false;
  var reference = get('reference');
  if (Ext.isDefined(reference) && reference !== null) {
    refDefined = true;
  }
  return refDefined;
}, formattedTopicId:function(get) {
  var refDefined = '';
  var reference = get('topicId');
  if (Ext.isDefined(reference) && reference !== null) {
    refDefined = collaboration.Api.getTopicById(reference);
    if (refDefined !== null) {
      refDefined = '\x3cspan class\x3d"x-fal fa-comment-o" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + refDefined.topic;
    }
  }
  return refDefined;
}}});
Ext.define('collaboration.components.base.StreamGrid', {extend:'Ext.grid.Panel', alias:'widget.collabStreamGrid', title:false, hideHeaders:true, trackMouseOver:false, scrollable:'y', collabType:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.columns = [{text:'Widget', flex:1, xtype:'widgetcolumn', dataIndex:'id', padding:'3 3 3 3', widget:{xtype:'collabActivityCard', instanceMode:'widget', header:false, collabType:this.collabType, moduleId:this.moduleId}, onWidgetAttach:function(col, widget, rec) {
    widget.refreshData(rec);
  }}];
  this.store = Ext.create('Ext.data.BufferedStore', {model:'collaboration.model.CollabActivityData', pageSize:100, proxy:{type:'ajax', url:collaboration.Module.getBaseUrl() + (collaboration.Support.artefactTypes.indexOf(this.collabType) !== -1 ? 'artefacts?type\x3d' + this.collabType : this.moduleId + '/' + this.collabType), disableCaching:false, useDefaultXhrHeader:false, reader:{type:'json', rootProperty:'items', totalProperty:'total'}}, autoLoad:false, listeners:{prefetch:function(store, records, 
  success, op) {
    $jscomp$this.updateParentComponent(store.getTotalCount());
  }}});
  this.on('afterrender', function(c) {
    c.store.load();
  });
  this.callParent(arguments);
}, updateParentComponent:function(totalCount) {
  if (this.updateParentCmpFn !== null) {
    this.updateParentCmpFn(totalCount);
  }
}, updateParentCmpFn:null});
Ext.define('collaboration.components.Tag', {extend:'Ext.container.Container', alias:'widget.bpcCollabTag', layout:{type:'hbox', align:'stretch'}, margin:'0 1 0 1', initComponent:function() {
  var $jscomp$this = this;
  var baseButton = {xtype:'button', overCls:'', focusCls:'', handleMouseEvents:false, text:this._tagName || '-1', flex:1};
  if (this._tagCfg && this._tagCfg.color && this._tagCfg.color.length) {
    baseButton.style = 'background-color:' + this._tagCfg.color + ';';
  }
  if (this._tagCfg && this._tagCfg.iconCls && this._tagCfg.iconCls.length) {
    baseButton.iconCls = collaboration.Support.getBpcIconCls(this._tagCfg.iconCls);
  }
  var tagDisplay = Ext.create(baseButton);
  this.items = [tagDisplay, {xtype:'button', tooltip:collaboration.Support.TR('REMOVE'), iconCls:collaboration.Support.getBpcIconCls('fa-times'), handler:function(b) {
    $jscomp$this.destroy();
  }}];
  this.callParent();
}, getTagId:function() {
  return this._tagId;
}});
Ext.define('collaboration.components.base.Typewrapper', {extend:'Ext.panel.Panel', alias:'widget.collabTypewrapper', collabType:undefined, moduleId:undefined, layout:{type:'hbox', align:'stretch'}, title:'', baseTitle:'', initComponent:function() {
  this.itemId = this.collabType + 'Wrapper';
  this.iconCls = collaboration.Module.getAssetForType(this.collabType, 'iconCls');
  this.baseTitle = collaboration.Module.getAssetForType(this.collabType, 'plural');
  if (collaboration.Support.artefactTypes.indexOf(this.collabType) !== -1) {
    var cfg;
    var inboxViews = this.getInboxViews();
    if (inboxViews.length) {
      inboxViews = inboxViews.map(function(item) {
        if (!item.hasOwnProperty('title') && !item.hasOwnProperty('titleKey')) {
          item.title = collaboration.Module.getAssetForType(item.artefactType, 'plural');
        }
        return item;
      });
      cfg = {items:inboxViews, title:this.collabType};
    }
    if (cfg) {
      this.items = [{xtype:'tabpanel', stateful:true, stateId:'collabGenericGrid' + this.collabType + 'State', stateEvents:['tabchange'], defaults:{xtype:'collabGenericGrid', collabType:this.collabType, moduleId:this.moduleId, iconCls:this.iconCls, margin:'10 10 10 10'}, items:cfg.items, applyState:function(state) {
        if (state && state.activeTab) {
          this.setActiveTab(state.activeTab);
          Ext.apply(this, state);
        }
      }, listeners:{tabchange:function(tabpanel, newcard, oldcard, eOpts) {
        tabpanel.up('collabTypewrapper').updateTypeWrapperTitle(newcard.lastTotalCount || 0);
      }, beforestatesave:function(tab, state) {
        tab.addPropertyToState(state, 'activeTab', tab.items.indexOf(tab.getActiveTab()));
      }}, flex:2}];
    }
  }
  this.callParent(arguments);
}, getInboxViews:function() {
  var $jscomp$this = this;
  var viewConfig = BpcCommon.Api.getSetting(this.moduleId, 'viewConfig');
  if (Ext.isObject(viewConfig) && viewConfig.hasOwnProperty('tabConfig') && Ext.isArray(viewConfig.tabConfig)) {
    var customTabs = viewConfig.tabConfig.filter(function(customCfg) {
      return Ext.isDefined(customCfg.artefactType) && customCfg.artefactType === $jscomp$this.collabType;
    });
    return customTabs.map(function(item) {
      if (item.titleKey) {
        item.title = collaboration.Support.TR(item.titleKey);
      }
      if (item.iconCls) {
        item.iconCls = collaboration.Support.getBpcIconCls(item.iconCls);
      }
      item.isCustomView = true;
      return item;
    });
  }
  return [];
}, updateTypeWrapperTitle:function(totalCount) {
  this.setTitle(collaboration.Module.getCollabInboxTitle(this.baseTitle, totalCount));
}});
Ext.define('collaboration.components.column.widget.WidgetButton', {extend:'Ext.button.Button', alias:['widget.collabColumnWidgetButton', 'widget.collabColumnWidget'], controller:'collabColumnWidgetButtonController', iconCls:collaboration.Module.getAssetForType('comments', 'iconCls'), text:'Collaboration', tooltip:'!', padding:'2 2 2 2', tasks:0, messages:0, comments:0, total:0, collabData:undefined, instanceId:undefined, moduleId:undefined, referenceId:undefined, sourceColumn:undefined, instanceType:undefined, 
signals:{empty:'rgba(151,193,199,0.25)', comments:'#46C5BE', messages:'#FF935D', tasks:{state:{open:'#F23C4D', resolved:'#90EE90'}}}, initComponent:function() {
  this.style = 'background-color:' + this.signals.empty + ';';
  this.handler = this.openArtefactStream;
  this.callParent(arguments);
}, initCollabData:function(collabData) {
  var $jscomp$this = this;
  if (collabData && Ext.isObject(collabData)) {
    this.collabData = collabData;
    var type = 'empty';
    if (collabData.details) {
      var tooltip = '\x3ctable\x3e';
      var supportedTypes = collaboration.Support.getSupportedTypes().map(function(i) {
        return i.id;
      });
      supportedTypes.push('total');
      supportedTypes.map(function(k) {
        var key = k + 'Count';
        $jscomp$this[k] = key in collabData.details ? collabData.details[key] : 0;
        type = k !== 'total' && $jscomp$this[k] > 0 ? k : type;
        tooltip += '\x3ctr\x3e\x3ctd\x3e' + collaboration.Module.getAssetForType(k, 'text') + '\x3c/td\x3e\x3ctd\x3e' + $jscomp$this[k] + '\x3ctr\x3e\x3ctd\x3e';
      });
      this.styleComponent(type);
      tooltip += '\x3c/table\x3e';
      this.setTooltip(tooltip);
      this.setText(this.total === 0 ? '' : String(this.total));
    }
    if (collabData.instance) {
      this.instanceId = collabData.instance;
    }
    if (collabData.module) {
      this.moduleId = collabData.module;
    }
    if (collabData.reference) {
      this.referenceId = collabData.reference;
    }
    return;
  }
  this.setIconCls('x-fal fa-comment-slash');
  this.setText('');
}, styleComponent:function(type) {
  var styleDef = this.signals[type];
  if (type === 'tasks' && Ext.isObject(styleDef)) {
    styleDef = styleDef.state.open;
  }
  var elem = this.getEl();
  if (Ext.isDefined(elem)) {
    if ('dom' in elem && 'style' in elem.dom) {
      elem.dom.style.backgroundColor = styleDef;
    }
  }
  this.setIconCls(collaboration.Module.getAssetForType(type, 'iconCls'));
}, openArtefactStream:function(b) {
  try {
    if ('instanceType' in this && Ext.isDefined(this.instanceType) && this.instanceType === 'blog') {
      if ('record' in this && Ext.isDefined(this.record)) {
        var channelId = this.record.get('channelId');
        var topicId = this.record.get('topicId');
        if (Ext.isDefined(channelId) && Ext.isDefined(topicId)) {
          collaboration.Support.showSteamWindow(channelId, topicId, this.moduleId, this.record);
        }
      }
    } else {
      collaboration.Api.showActivityFlowWindow(this.referenceId, this.instanceId, this.moduleId, collaboration.Support.getSetting(this.instanceId, 'column_id') + ' ' + this.referenceId);
    }
  } catch (e) {
    collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_ARTEFACT_WINDOW_COULD_NOT_BE_OPENED'));
  }
}});
Ext.define('collaboration.components.column.widget.WidgetButtonController', {extend:'Ext.app.ViewController', alias:'controller.collabColumnWidgetButtonController', listen:{global:{artefact:'onNewData'}}, onNewData:function(obj) {
  if (this.parseMessage(obj)) {
    try {
      this.refreshCollabData();
    } catch (e) {
      collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_WHILE_UPDATING_COLLAB_BUTTON'));
    }
  }
}, refreshCollabData:function() {
  var view = this.getView();
  if (view.sourceColumn) {
    view.sourceColumn.loadData();
  }
  var blog = view.up('collabTopicChannelBase');
  if (Ext.isDefined(blog)) {
    blog.refreshTopics();
  } else {
  }
}, getCollabReference:function() {
  var $jscomp$destructuring$var5 = this.getView().collabData || {};
  var reference = $jscomp$destructuring$var5.reference;
  return reference;
}, parseMessage:function(obj) {
  obj = obj === undefined ? {} : obj;
  var view = this.getView();
  var instanceId = view.instanceId;
  var integrationReference;
  var $jscomp$destructuring$var6 = obj.properties || {};
  var referenceData = $jscomp$destructuring$var6.referenceData;
  var $jscomp$destructuring$var7 = referenceData || {};
  var reference = $jscomp$destructuring$var7.reference;
  var src_instanceId = $jscomp$destructuring$var7.src_instanceId;
  var src_moduleId = $jscomp$destructuring$var7.src_moduleId;
  var sourceModule = view.sourceColumn.getBaseModule();
  if (sourceModule === 'monitor' && src_moduleId === 'monitor') {
  } else {
    if (sourceModule === 'vam' && src_moduleId === 'vam') {
      integrationReference = collaboration.Support.getSetting(instanceId, 'collab_integrationReference');
    }
  }
  if (instanceId && instanceId === src_instanceId) {
    var processId = this.getCollabReference();
    if (processId === reference) {
      return true;
    }
  }
  if (src_moduleId === 'collaboration') {
    return true;
  }
  return false;
}});
Ext.define('collaboration.components.column.collabcolumn', {extend:'Ext.grid.column.Widget', alias:'widget.collabWidgetColumn', text:' ', dataIndex:'CollabData', widget:{xtype:'collabColumnWidgetButton'}, moduleId:undefined, items:[{xtype:'bpcTextField', readOnly:true}], instanceType:undefined, width:70, onWidgetAttach:function(col, widget, rec) {
  if (widget) {
    widget.initCollabData(rec ? rec.get('collabData') || {} : undefined);
  }
}});
Ext.define('collaboration..mixins.BpcPluginMixin', {getValueFromPluginConfiguration:function(key, fromContext) {
  if (Ext.isObject(this.pluginConfiguration)) {
    if (!fromContext) {
      if (this.pluginConfiguration.configuration && Ext.isObject(this.pluginConfiguration.configuration)) {
        var pluginConfig = this.pluginConfiguration.configuration;
        if (pluginConfig.hasOwnProperty(key)) {
          return pluginConfig[key];
        }
      }
    } else {
      if (fromContext) {
        if (this.pluginConfiguration.context && Ext.isObject(this.pluginConfiguration.context)) {
          var ctx = this.pluginConfiguration.context;
          if (ctx.hasOwnProperty(key)) {
            return ctx[key];
          }
        }
      }
    }
  }
  return undefined;
}, getBaseModule:function() {
  return this.getValueFromPluginConfiguration('moduleId', true);
}, isModule:function(isModule) {
  isModule = isModule === undefined ? '' : isModule;
  var baseModuleId = this.getBaseModule();
  return baseModuleId && baseModuleId === isModule;
}});
Ext.define('collaboration.components.column.CollabArtefactColumn', {extend:'collaboration.components.column.collabcolumn', alias:'widget.collabArtefactGridColumn', requires:['collaboration.components.column.collabcolumn'], mixins:['collaboration..mixins.BpcPluginMixin'], text:'Collaboration', dataIndex:'CollabData', widget:{xtype:'collabColumnWidgetButton'}, moduleId:undefined, items:[{xtype:'bpcTextField', readOnly:true}], width:70, instanceType:undefined, grid:undefined, store:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.moduleId = collaboration.Api.getGlobalDefaultInstance();
  this.targetInstanceId = this.getValueFromPluginConfiguration('instanceId', true);
  this.listeners = {boxready:function(c) {
    var g = $jscomp$this.up('grid');
    if (g) {
      $jscomp$this.grid = g;
      $jscomp$this.store = g.store;
      g.store.getProxy().setExtraParam('collaboration', true);
      $jscomp$this.loadData();
    }
  }};
  Ext.on('monitor_contextMenu_created', this.onContextMenuCreated.bind(this));
  this.callParent(arguments);
}, onWidgetAttach:function(col, widget, rec) {
  if (widget) {
    widget.sourceColumn = col;
    widget.initCollabData(rec ? rec.get('collabData') || {} : undefined);
  }
}, loadData:function() {
  if (this.store) {
    this.store.load();
  }
}, onContextMenuCreated:function(menu) {
  if (this.isVisible(true)) {
    menu.add({xtype:'menuitem', text:'Collaboration', menu:{xtype:'collabContextMenu', src_record:menu.record, src_instanceId:this.targetInstanceId}, moduleId:this.moduleId});
  }
}});
Ext.define('collaboration.components.column.WidgetActionColumn', {extend:'Ext.grid.column.Action', alias:'widget.collabWidgetActionColumn', width:70, items:[{iconCls:null, tooltip:collaboration.Support.TR('ACKNOWLEDGE'), isActionDisabled:function(tableView, rowIndex, colIndex, item, record) {
  if (record.get('acknowledgeAble')) {
    var flags = record.get('flags') || {};
    item.iconCls = collaboration.Support.getBpcIconCls(flags.acknowledged ? 'fa-check' : 'fa-warning');
  } else {
    item.iconCls = null;
  }
  return true;
}}, {iconCls:collaboration.assets.Icons.externalLink, tooltip:collaboration.Support.TR('OPEN_REFRENCEMODULE'), handler:function(grid, rowIndex, colIndex) {
  var rec = grid.getStore().getAt(rowIndex);
  collaboration.Api.jumpToReference(rec.get('src_instanceId'), rec.get('reference') || null);
}, isActionDisabled:function(view, rowIndex, colIndex, item, record) {
  return !(record && Ext.isDefined('src_instanceId'));
}}]});
Ext.define('collaboration.components.container.ErrorCmp', {extend:'Ext.container.Container', alias:['widget.collabGenericErrorCmp'], layout:'center', iconCls:'x-fa fa-wrench', text:collaboration.Support.TR('CONFIG_NOT_VALID'), style:'color:grey;cursor: pointer;', initComponent:function() {
  this.items = [{xtype:'box', html:this.createHtml()}];
  this.callParent(arguments);
}, createHtml:function() {
  var iconCls = this.iconCls;
  var style = this.style;
  var text = this.text;
  return '\x3cspan class\x3d"' + iconCls + '" style\x3d"' + style + '"\x3e ' + text + '\x3c/span\x3e';
}});
Ext.define('collaboration.components.contextmenu.CollabGridMenu', {extend:'Ext.menu.Menu', alias:'widget.collabGridContextMenu', viewModel:{data:{}}, selectedRecord:undefined, gridReference:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.setViewModelData();
  var users = collaboration.Module.users || [];
  var delegateDisabled = !(Ext.isArray(users) && users.length > 0);
  this.tagMenuItem = Ext.create({xtype:'menuitem', iconCls:collaboration.Support.getBpcIconCls('fa-tags'), text:collaboration.Support.TR('TAGS'), menu:[]});
  this.getTags();
  this.items = [{iconCls:collaboration.assets.Artefacts.takeIcon, text:collaboration.Support.TR('ASSIGN_TO_ME'), disabled:!this.toAll(), handler:function(menuEntry) {
    try {
      collaboration.Module.takeItem($jscomp$this.selectedRecord.get('artefactId'));
      $jscomp$this.gridReference.getStore().load();
    } catch (e) {
    }
  }}, this.tagMenuItem, {iconCls:collaboration.assets.Icons.users, text:collaboration.Support.TR('DELEGATE'), disabled:delegateDisabled, menu:!delegateDisabled ? this.usersMenu(users || []) : []}, {iconCls:collaboration.assets.Icons.calendar, text:collaboration.Support.TR('STATE'), hidden:this.gridReference.collabType !== 'tasks', menu:this.statesMenu(collaboration.Support.getStatesAsArray())}];
  this.callParent(arguments);
}, getTags:function() {
  var $jscomp$this = this;
  var tags = this.gridReference.tags || [];
  this.tagMenuItem.getMenu().add(tags.map(function(t) {
    var menuitem = {iconCls:collaboration.Support.getBpcIconCls(t.iconCls), text:t.name, tagId:t.artefactId, documentRecord:$jscomp$this.selectedRecord, handler:function(m) {
      var selection = $jscomp$this.gridReference.getSelection();
      if (selection.length === 0) {
        selection = [m.documentRecord];
      }
      var requests = selection.map(function(r) {
        var tags = r.get('tagIds') || [];
        if (Ext.isString(tags)) {
          tags = [tags];
        }
        if (tags.indexOf(m.tagId) === -1) {
          tags.push(m.tagId);
        } else {
          tags = tags.filter(function(t) {
            return t !== m.tagId;
          });
        }
        return {artefactId:r.getId(), payload:{tagIds:tags}};
      }).map(function(obj) {
        return collaboration.Support.newRequest('artefacts/' + obj.artefactId, {updateSource:true}, obj.payload, 'PUT');
      });
      Promise.all(requests).then(function(arr) {
        $jscomp$this.gridReference.store.load();
      });
    }};
    if (t.color && t.color.length) {
      menuitem.style = 'background-color:' + t.color + ';';
    }
    return menuitem;
  }));
}, setViewModelData:function() {
  var vm = this.getViewModel();
  var data = this.selectedRecord.getData();
  data = collaboration.Module.cleanRecord(data);
  vm.set(data);
}, toAll:function() {
  return this.getViewModel().get('to').indexOf(collaboration.assets.Artefacts.allSpecifier) > -1;
}, statesMenu:function(configuredStates) {
  var $jscomp$this = this;
  configuredStates = configuredStates || [];
  return configuredStates.map(function(item) {
    delete item.id;
    return item;
  }).map(function(state) {
    return {iconCls:collaboration.Support.getBpcIconCls(state.iconCls), text:collaboration.Support.TR(state.languageKey), name:state.state, handler:function(menuItem) {
      var data = $jscomp$this.getViewModel().getData();
      data.state = menuItem.name;
      collaboration.Api.modifyArtefactByType('tasks', $jscomp$this.selectedRecord.id, data).then(function(response, opts) {
        collaboration.Support.notify(collaboration.Support.TR('TASK_WAS_SUCCESSFULLY_UPDATED'));
      }, function(response, opts) {
        collaboration.Support.notify(collaboration.Support.TR('ERROR_WHILE_PROCESSING_THE_UPDATE'));
      });
      $jscomp$this.destroy();
    }};
  });
}, usersMenu:function(users) {
  var $jscomp$this = this;
  users = users.filter(function(user) {
    return user && user.userName;
  });
  return users.map(function(user) {
    var name = user['userName'];
    if (user.firstName) {
      name = user.firstName + ' ';
    }
    if (user.lastName) {
      name += user.lastName;
    }
    var color = collaboration.assets.Artefacts.userColor;
    var iconCls = collaboration.assets.Artefacts.userIcon;
    if (user.type === 'group') {
      color = collaboration.assets.Artefacts.groupColor;
      iconCls = collaboration.assets.Artefacts.groupIcon;
    }
    if (user.type === 'role') {
      color = collaboration.assets.Artefacts.roleColor;
      iconCls = collaboration.assets.Artefacts.roleIcon;
    }
    return {name:name, text:collaboration.Support.getSpanForIconCls(iconCls, color, undefined, '8px', '-25px') + ' ' + name, userName:user['userName'], scope:$jscomp$this, handler:function(menuEntry) {
      collaboration.Api.assignArtefact($jscomp$this.selectedRecord.get('artefactId'), menuEntry.userName, $jscomp$this.collabType).then(function(successCode) {
        collaboration.Support.notify('Delegation an "' + menuEntry.name + '" war erfolgreich');
        this.gridReference.store.load();
      });
    }};
  });
}});
Ext.define('collaboration.components.contextmenu.CollabMenu', {extend:'Ext.menu.Menu', alias:'widget.collabContextMenu', viewModel:{data:{collabDataCount:0, artefactResponse:{}, url:'', addNewComment:false}, formulas:{showOverview:function(get) {
  var value = false;
  if (get('collabDataCount') > 0) {
    value = true;
  }
  return value;
}}}, moduleId:undefined, initComponent:function() {
  var $jscomp$this = this;
  var featuresEnabled = {'comments':true, 'messages':true, 'tasks':true};
  if (Ext.isDefined(this.moduleId)) {
    var settingValue = collaboration.Support.getSetting(this.moduleId, 'module_collab_descriptor');
    if (Ext.isDefined(settingValue) && Ext.isObject(settingValue)) {
      if ('restrictions' in settingValue) {
        var restrictionsO = settingValue.restrictions;
        if ('user' in restrictionsO) {
          var userRestrictionsO = restrictionsO.user;
          var logonName = BpcCommon.Api.getUserName();
          if (logonName in userRestrictionsO) {
            featuresEnabled = Ext.apply(featuresEnabled, userRestrictionsO[logonName]);
          }
        }
        if ('module' in restrictionsO) {
          featuresEnabled = {'comments':true, 'messages':true, 'tasks':true};
          featuresEnabled = Ext.apply(featuresEnabled, restrictionsO['module']);
        }
      }
    }
  }
  this.restrictions = featuresEnabled;
  this.items = [{iconCls:collaboration.assets.Icons.collabOverviewIcon, bind:{text:collaboration.Support.TR('OVERVIEW') + ' ({collabDataCount})', disabled:'{!showOverview}'}, handler:function() {
    collaboration.Api.showActivityFlowWindow($jscomp$this.referenceId, $jscomp$this.src_instanceId, $jscomp$this.src_moduleId, collaboration.Support.getSetting($jscomp$this.src_instanceId, 'column_id') + ' ' + $jscomp$this.referenceId);
  }}, {iconCls:collaboration.assets.Icons.commentsIcon, text:collaboration.Support.TR('ADD_COMMENT'), disabled:!featuresEnabled.comments, handler:function(menuEntry) {
    $jscomp$this.showCollabWindow('comments', menuEntry.iconCls);
  }}, {iconCls:collaboration.assets.Artefacts.messageIcon, text:collaboration.Support.TR('SEND_MESSAGE'), disabled:!featuresEnabled.messages, handler:function(menuEntry) {
    $jscomp$this.showCollabWindow('messages', menuEntry.iconCls);
  }}, {iconCls:collaboration.assets.Artefacts.tasksIcon, text:collaboration.Support.TR('DISTRIBUTE_TASK'), disabled:!featuresEnabled.tasks, handler:function(menuEntry) {
    $jscomp$this.showCollabWindow('tasks', menuEntry.iconCls);
  }}, {iconCls:collaboration.assets.Artefacts.tasksIcon, text:collaboration.Support.TR('SHARE_VIEW'), disabled:true}, {iconCls:collaboration.assets.Artefacts.tasksIcon, text:collaboration.Support.TR('CREATE_BLOGENTRY'), disabled:true}, {iconCls:collaboration.assets.Artefacts.tasksIcon, text:BpcCommon.Api.getTranslation('COLLABORATION_CREATE_FORUMTHREAD'), disabled:true}];
  this.callParent(arguments);
  if (this.src_record) {
    var collabData = this.src_record.get('collabData');
    if (collabData) {
      this.referenceId = collabData.reference;
      this.getViewModel().set('collabDataCount', collabData.details && 'totalCount' in collabData.details ? collabData.details.totalCount || 0 : 0);
    }
  }
}, showCollabWindow:function(collabType, iconCls) {
  try {
    if (collabType) {
      if (this.src_record && this.src_instanceId) {
        var idKey = collaboration.Support.getSetting(this.src_instanceId, 'column_id');
        var reference = this.src_record.get(idKey);
        if (reference) {
          collaboration.Api.applyGlobalReference(reference);
        }
      }
      collaboration.Api.getEditorWindow(null, collabType, collaboration.Module.getDefaultCollabInstance(), iconCls, {restrictions:this.restrictions});
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR showCollabWindow', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('collaboration.widgets.components.contextMenu.ForumWidgetGroupMenu', {extend:'Ext.menu.Menu', alias:['widget.forumWidgetGroupMenu', 'widget.forumwidgetgroupmenu'], moduleId:undefined, gridReference:undefined, groupName:undefined, items:[{text:collaboration.Support.TR('OPEN_ALL'), iconCls:collaboration.assets.Artefacts.forumContextAufklappen, handler:function(menuItem) {
  var menu = menuItem.up('forumwidgetgroupmenu');
  var me = menu.gridReference.features[0], metaGroupCache = me.getCache(), lockingPartner = me.lockingPartner, groupName;
  for (groupName in metaGroupCache) {
    if (metaGroupCache.hasOwnProperty(groupName)) {
      if (Ext.isDefined(metaGroupCache[groupName])) {
        metaGroupCache[groupName].isCollapsed = false;
      }
    }
  }
  Ext.suspendLayouts();
  me.dataSource.onRefresh();
  Ext.resumeLayouts(true);
  for (groupName in metaGroupCache) {
    if (metaGroupCache.hasOwnProperty(groupName)) {
      me.afterCollapseExpand(false, groupName);
      if (lockingPartner) {
        lockingPartner.afterCollapseExpand(false, groupName);
      }
    }
  }
}}, {text:collaboration.Support.TR('CLOSE_ALL'), iconCls:collaboration.assets.Artefacts.forumContextZuklappen, handler:function(menuItem) {
  var menu = menuItem.up('forumwidgetgroupmenu');
  var me = menu.gridReference.features[0], metaGroupCache = me.getCache(), groupName, lockingPartner = me.lockingPartner;
  for (groupName in metaGroupCache) {
    if (metaGroupCache.hasOwnProperty(groupName)) {
      if (Ext.isDefined(metaGroupCache[groupName])) {
        metaGroupCache[groupName].isCollapsed = true;
      }
    }
  }
  Ext.suspendLayouts();
  me.dataSource.onRefresh();
  Ext.resumeLayouts(true);
  for (groupName in metaGroupCache) {
    if (metaGroupCache.hasOwnProperty(groupName)) {
      me.afterCollapseExpand(true, groupName);
      if (lockingPartner) {
        lockingPartner.afterCollapseExpand(true, groupName);
      }
    }
  }
}}]});
Ext.define('collaboration.components.editor.CollabArtefactHistoryGrid', {extend:'Ext.grid.Panel', alias:'widget.collabArtefactHistoryGrid', moduleId:undefined, record:undefined, scrollable:true, margin:'0 3 10 80', initComponent:function() {
  this.langKeys = {created:'COLLABORATION_CREATED', msg:'COLLABORATION_TYPE_MESSAGE_SINGLE', header:'COLLABORATION_HEADER', to:'COLLABORATION_TO', state:'COLLABORATION_STATE', duedate:'COLLABORATION_DUE', lastupdate:'COLLABORATION_LAST_UPDATE', topicId:'COLLABORATION_TOPIC', channelId:'COLLABORATION_CHANNEL_NAME', assigneeType:'COLLABORATION_TYPE', user:'COLLABORATION_USER', role:'COLLABORATION_ROLE', org:'COLLABORATION_ORG'};
  this.columns = [{dataIndex:'header', width:50, renderer:function(value, meta) {
    if (value) {
      var icon = 'fa-info', text = 'unknown';
      if (value === 'create') {
        icon = 'fa-plus-circle';
        text = 'CREATED';
      } else {
        if (value === 'change') {
          icon = 'fa-edit';
          text = 'UPDATED';
        } else {
          if (value === 'acknowledge' || value === 'ack') {
            icon = 'fa-check';
            text = 'ACKNOWLEDGED';
          } else {
            if (value === 'artefactComment') {
              icon = 'fa-comment-dots';
              text = 'TYPE_COMMENT_SINGLE';
            }
          }
        }
      }
      meta.tdAttr = 'data-qtip\x3d' + collaboration.Support.TR(text);
      value = collaboration.Module.getColumnHeaderHtmlString(collaboration.Support.getBpcIconCls(icon), 'green', '');
    }
    return value;
  }}, {xtype:'datecolumn', dataIndex:'created', width:150, title:'Änderung', format:'d.m.Y H:i:s'}, {dataIndex:'creator', title:'Ersteller', width:120}, {dataIndex:'message', title:'Message', flex:1}, {flex:1, dataIndex:'artefactLog', title:'Änderungen', renderer:function(value) {
    var val = '';
    if (value && Ext.isObject(value) && value.changed && (value.changed || {}).newValues) {
      var newData = (value.changed || {}).newValues || {};
      val = Object.keys(newData).join(', ');
    }
    return val;
  }}];
  this.store = Ext.create('Ext.data.BufferedStore', {proxy:{type:'ajax', url:collaboration.Module.getBaseUrl() + 'artefacts?type\x3dactivities\x26id\x3d' + this.record.get('artefactId'), reader:{type:'json', rootProperty:'items', totalProperty:'total'}}, autoLoad:true});
  this.callParent(arguments);
}, getArtifactValue:function(value, type) {
  var me = this;
  var obj = {};
  var moduleId = collaboration.Module.getDefaultCollabInstance();
  var states = collaboration.Support.getSetting(moduleId, 'default_taskStates');
  if (value && type) {
    Ext.Object.each(value, function(key, val) {
      if (key.includes(type)) {
        Ext.Object.each(val, function(k, v) {
          var langKey = me.langKeys[k] || k;
          if (k === 'state') {
            v = states[v].defaultText || v;
          }
          if (k === 'topicId') {
            var topicObj = collaboration.Api.getTopicById(v);
            if (topicObj && topicObj !== null) {
              obj[BpcCommon.Api.getTranslation(langKey)] = topicObj.topic;
            } else {
              obj[BpcCommon.Api.getTranslation(langKey)] = v;
            }
          } else {
            if (k === 'channelId') {
              var channelObj = collaboration.Api.getChannelById(v);
              if (channelObj && channelObj !== null) {
                obj[BpcCommon.Api.getTranslation(langKey)] = channelObj.channel;
              } else {
                obj[BpcCommon.Api.getTranslation(langKey)] = v;
              }
            } else {
              if (k === 'assigneeType') {
                obj[BpcCommon.Api.getTranslation(langKey)] = BpcCommon.Api.getTranslation(me.langKeys[v] || v);
              } else {
                obj[BpcCommon.Api.getTranslation(langKey)] = v;
              }
            }
          }
        });
      }
    });
  }
  return obj;
}});
Ext.define('collaboration.components.editor.EditorPanel', {extend:'Ext.panel.Panel', alias:'widget.collabEditorPanel', controller:'collabEditorPanelController', viewModel:{type:'collabEditorPanelViewModel'}, width:810, minWidth:810, maxWidth:810, minButtonWidth:35, streamView:false, moduleId:undefined, collabType:undefined, record:undefined, restrictions:undefined, layout:{type:'vbox', align:'stretch'}, isAnswer:false, typeMap:{date:function() {
  return {xtype:'datefield', format:'d.m.Y'};
}}, hoursField:undefined, minutesField:undefined, mode:undefined, getMetaByType:function(type) {
  var viewConfig = BpcCommon.Api.getSetting(this.moduleId, 'viewConfig') || {};
  var $jscomp$destructuring$var8 = viewConfig;
  var meta = $jscomp$destructuring$var8.meta;
  meta = meta || {};
  return meta[type] || {};
}, getCustomMetaFields:function(type) {
  var $jscomp$this = this;
  var def = this.getMetaByType(type);
  return Object.keys(def).map(function(name) {
    var fieldCfg = def[name].type ? $jscomp$this.typeMap[def[name].type]() || {xtype:'textfield'} : {xtype:'textfield'};
    fieldCfg.metaId = name;
    fieldCfg.fieldLabel = def[name].label ? def[name].label : name;
    if (fieldCfg.xtype === 'datefield') {
      fieldCfg.format = def[name].format ? def[name].format : 'd.m.Y';
    }
    return fieldCfg;
  });
}, isBoxed:function() {
  return this.mode && this.mode === 'boxed';
}, initComponent:function() {
  var $jscomp$this = this;
  var btns = [{collabType:'comments', iconCls:collaboration.assets.Icons.comments, tooltip:collaboration.Support.TR('TYPE_COMMENT_NEW'), bind:{hidden:'{isModeEdit}', disabled:'{isComment}'}, margin:'0 0 0 75', collabAction:'createComment', handler:'changeType'}, {xtype:'tbspacer', width:2}, {collabType:'messages', iconCls:collaboration.assets.Icons.messageIcon, tooltip:collaboration.Support.TR('TYPE_MESSAGE_NEW'), bind:{hidden:'{isModeEdit}', disabled:'{isMessage}'}, collabAction:'createMessage', 
  handler:'changeType'}, {collabType:'tasks', iconCls:collaboration.assets.Icons.tasksIcon, tooltip:collaboration.Support.TR('TYPE_TASK_NEW'), collabAction:'createTask', handler:'changeType', bind:{hidden:'{isModeEdit}', disabled:'{isTask}'}}, {xtype:'tbspacer', width:62, bind:{hidden:'{editor.isNewItem}'}}, {iconCls:collaboration.assets.Icons.bug, enableToggle:true, toggleHandler:'toggleWebhook', tooltip:BpcCommon.Api.getTranslation('COLLABORATION_WEBHOOK_DE_ACTIVATE'), itemId:'defaultWebhook', 
  bind:{hidden:true}}, {iconCls:'x-fal fa-history', handler:'showArtefactHistory', localized:{tooltip:'COLLABORATION_HISTORY'}, bind:{hidden:'{editor.isNewItem}'}}, {xtype:'tbfill'}, {iconCls:collaboration.assets.Icons.trash, text:collaboration.Support.TR('DELETE'), bind:{hidden:'{editor.isNewItem}', disabled:'{!canDelete}'}, handler:'deleteItem', ui:'secondary'}, {iconCls:collaboration.assets.Icons.times, text:collaboration.Support.TR('CLOSE'), ui:'secondary', hidden:this.isBoxed(), handler:function(btn) {
    $jscomp$this.closeWindow();
  }}, {iconCls:collaboration.assets.Icons.envelope, text:collaboration.Support.TR('REPLY'), handler:'onAnswer', bind:{hidden:'{answerActive}'}, ui:'primary'}, {iconCls:collaboration.assets.Icons.save, text:collaboration.Support.TR('UPDATE'), bind:{hidden:'{editor.isNewItem}'}, handler:'updateItem', ui:'primary'}, {iconCls:collaboration.assets.Icons.save, text:collaboration.Support.TR('ACKNOWLEDGE'), bind:{hidden:'{!acknowledgeAble}', disabled:'{IsAcknowledged}'}, handler:'acknowledgeItem', ui:'primary'}, 
  {iconCls:collaboration.assets.Icons.envelope, text:collaboration.Support.TR('SEND'), handler:'onSend', bind:{hidden:'{!editor.isNewItem}'}, ui:'primary'}];
  var types = collaboration.Support.getSupportedTypes().map(function(i) {
    return i.id;
  });
  this.buttons = btns.filter(function(b) {
    return b.hasOwnProperty('collabType') && !$jscomp$this.isBoxed() && types.indexOf(b.collabType) !== -1 || !b.hasOwnProperty('collabType');
  });
  this.collabType = !this.collabType ? types[0] : this.collabType;
  var me = this, skipLogic = false;
  try {
    if (!Ext.isDefined(this.moduleId)) {
      this.moduleId = collaboration.Module.getDefaultCollabInstance();
    }
    var vm = this.getViewModel();
    if (this.collabType === 'tasks' && vm.getData().editor.isNewItem) {
      var states = collaboration.Support.getSetting(me.moduleId, 'default_taskStates');
      if (states.states && Ext.isArray(states.states)) {
        states = states.states;
        vm.set('collabData.state', states[0]);
      } else {
        Ext.iterate(states, function(k, v) {
          if (v && Ext.isDefined(v) && Ext.isObject(v)) {
            vm.set('collabData.state', k);
            return false;
          }
        });
      }
      vm.set('collabData.duedate', new Date);
    }
    if (this.channelId && me.topicId) {
      vm.set('collabData.channelId', me.channelId);
      vm.set('collabData.topicId', me.topicId);
    } else {
      var globalPredefinedChannelId = collaboration.Support.getSetting('collaboration', 'default_globalChannelId');
      if (globalPredefinedChannelId) {
        me.channelId = globalPredefinedChannelId;
        skipLogic = true;
        vm.set('collabData.channelId', me.channelId);
      }
    }
    var topicStoreData = [];
    var channelId = me.channelId;
    if (skipLogic) {
    } else {
      if (me.record && Ext.isDefined(me.record)) {
        channelId = me.record.get('channelId');
      }
    }
    if (Ext.isDefined(channelId)) {
      topicStoreData = collaboration.Api.streams.topics.getByChannelId(channelId);
    }
    this.topicsStore = Ext.create('Ext.data.Store', {data:topicStoreData});
    if (this.restrictions && Ext.isObject(this.restrictions)) {
      this.getViewModel().set('restrictions', this.restrictions);
    }
    var moduleFeatures = BpcCommon.Api.getSetting('collaboration', 'module_features') || [];
    moduleFeatures = moduleFeatures.filter(function(feature) {
      return feature.enabled;
    });
    var hideChannelAndTopic = this.isAnswer === true ? true : moduleFeatures.length === 0;
    var baseNumField = {xtype:'numberfield', maxWidth:65, width:65};
    this.hoursField = Ext.create(Ext.apply({}, {}, baseNumField));
    this.minutesField = Ext.create(Ext.apply({}, {}, baseNumField));
    this.tagField = Ext.create({xtype:'toolbar', layout:{type:'hbox', align:'stretch'}, items:[]});
    this.items = [{xtype:'container', layout:{type:'hbox', align:'stretch'}, defaults:{xtype:'container', flex:1, layout:{type:'vbox', align:'stretch'}}, items:[{items:[{fieldLabel:collaboration.Support.TR('HEADER') + ':', itemId:'artefactHeader', xtype:'bpcTextArea', emptyText:BpcCommon.Api.getTranslation('COLLABORATION_HEADER_PLS_ADD'), allowBlank:false, bind:{value:'{collabData.header}', hidden:'{isComment}'}}, {xtype:'stateselectcombo', record:me.record, bind:{value:'{collabData.state}', hidden:'{!isTask}'}}]}, 
    {items:[{xtype:'collabFieldReferenceContainer'}, {xtype:'colReferenceTypeContainer', hidden:!BpcCommon.Api.getSetting('collaboration', 'showReferenceType')}, {xtype:'userSelectCombo', bind:{value:'{toUsers}', hidden:'{isComment}'}}, {xtype:'fieldcontainer', fieldLabel:collaboration.Support.TR('TASK_DATE'), bind:{hidden:'{!isTask}'}, layout:{type:'hbox', align:'stretch'}, items:[{itemId:'taskDate', xtype:'datefield', bind:{value:'{collabData.duedate}'}}, this.hoursField, this.minutesField]}]}]}, 
    {xtype:'container', layout:{type:'hbox', align:'stretch'}, hidden:hideChannelAndTopic, itemId:'streamSelectContainer', defaults:{xtype:'container', flex:1, layout:{type:'vbox', align:'stretch'}}, items:[{items:[{xtype:'channelselcetcombo', fieldLabel:collaboration.Support.TR('CHANNEL'), itemId:'selectedChannel', bind:{value:'{collabData.channelId}'}, listeners:{change:'onChannelChange'}}]}, {items:[{xtype:'collabTopicFieldContainer', enableTopics:skipLogic, topicsStore:this.topicsStore}]}]}, 
    {xtype:'container', layout:{type:'hbox', align:'stretch'}, itemId:'tagsWrapper', items:[{xtype:'collabTagsCombo', fieldLabel:collaboration.Support.TR('TAGS'), flex:1, editorPanel:this, onTagSelect:function(field, record) {
      var vm = field.editorPanel.getViewModel();
      vm.set('collabData.tags', record.get('name') || '');
      vm.set('collabData.tagIds', record.get('artefactId') || '');
    }}, {xtype:'button', iconCls:collaboration.Support.getBpcIconCls('fa-plus'), handler:function(b) {
      var wrapper = b.up('#tagsWrapper');
      var combo = wrapper.down('collabTagsCombo');
      var value = combo.getValue();
      if (value) {
        collaboration.Api.getCreateRequestPromise(combo.getStore().getProxy().url + '?name\x3d' + encodeURIComponent(value)).then(function(obj) {
          combo.getStore().load();
          Ext.defer(function() {
            var index = combo.getStore().find('name', obj.name);
            var record = combo.getStore().getAt(index);
            if (record) {
              combo.setSelection(record);
            }
          }, 500);
        }, function(statucCode) {
        });
      } else {
        Ext.toast('No Tag');
      }
    }}]}, this.tagField, {xtype:'container', layout:{type:'hbox', align:'stretch'}, defaults:{flex:1}, items:this.getCustomMetaFields('tasks')}, {xtype:'bpcTextArea', itemId:'messageTextArea', allowBlank:false, bind:{emptyText:'{getEmptyText}', value:'{collabData.msg}', fieldLabel:'{textAreaLabel}'}}].concat(this.record ? [{xtype:'container', layout:{type:'hbox', align:'stretch'}, bind:{hidden:'{editor.historyGridHidden}'}, items:[{xtype:'label', margin:'0 5 10 5', html:BpcCommon.Api.getTranslation('COLLABORATION_HISTORY') + 
    ':'}, {xtype:'collabArtefactHistoryGrid', moduleId:this.moduleId, margin:'0 3 10 25', record:me.record, height:200, flex:1}]}] : []);
  } catch (e) {
  }
  this.callParent(arguments);
  if (me.record) {
    me.updateRecord(me.record);
    this.fillMetaFields(me.record);
    me.getViewModel().set('temp.record', me.record);
  } else {
    this.updateWindowTitle(this.deriveTitle());
    if (me.sourceInstance && me.sourceRecord) {
      if (me.isAnswer && me.isAnswer === true) {
        var taskO = {creator:me.sourceRecord.get('creator'), recipients:[{name:me.sourceRecord.get('to'), type:'user'}], msg:me.sourceRecord.get('msg')};
        var msg = collaboration.Api.actions.tasks.transformMsg(' ', taskO);
        vm.set('collabData.reference', me.sourceRecord.get('reference'));
        vm.set('collabData.header', 'Re: ' + me.sourceRecord.get('header'));
        vm.set('collabData.recipients', [{name:me.sourceRecord.get('creator'), type:'user'}]);
        vm.set('collabData.msg', msg);
        if (me.sourceRecord.get('topicId')) {
          vm.set('collabData.topicId', me.sourceRecord.get('topicId'));
          vm.set('collabData.channelId', me.sourceRecord.get('channelId'));
        }
      } else {
        vm.set('collabData.reference', me.getCollabReference(me.sourceRecord, me.sourceInstance));
      }
      if (me.sourceModule) {
        me.applyModuleAndInstanceId(me.sourceModule, me.sourceInstance);
      }
    } else {
      me.getBpcContext();
    }
  }
  me.getViewModel().set('editor.collabType', me.collabType);
  me.getViewModel().notify();
  var value = collaboration.Support.getSetting('collaboration', 'tasks_flowOnlyCreatorCanFinish');
  if (Ext.isDefined(value) && value !== null) {
    me.getViewModel().set('editor.onlyCreatorCanfinish', value);
  }
  if (me.getViewModel().get('editor.collabType') === 'tasks') {
    me.down('#taskState').allowBlank = false;
    me.down('#taskDate').allowBlank = false;
    me.down('#to').allowBlank = false;
    this.hoursField.setValue((new Date).getHours());
    this.minutesField.setValue((new Date).getMinutes());
  } else {
    if (me.getViewModel().get('editor.collabType') === 'messages') {
      me.down('#to').allowBlank = false;
    }
  }
  if (skipLogic) {
    var channelCombo = me.down('#selectedChannel');
    if (channelCombo) {
      Ext.defer(function() {
        me.getController().onChannelChange(channelCombo);
      }, 1500);
    }
  }
  if (this.record) {
    var tagsIds = this.record.get('tagIds') || [];
    if (Ext.isString(tagsIds)) {
      tagsIds = [tagsIds];
    }
    collaboration.Api.getTags().then(function(json) {
      var $jscomp$destructuring$var9 = json || {};
      var items = $jscomp$destructuring$var9.items;
      items = items || [];
      tagsIds.map(function(tagId) {
        var match = items.filter(function(obj) {
          return obj.artefactId === tagId;
        });
        $jscomp$this.addDisplayTag(match.length ? match[0].name : '-1', tagId, match.length ? match[0] : undefined);
      });
    });
  }
}, addDisplayTag:function(name, id, tagCfg) {
  var f = Ext.create({xtype:'bpcCollabTag', _tagName:name, _tagId:id, _tagCfg:tagCfg});
  this.tagField.add(f);
}, fillMetaFields:function(record) {
  var $jscomp$this = this;
  var obj = this.getMetaByType('tasks');
  var meta = record.get('meta') || {};
  Object.keys(obj).map(function(name) {
    if (meta[name]) {
      var value = meta[name];
      if (obj[name].type && obj[name].type === 'date') {
        value = Ext.Date.parse(value);
      }
      var f = $jscomp$this.down('field[metaId\x3d' + name + ']');
      f.setValue(value);
    }
  });
}, getBpcContext:function() {
  var obj = BpcCommon.Api.getActiveModuleMeta();
  if (obj !== null && Ext.isObject(obj)) {
    if (obj.moduleId && obj.instanceId) {
      this.applyModuleAndInstanceId(obj.moduleId, obj.instanceId);
    }
  }
}, setReferenceId:function(referenceId, moduleId, instanceId) {
  var vm = this.getViewModel();
  vm.set('collabData.reference', referenceId);
  vm.set('collabData.src_moduleId', moduleId);
  vm.set('collabData.src_instanceId', instanceId);
}, applyModuleAndInstanceId:function(moduleId, instanceId) {
  var vm = this.getViewModel();
  if (moduleId && instanceId) {
    if (moduleId === 'monitor') {
      var collabDescriptor = collaboration.Support.getSetting(instanceId, 'module_collab_descriptor');
      if (collabDescriptor && 'integrationReference' in collabDescriptor) {
        vm.set('collabData.src_integrationReference', collabDescriptor.integrationReference);
      }
    } else {
      if (moduleId === 'vam') {
        var integrationReference = collaboration.Support.getSetting(instanceId, 'collab_integrationReference');
        if (integrationReference && integrationReference.length > 0) {
          vm.set('collabData.src_integrationReference', integrationReference);
        }
      }
    }
    vm.set('collabData.src_instanceId', instanceId);
    vm.set('collabData.src_moduleId', moduleId);
  }
  var ref = collaboration.Module.lastReferenceId;
  if (ref !== null) {
    vm.set('collabData.reference', ref);
    this.loadReferenceData(moduleId, instanceId, ref);
  }
}, getCollabReference:function(record, instanceId) {
  var processId = record.get(collaboration.Support.getSetting(instanceId, 'column_id'));
  this.loadReferenceData('vam', instanceId, processId);
  return processId;
}, deriveTitle:function() {
  return this.collabType ? collaboration.Module.getAssetForType(this.collabType, 'new') : '';
}, deriveAssetTitle:function() {
  return this.collabType ? collaboration.Module.getAssetForType(this.collabType, 'singleText') : '';
}, updateRecord:function(record) {
  var $jscomp$this = this;
  try {
    var vm = this.getViewModel();
    vm.set('editor.isNewItem', false);
    vm.set('editor.mode', 'edit');
    var recordData = record.getData() || {};
    Object.keys(recordData).map(function(name) {
      vm.set('collabData.' + name, recordData[name]);
      if (name && name === 'duedate' && recordData[name]) {
        $jscomp$this.hoursField.setValue(recordData[name].getHours());
        $jscomp$this.minutesField.setValue(recordData[name].getMinutes());
      }
    });
    vm.notify();
    if (record.get('src_instanceId') && record.get('src_moduleId') && record.get('reference')) {
      this.loadReferenceData(record.get('src_moduleId'), record.get('src_instanceId'), record.get('reference'));
    }
    this.updateWindowTitle(this.deriveAssetTitle());
    this.down('colReferenceTypeContainer').loadAvailableReferenceTypes();
    var tagIds = record.get('tagIds') || '';
    var c = this.down('collabTagsCombo');
    if (tagIds.length) {
      c.setValue(tagIds);
    }
  } catch (e) {
    Ext.log('error while updating the record');
  }
}, loadReferenceData:function(moduleId, instanceId, reference) {
  var $jscomp$this = this;
  if (moduleId === 'monitor') {
    collaboration.Support.loadReference(moduleId, instanceId, reference).then(function(response) {
      var obj = Ext.decode(response.responseText, true);
      var processData = Ext.isObject(obj) && obj.totalCount ? obj.data[0] || {} : undefined;
      var descriptor = collaboration.Module.getCollabIntegrationDescriptor(instanceId);
      if (processData && descriptor && descriptor.labelColumn) {
        var label = processData[descriptor.labelColumn];
        $jscomp$this.getViewModel().set('referenceLabel', label ? label : reference);
      }
    }, function(error) {
      Ext.log({dump:error, msg:'fehler'});
    });
  } else {
    if (moduleId === 'vam') {
      collaboration.Support.loadReference(moduleId, instanceId, reference).then(function(response) {
        var respObj = collaboration.Module.xml2json(response.responseXML);
        if (respObj && Ext.isObject(respObj) && respObj.response.overallResult === 'true') {
          var asset = respObj.response.responses.response.body.assets.asset;
          var label = asset.version.customId;
          this.getViewModel().set('referenceLabel', label ? label : reference);
        }
      }.bind(this), function(error) {
        Ext.log({dump:error, msg:'fehler'});
      });
    }
  }
}, updateWindowTitle:function(title) {
  var win = this.up('window');
  if (win && (this.isAnswer || this.streamView === false) && win.updateTitle) {
    win.updateTitle(this.isAnswer ? collaboration.Support.TR('REPLY') : title);
  }
}, closeWindow:function() {
  if (this.isBoxed()) {
    this.wrapper.destroy();
  } else {
    var win = this.up('win');
    if (win) {
      win.close();
    }
  }
}});
Ext.define('collaboration.components.EditorPanelController', {extend:'Ext.app.ViewController', alias:'controller.collabEditorPanelController', listen:{global:{collabSetReference:'onChangeReferenceTo'}}, control:{'userSelectCombo':{select:'setRecipients'}}, setRecipients:function(combo, selectedRecords, eOpts) {
  var recipients = Ext.isArray(selectedRecords) ? selectedRecords.map(function(r) {
    return {name:r.get('userName'), type:r.get('type')};
  }) : [];
  var vm = this.getViewModel();
  if (vm) {
    vm.set('collabData.recipients', recipients);
  }
}, onChangeReferenceTo:function(refId) {
  this.getViewModel().set('collabData.reference', refId);
}, changeType:function(btn) {
  try {
    if (btn && btn.collabAction) {
      var me = this.getView();
      var vm = this.getViewModel();
      if (btn.collabAction === 'createComment' || btn.collabAction === 'createMessage') {
        var isComment = btn.collabAction === 'createComment';
        vm.set('editor.collabType', isComment ? 'comments' : 'messages');
        vm.set('collabData.state', '');
        vm.set('collabData.duedate', '');
        if (isComment) {
          me.down('#to').allowBlank = false;
        }
        this.updateWindow(collaboration.Support.TR(isComment ? 'TYPE_COMMENT_NEW' : 'TYPE_MESSAGE_NEW'), isComment ? collaboration.assets.Icons.commentsIcon : collaboration.assets.Artefacts.messageIcon);
        return;
      }
      if (btn.collabAction === 'createTask') {
        var initialState;
        if (vm.getData().editor.isNewItem) {
          var states = collaboration.Support.getSetting(me.moduleId, 'default_taskStates');
          if (states.states && Ext.isArray(states.states)) {
            states = states.states;
            initialState = states[0];
          } else {
            Ext.iterate(states, function(k, v) {
              if (v && Ext.isDefined(v) && Ext.isObject(v)) {
                initialState = k;
                return false;
              }
            });
          }
        }
        vm.set('editor.collabType', 'tasks');
        var date = new Date;
        me.hoursField.setValue(date.getHours());
        me.minutesField.setValue(date.getMinutes());
        vm.set('collabData.duedate', new Date);
        vm.set('collabData.state', initialState);
        me.down('#taskState').allowBlank = false;
        me.down('#taskDate').allowBlank = false;
        me.down('#to').allowBlank = false;
        this.updateWindow(collaboration.Support.TR('TYPE_TASK_NEW'), collaboration.assets.Artefacts.tasksIcon);
      }
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR changeType', dump:e, stack:true, level:'error'});
  }
}, updateWindow:function(title, iconCls) {
  var me = this.getView();
  var extWindow = me.up('window');
  if (extWindow) {
    extWindow.updateTitle(title);
    extWindow.setIconCls(iconCls);
  }
}, onHandBackTaskClick:function() {
  this.runAction('tasks', 'handback');
}, onFinishTaskClick:function() {
  this.runAction('tasks', 'finish');
}, runAction:function(type, actionName) {
  var vm = this.getViewModel();
  var lastRecord = vm.get('temp.record');
  if (lastRecord !== null) {
    collaboration.assets.Artefacts.collabTypes[type].actions[actionName].fn(lastRecord.get('artefactId'), lastRecord.data);
    this.closeWindow();
  }
}, acknowledgeItem:function() {
  this.runAction('messages', 'ack');
}, fieldsValid:function(fields) {
  fields = fields === undefined ? [] : fields;
  var view = this.getView();
  return fields.map(function(itemId) {
    return view.down('#' + itemId).isValid();
  }).indexOf(false) === -1;
}, onSend:function(btn, evt, eOpts) {
  var valid = false;
  var editPanel = this.getView();
  var vm = this.getViewModel();
  var type = vm.get('editor.collabType');
  if (type === 'tasks') {
    valid = this.fieldsValid(['artefactHeader', 'to', 'taskState', 'taskDate', 'messageTextArea']);
  } else {
    if (type === 'messages') {
      valid = this.fieldsValid(['artefactHeader', 'to', 'messageTextArea']);
    } else {
      if (editPanel.down('#messageTextArea').isValid()) {
        valid = true;
      }
    }
  }
  if (editPanel.down('#selectedChannel').getValue()) {
    if (!editPanel.down('#selectedTopic').isValid()) {
      valid = false;
    }
  }
  if (valid) {
    try {
      var payload = Ext.clone(vm.get('collabData'));
      if (type === 'tasks') {
        payload.state = editPanel.down('#taskState').getValue();
        payload.recipients = vm.get('collabData.recipients');
        var dt = vm.get('collabData.duedate');
        dt.setHours(editPanel.hoursField.getValue(), editPanel.minutesField.getValue(), 0, 0);
        payload.duedate = dt;
      } else {
        if (type === 'messages') {
          payload.recipients = vm.get('collabData.recipients');
        } else {
          delete payload.to;
          delete payload.recipients;
        }
      }
      if (payload.meta && Ext.isObject(payload.meta)) {
      } else {
        payload.meta = {};
      }
      payload.meta = Ext.apply(payload.meta, this.fetchMetaFieldsValues());
      Ext.iterate(payload, function(k, v) {
        if (v === '') {
          delete payload[k];
        }
      });
      delete payload.to;
      var referenceName = vm.get('referenceLabel') || '';
      if (referenceName && referenceName.length) {
        payload.referenceName = referenceName;
      }
      collaboration.Api.createArtefact(collaboration.Api.createArtefactUrlFromType(type), JSON.stringify(payload)).then(function(success) {
        collaboration.Support.notify(type + ' ' + BpcCommon.Api.getTranslation('COLLABORATION_WAS_UPDATED_SUCCESSFULLY'));
        this.closeWindow();
      }.bind(this), function(error) {
        collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ERROR_WHILE_UPDATING'));
        this.closeWindow();
      }.bind(this));
    } catch (e) {
      Ext.log({msg:this.self.getName() + ': ERROR onSend', dump:e, stack:true, level:'error'});
    }
  }
}, showArtefactHistory:function() {
  var vm = this.getViewModel();
  vm.set('editor.historyGridHidden', !vm.get('editor.historyGridHidden'));
}, onAnswer:function() {
  try {
    var sourceRecord = null, sourceModule = null, sourceInstance = null, moduleId = null, collabType = this.getView().collabType;
    if (this.getView().record) {
      sourceRecord = Ext.clone(this.getView().record);
      sourceModule = Ext.clone(this.getView().record.get('src_moduleId'));
      sourceInstance = Ext.clone(this.getView().record.get('src_instanceId'));
      moduleId = this.getView().moduleId;
    }
    if (sourceInstance !== null && sourceModule !== null && sourceRecord !== null) {
      var window = this.getView().up('window');
      if (window) {
        var record = null;
        window.removeAll();
        window.add({xtype:'collabEditorPanel', moduleId:moduleId, collabType:collabType, record:record, sourceRecord:sourceRecord, sourceModule:sourceModule, sourceInstance:sourceInstance, isAnswer:true});
      }
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onAnswer', dump:e, stack:true, level:'error'});
  }
}, fetchMetaFieldsValues:function() {
  var metaFields = this.getView().query('field[metaId]');
  var customMeta = {};
  metaFields.map(function(field) {
    customMeta[field.metaId] = field.getValue();
  });
  return customMeta;
}, updateItem:function(btn, evt, eOpts, leaveWindowOpen) {
  try {
    var vm = this.getViewModel();
    var view = this.getView();
    var itemId = vm.get('collabData.artefactId');
    var payload = Ext.clone(vm.get('collabData'));
    if (payload.id) {
      delete payload['id'];
    }
    payload = collaboration.Module.cleanRecord(payload);
    if (payload.meta && Ext.isObject(payload.meta)) {
    } else {
      payload.meta = {};
    }
    payload.meta = Ext.apply(payload.meta, this.fetchMetaFieldsValues());
    if (view.collabType === 'tasks') {
      payload.state = view.down('#taskState').getValue();
      payload.recipients = vm.get('collabData.recipients');
      var dt = vm.get('collabData.duedate');
      dt.setHours(view.hoursField.getValue(), view.minutesField.getValue(), 0, 0);
      payload.duedate = dt;
    } else {
      if (view.collabType === 'messages') {
        payload.recipients = vm.get('collabData.recipients');
      } else {
        if ('recipients' in payload) {
          delete payload.recipients;
        }
      }
    }
    delete payload.to;
    delete payload.assigneeType;
    payload.lastupdate = new Date;
    collaboration.Api.modifyArtefactByType(view.collabType, itemId, JSON.stringify(payload)).then(function(response, opts) {
      collaboration.Support.notify(view.deriveAssetTitle() + ' ' + BpcCommon.Api.getTranslation('COLLABORATION_WAS_UPDATED_SUCCESSFULLY'));
      if (Ext.isDefined(leaveWindowOpen) && leaveWindowOpen !== null && leaveWindowOpen === true) {
      } else {
        this.closeWindow();
      }
    }.bind(this), function(response, opts) {
      if (Ext.isDefined(response) && Ext.isObject(response) && 'error' in response) {
        var message = response.error;
        if ('errorBpcLangId' in response && Ext.isDefined(response.errorBpcLangId)) {
          message = collaboration.Support.TR(response.errorBpcLangId);
        }
        Ext.toast({iconCls:'x-fal fa-warning', title:BpcCommon.Api.getTranslation('COLLABORATION_ERROR_WHILE_UPDATING'), html:message, anchor:view.getId()});
      } else {
        collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ERROR_WHILE_UPDATING'));
      }
    });
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR updateItem', dump:e, stack:true, level:'error'});
  }
}, getSelectUserMeta:function() {
  var userField = this.getView().down('userSelectCombo');
  return userField.assigneeType;
}, onChannelChange:function(combo) {
  var data = combo.up('collabEditorPanel').getViewModel().get('collabData');
  var record = combo.getSelection();
  if (!record) {
    return;
  }
  var editorPanel = combo.up('collabEditorPanel');
  var topicCombo = editorPanel.down('#selectedTopic');
  topicCombo.setValue('');
  var instanceType = record.get('instanceType');
  var channelId = record.get('channelId');
  var topicsStore = editorPanel.topicsStore;
  if (Ext.isDefined(instanceType) && Ext.isDefined(channelId)) {
    Ext.log({dump:channelId, msg:'get topics for channelId'});
    var topicsArr = collaboration.Api.streams.topics.getByChannelId(channelId);
    if (Ext.isDefined(topicsArr) && Ext.isArray(topicsArr)) {
      topicsStore.loadData(topicsArr);
      topicCombo.enable();
      if (data && data.topicId) {
        var topicInfo = collaboration.Api.getTopicById(data.topicId);
        if (topicInfo && topicInfo.channelId === channelId) {
          editorPanel.down('#selectedTopic').setValue(data.topicId);
        }
      }
    } else {
      Ext.log({dump:channelId, msg:'channelId'});
      collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ERROR_WHILE_FETCHING_TOPICS_FOR_CHANNELID') + ' ' + channelId);
    }
  } else {
    collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ERROR_CHANNELCONFIG_NOT_CORRECT'));
  }
}, closeWindow:function() {
  var view = this.getView();
  view.closeWindow();
}, toggleWebhook:function(btn) {
  var vm = this.getViewModel();
  var vmValue = 'editor.webhookActivated';
  var isActivated = vm.get(vmValue) || false, style = 'color:red', iconCls = isActivated === false ? collaboration.assets.Artefacts.webhookIcon : collaboration.assets.Icons.bug;
  vm.set(vmValue, !isActivated);
  btn.setIconCls(iconCls);
  btn.setStyle(style);
}, deleteItem:function(btn) {
  var $jscomp$this = this;
  var vm = btn.up('collabEditorPanel').getViewModel();
  var id = vm.get('collabData.artefactId');
  if (Ext.isDefined(id)) {
    Ext.Msg.confirm(collaboration.Support.TR('DELETE'), BpcCommon.Api.getTranslation('COLLABORATION_DELETE_ENTRY_WARNING_Q'), function(btnText) {
      if (btnText === 'yes') {
        collaboration.Api.deleteArtefacts([id]).then(function() {
          collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ENTRY_DELETED_SUCCESS'));
          $jscomp$this.closeWindow();
        }, function() {
          collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ENTRY_DELETED_ERROR'));
          $jscomp$this.closeWindow();
        });
      }
    });
  } else {
    collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ERROR_NO_ID_CANT_BE_DELETED'));
  }
}, toggleReferences:function(btn) {
  var vm = this.getViewModel(), iconCls = collaboration.assets.Icons.chain;
  if (vm.get('getModuleName') === null && vm.get('getInstanceName') === null) {
    this.getView().getBpcContext();
  } else {
    this.getView().applyModuleAndInstanceId(null, null);
    iconCls += 'x-fal fa-unlink';
  }
  btn.setIconCls(iconCls);
}});
Ext.define('collaboration.components.EditorPanelViewModel', {extend:'Ext.app.ViewModel', alias:'viewmodel.collabEditorPanelViewModel', data:{collabData:{created:'', creator:'', duedate:'', header:'', lastupdate:'', msg:'', reference:'', referenceType:null, src_instanceId:null, src_moduleId:null, state:null, to:null, meta:{}, flags:{}, tags:'', tagIds:'', priority:'', recipients:[], assigneeType:null, acknowledgeAble:false}, temp:{record:null}, editor:{collabType:'', isNewItem:true, mode:null, historyGridHidden:true, 
webhookActivated:false, onlyCreatorCanfinish:false}, restrictions:{comments:true, messages:true, tasks:true}}, formulas:{acknowledgeAble:function(get) {
  return get('collabData.acknowledgeAble');
}, IsAcknowledged:function(get) {
  var canAck = get('collabData.acknowledgeAble');
  if (canAck) {
    var flags = get('collabData.flags') || {};
    return flags.acknowledged;
  }
  return false;
}, toUsers:{get:function(get) {
  var recipients = get('collabData.recipients');
  var toUsers = [];
  if (Ext.isArray(recipients) && recipients.length > 0) {
    Ext.Array.each(recipients, function(recipient, index, recipients) {
      toUsers.push(recipient.name);
    });
  }
  return toUsers;
}, set:function(val) {
  this.set('to', val);
}}, canDelete:function(get) {
  var can = true;
  var collabType = get('editor.collabType');
  if (Ext.isDefined(collabType) && collabType !== '') {
    can = collaboration.Support.canDelete(collabType);
  }
  return can;
}, isMessage:function(get) {
  var is = false;
  if (get('editor.collabType') === 'messages') {
    is = true;
  }
  return is || !get('restrictions').messages;
}, isComment:function(get) {
  var is = false;
  if (get('editor.collabType') === 'comments') {
    is = true;
  }
  return is || !get('restrictions').comments;
}, isTask:function(get) {
  var is = false;
  if (get('editor.collabType') === 'tasks') {
    is = true;
  }
  return is || !get('restrictions').tasks;
}, isModeEdit:function(get) {
  var is = false;
  if (get('editor.mode') === 'edit') {
    is = true;
  }
  return is;
}, getModuleLink:function(get) {
  var instanceName = get('getInstanceName'), moduleName = get('getModuleName'), text = collaboration.Support.TR('COLLABORATION_ARTEFACT_WILL_BE') + ' ' + instanceName + ' ' + collaboration.Support.TR('LINKED');
  if (instanceName === null && moduleName === null) {
    text = collaboration.Support.TR('COLLABORATION_NO_LINK_ACTIVE');
  }
  return text;
}, getModuleName:function(get) {
  var name = null, id = get('collabData.src_moduleId');
  if (id !== '') {
    var settingName = collaboration.Support.getSetting(id, 'module_name');
    if (Ext.isDefined(settingName)) {
      name = settingName;
    }
  }
  return name;
}, getInstanceName:function(get) {
  var name = null, id = get('collabData.src_instanceId');
  if (id !== '') {
    var settingName = collaboration.Support.getSetting(id, 'module_name');
    if (Ext.isDefined(settingName)) {
      name = settingName;
    }
  }
  return name;
}, answerActive:function(get) {
  var is = false;
  if (get('isComment')) {
    is = true;
  } else {
    if (get('editor.isNewItem') === true || get('isTask') === true) {
      is = true;
    }
  }
  return is;
}, inititalState:function(get) {
  var state = get('collabData.state');
  if (get('editor.isNewItem') === true && get('isTask') === true) {
    state = 'new';
  }
  return state;
}}});
Ext.define('collaboration.components.editor.FileUploadGrid', {extend:'Ext.grid.Panel', alias:'widget.fileUploadGrid', scrollable:true, sortableColumns:false, moduleId:undefined, collabType:undefined, vmHolderComponent:undefined, store:{fields:['created', 'filename', 'contentType'], autoLoad:true}, margin:'0 3 10 80', emptyText:BpcCommon.Api.getTranslation('COLLABORATION_NO_FILE_SELECTED'), bbar:[{xtype:'form', layout:{type:'hbox', align:'stretch'}, items:[{xtype:'filefield', name:'file', fieldLabel:collaboration.Support.TR('FILE'), 
labelWidth:50, allowBlank:false, buttonText:'\x3cspan class\x3d"x-fal fa-folder-open" style\x3d"color:white"\x3e\x3c/span\x3e', listeners:{change:function(fileField, value, eOpts) {
  var newValue = value.replace(/(^.*(\\|\/))?/, '');
  fileField.setRawValue(newValue);
}}}, {xtype:'tbspacer', width:3}, {xtype:'button', iconCls:collaboration.assets.Icons.upload, text:BpcCommon.Api.getTranslation('COLLABORATION_UPLOAD'), handler:function(uploadBtn) {
  var formPanel = uploadBtn.up('fileUploadGrid').down('form');
  var form = formPanel.getForm();
  if (form.isValid()) {
    form.submit({url:collaboration.Module.getBaseUrl() + 'upload', waitMsg:BpcCommon.Api.getTranslation('COLLABORATION_FILES_WILL_BE_UPLOADED'), success:function(fp, o) {
      var fileGrid = uploadBtn.up('fileUploadGrid');
      if (o && o.result && o.result.uploadId) {
        var storeObj = {};
        storeObj['created'] = new Date;
        storeObj['filename'] = o.result.fileName;
        storeObj['contentType'] = o.result.contentType;
        storeObj['_id'] = o.result.uploadId;
        var store = fileGrid.getStore();
        store.add(storeObj);
      }
      fileGrid.updateCountInParentCmp();
    }, failure:function(fp, o) {
      collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_FILES_UPLOAD_ERROR'));
    }});
  }
}}]}], initComponent:function() {
  this.columns = [{text:'\x3cspan class\x3d"' + collaboration.assets.Icons.edit + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + ' ' + collaboration.Support.TR('CREATED'), dataIndex:'created', xtype:'datecolumn', flex:1, format:'d.m.Y H:i:s'}, {text:'\x3cspan class\x3d"' + collaboration.assets.Icons.file + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + ' ' + collaboration.Support.TR('FILENAME'), dataIndex:'filename', flex:2}, {text:'Content-Typ', dataIndex:'contentType', flex:1, 
  renderer:function(value, meta, record, colIndex, rowIndex, store) {
    if (value && Ext.isDefined(value)) {
      var array = value.split('/');
      if (array.length > 0 && array[0] === 'image') {
        value = '\x3cspan class\x3d"' + collaboration.assets.Icons.picture + '" style\x3d"color:grey"\x3e\x3c/span\x3e\x26nbsp;' + value;
        meta.tdAttr = 'data-qtip\x3d"Bilddatei"';
      }
    }
    return value;
  }}, {xtype:'actioncolumn', text:'\x3cspan class\x3d"' + collaboration.assets.Icons.edit + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + ' ' + BpcCommon.Api.getTranslation('COLLABORATION_ADMINISTRATION'), flex:1, items:[{iconCls:collaboration.assets.Icons.download, tooltip:collaboration.Support.TR('DOWNLOAD'), handler:function(grid, rowIndex, colIndex) {
    var rec = grid.getStore().getAt(rowIndex);
    var url = collaboration.Module.getBaseUrl();
    url += 'artefacts?type\x3dfiles\x26id\x3d' + rec.get('artefactId');
    BpcCommon.Util.downloadFile(url);
  }}, {iconCls:collaboration.assets.Icons.remove, tooltip:collaboration.Support.TR('DELETE'), handler:function(tv, rowIndex, colIndex) {
    var store = tv.getStore();
    var rec = store.getAt(rowIndex);
    collaboration.Api.deleteArtefacts([rec.get('artefactId')]).then(function(statusCode) {
      store.removeAt(rowIndex);
      collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_FILE_DELETE_SUCCESS'));
      var fileGrid = tv.up('fileUploadGrid');
      fileGrid.updateCountInParentCmp();
    });
  }}]}];
  this.callParent(arguments);
}, updateCountInParentCmp:function() {
  if (!this.vmHolderComponent) {
    return;
  }
  var holderCmp = this.vmHolderComponent;
  var mv = this.up(holderCmp);
  var store = this.getStore();
  var btns = mv.query('#files');
  btns[0].setText('( ' + store.getCount() + ' )');
  var isNewItem = mv.getViewModel().get('editor.isNewItem');
  if (Ext.isDefined(isNewItem) && isNewItem != null && isNewItem === false) {
    mv.getController().updateItem(undefined, undefined, undefined, true);
  }
}, loadFiles:function(uploadArray) {
  var filesArray = [], me = this, storeData = [];
  Ext.Array.each(uploadArray, function(uploadId) {
    filesArray.push(collaboration.Api.getFile(uploadId));
  });
  Ext.Promise.all(filesArray).then(function(value) {
    Ext.Array.each(value, function(upload, index) {
      var storeObj = {};
      if (upload !== null) {
        storeObj.created = Ext.Date.parse(upload.created, 'c');
        storeObj['artefactId'] = uploadArray[index];
        storeObj.contentType = upload.contentType;
        storeObj.filename = upload.filename;
      } else {
        storeObj.filename = collaboration.Support.TR('FILE_LOAD_ERROR');
      }
      storeData.push(storeObj);
    });
    me.getStore().loadData(storeData);
  });
}, getUploadIds:function() {
  var ids = [];
  var modelArray = this.getStore().getData().getRange();
  Ext.Array.each(modelArray, function(record) {
    var id = record.get('artefactId');
    if (Ext.isDefined(id)) {
      ids.push(id);
    }
  });
  return ids;
}});
Ext.define('collaboration.components.editor.StreamEditorPanel', {extend:'Ext.panel.Panel', alias:'widget.collabStreamEditorPanel', controller:'collabStreamEditorPanelController', streamView:false, viewModel:{data:{collabData:{created:'', creator:'', duedate:'', header:'', lastupdate:'', msg:'', reference:'', src_instanceId:'', src_moduleId:'', state:'', to:'', tags:'', priority:''}, helper:{lastUserRecord:null}, moduleId:null, editor:{collabType:'comments', emptyText:BpcCommon.Api.getTranslation('COLLABORATION_PLEASE_ENTER_YOUR_COMMENT'), 
isNewItem:true, mode:null}, validations:{validMsg:false, validHeader:false, validTo:false, validState:false, validDueDate:false}}, formulas:{isMessage:function(get) {
  var is = false;
  if (get('editor.collabType') === 'messages') {
    is = true;
  }
  return is;
}, isComment:function(get) {
  var is = false;
  if (get('editor.collabType') === 'comments') {
    is = true;
  }
  return is;
}, isTask:function(get) {
  var is = false;
  if (get('editor.collabType') === 'tasks') {
    is = true;
  }
  return is;
}, answerActive:function(get) {
  var is = false;
  if (get('editor.isNewItem') === true || get('isTask') === true) {
    is = true;
  }
  return is;
}, inititalState:function(get) {
  var state = get('collabData.state');
  if (get('editor.isNewItem') === true && get('isTask') === true) {
    state = 'new';
  }
  return state;
}, isEditorValid:function(get) {
  var is = false;
  if (get('editor.collabType') === 'comments') {
    if (get('validations.validMsg')) {
      is = true;
    }
  }
  if (get('editor.collabType') === 'messages') {
    if (get('validations.validMsg') && get('validations.validTo') && get('validations.validHeader')) {
      is = true;
    }
  }
  if (get('editor.collabType') === 'tasks') {
    if (get('validations.validMsg') && get('validations.validTo') && get('validations.validHeader') && get('validations.validState') && get('validations.validDueDate')) {
      is = true;
    }
  }
  return is;
}}}, padding:'5 5 5 5', moduleId:undefined, record:undefined, topicId:undefined, channelId:undefined, layout:'fit', items:[{xtype:'bpcTextArea', itemId:'messageTextArea', border:1, style:{borderColor:'grey', borderStyle:'solid'}, bind:{value:'{collabData.msg}', emptyText:'{editor.emptyText}'}, listeners:{change:function(cmp, newValue, oldValue, eOpts) {
  var mainPanel = cmp.up('collabStreamEditorPanel');
  if (newValue === '') {
    mainPanel.getViewModel().set('validations.validMsg', false);
  } else {
    mainPanel.getViewModel().set('validations.validMsg', true);
  }
}}}], initComponent:function() {
  try {
    var topicsStore = Ext.create('Ext.data.Store', {data:[]});
    this.topicsStore = topicsStore;
    this.tbar = [{xtype:'bpcTextField', labelWidth:'auto', fieldLabel:collaboration.Support.TR('HEADER') + ':', bind:{value:'{collabData.header}', hidden:'{isComment}'}, listeners:{change:function(cmp, newValue, oldValue, eOpts) {
      var mainPanel = cmp.up('collabStreamEditorPanel');
      if (newValue === '') {
        mainPanel.getViewModel().set('validations.validHeader', false);
      } else {
        mainPanel.getViewModel().set('validations.validHeader', true);
      }
    }}}, {xtype:'tbspacer', width:40, bind:{hidden:'{isComment}'}}, {xtype:'bpcTextField', labelWidth:'auto', fieldLabel:collaboration.Support.TR('REFERENCE'), bind:{value:'{collabData.reference}'}}, {iconCls:collaboration.assets.Icons.externalLink, tooltip:'jump to module', handler:function(btn) {
      var instanceId = btn.up('collabStreamEditorPanel').getViewModel().get('collabData.src_instanceId');
      var recordId = btn.up('collabStreamEditorPanel').getViewModel().get('collabData.reference');
      collaboration.Api.jumpToReference(instanceId, recordId);
    }}];
    this.bbar = [{iconCls:collaboration.assets.Artefacts.messageIcon, tooltip:BpcCommon.Api.getTranslation('COLLABORATION_TYPE_MESSAGE_NEW'), bind:{disabled:'{!isComment}'}, collabAction:'createMessage', handler:'changeType'}, {xtype:'userSelectCombo', labelWidth:'auto', bind:{hidden:'{isComment}'}, listeners:{select:function(cmp, record, eOpts) {
      var mainPanel = cmp.up('collabStreamEditorPanel');
      var vm = mainPanel.getViewModel();
      vm.set('helper.lastUserRecord', record);
    }, change:function(cmp, newValue, oldValue, eOpts) {
      var mainPanel = cmp.up('collabStreamEditorPanel'), vm = mainPanel.getViewModel(), value = true;
      if (newValue === '') {
        value = false;
      }
      vm.set('validations.validTo', value);
    }}}, {xtype:'tbspacer', width:15, bind:{hidden:'{isComment}'}}, {iconCls:collaboration.assets.Artefacts.tasksIcon, tooltip:BpcCommon.Api.getTranslation('COLLABORATION_TYPE_TASK_NEW'), collabAction:'createTask', handler:'changeType', bind:{disabled:'{isTask}'}}, {xtype:'stateselectcombo', hideLabel:true, bind:{hidden:'{!isTask}'}, listeners:{change:function(cmp, newValue, oldValue, eOpts) {
      var mainPanel = cmp.up('collabStreamEditorPanel');
      if (newValue === '') {
        mainPanel.getViewModel().set('validations.validState', false);
      } else {
        mainPanel.getViewModel().set('validations.validState', true);
      }
    }}}, {xtype:'bpcDateField', itemId:'taskDate', tooltip:'Fälligkeitsdatum', bind:{value:'{collabData.duedate}', hidden:'{!isTask}'}, listeners:{change:function(cmp, newValue, oldValue, eOpts) {
      var mainPanel = cmp.up('collabStreamEditorPanel');
      if (newValue === '') {
        mainPanel.getViewModel().set('validations.validDueDate', false);
      } else {
        mainPanel.getViewModel().set('validations.validDueDate', true);
      }
    }}}, {iconCls:collaboration.assets.Icons.times, bind:{hidden:'{isComment}'}, handler:function(btn) {
      var vm = btn.up('collabStreamEditorPanel').getViewModel();
      vm.set('editor.collabType', 'comments');
      vm.set('editor.emptyText', BpcCommon.Api.getTranslation('COLLABORATION_PLEASE_ENTER_YOUR_COMMENT'));
    }}, {xtype:'tbspacer', width:15}, {xtype:'tbfill'}, {iconCls:collaboration.assets.Icons.envelope, text:collaboration.Support.TR('SEND'), handler:'onSend', bind:{hidden:'{!editor.isNewItem}', disabled:'{!isEditorValid}'}}];
    var me = this;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR initComponent', dump:e, stack:true, level:'error'});
  }
  this.callParent(arguments);
  if (Ext.isDefined(this.topicId) && Ext.isDefined(this.channelId)) {
    var vm = this.getViewModel();
    vm.set('collabData.topicId', this.topicId);
    vm.set('collabData.channelId', this.channelId);
    vm.set('moduleId', this.moduleId);
  }
}, getCollabReference:function(record, instanceId) {
  var idKey = collaboration.Support.getSetting(instanceId, 'column_id');
  return record && idKey ? record.get(idKey) : undefined;
}, deriveTitle:function(single) {
  if (this.collabType) {
    if (this.collabType === 'comments') {
      if (single) {
        return BpcCommon.Api.getTranslation('COLLABORATION_TYPE_COMMENT_SINGLE');
      }
      return BpcCommon.Api.getTranslation('COLLABORATION_TYPE_COMMENT_NEW');
    }
    if (this.collabType === 'messages') {
      if (single) {
        return BpcCommon.Api.getTranslation('COLLABORATION_TYPE_MESSAGE_SINGLE');
      }
      return BpcCommon.Api.getTranslation('COLLABORATION_TYPE_MESSAGE_NEW');
    }
    if (this.collabType === 'tasks') {
      if (single) {
        return BpcCommon.Api.getTranslation('COLLABORATION_TYPE_TASK_SINGLE');
      }
      return BpcCommon.Api.getTranslation('COLLABORATION_TYPE_TASK_NEW');
    }
  }
  return '';
}, deriveAssetTitle:function() {
  var titleString = '';
  if (this.collabType) {
    if (this.collabType === 'comments') {
      titleString = BpcCommon.Api.getTranslation('COLLABORATION_TYPE_COMMENT_SINGLE');
    }
    if (this.collabType === 'messages') {
      titleString = BpcCommon.Api.getTranslation('COLLABORATION_TYPE_MESSAGE_SINGLE');
    }
    if (this.collabType === 'tasks') {
      titleString = BpcCommon.Api.getTranslation('COLLABORATION_TYPE_TASK_SINGLE');
    }
  }
  return titleString;
}, updateRecord:function(record) {
  Ext.log(this.self.getName() + ': updateRecord');
  try {
    Ext.log({dump:record});
    var me = this;
    me.getViewModel().set('editor.isNewItem', false);
    me.getViewModel().set('editor.mode', 'edit');
    Ext.iterate(record.getData(), function(k, v) {
      me.getViewModel().set('collabData.' + k, v);
    });
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR updateRecord', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('collaboration.components.StreamEditorPanelController', {extend:'Ext.app.ViewController', alias:'controller.collabStreamEditorPanelController', changeType:function(btn) {
  try {
    if (btn && btn.isComponent && btn.collabAction) {
      var me = this.getView();
      if (btn.collabAction === 'createMessage') {
        this.getViewModel().set('editor.collabType', 'messages');
        this.getViewModel().set('editor.emptyText', BpcCommon.Api.getTranslation('COLLABORATION_PLEASE_ENTER_YOUR_MESSAGE'));
        me.down('#to').allowBlank = false;
      }
      if (btn.collabAction === 'createTask') {
        this.getViewModel().set('editor.collabType', 'tasks');
        this.getViewModel().set('editor.emptyText', BpcCommon.Api.getTranslation('COLLABORATION_PLEASE_ENTER_YOUR_TASK'));
        me.down('#taskState').allowBlank = false;
        me.down('#taskDate').allowBlank = false;
        me.down('#to').allowBlank = false;
      }
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR changeType', dump:e, stack:true, level:'error'});
  }
}, getSelectUserMeta:function() {
  var vm = this.getViewModel();
  var selType = null;
  var comboRecordData = vm.get('helper.lastUserRecord');
  if (comboRecordData && comboRecordData.length > 0) {
    if (comboRecordData[0].get('type')) {
      selType = comboRecordData[0].get('type');
    }
  }
  return selType;
}, onSend:function(btn, evt, eOpts) {
  var vm = btn.up('collabStreamEditorPanel').getViewModel();
  var streamEditPanel = btn.up('collabStreamEditorPanel');
  try {
    var type = this.getViewModel().get('editor.collabType');
    var payload = Ext.clone(this.getViewModel().get('collabData'));
    Ext.iterate(payload, function(k, v) {
      if (v === '') {
        delete payload[k];
      }
    });
    if (type === 'tasks') {
      payload.state = streamEditPanel.down('#taskState').getValue();
      payload.to = streamEditPanel.down('#to').getValue();
      payload.msg = streamEditPanel.down('#messageTextArea').getValue();
      payload.assigneeType = this.getSelectUserMeta();
    } else {
      if (type === 'messages') {
        payload.to = streamEditPanel.down('#to').getValue();
        payload.msg = streamEditPanel.down('#messageTextArea').getValue();
        payload.assigneeType = this.getSelectUserMeta();
      } else {
        payload.msg = streamEditPanel.down('#messageTextArea').getValue();
      }
    }
    streamEditPanel.down('#messageTextArea').setValue('');
    payload.topicId = this.getView().topicId;
    payload.channelId = this.getView().channelId;
    var types = ['comments', 'tasks', 'messages'];
    if (types.indexOf(type) !== -1) {
      var messages = {comments:{success:'COLLABORATION_COMMENT_CREATED_SUCCESS', error:'COLLABORATION_COMMENT_CREATED_ERROR'}, tasks:{success:'COLLABORATION_TASK_CREATED_SUCCESS', error:'COLLABORATION_TASK_CREATED_ERROR'}, messages:{success:'COLLABORATION_MESSAGE_CREATED_SUCCESS', failure:'COLLABORATION_MESSAGE_CREATED_ERROR'}};
      collaboration.Api.createArtefact(collaboration.Api.createArtefactUrlFromType(type), JSON.stringify(payload)).then(function(response, opts) {
        this.resetCmp(BpcCommon.Api.getTranslation(messages[type].success));
      }.bind(this), function(response, opts) {
        collaboration.Support.notify(BpcCommon.Api.getTranslation(messages[type].failure));
      });
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onSend', dump:e, stack:true, level:'error'});
  }
}, resetCmp:function(text) {
  try {
    if (Ext.isDefined(text)) {
      collaboration.Support.notify(text);
    }
    var vm = this.getViewModel();
    var view = this.getView();
    if (vm && view) {
      vm.set('collabData', {created:'', channelId:view.channelId, topicId:view.topicId, creator:'', duedate:'', header:'', lastupdate:'', msg:'', reference:'', src_instanceId:'', src_moduleId:'', state:'', to:'', tags:'', priority:''});
      vm.set('editor.collabType', 'comments');
    }
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR onSend', dump:e, stack:true, level:'error'});
  }
}});
Ext.define('collaboration.components.fields.BaseFilterCombo', {extend:'Ext.form.field.ComboBox', alias:'widget.collabbasefiltercombo', moduleId:undefined, queryMode:'local', displayField:'key', valueField:'key', emptyText:'Filter', forceSelection:true, enforceMaxLength:true, viewModel:{data:{valueSelected:false}}, triggers:{reset:{cls:'x-fal fa-times', weight:-1, handler:function() {
  this.setValue('');
}}}, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{[this.returnRenderer(values)]}"\x3e{[this.returnLabel(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnLabel:function(recordData) {
  return Ext.util.Format.htmlEncode2(recordData.key);
}, returnRenderer:function(recordData) {
  return Ext.util.Format.htmlDoubleEncode(JSON.stringify(recordData.val, null, 2));
}})}, bind:{triggerResetHidden:'{!valueSelected}'}, setTriggerResetHidden:function(hidden) {
  Ext.log({msg:this.self.getName() + ': setTriggerResetHidden', level:'info'});
  this.getTrigger('reset').setVisible(!hidden);
}, getTriggerResetHidden:function() {
  Ext.log({msg:this.self.getName() + ': getTriggerResetHidden', level:'info'});
  return !this.getTrigger('reset').isVisible();
}, initComponent:function() {
  var me = this;
  if (!me.moduleId) {
    me.moduleId = collaboration.Module.getDefaultCollabInstance();
  }
  me.triggers.picker.extraCls = 'x-fal fa-filter';
  var filterConfig = collaboration.Support.getSetting(me.moduleId, 'basefilter');
  var store = Ext.create('Ext.data.Store', {});
  if (filterConfig && Ext.isObject(filterConfig)) {
    var data = [];
    Ext.iterate(filterConfig, function(key, val) {
      data.push({key:key, val:val});
    });
    store.loadData(data);
  }
  me.store = store;
  me.callParent(arguments);
}});
Ext.define('collaboration.components.fields.CollabDateRangeField', {extend:'BpcCommon.component.form.field.DateRange', alias:'widget.collabDateRangeField', viewModel:{data:{collabDateRangeEntered:false, globalDateRange:undefined}}, emptyText:BpcCommon.Api.getTranslation('COLLABORATION_DATE_FILTER'), gridReference:undefined, bind:{value:'{globalDateRange.value}', triggerResetHidden:'{!collabDateRangeEntered}'}, triggers:{reset:{cls:'x-fal fa-times', weight:-1, scope:'this', handler:function() {
  this.setValue('');
}}}, setTriggerResetHidden:function(hidden) {
  this.getTrigger('reset').setVisible(!hidden);
}, getTriggerResetHidden:function() {
  return !this.getTrigger('reset').isVisible();
}, listeners:{change:function(cmp, newVal, oldVal, eOpts) {
  try {
    var me = this;
    var grid, store;
    if (me.gridReference) {
      grid = cmp.up(me.gridReference);
    } else {
      var mainView = cmp.up('widgetGridWrapper');
      grid = mainView.down(mainView.childCmp);
    }
    if (this.inTabpanel) {
      var tabs = this.up('tabpanel').query(me.gridReference);
      if (tabs && tabs.length > 0) {
        Ext.each(tabs, function(tab, index) {
          me.processDateQuery(tab, newVal);
        });
      }
      var val = newVal;
      var start = val.start && Ext.isDate(val.start) ? val.start : val.calculatedAbsoluteStartDate && Ext.isDate(val.calculatedAbsoluteStartDate) ? val.calculatedAbsoluteStartDate : undefined;
      var end = val.end && Ext.isDate(val.end) ? val.end : val.calculatedAbsoluteEndDate && Ext.isDate(val.calculatedAbsoluteEndDate) ? val.calculatedAbsoluteEndDate : undefined;
      if (Ext.isDefined(start) && Ext.isDefined(end)) {
        var chart = me.up('tabpanel').query('collabChart')[0];
        if (chart) {
        }
      }
    } else {
      if (grid && grid.isComponent) {
        me.processDateQuery(grid, newVal);
      }
    }
  } catch (e) {
  }
}}, processDateQuery:function(grid, val) {
  try {
    var store = grid.getStore(), p = store.getProxy();
    if (store) {
      var vm = this.getViewModel();
      if (val && !Ext.Object.isEmpty(val)) {
        vm.set('collabDateRangeEntered', true);
        var start = val.start && Ext.isDate(val.start) ? val.start : val.calculatedAbsoluteStartDate && Ext.isDate(val.calculatedAbsoluteStartDate) ? val.calculatedAbsoluteStartDate : undefined;
        var end = val.end && Ext.isDate(val.end) ? val.end : val.calculatedAbsoluteEndDate && Ext.isDate(val.calculatedAbsoluteEndDate) ? val.calculatedAbsoluteEndDate : undefined;
        if (Ext.isDefined(start) && Ext.isDefined(end)) {
          p.setExtraParams({startDate:start, endDate:end});
        }
      } else {
        vm.set('collabDateRangeEntered', false);
        var o = {};
        if (this.gridReference === 'collabTopicList') {
          o.includeCounts = true;
        }
        p.setExtraParams(o);
      }
      store.load();
    }
  } catch (e) {
  }
}, initComponent:function() {
  this.value = {};
  this.callParent(arguments);
  this.getViewModel().bind('{global.dateRange}', this.setDateRange, this, {deep:true});
}, setDateRange:function(value) {
  try {
    if (value.active === true && !Ext.Object.isEmpty(value.value)) {
      this.setValue(value.value);
      this.getViewModel().set('globalDateRange', value);
    }
  } catch (e) {
  }
}});
Ext.define('collaboration.components.base.fields.CollabGridInputField', {extend:'Ext.form.field.Text', alias:['widget.collabGridInput'], emptyText:collaboration.Support.TR('SEARCH'), collabType:undefined, initComponent:function() {
  if (Ext.isDefined(this.collabType)) {
    this.emptyText += ' in ' + collaboration.Module.getAssetForType(this.collabType, 'plural');
  }
  this.listeners = {change:{fn:this.onChange.bind(this), buffer:500}};
  this.callParent(arguments);
}, triggers:{reset:{cls:'x-fal fa-times', hidden:true, scope:'this', handler:function() {
  this.setValue(undefined);
}}}, onChange:function(cmp, newVal, oldVal, eOpts) {
  this.getTrigger('reset').setVisible(newVal && newVal.length > 0);
  this.processQuery(this._grid, newVal);
}, gridStoreLocal:false, applicationToolbarLeftReference:undefined, gridFilterField:undefined, processQuery:function(grid, query) {
  var store = grid.getStore();
  if (!this.gridStoreLocal) {
    var proxy = store.getProxy();
    var params = proxy.getExtraParams() || {};
    params.query = '' + query;
    proxy.setExtraParams(params);
    store.load();
  } else {
    if (query && query === '') {
      store.removeFilter();
    } else {
      store.filter(this.gridFilterField, query);
    }
  }
}});
Ext.define('collaboration.components.base.fields.FilterCombo', {extend:'Ext.form.field.ComboBox', alias:['widget.collabFilterSelector', 'widget.bpcconfig.collabFilterSelector'], viewModel:{data:{setting:undefined, instanceData:[]}, stores:{instanceStore:{data:'{instanceData}'}}}, bind:{value:'{setting.value}', store:'{instanceStore}'}, queryMode:'local', displayField:'filterName', valueField:'filterName', forceSelection:true, enforceMaxLength:true, fieldLabel:collaboration.Support.TR('FILTERNAME'), 
initComponent:function() {
  var storeData = [];
  this.getViewModel().set('setting', this.setting);
  var filterObj = collaboration.Support.getSetting(collaboration.Module.getDefaultCollabInstance(), 'basefilter');
  if (Ext.isObject(filterObj)) {
    var array = Ext.Object.getKeys(filterObj);
    if (array.length > 0) {
      Ext.Array.each(array, function(item, index, array) {
        storeData.push({filterName:item});
      });
    }
  }
  this.callParent(arguments);
  this.getViewModel().set('instanceData', storeData);
}});
Ext.define('collaboration.components.base.fields.RadioFieldTimeRange', {extend:'Ext.form.FieldContainer', alias:['widget.collabRadioTimeRangeContainer'], flex:2, minWidth:180, defaultType:'radiofield', defaults:{flex:1, listeners:{change:function(radio, newValue, oldValue, eOpts) {
  if (radio.getValue() && 'inputValue' in radio) {
    var startDate = radio.inputValue;
    var grid = radio.up('grid');
    var store = grid.getStore();
    if (startDate !== null) {
      store.getProxy().setExtraParam('startDate', radio.inputValue);
      store.getProxy().setExtraParam('endDate', 'now');
    } else {
      var o = {};
      store.getProxy().setExtraParams(o);
    }
    store.load();
  }
}, afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.getId(), html:cmp.tooltipText});
}}, name:'inbox_' + Ext.id()}, layout:{type:'hbox', layout:'stretch'}, items:[{boxLabel:collaboration.Support.TR('ALL_SHORT'), inputValue:null, tooltipText:collaboration.Support.TR('ALL_TASKS')}, {boxLabel:collaboration.Support.TR('LAST_HOUR_SHORT'), inputValue:'now-1h', tooltipText:collaboration.Support.TR('LAST_HOUR')}, {boxLabel:collaboration.Support.TR('LAST_DAY_SHORT'), inputValue:'now-1d', tooltipText:collaboration.Support.TR('LAST_DAY')}, {boxLabel:collaboration.Support.TR('LAST_7_DAYS_SHORT'), 
inputValue:'now-7d', tooltipText:collaboration.Support.TR('LAST_7_DAYS')}]});
Ext.define('collaboration.components.editor.fields.ReferenceFieldContainer', {extend:'Ext.form.FieldContainer', alias:'widget.collabFieldReferenceContainer', layout:{type:'hbox', align:'stretch'}, fieldLabel:collaboration.Support.TR('REFERENCE'), initComponent:function() {
  var instances = BpcCommon.Api.getModuleInstances('vam'), instanceArray = [];
  if (Ext.isDefined(instances)) {
    instanceArray = instances.getData().getRange();
  }
  var menu = {width:100, plain:true, items:[]};
  var items = [{text:'VAM', menu:[]}];
  Ext.Array.each(instanceArray, function(instance) {
    var $jscomp$this = this;
    items[0].menu.push({moduleId:'vam', instanceId:instance.get('moduleId'), text:instance.get('instanceName'), handler:function(btn) {
      var editorPanel = $jscomp$this.up('collabEditorPanel');
      editorPanel.applyModuleAndInstanceId(btn.moduleId, btn.instanceId);
    }});
  });
  menu.items = items;
  this.items = [{xtype:'bpcTextField', emptyText:BpcCommon.Api.getTranslation('COLLABORATION_NO_REFERENCE_SELECTED'), bind:{value:'{collabData.reference}'}, flex:1, listeners:{'change':function(cmp, newVal, oldVal, eOpts) {
    var trigger = cmp.getTrigger('resetField');
    if (cmp.getValue()) {
      trigger.show();
    } else {
      trigger.hide();
    }
  }}, triggers:{resetField:{cls:collaboration.assets.Icons.remove, hidden:true, handler:function() {
    this.up('collabEditorPanel').getViewModel().set('collabData.reference', null);
  }}}}, {xtype:'button', iconCls:collaboration.assets.Icons.chain, enableToggle:true, arrowCls:'', width:32, toggleHandler:'toggleReferences', bind:{tooltip:' {getModuleLink}'}, maybeShowMenu:function(e) {
    return false;
  }, menu:menu, listeners:{afterrender:function(btn) {
    btn.el.on('contextmenu', function(e) {
      e.preventDefault();
      var menu = btn.getMenu();
      if (menu.isVisible(true)) {
        btn.hideMenu();
      } else {
        btn.showMenu();
      }
    });
  }}}, {xtype:'button', iconCls:collaboration.assets.Icons.externalLink, tooltip:BpcCommon.Api.getTranslation('COLLABORATION_JUMP_TP_MODULE'), width:32, handler:function(btn) {
    var editorPanel = btn.up('collabEditorPanel');
    var editorPanelVm = btn.up('collabEditorPanel').getViewModel();
    var instanceId = editorPanelVm.get('collabData.src_instanceId');
    var recordId = editorPanelVm.get('collabData.reference');
    collaboration.Api.jumpToReference(instanceId, recordId);
  }}];
  this.callParent(arguments);
}});
Ext.define('collaboration.components.editor.fields.ReferenceTypeFieldContainer', {extend:'Ext.form.FieldContainer', alias:'widget.colReferenceTypeContainer', layout:{type:'hbox', align:'stretch'}, fieldLabel:collaboration.Support.TR('TYPE'), items:[{xtype:'bpcTextField', emptyText:BpcCommon.Api.getTranslation('COLLABORATION_NO_REFERENCE_TYPE_SELECTED'), bind:{value:'{collabData.referenceType}'}, flex:1, listeners:{'change':function(cmp, newVal, oldVal, eOpts) {
  var trigger = cmp.getTrigger('resetField');
  if (cmp.getValue()) {
    trigger.show();
  } else {
    trigger.hide();
  }
}}, triggers:{resetField:{cls:collaboration.assets.Icons.remove, hidden:true, handler:function() {
  this.up('collabEditorPanel').getViewModel().set('collabData.referenceType', null);
}}}}, {xtype:'button', iconCls:collaboration.assets.Icons.arrowDown, enableToggle:true, arrowCls:'', width:32, bind:{tooltip:BpcCommon.Api.getTranslation('COLLABORATION_CHOOSE_REFERENCE_TYPE')}, menu:null}], initComponent:function() {
  this.callParent(arguments);
  this.loadAvailableReferenceTypes();
}, loadAvailableReferenceTypes:function() {
  var editorPanelVm = this.up('collabEditorPanel').getViewModel();
  var moduleId = editorPanelVm.get('collabData.src_moduleId');
  var instanceId = editorPanelVm.get('collabData.src_instanceId');
  if (Ext.isDefined(moduleId) && Ext.isDefined(instanceId)) {
    if (moduleId !== null && instanceId !== null) {
      var me = this;
      collaboration.Api.getReferenceTypes(instanceId).then(function(esResponse) {
        if (esResponse !== null) {
          var menu = [];
          var module = BpcCommon.Api.getBaseModuleIdById(moduleId);
          Ext.Array.each(esResponse.items, function(item, index, array) {
            var source = item;
            if (source.referenceType && source.referenceType !== null) {
              var btnTitle = collaboration.Support.TR(source.referenceType, module);
              menu.push({refType:source.referenceType, text:btnTitle, handler:function(btn) {
                var container = btn.up('colReferenceTypeContainer');
                editorPanel = container.up('collabEditorPanel');
                editorPanel.getViewModel().set('collabData.referenceType', btn.refType);
              }});
            }
          });
          me.down('button').setMenu(menu);
        }
      });
    }
  }
}});
Ext.define('collaboration.widgets.components.StateSelectTagField', {extend:'Ext.form.field.Tag', alias:['widget.bpcconfig.stateSelectTagField', 'widget.bpcconfig.stateselecttagfield'], viewModel:{data:{setting:undefined, stateList:[]}, stores:{stateListStore:{data:'{stateList}'}}}, fieldLabel:BpcCommon.Api.getTranslation('COLLABORATION_STATE_TO_HIDE'), displayField:'defaultText', valueField:'state', queryMode:'local', bind:{value:'{setting.value}', store:'{stateListStore}'}, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{[this.returnLabel(values)]}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', 
{returnRenderer:function(recordData) {
  var color = '';
  var iconCls = '';
  if (recordData.iconCls) {
    iconCls = recordData.iconCls;
  } else {
    if (recordData.state === 'closed') {
      iconCls = collaboration.assets.Artefacts.states[recordData.state];
    } else {
      if (recordData.state === 'open') {
        iconCls = collaboration.assets.Artefacts.states[recordData.state];
      } else {
        if (recordData.state === 'in_progress') {
          iconCls = collaboration.assets.Artefacts.states[recordData.state];
        } else {
          if (recordData.state === 'new') {
            iconCls = collaboration.assets.Artefacts.states[recordData.state];
          } else {
            if (recordData.state === 'resolved') {
              iconCls = collaboration.assets.Artefacts.states[recordData.state];
            } else {
              iconCls = collaboration.assets.Icons.retweet;
            }
          }
        }
      }
    }
  }
  if (recordData.state === 'closed') {
    color = '#55AA55';
  } else {
    if (recordData.state === 'open') {
      color = '#D49A6A';
    } else {
      if (recordData.state === 'in_progress') {
        color = '#407F7F';
      } else {
        if (recordData.state === 'new') {
          color = '#42A5F5';
        } else {
          if (recordData.state === 'resolved') {
            color = '#42A5F5';
          } else {
            color = '#42A5F5';
          }
        }
      }
    }
  }
  var icon = '\x3cspan class\x3d"' + iconCls + '" style\x3d"color:' + color + ' ; font-size: 18px; margin-right: 3px;"\x3e\x3c/span\x3e';
  return icon + ' ' + Ext.util.Format.htmlEncode2(recordData.defaultText);
}, returnLabel:function(recordData) {
  return Ext.util.Format.htmlDoubleEncode(recordData.defaultText);
}})}, initComponent:function() {
  try {
    this.getViewModel().set('setting', this.setting);
    var moduleId = collaboration.Module.getDefaultCollabInstance();
    var stateArr = [];
    var states = collaboration.Support.getSetting(moduleId, 'default_taskStates');
    if (states.states && Ext.isArray(states.states)) {
      states = states.states;
      Ext.Array.each(states, function(state) {
        stateArr.push({state:state, defaultText:state});
      });
    } else {
      Ext.iterate(states, function(k, v) {
        if (v && Ext.isDefined(v) && Ext.isObject(v)) {
          v['state'] = k;
          stateArr.push(v);
        }
      });
    }
    this.getViewModel().set('stateList', stateArr);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR loadStateList', dump:e, stack:true, level:'error'});
  }
  this.callParent(arguments);
}});
Ext.define('collaboration.components.fields.TagCombo', {extend:'Ext.form.field.ComboBox', alias:'widget.collabTagsCombo', tagWrapperItemId:undefined, queryMode:'local', displayField:'name', emptyText:'Bitte wählen', valueField:'artefactId', listeners:{select:function(c, record) {
  var name = record.get('name');
  if (name && name.length > 0) {
    c.onTagSelect(c, record);
  }
}}, onTagSelect:function(field, record) {
  Ext.log({dump:record, msg:'implement onTagSelect'});
}, initComponent:function() {
  this.fieldLabel = collaboration.Support.TR('TAGS');
  this.store = Ext.create('Ext.data.Store', {proxy:{type:'ajax', url:collaboration.Module.getBaseUrl() + 'tags', reader:{type:'json', rootProperty:'items', totalProperty:'total'}}, autoLoad:true});
  this.callParent(arguments);
}});
Ext.define('collaboration.components.editor.fields.TopicFieldContainer', {extend:'Ext.form.FieldContainer', alias:'widget.collabTopicFieldContainer', layout:{type:'hbox', align:'stretch'}, fieldLabel:collaboration.Support.TR('TOPIC'), topicsStore:undefined, createTopic:false, enableTopics:false, initComponent:function() {
  Ext.log({msg:this.self.getName() + ': initComponent', level:'info'});
  var me = this;
  this.items = [{xtype:'bpcComboBox', forceSelection:true, allowBlank:false, itemId:'selectedTopic', queryMode:'local', emptyText:collaboration.Support.TR('NO_TOPIC_SELECTED'), store:this.topicsStore, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{[this.returnLabel(values)]}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnRenderer:function(recordData) {
    var renderedValue = recordData.topic;
    return Ext.util.Format.htmlEncode2(renderedValue);
  }, returnLabel:function(recordData) {
    var renderedValue = recordData.topic;
    return Ext.util.Format.htmlDoubleEncode(renderedValue);
  }})}, filters:[function(item) {
    var topicId = item.get('topicId') || item.get('id');
    if (topicId) {
      var topicObj = collaboration.Api.getTopicById(topicId);
      if (!topicObj) {
        return;
      }
      return topicObj.active;
    }
    return true;
  }], disabled:!this.enableTopics, valueField:'id', bind:{value:'{collabData.topicId}'}, displayField:'topic', listeners:{select:function(combo, record, eOpts) {
    var channelId = record.get('channelId');
    var topicId = record.get('topicId');
    combo.up('collabEditorPanel').getViewModel().set('collabData.channelId', channelId);
    combo.up('collabEditorPanel').getViewModel().set('collabData.topicId', topicId);
  }}, flex:1}, {xtype:'bpcTextField', itemId:'newTopicName', emptyText:BpcCommon.Api.getTranslation('COLLABORATION_TOPIC_CHOOSE_NAME'), hidden:true, flex:1}, {xtype:'button', iconCls:collaboration.assets.Icons.plus, tooltip:'Topic anlegen', width:32, handler:function(btn) {
    var container = btn.up('collabTopicFieldContainer');
    var selTopic = container.down('#selectedTopic');
    var newTopic = container.down('#newTopicName');
    if (!container.createTopic) {
      selTopic.hide();
      newTopic.show();
      btn.setIconCls(collaboration.assets.Icons.check);
      container.createTopic = true;
    } else {
      newTopic.hide();
      selTopic.show();
      var selectedRecord = btn.up('collabEditorPanel').down('#selectedChannel').getSelection();
      if (Ext.isDefined(selectedRecord)) {
        var instanceId = selectedRecord.get('instanceId'), channelId = selectedRecord.get('channelId');
        if (Ext.isDefined(instanceId)) {
          var topicName = newTopic.getValue();
          var template = {};
          template.topic = topicName;
          template.active = true;
          template.activeFrom = new Date;
          template.lastupdate = new Date;
          template.activeTo = null;
          template.priority = '';
          template.team = '';
          template.channelId = channelId;
          template.instanceId = instanceId;
          template.description = '';
          collaboration.Api.createTopic(instanceId, topicName, template).then(function(successObj) {
            collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_TOPIC_CREATE_SUCCESS'));
            collaboration.Api.streams.topics.refresh(instanceId, channelId).then(function(value) {
              var topicStoreData = collaboration.Api.streams.topics.getByChannelId(channelId);
              selTopic.getStore().loadData(topicStoreData);
              if (successObj && successObj.id) {
                var topicId = successObj.id;
              }
              btn.setIconCls(collaboration.assets.Icons.plus);
              container.up('collabEditorPanel').getViewModel().set('collabData.topicId', topicId);
            }, function(reason) {
              collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_TOPIC_CREATE_ERROR'));
            });
          });
        }
      }
      container.createTopic = false;
    }
  }}];
  this.callParent(arguments);
  me.topicsStore.on('datachanged', me.checkSelection.bind(me));
  me.checkSelection();
}, checkSelection:function() {
  Ext.log({msg:this.self.getName() + ': checkSelection', level:'info'});
  var me = this;
  if (me.topicsStore.getCount() === 1) {
    me.down('#selectedTopic').setValue(me.topicsStore.first());
  }
}});
Ext.define('collaboration.components.webhookViewer.WebhookViewer', {extend:'Ext.panel.Panel', alias:'widget.collabWebhookViewer', width:650, viewModel:{data:{webhook:{artefactId:null, statusCode:null, request:{}, response:{}}}, formulas:{getRequestString:function(get) {
  var string = null, obj = get('webhook.request'), keys = Ext.Object.getKeys(obj);
  if (keys.length > 0) {
    string = JSON.stringify(obj, undefined, 2);
  }
  return string;
}, getResponseString:function(get) {
  var string = null, obj = get('webhook.response'), keys = Ext.Object.getKeys(obj);
  if (keys.length > 0) {
    string = JSON.stringify(obj, undefined, 2);
  }
  return string;
}}}, padding:'5 5 5 5', recordData:undefined, layout:{type:'vbox', align:'stretch'}, items:[{xtype:'bpcTextField', fieldLabel:'ArtefaktId', bind:{value:'{webhook.artefactId}'}}, {xtype:'numberfield', fieldLabel:'StatusCode', readOnly:true, bind:{value:'{webhook.statusCode}'}}, {xtype:'bpcCommonJsonFieldContainer', flex:1, itemId:'request', fieldLabel:'Request'}, {xtype:'bpcCommonJsonFieldContainer', flex:1, itemId:'response', fieldLabel:'Response'}], initComponent:function() {
  this.callParent(arguments);
  if (Ext.isDefined(this.recordData) && Ext.isObject(this.recordData)) {
    if (this.recordData.id) {
      delete this.recordData.id;
    }
    this.getViewModel().set('webhook', this.recordData);
    this.changeWindowTitle(this.recordData);
    if (Ext.isObject(this.recordData.response)) {
      this.down('#response').updateJson(this.recordData.response);
    }
    if (Ext.isObject(this.recordData.request)) {
      this.down('#request').updateJson(this.recordData.request);
    }
  }
}, changeWindowTitle:function(recordData) {
  var win = this.up('window');
  if (win) {
    var title = win.getTitle();
    if (recordData.statusCode) {
      win.setTitle(title + ' - ' + recordData.statusCode);
    }
  }
}});
Ext.define('collaboration.components.window.Window', {extend:'BpcCommon.component.window.WindowMinimize', alias:'widget.collabGenericWindow', mixins:['BpcCommon.mixin.MinimizeWindowTool'], minWidth:275, constrain:true, layout:'fit'});
Ext.define('collaboration.components.editor.EditorWindow', {extend:'collaboration.components.window.Window', alias:'widget.collabEditorWindow', modal:true, width:830, layout:'fit', padding:'5 5 5 5', title:BpcCommon.Api.getTranslation('COLLABORATION_TYPE_COMMENT_NEW'), resizable:false, updateTitle:function(title) {
  this.setTitle(title);
}});
Ext.define('collaboration.config.GenericInstanceSelector', {extend:'Ext.form.field.ComboBox', viewModel:{data:{setting:undefined, instanceData:[]}, stores:{instanceStore:{data:'{instanceData}'}}}, bind:{value:'{setting.value}', store:'{instanceStore}'}, queryMode:'local', displayField:'instanceName', valueField:'moduleId', forceSelection:true, editable:false, fieldLabel:collaboration.Support.TR('INSTANCETYPE'), instanceMatchArray:['TypeToMatch'], initComponent:function() {
  this.getViewModel().set('setting', this.setting);
  this.callParent();
  this.fillStoreWithMatchingInstances(this.instanceMatchArray);
}, fillStoreWithMatchingInstances:function(matchingArray) {
  var instances = collaboration.Support.getStreamInstancesByType(matchingArray);
  this.getViewModel().set('instanceData', instances);
}});
Ext.define('collaboration.config.BlogJournalSelector', {extend:'collaboration.config.GenericInstanceSelector', alias:['widget.bpcconfig.blogJournalSelector'], fieldLabel:collaboration.Support.TR('BLOG_JOURNAL'), instanceMatchArray:['blog', 'journal']});
Ext.define('collaboration.support.ChannelSelector', {extend:'Ext.form.ComboBox', alias:['widget.channelselcetcombo', 'widget.channelSelectCombo'], forceSelection:true, viewModel:{data:{to:null, channelList:[]}, stores:{channelListStore:{data:'{channelList}'}}}, bind:{store:'{channelListStore}', emptyText:collaboration.Support.TR('COLLABORATION_NO_CHANNEL_SELECTED')}, itemId:'channelCombo', valueField:'channelId', displayField:'channel', queryMode:'local', listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{[this.returnRenderer(values)]}"\x3e{[this.returnLabel(values)]}\x3c/div\x3e\x3c/tpl\x3e', 
{returnLabel:function(recordData) {
  var renderedValue = recordData.channel;
  if (recordData.instanceType === 'forum') {
    var channelInfo = collaboration.Api.getChannelById(recordData.channelId);
    if (channelInfo) {
      var icon = '\x3cspan style\x3d"color:' + channelInfo.color + '"\x3e#\x3c/span\x3e';
      return icon + ' ' + Ext.util.Format.htmlEncode2(channelInfo.channel);
    }
  } else {
    var iconCls = collaboration.Support.getSetting(recordData.instanceId, 'module_iconCls');
    renderedValue = collaboration.Module.getIconClsSpan(iconCls, '#333') + ' ' + Ext.util.Format.htmlEncode2(renderedValue);
  }
  return renderedValue;
}, returnRenderer:function(recordData) {
  if (recordData.instanceType === 'forum') {
    return collaboration.Support.TR('FORUM') + ': ' + Ext.util.Format.htmlDoubleEncode(recordData.instanceName);
  }
}})}, initComponent:function() {
  this.callParent(arguments);
  var channels = collaboration.Api.streams.channels.getAll();
  var channelModified = Ext.clone(channels);
  Ext.Array.each(channelModified, function(channel) {
    var name = channel['channel'];
    if (channel['channel'].includes(channel['instanceId'])) {
      name = channel.instanceName;
    }
    channel['channel'] = name;
  });
  this.getViewModel().set('channelList', channelModified);
  if (this.store.getCount() === 1) {
    this.setValue(this.store.first());
  }
}});
Ext.define('collaboration.config.ChannelSetting', {extend:'collaboration.support.ChannelSelector', alias:['widget.bpcconfig.collaborationChannel', 'widget.collaborationChannelSetting']});
Ext.define('collaboration.config.ForumSelector', {extend:'collaboration.config.GenericInstanceSelector', alias:['widget.bpcconfig.forumSelector'], fieldLabel:undefined, instanceMatchArray:['forum']});
Ext.define('collaboration.config.ModuleInstanceSelector', {extend:'collaboration.config.GenericInstanceSelector', alias:['widget.bpcconfig.collabInstanceSelector'], fieldLabel:collaboration.Support.TR('COLLABORATION_MODULEINSTANCE'), instanceMatchArray:['vam', 'monitor'], valueField:'instanceId', fillStoreWithMatchingInstances:function(matchingArray) {
  var instances = collaboration.Support.getBpcModuleInstances(matchingArray);
  this.getViewModel().set('instanceData', instances);
}});
Ext.define('collaboration.config.TopicSelector', {extend:'Ext.form.field.ComboBox', alias:['widget.bpcconfig.topicSelector'], viewModel:{data:{initialValue:'null'}}, bind:{value:'{initialValue}', fieldLabel:'Topic'}, store:Ext.create('Ext.data.Store', {data:[{'topicId':'null', 'topicName':'keine Topic'}, {'topicId':'AL', 'topicName':'Alaska'}]}), queryMode:'local', displayField:'topicName', valueField:'topicId', initComponent:function() {
  this.callParent(arguments);
  this.getTopics(this.settingStore.findRecord('name', 'moduleId', 0, false, false, true));
}, getTopics:function(moduleId) {
  var channelUI = collaboration.Support.getSetting(moduleId, 'instance_channelUi');
  if (channelUI && channelUI === 'blog') {
    var chanConfig = collaboration.Support.getSetting(moduleId, 'instance_channelConfig');
    if (Ext.isDefined(chanConfig) && chanConfig.streamConfig.length > 0) {
      var blogConfig = chanConfig.streamConfig[0];
      if (blogConfig.channelId) {
        collaboration.Api.getTopics(moduleId).then(function(response) {
          if (response && response.total > 0) {
            var topics = response.items;
            var topicsArr = [];
            Ext.Array.each(topics, function(topic) {
              if (!topic['state']) {
                topicsArr.push({id:topic['artefactId'], topicId:topic['artefactId'], topic:topic['topic'], channelId:blogConfig.channelId});
              }
            });
          }
        });
      }
    }
  }
}});
Ext.define('collaboration.config.UiSelector', {extend:'Ext.form.field.ComboBox', alias:['widget.bpcconfig.collaborationui'], viewModel:{data:{}, stores:{uiStore:{data:collaboration.Support.uiTypes}}}, displayField:'viewName', valueField:'type', bind:{store:'{uiStore}', value:'{setting.value}'}, initComponent:function() {
  this.callParent();
  this.getViewModel().set('setting', this.setting);
}});
Ext.define('collaboration.model.CollabData', {extend:'Ext.data.Model', fields:[{name:'created', mapping:'created', type:'date'}, {name:'creator', mapping:'creator'}, {name:'src_instanceId', mapping:'src_instanceId'}, {name:'src_moduleId', mapping:'src_moduleId'}, {name:'src_integrationReference', mapping:'src_integrationReference'}, {name:'reference', mapping:'reference'}, {name:'referenceType', mapping:'referenceType'}, {name:'referenceName', mapping:'referenceName'}, {name:'tags', mapping:'tags'}, 
{name:'tagIds', mapping:'tagIds'}, {name:'lastupdate', mapping:'lastupdate', type:'date'}, {name:'header', mapping:'header'}, {name:'msg', mapping:'msg'}, {name:'recipients', mapping:'recipients'}, {name:'state', mapping:'state'}, {name:'duedate', mapping:'duedate', type:'date'}, {name:'meta', mapping:'meta'}, {name:'channelId', mapping:'channelId'}, {name:'topicId', mapping:'topicId'}, {name:'replyTo', mapping:'replyTo'}, {name:'replyId', mapping:'replyId'}, {name:'to', calculate:function(data) {
  var recipients = data.recipients;
  var toUsers = [];
  if (Ext.isArray(recipients) && recipients.length > 0) {
    Ext.Array.each(recipients, function(recipient, index, recipients) {
      toUsers.push(recipient.name);
    });
  }
  return toUsers;
}}, {name:'overdue', calculate:function(data) {
  var now = new Date;
  var date = new Date(data.duedate);
  var value;
  value = now > date && data.state !== 'resolved';
  return value;
}}, {name:'collabDataType', calculate:function(data) {
  var value = 'comment';
  if (data.duedate) {
    value = 'task';
  }
  if (data.recipients && !data.duedate) {
    value = 'message';
  }
  return value;
}}, {name:'groupCreated', type:'date', calculate:function(data) {
  var date = new Date(data.created);
  date.setSeconds(0);
  date.setMinutes(0);
  date.setHours(0);
  return date;
}}, {name:'acknowledgeAble', calculate:function(data) {
  var $jscomp$destructuring$var10 = data;
  var flags = $jscomp$destructuring$var10.flags;
  flags = flags || {};
  return flags.hasOwnProperty('acknowledged');
}}], idProperty:'artefactId'});
Ext.define('collaboration.model.CollabActivityData', {extend:'collaboration.model.CollabData', fields:[{name:'artefactType', mapping:'artefactType'}]});
Ext.define('collaboration.model.CollabDataChannels', {extend:'Ext.data.Model', fields:[{name:'channel', mapping:'channel'}, {name:'created', mapping:'created', type:'date'}, {name:'creator', mapping:'creator'}, {name:'active', mapping:'active'}, {name:'color', mapping:'color'}, {name:'lastupdate', mapping:'lastupdate', type:'date'}, {name:'instanceId', mapping:'instanceId'}]});
Ext.define('collaboration.model.CollabDataOverview', {extend:'collaboration.model.CollabData', fields:[{name:'collabDataType', calculate:function(data) {
  var value = 'comment';
  if (data.duedate) {
    value = 'task';
  }
  if (data.to && !data.duedate) {
    value = 'message';
  }
  return value;
}}]});
Ext.define('collaboration.model.CollabDataTopics', {extend:'collaboration.model.CollabData', fields:[{name:'topicName', mapping:'topic'}]});
Ext.define('collaboration.model.CollabWebhooks', {extend:'Ext.data.Model', fields:[{name:'created', mapping:'created'}, {name:'hook', mapping:'hookId'}, {name:'statusCode', mapping:'code'}, {name:'artefactId', mapping:'artefactId'}, {name:'artefactType', mapping:'request.artefactType'}, {name:'artefactHeader', mapping:'request.header'}, {name:'response', mapping:'response'}, {name:'request', mapping:'request'}], idProperty:'artefactId'});
Ext.define('collaboration.moduleConfigurationComponents.Main', {extend:'BpcCommon.component.panel.moduleConfigurationComponents.Wrapper', alias:'widget.collaborationConfigMain', initComponent:function() {
  this.callParent(arguments);
  this.setTitle(collaboration.Support.TR('ADMIN'));
  this.setIconCls(collaboration.Support.getBpcIconCls(collaboration.Support.icons.admin));
  this.saveButton.setVisible(false);
  this.resetButton.setVisible(false);
}, onSelect:function(grid, record) {
  this.settingsWrapper.removeAll();
  var collabConfigPanel = Ext.create({xtype:'tabpanel', flex:7, defaults:{moduleId:record.get('moduleId')}, items:[{xtype:'collabAdminGenericPanel'}, {xtype:'collabAdminUserGrid'}, {xtype:'collabAdminStatesGrid'}, {xtype:'collabAdminTags'}, {xtype:'collabAdminTranslationsGrid'}]});
  this.settingsWrapper.add(collabConfigPanel);
}});
Ext.define('collaboration.view.admin.general', {extend:'Ext.panel.Panel', alias:'widget.collabAdminGenericPanel', keysToPersist:[], layout:'fit', items:[], initComponent:function() {
  this.iconCls = collaboration.Support.getBpcIconCls('fa-cog');
  this.title = collaboration.Support.TR('ADMIN_TITLE_GENERAL');
  var fieldset = Ext.create({xtype:'fieldset', layout:'anchor', title:collaboration.Support.TR('ADMIN_GENERAL'), defaults:{anchor:'90%', labelWidth:150}, storeSetting:function(moduleId, id, value) {
    return new Ext.Promise(function(resolve, reject) {
      moduleId = moduleId || 'collaboration';
      var r = BpcCommon.Api.getSetting(moduleId, id, true);
      r.data.value = value;
      r.save({callback:function(r, operation, success) {
        resolve(success);
      }});
    });
  }});
  this.items = [fieldset];
  this.callParent(arguments);
  var cfg = BpcCommon.Api.getSetting('collaboration', 'module_supportedArtefactTypes');
  fieldset.add({xtype:'fieldcontainer', itemId:'artefactTypes', fieldLabel:collaboration.Support.TR('ARTEFACT_TYPES'), defaults:{xtype:'checkboxfield', fieldSet:fieldset, listeners:{change:function(cb) {
    var checkBoxes = cb.up('#artefactTypes').query('checkboxfield');
    var cfg = checkBoxes.map(function(field) {
      return {id:field.collabType, enabled:field.getValue()};
    });
    cb.fieldSet.storeSetting(undefined, 'module_supportedArtefactTypes', cfg).then(function(success) {
      collaboration.Support.notify('UPDATE WAS ' + (success ? '' : 'NOT ') + 'SUCCESSFUL');
    });
  }}}, items:cfg.map(function(typeCfg) {
    return {boxLabel:collaboration.Support.TR('TYPE_' + typeCfg.id), collabType:typeCfg.id, checked:typeCfg.enabled};
  })});
  var showRefType = BpcCommon.Api.getSetting('collaboration', 'showReferenceType');
  var editorConfig = Ext.create({xtype:'fieldcontainer', itemId:'editorConfig', fieldLabel:collaboration.Support.TR('LABEL_EDITOR_SETTINGS'), defaults:{xtype:'checkboxfield', fieldSet:fieldset, listeners:{change:function(cb) {
    cb.fieldSet.storeSetting(undefined, 'showReferenceType', cb.getValue()).then(function(success) {
      collaboration.Support.notify('UPDATE WAS ' + (success ? '' : 'NOT ') + 'SUCCESSFUL');
    });
  }}}, items:[{boxLabel:collaboration.Support.TR('SHOW_REFERENCE_TYPE'), checked:showRefType}]});
  fieldset.add(editorConfig);
  var moduleFeatures = BpcCommon.Api.getSetting('collaboration', 'module_features') || [];
  moduleFeatures.map(function(feature) {
    editorConfig.add({checked:feature.enabled, boxLabel:feature.id, name:'module_features', featureType:feature.id, listeners:{change:function(cb) {
      var checkBoxes = cb.up('#editorConfig').query('checkboxfield');
      checkBoxes = checkBoxes.filter(function(cb) {
        return cb.name === 'module_features';
      });
      var cfg = checkBoxes.map(function(field) {
        return {id:field.featureType, enabled:field.getValue()};
      });
      cb.fieldSet.storeSetting(undefined, 'module_features', cfg).then(function(success) {
        collaboration.Support.notify('UPDATE WAS ' + (success ? '' : 'NOT ') + 'SUCCESSFUL');
      });
    }}});
  });
}});
Ext.define('collaboration.view.admin.states', {extend:'Ext.grid.Panel', alias:'widget.collabAdminStatesGrid', title:'States', keysToPersist:[], initComponent:function() {
  this.iconCls = collaboration.Support.getBpcIconCls('fa-battery-half');
  this.store = Ext.create('Ext.data.Store', {data:collaboration.Support.getStatesAsArray(true), listeners:{datachanged:function(store, eOpts0) {
    this.updateSetting();
  }.bind(this)}});
  this.columns = [{dataIndex:'state', flex:1}, {dataIndex:'state', languageKey:'state_visual', renderer:function(v) {
    return collaboration.Support.renderState(v, undefined, true);
  }, flex:1}, {dataIndex:'color'}, {dataIndex:'iconCls'}, {dataIndex:'languageKey', flex:1}, {dataIndex:'languageKey', flex:1, languageKey:'languageKey_translated', renderer:function(v) {
    return collaboration.Support.TR(v);
  }}, {xtype:'actioncolumn', width:50, items:[{iconCls:collaboration.Support.getBpcIconCls('fa-toggle-on'), tooltip:'Enable / Disable', isActionDisabled:function(view, rowI, colI, item, record) {
    item.iconCls = collaboration.Support.getBpcIconCls(record.get('isDisabled') ? 'fa-toggle-off' : 'fa-toggle-on');
  }, handler:function(grid, rowIndex, colIndex) {
    var rec = grid.getStore().getAt(rowIndex);
    rec.set('isDisabled', !rec.get('isDisabled'));
  }}]}].map(function(i) {
    i.text = collaboration.Support.TR('COL_' + (i.languageKey ? i.languageKey : i.dataIndex));
    return i;
  });
  this.callParent(arguments);
}, updateSetting:function() {
  var settingsData = {};
  this.store.each(function(r) {
    settingsData[r.data.state] = {color:r.data.color, iconCls:r.data.iconCls, languageKey:r.data.languageKey, isDisabled:r.data.isDisabled === true};
  });
  var m = BpcCommon.Api.getSetting(this.moduleId, 'default_taskStates', true);
  m.data.value = settingsData;
  m.save();
}});
Ext.define('collaboration.view.admin.tags', {extend:'Ext.grid.Panel', alias:'widget.collabAdminTags', title:'Tags', keysToPersist:[], getTagObject:function() {
  var id = this.idField.getValue();
  var tag = {instanceId:this.moduleId, name:this.tagNameField.getValue() || '', iconCls:this.iconClsField.getValue(), color:this.previewBtn.color};
  if (id && id.length) {
    tag.artefactId = id;
  }
  return tag;
}, initComponent:function() {
  var $jscomp$this = this;
  this.iconCls = collaboration.Support.getBpcIconCls('fa-tags');
  this.tagNameField = Ext.create({xtype:'textfield', flex:1, emptyText:collaboration.Support.TR('PLEASE_ENTER'), fieldLabel:collaboration.Support.TR('TAG_NAME'), labelWidth:120, listeners:{change:function(f, val) {
    $jscomp$this.tagAddBtn.setDisabled(!val || val && val.length === 0);
  }}});
  this.iconClsField = Ext.create({xtype:'textfield', flex:1, emptyText:collaboration.Support.TR('PLEASE_ENTER'), fieldLabel:collaboration.Support.TR('ICON'), labelWidth:60});
  this.idField = Ext.create({xtype:'textfield', flex:1, readOnly:true, emptyText:collaboration.Support.TR('NEW'), listeners:{change:function(f, val) {
    $jscomp$this.deleteBtn.setDisabled(!val || val && val.length === 0);
  }}});
  this.previewBtn = Ext.create({xtype:'button', width:30, padding:'2 2 2 2', margin:'0 1 0 1', changeColor:function(c) {
    this.setStyle({backgroundColor:c, color:c});
    this.color = c;
  }});
  this.previewBtn.changeColor('#fff');
  this.tagAddBtn = Ext.create({xtype:'button', tagMode:'create', iconCls:collaboration.Support.getBpcIconCls('fa-plus'), handler:function(b) {
    var tagName = $jscomp$this.tagNameField.getValue() || '';
    if (tagName.length > 2) {
      if (b.tagMode === 'create') {
        $jscomp$this.createTag();
      } else {
        $jscomp$this.updateTag();
      }
    }
  }});
  this.resetBtn = Ext.create({xtype:'button', iconCls:collaboration.Support.getBpcIconCls('fa-times'), handler:this.resetForms.bind(this)});
  this.deleteBtn = Ext.create({xtype:'button', iconCls:collaboration.Support.getBpcIconCls('fa-trash'), handler:this.deleteTag.bind(this)});
  this.topContainer = Ext.create({xtype:'container', flex:1, layout:{type:'vbox', align:'stretch'}, defaults:{xtype:'toolbar', flex:1, layout:{type:'hbox', align:'stretch'}}, items:[{items:[this.tagNameField, this.iconClsField, this.previewBtn, this.idField, this.tagAddBtn, this.resetBtn, this.deleteBtn]}, {items:collaboration.Support.virtimo.map(function(color) {
    var style = 'background-color:' + color + ';color:' + color + ';';
    return {xtype:'button', width:30, padding:'2 2 2 2', margin:'0 1 0 1', style:style, handler:function(b) {
      if (collaboration.Support.copyToClipBoard(color, b)) {
        BpcCommon.Api.showNotification(color + ' ' + 'copied to clipboard');
      }
      $jscomp$this.previewBtn.changeColor(color);
    }};
  })}]});
  this.tbar = [this.topContainer];
  this.listeners = {select:function(rowModel, record) {
    $jscomp$this.setEditableTag(record);
  }};
  this.store = Ext.create('Ext.data.Store', {proxy:{type:'ajax', url:collaboration.Module.getBaseUrl() + 'tags', reader:{type:'json', rootProperty:'items', totalProperty:'total'}}, autoLoad:true, listeners:{load:function(s, records, success, operation, eOpts) {
    if (success) {
      $jscomp$this.setTitle('TAGS' + '( ' + s.getTotalCount() + ' ) ');
      $jscomp$this.totalBox.setTitle(success ? s.getTotalCount() : '0');
    }
  }}});
  this.columns = [{dataIndex:'name', text:'Tag', width:240, renderer:function(val, m, r) {
    return '\x3cdiv style\x3d"width:200px;' + collaboration.Support.getTagBoxStyle(false, r.get('color')) + '"\x3e' + val + '\x3c/div\x3e';
  }}, {dataIndex:'color', text:'Color', width:40, renderer:function(val) {
    return '\x3cdiv style\x3d"width:100%;height:100%;background-color:' + val + ';"\x3e\x26nbsp;\x26nbsp;\x26nbsp;\x26nbsp;\x26nbsp;\x26nbsp;\x26nbsp;\x3c/div\x3e';
  }}, {dataIndex:'iconCls', text:'ICON', width:100, renderer:function(v) {
    return v && v.length > 0 ? BpcCommon.util.Icons.getSpanForIconCls(v, '#000') : '';
  }}, {dataIndex:'artefactId', hidden:true}, {dataIndex:'name', flex:1}, {dataIndex:'slug', flex:1, hidden:true}, {dataIndex:'created', width:150, renderer:function(v) {
    return collaboration.Support.dateFormatter(v);
  }}, {dataIndex:'creator'}, {xtype:'actioncolumn', width:60, items:[{iconCls:collaboration.Support.getBpcIconCls('fa-clone'), tooltip:collaboration.Support.TR('CLONE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    $jscomp$this.setEditableTag(r);
    $jscomp$this.createTag();
  }}, {iconCls:collaboration.Support.getBpcIconCls('fa-trash'), tooltip:collaboration.Support.TR('DELETE'), handler:function(grid, rowIndex, colIndex, item, e, r) {
    $jscomp$this.deleteTag(r.get('artefactId'));
  }}]}].map(function(i) {
    i.text = collaboration.Support.TR('COL_' + i.dataIndex);
    return i;
  });
  this.totalBox = Ext.create({xtype:'box', setTitle:function(count) {
    count = count === undefined ? 0 : count;
    this.update('\x26nbsp;\x26nbsp; ' + collaboration.Support.TR('TXT_TOTAL') + ': ' + count);
  }, html:''});
  this.bbar = [this.totalBox, {xtype:'tbfill'}, {iconCls:collaboration.Support.getBpcIconCls('fa-refresh'), handler:function() {
    $jscomp$this.refreshData();
  }}];
  this.callParent(arguments);
}, createTag:function() {
  var $jscomp$this = this;
  return collaboration.Api.createTag(this.getTagObject()).then(function(data) {
    $jscomp$this.resetForms();
    $jscomp$this.refreshData();
  });
}, refreshData:function() {
  this.store.load();
}, updateTag:function() {
  var $jscomp$this = this;
  return collaboration.Api.updateTag(this.getTagObject()).then(function(data) {
    $jscomp$this.resetForms();
    $jscomp$this.refreshData();
  });
}, deleteTag:function(id) {
  var $jscomp$this = this;
  id = id || this.idField.getValue();
  return collaboration.Api.deleteTag(id).then(function(data) {
    $jscomp$this.resetForms();
    $jscomp$this.refreshData();
  });
}, setEditableTag:function(record) {
  this.previewBtn.changeColor(record.get('color') || '#fff');
  this.idField.setValue(record.get('artefactId'));
  this.tagNameField.setValue(record.get('name'));
  this.iconClsField.setValue(record.get('iconCls'));
  this.tagAddBtn.setIconCls(BpcCommon.util.Icons.getBpcIconCls('fa-save'));
  this.tagAddBtn.tagMode = 'update';
}, resetForms:function() {
  this.tagAddBtn.setIconCls(BpcCommon.util.Icons.getBpcIconCls('fa-plus'));
  this.previewBtn.changeColor('#fff');
  this.iconClsField.setValue(undefined);
  this.idField.setValue(undefined);
  this.tagNameField.setValue(undefined);
  this.tagAddBtn.tagMode = 'create';
}});
Ext.define('collaboration.view.admin.translations', {extend:'Ext.grid.Panel', alias:'widget.collabAdminTranslationsGrid', title:'Translations', keysToPersist:[], initComponent:function() {
  this.iconCls = collaboration.Support.getBpcIconCls('fa-globe');
  var translations = BpcCommon.Api.getSetting('collaboration', 'customLanguageKeys') || {};
  this.store = Ext.create('Ext.data.Store', {data:Object.keys(translations).map(function(k) {
    var i = translations[k];
    i.key = k;
    return i;
  })});
  this.columns = [{dataIndex:'key'}, {dataIndex:'de'}, {dataIndex:'en'}, {dataIndex:'key', languageKey:'DEFAULT', renderer:function(v) {
    return BpcCommon.Api.getTranslation(v);
  }}, {xtype:'actioncolumn', width:35, items:[{iconCls:collaboration.Support.getBpcIconCls('fa-times'), tooltip:collaboration.Support.TR('REMOVE'), handler:function(grid, rowIndex, colIndex) {
    grid.getStore().removeAt(rowIndex);
  }}]}].map(function(i) {
    i.flex = 1;
    i.text = collaboration.Support.TR('COL_' + (i.languageKey ? i.languageKey : i.dataIndex || 'ACTIONS'));
    return i;
  });
  this.langKeySelectCombo = Ext.create({xtype:'combo', store:Ext.create('Ext.data.Store', {data:this.getLangKeys()}), queryMode:'local', displayField:'key', valueField:'key', emptyText:collaboration.Support.TR('PLEASE_SELECT'), fieldLabel:collaboration.Support.TR('COL_LANGUAGEKEY'), flex:1, listeners:{change:function(f, newV, oldV) {
    if (newV) {
      this.defaultField.setValue(BpcCommon.Api.getTranslation(newV));
    }
  }.bind(this)}});
  this.deField = Ext.create({xtype:'textfield', emptyText:collaboration.Support.TR('PLEASE_SELECT'), fieldLabel:collaboration.Support.TR('COL_DE'), flex:1});
  this.enField = Ext.create({xtype:'textfield', fieldLabel:collaboration.Support.TR('COL_EN'), emptyText:collaboration.Support.TR('PLEASE_SELECT'), flex:1});
  this.defaultField = Ext.create({xtype:'displayfield', fieldLabel:collaboration.Support.TR('COL_DEFAULT'), flex:1});
  this.dockedItems = [{xtype:'toolbar', dock:'top', padding:'5 5 5 5', items:[this.langKeySelectCombo, this.deField, this.enField, this.defaultField, {xtype:'tbfill'}, {iconCls:collaboration.Support.getBpcIconCls('fa-plus'), text:collaboration.Support.TR('ADD'), grid:this, handler:function(b) {
    var translation = {key:b.grid.langKeySelectCombo.getValue(), de:b.grid.deField.getValue() || b.grid.enField.getValue() || '', en:b.grid.enField.getValue() || b.grid.deField.getValue() || ''};
    b.grid.store.add(translation);
    b.grid.langKeySelectCombo.reset();
    b.grid.deField.reset();
    b.grid.enField.reset();
  }}]}];
  this.bbar = [{xtype:'tbfill'}, {text:collaboration.Support.TR('RESET'), grid:this, handler:function(b) {
    b.grid.storeList({});
  }}, {text:collaboration.Support.TR('SAVE'), grid:this, handler:function(b) {
    var translations = {};
    b.grid.getStore().each(function(r) {
      translations[r.get('key')] = {de:r.get('de'), en:r.get('en')};
    });
    b.grid.storeList(translations);
  }}];
  this.callParent(arguments);
}, getLangKeys:function() {
  if (BpcCommon && BpcCommon.lang && BpcCommon.lang.Current) {
    return Object.keys(BpcCommon.lang.Current).filter(function(i) {
      return i.startsWith('COLLABORATION_');
    }).map(function(k) {
      return {key:k};
    });
  }
}, storeList:function(list) {
  var m = BpcCommon.Api.getSetting('collaboration', 'customLanguageKeys', true);
  m.data.value = list;
  m.save();
}});
Ext.define('collaboration.view.admin.users', {extend:'Ext.grid.Panel', alias:'widget.collabAdminUserGrid', title:'Users', keysToPersist:[], initComponent:function() {
  this.iconCls = collaboration.Support.getBpcIconCls('fa-users');
  this.store = Ext.create('Ext.data.Store', {data:[]});
  this.columns = [{dataIndex:'id'}, {dataIndex:'userName', flex:1}, {dataIndex:'displayName', flex:1}, {dataIndex:'firstName'}, {dataIndex:'lastName'}, {dataIndex:'email'}, {xtype:'checkcolumn', dataIndex:'selected'}].map(function(i) {
    i.text = collaboration.Support.TR('COL_' + i.dataIndex);
    return i;
  });
  this.bbar = [{xtype:'tbfill'}, {text:'Liste zurücksetzen', grid:this, handler:function(b) {
    b.grid.storeList([]);
  }}, {text:'Auswahl übernehmen', grid:this, handler:function(b) {
    var userList = [];
    b.grid.getStore().each(function(r) {
      if (r.get('selected')) {
        var item = {};
        b.grid.keysToPersist.map(function(k) {
          item[k] = r.get(k) || null;
        });
        userList.push(item);
      }
    });
    b.grid.storeList(userList);
    Ext.log({dump:userList, msg:'user'});
  }}];
  this.callParent(arguments);
  this.loadList();
}, loadList:function() {
  collaboration.Api.getUsersSave().then(function(array) {
    this.keysToPersist = array.length ? Object.keys(array[0]) : [];
    var list = BpcCommon.Api.getSetting('collaboration', 'custom_userList');
    if (list.length) {
      var userNames = list.map(function(user) {
        return user.userName;
      });
      array = array.map(function(u) {
        u.selected = userNames.indexOf(u.userName) !== -1;
        return u;
      });
    }
    this.store.add(array);
  }.bind(this));
}, storeList:function(list) {
  var m = BpcCommon.Api.getSetting('collaboration', 'custom_userList', true);
  m.data.value = list;
  m.save();
}});
Ext.define('collaboration.components.plugin.Global', {extend:'BpcCommon.plugin.Interface', inheritableStatics:{NAME:'Collaboration Button', COMPONENT_XTYPE:'collabToolbarKnad', PLUGIN_ID:'collabGlobalMenuButton'}});
Ext.define('collaboration.bpc.plugin.interfaces.GridColumn', {extend:'BpcCommon.plugin.Interface', inheritableStatics:{NAME:'Collaboration Column', COMPONENT_XTYPE:'collabArtefactGridColumn', PLUGIN_ID:'collabArtefactGridColumn'}});
Ext.define('collaboration.components.plugin.global.Button', {extend:'Ext.button.Button', alias:'widget.collabToolbarKnad', viewModel:{data:{commentsCount:0, tasksCount:0, messagesCount:0, actualModuleText:BpcCommon.Api.getTranslation('COLLABORATION_NOT_SET'), actualModuleOverallCount:0, actualModuleIconCls:collaboration.assets.Icons.fire}, formulas:{getActualModuleText:function(get) {
  var text = BpcCommon.Api.getTranslation(get('actualModuleText')), count = get('actualModuleOverallCount');
  text += ' ( ' + count + ' ) ';
  return text;
}, getSumCount:function(get) {
  var commentsCount = get('commentsCount'), tasksCount = get('tasksCount'), messagesCount = get('messagesCount');
  return commentsCount + tasksCount + messagesCount;
}}}, menuAlign:'tr-br?', defaults:{xtype:'menuitem'}, controller:'collabButtonController', initComponent:function() {
  var menuItems = [{text:collaboration.Support.TR('NEW'), iconCls:collaboration.assets.Artefacts.newCollabArtefactIcon, handler:function(b) {
    collaboration.Api.getEditorWindow(null, undefined, collaboration.Module.getDefaultCollabInstance());
  }}, {xtype:'menuseparator'}, {iconCls:collaboration.assets.Icons.commentsIcon, bind:{text:BpcCommon.Api.getTranslation('COLLABORATION_TYPE_COMMENT_PLURAL') + '  ( {commentsCount} )'}, collabType:'comments', handler:'onShowModule'}, {iconCls:collaboration.assets.Artefacts.messageIcon, bind:{text:BpcCommon.Api.getTranslation('COLLABORATION_TYPE_MESSAGE_PLURAL') + '  ( {messagesCount} )'}, collabType:'messages', handler:'onShowModule'}, {iconCls:collaboration.assets.Artefacts.tasksIcon, bind:{text:collaboration.assets.Artefacts.tasksTooltip + 
  '  ( {tasksCount} )'}, collabType:'tasks', handler:'onShowModule'}, {xtype:'menuseparator'}, {iconCls:collaboration.assets.Icons.commentsIcon, bind:{text:BpcCommon.Api.getTranslation('COLLABORATION_INBOX_EN') + '  ( {getSumCount} )'}, linkedInstanceId:collaboration.Module.getDefaultCollabInstance(), handler:'onShowModule'}];
  var typesArraySimple = collaboration.Support.getSupportedTypes().map(function(item) {
    return item.id;
  });
  menuItems = menuItems.filter(function(item) {
    return item.hasOwnProperty('collabType') && typesArraySimple.indexOf(item.collabType) !== -1 || !item.hasOwnProperty('collabType');
  });
  var linkedInstances = this.getLinkedInstances();
  menuItems.concat(linkedInstances).concat(linkedInstances.length > 0 ? [{xtype:'menuseparator'}] : []);
  this.menu = {plain:true, items:menuItems};
  this.iconCls = collaboration.Support.getSetting('collaboration', 'module_iconCls') || collaboration.assets.Icons.commentsIcon;
  this.callParent(arguments);
  this.getController().initOverview();
}, getLinkedInstances:function() {
  var $jscomp$this = this;
  var CoreToolbarConfig = collaboration.Support.getSetting('collaboration', 'module_integrationCoreToolbarConfig') || {};
  if (CoreToolbarConfig && CoreToolbarConfig.linkedInstances) {
    var config = CoreToolbarConfig['linkedInstances'] || [];
    return config.filter(function(item) {
      return Ext.isString(item) || Ext.isObject(item);
    }).map(function(i) {
      return Ext.isString(i) ? $jscomp$this.getLinkedInstanceStringItem(i) : $jscomp$this.getLinkedInstanceObjectItem(i);
    });
  }
  return [];
}, getLinkedInstanceStringItem:function(item) {
  return {text:collaboration.Support.getSetting(item, 'module_name'), iconCls:collaboration.Support.getSetting(item, 'module_iconCls'), linkedInstanceId:item, handler:'onShowModule'};
}, getLinkedInstanceObjectItem:function(item) {
  var obj = item.instanceId ? this.getLinkedInstanceStringItem(item.instanceId) : {};
  if (item.title) {
    obj.text = item.title;
  }
  return obj;
}});
Ext.define('collaboration.components.plugin.global.ButtonController', {extend:'Ext.app.ViewController', alias:'controller.collabButtonController', listen:{global:{artefact:'initOverview', messageUpdated:'initOverview', taskUpdated:'initOverview', itemUpdated:'initOverview', bpcActiveModuleChanged:'onModuleChanged', collabSetReference:'onChangeReferenceTo'}}, onShowModule:function(btn, e, eOpts) {
  if (btn.collabType) {
    var instanceId = collaboration.Module.getDefaultCollabInstance();
    if (instanceId) {
      BpcCommon.Api.showModule(instanceId, false, {tab:btn.collabType});
    } else {
      collaboration.Support.notify(collaboration.Support.TR('NO_INBOX_INSTANCE_ID_FOUND'));
    }
  } else {
    if ('linkedInstanceId' in btn) {
      BpcCommon.Api.showModule(btn.linkedInstanceId);
    }
  }
}, onChangeReferenceTo:function(referenceId) {
  collaboration.Module.lastReferenceId = referenceId;
}, onModuleChanged:function(moduleId) {
  var obj = BpcCommon.Api.getActiveModuleMeta();
  if (obj) {
    this.updateActiveModule(obj);
  }
  collaboration.Api.applyGlobalReference(null);
}, updateActiveModule:function(obj) {
}, initOverview:function() {
  if (collaboration.Api.getGlobalDefaultInstance()) {
    collaboration.Api.getOverview().then(function(successJson) {
      var vm = this.getViewModel();
      if (Ext.isObject(successJson)) {
        Ext.iterate(successJson, function(k, v) {
          vm.set(k, v);
        });
      }
    }.bind(this), function(error) {
    });
  } else {
    collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_ERROR_OVERVIEW_CAN_NOT_BE_LOADED_CONFIG_NOT_CORRECT'));
  }
  this.onModuleChanged();
}});
Ext.define('collaboration.support.generics.Box', {extend:'Ext.Component', alias:['widget.collabGenericBox'], padding:'5 5 5 5'});
Ext.define('collaboration.support.BlogInfoBox', {extend:'collaboration.support.generics.Box', alias:['widget.blogInfoBox', 'widget.bloginfobox'], bind:{html:'{formattedTopicId}'}, vmHolderComponent:undefined, ttPrefix:undefined, listeners:{afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, prefix:cmp.prefix, listeners:{beforeshow:function(tt) {
    var topicId = tt.targetCard.up(tt.targetCard.vmHolderComponent).getViewModel().get('topicId');
    var topicObj = collaboration.Api.getTopicById(topicId);
    if (!topicObj) {
      return;
    }
    var collabType = collaboration.Support.getSetting(topicObj.instanceId, 'instance_channelUi');
    if (collabType && collabType === 'blog') {
      collabType = 'Blog';
    } else {
      if (collabType && collabType === 'journal') {
        collabType = 'Journal';
      }
    }
    var prefix = tt.targetCard.ttPrefix;
    if (!Ext.isDefined(prefix)) {
      prefix = '\x3ctable\x3e\x3ctr\x3e\x3ctd\x3e\x3cb\x3eCreator:\x3c/b\x3e\x3c/td\x3e\x3ctd\x3e' + topicObj.creator + '\x3c/td\x3e\x3c/tr\x3e';
      prefix += '\x3ctr\x3e\x3ctd\x3e\x3cb\x3eCollabType:\x3c/b\x3e\x3c/td\x3e\x3ctd\x3e' + collabType + '\x3c/td\x3e\x3c/tr\x3e';
      if (topicObj.team && topicObj.team !== '') {
        prefix += '\x3ctr\x3e\x3ctd\x3e\x3cb\x3eTeam:\x3c/b\x3e\x3c/td\x3e\x3ctd\x3e' + topicObj.team + '\x3c/td\x3e\x3c/tr\x3e';
      }
      prefix += '\x3c/table\x3e';
    }
    tt.update(prefix);
  }}});
}}});
Ext.define('collaboration.support.BolgUpdatePanel', {extend:'Ext.panel.Panel', alias:'widget.blogUpdatepanel', itemId:'editorPanel', layout:'fit', vmHolderComponent:null, record:null, viewModel:{data:{collabData:undefined, artefactId:'', state_new:''}, formulas:{isTask:function(get) {
  var is = false;
  if (get('collabData.collabType') === 'tasks') {
    is = true;
  }
  return is;
}, inititalState:function(get) {
  return get('collabData.state');
}}}, items:[{xtype:'bpcTextArea', bind:'{collabData.msg}', width:280}], tbar:[{xtype:'stateselectcombo', bind:{hidden:'{!isTask}', value:'{collabData.state}', selection:'{state_new}'}}], bbar:[{iconCls:collaboration.assets.Icons.refresh, text:collaboration.Support.TR('REFRESH'), flex:1, handler:function(btn) {
  try {
    var vm = this.up('blogUpdatepanel').getViewModel();
    var itemId = vm.get('artefactId');
    var payload = vm.get('collabData');
    var collabType;
    if (vm.get('collabData.collabType')) {
      collabType = vm.get('collabData.collabType');
    }
    if (collabType === 'tasks') {
      payload['state'] = vm.get('state_new').get('state');
    }
    if (collabType === 'messages' || collabType === 'comments') {
      delete payload['state'];
    }
    if (payload['state_new']) {
      delete payload['state_new'];
    }
    payload['lastupdate'] = new Date;
    payload = JSON.stringify(payload);
    Ext.log({dump:payload});
    var derivedType = '';
    if (collabType === 'comments') {
      derivedType = collaboration.Support.TR('COLLABORATION_TYPE_COMMENT_SINGLE');
      collaboration.Api.modifyArtefactByType(collabType, itemId, payload).then(function(response, opts) {
        collaboration.Support.notify(derivedType + ' ' + collaboration.Support.TR('COLLABORATION_WAS_UPDATED_SUCCESS') + '.');
      }, function(response, opts) {
        collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_WHILE_UPDATING'));
      });
    } else {
      if (collabType === 'messages') {
        derivedType = collaboration.Support.TR('COLLABORATION_TYPE_MESSAGE_SINGLE');
        collaboration.Api.modifyArtefactByType(collabType, itemId, payload).then(function(response, opts) {
          collaboration.Support.notify(derivedType + ' ' + collaboration.Support.TR('COLLABORATION_WAS_UPDATED_SUCCESS') + '.');
        }, function(response, opts) {
          collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_WHILE_UPDATING'));
        });
      } else {
        if (collabType === 'tasks') {
          derivedType = collaboration.Support.TR('COLLABORATION_TYPE_TASK_SINGLE');
          collaboration.Api.modifyArtefactByType(collabType, itemId, payload).then(function(response, opts) {
            collaboration.Support.notify(derivedType + ' ' + collaboration.Support.TR('COLLABORATION_WAS_UPDATED_SUCCESS') + '.');
          }, function(response, opts) {
            collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_WHILE_UPDATING'));
          });
        }
      }
    }
    var obj = btn.up('#editorPanel');
    obj.destroy();
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR updateItem', dump:e, stack:true, level:'error'});
  }
}}, {iconCls:collaboration.assets.Icons.trash, tooltip:collaboration.Support.TR('DELETE'), handler:function(btn) {
  Ext.Msg.confirm(collaboration.Support.TR('DELETE'), collaboration.Support.TR('COLLABORATION_DELETE_ENTRY_WARNING_Q'), function(btnText) {
    if (btnText === 'yes') {
      try {
        var vm = this.up('blogUpdatepanel').getViewModel();
        var id = vm.get('artefactId');
        if (id && Ext.isString(id)) {
          var url = collaboration.Module.getBaseUrl() + vm.get('moduleId') + '/' + id;
          Ext.Ajax.request({url:url, method:'DELETE', headers:{'Content-Type':'application/json'}, success:function(response, opts) {
            collaboration.Support.notify(collaboration.Support.TR('ENTRY_DELETED_SUCCESS'));
          }, failure:function(response, opts) {
            collaboration.Support.notify(collaboration.Support.TR('ENTRY_DELETED_ERROR'));
          }});
        }
        var obj = btn.up('#editorPanel');
        obj.down('textarea').setValue('');
        obj.setHidden(true);
      } catch (e) {
        Ext.log({msg:this.self.getName() + ': ERROR deleteItem', dump:e, stack:true, level:'error'});
      }
    }
  }, this);
}}, {iconCls:collaboration.assets.Icons.times, handler:function(btn) {
  var obj = btn.up('#editorPanel');
  obj.destroy();
}}], initComponent:function() {
  this.setViewModelData();
  this.callParent(arguments);
}, setViewModelData:function() {
  var me = this;
  var vm = me.getViewModel();
  var recData = me.record;
  if (recData.stateInfo) {
    delete recData.stateInfo;
  }
  if (me.record.collabType) {
    recData.collabType = me.record.collabType;
  } else {
    if (me.record.collabDataType) {
      recData.collabType = me.record.collabDataType + 's';
    }
  }
  vm.set('artefactId', me.record.artefactId);
  recData = collaboration.Module.cleanRecord(recData);
  vm.set('collabData', recData);
}});
Ext.define('collaboration.support.EditBox', {extend:'collaboration.support.generics.Box', alias:['widget.collabEditbox', 'widget.collabEditBox'], bind:{html:'{formattedEdit}'}, vmHolderComponent:undefined, ttPrefix:undefined, style:'cursor: pointer;', listeners:{element:'el', click:'onEditBoxClick', afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, prefix:cmp.prefix, listeners:{beforeshow:function(tt) {
    tt.update(collaboration.Support.TR('EDIT_REPLY'));
  }}});
}}});
Ext.define('collaboration.support.FinishTask', {extend:'collaboration.support.generics.Box', alias:['widget.collabFinishTaskBox'], vmHolderComponent:undefined, ttPrefix:undefined, style:'cursor: pointer;', collabType:'tasks', action:'finish', initComponent:function() {
  this.html = this.createHtml(this.action);
  this.callParent(arguments);
}, createHtml:function(action) {
  var actions = collaboration.Module.getAssetForType(this.collabType, 'actions');
  var iconCls = actions[action].iconCls;
  var style = actions[action].style;
  var text = actions[action].text;
  return '\x3cspan class\x3d"' + iconCls + '" style\x3d"' + style + '"\x3e ' + text + '\x3c/span\x3e';
}, listeners:{element:'el', click:'onFinishTaskClick', afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, prefix:cmp.prefix, listeners:{beforeshow:function(tt) {
    tt.update(collaboration.Support.TR('COLLABORATION_TASK_FINISH'));
  }}});
}}});
Ext.define('collaboration.support.HandBackTask', {extend:'collaboration.support.generics.Box', alias:['widget.collabHandBackTaskBox'], vmHolderComponent:undefined, ttPrefix:undefined, style:'cursor: pointer;', collabType:'tasks', action:'handback', initComponent:function() {
  this.html = this.createHtml(this.action);
  var enabled = collaboration.Support.getSetting(collaboration.Module.getDefaultCollabInstance(), 'instance_quickActionHandbackEnabled');
  this.hidden = !enabled;
  this.callParent(arguments);
}, createHtml:function(action) {
  var actions = collaboration.Module.getAssetForType(this.collabType, 'actions');
  var iconCls = actions[action].iconCls;
  var style = actions[action].style;
  var text = actions[action].text;
  return '\x3cspan class\x3d"' + iconCls + '" style\x3d"' + style + '"\x3e ' + text + '\x3c/span\x3e';
}, listeners:{element:'el', click:'onHandBackTaskClick', afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, prefix:cmp.prefix, listeners:{beforeshow:function(tt) {
    tt.update(collaboration.Support.TR('COLLABORATION_TASK_FINISH'));
  }}});
}}});
Ext.define('collaboration.support.JumpBox', {extend:'collaboration.support.generics.Box', alias:['widget.collabJumpbox', 'widget.collabJumpBox'], bind:{html:'{formattedJumpReference}'}, vmHolderComponent:undefined, ttPrefix:undefined, style:'cursor: pointer;', listeners:{element:'el', click:'onJumpBoxClick', afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, prefix:cmp.prefix, listeners:{beforeshow:function(tt) {
    tt.update(collaboration.Support.TR('COLLABORATION_OPEN_REF_IN_MODUL'));
  }}});
}}});
Ext.define('collaboration.support.ModuleBox', {extend:'collaboration.support.generics.Box', alias:['widget.collabModulebox', 'widget.collabModuleBox'], bind:{html:'{formattedModule}'}, vmHolderComponent:undefined, ttPrefix:undefined});
Ext.define('collaboration.support.ReferenceBox', {extend:'collaboration.support.generics.Box', alias:['widget.collabReferenceBox'], bind:{html:'{formattedReference}'}, style:'cursor: pointer;', listeners:{element:'el', mouseover:'onReferenceBoxOver', mouseout:'onReferenceBoxOut'}});
Ext.define('collaboration.support.ReplyBox', {extend:'collaboration.support.generics.Box', alias:['widget.collabReplyBox', 'widget.collabReplyBox'], bind:{html:'{formattedReply}'}, vmHolderComponent:undefined, ttPrefix:undefined, style:'cursor: pointer;', listeners:{element:'el', click:'onReplyBoxClick', afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, prefix:cmp.prefix, listeners:{beforeshow:function(tt) {
    tt.update(collaboration.Support.TR('REPLY'));
  }}});
}}});
Ext.define('collaboration.support.ReplyPanel', {extend:'Ext.panel.Panel', alias:'widget.replypanel', itemId:'answerPanel', layout:'fit', vmHolderComponent:null, items:[{xtype:'bpcTextArea', emptyText:collaboration.assets.Artefacts.replyDefaultText, width:280}], bbar:[{iconCls:collaboration.assets.Artefacts.replyOnAnswerIcon, text:collaboration.assets.Artefacts.replyOnAnswerText, flex:1, handler:function(btn) {
  var view = btn.up('#answerPanel');
  var vmCmpHolder = view.up(view.vmHolderComponent);
  var obj = {};
  if (vmCmpHolder.instanceMode) {
    var instanceMode = vmCmpHolder.instanceMode;
    if (!instanceMode && instanceMode !== 'popup') {
      obj = vmCmpHolder.down('collabStreamCardCollab').getObjectPreset();
    } else {
      obj = vmCmpHolder.getObjectPreset();
    }
    if (obj.msgBefore) {
      delete obj.msgBefore;
    }
  }
  obj.msg = btn.up('replypanel').down('textarea').getValue();
  obj = view.addReplyProperties(obj);
  var types = ['comments', 'tasks', 'messages'];
  if (types.indexOf(obj.collabType) !== -1) {
    if (obj.collabType === 'messages') {
      obj = view.prepareReply(obj.collabType);
    }
    collaboration.Api.createArtefact(collaboration.Api.createArtefactUrlFromType(obj.collabType), JSON.stringify(obj)).then(function(content) {
      if (content !== null) {
        view.refreshReplyView();
      }
    }, view.handleError);
  }
}}, {iconCls:collaboration.assets.Artefacts.replyOnCancelIcon, tooltip:collaboration.Support.TR('CANCEL'), handler:function(btn) {
  var panel = btn.up('#answerPanel');
  panel.destroy();
}}], addReplyProperties:function(obj) {
  Ext.log(this.self.getName() + ': addReplyProperties');
  try {
    var view = this;
    var vmCmpHolder = view.up(view.vmHolderComponent);
    if (obj && Ext.isObject(obj)) {
      var rawArtefact = vmCmpHolder.getViewModel().get('_source');
      obj.replyTo = rawArtefact.creator + ';' + rawArtefact.created.toString();
      if (vmCmpHolder.getViewModel().get('artefactId')) {
        obj.replyId = vmCmpHolder.getViewModel().get('artefactId');
      }
    }
    return obj;
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR addReplyProperties', dump:e, stack:true, level:'error'});
  }
}, prepareReply:function(collabType) {
  var view = this;
  var vmCmpHolder = view.up(view.vmHolderComponent);
  var reply = null;
  if (collabType === 'messages') {
    var rawArtefact = Ext.clone(vmCmpHolder.getViewModel().get('_source'));
    delete rawArtefact.created;
    var from = rawArtefact.creator;
    delete rawArtefact.creator;
    delete rawArtefact.meta;
    rawArtefact.recipients = [{name:from, type:'user'}];
    rawArtefact.header = 'Re: ' + rawArtefact.header;
    rawArtefact.msg = this.down('textarea').getValue();
    reply = this.addReplyProperties(rawArtefact);
  }
  return reply;
}, handleError:function(content) {
  collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_OCCURED'));
}, resetReplyView:function() {
  this.down('textarea').setValue('');
  this.setHidden(true);
}, refreshReplyView:function() {
  var vmHolderComponent = this.vmHolderComponent;
  if (Ext.isDefined(vmHolderComponent)) {
    var vmCmpHolder = this.up(vmHolderComponent);
    if (Ext.isDefined(vmCmpHolder) && !vmCmpHolder.instanceMode) {
      this.up('collabCommentsWidget').refreshWidget();
    } else {
      this.resetReplyView();
    }
  }
}});
Ext.define('collaboration.support.plugin.ResolveTaskBox', {extend:'collaboration.support.generics.Box', alias:['widget.collabResolveTaskBox'], vmHolderComponent:undefined, ttPrefix:undefined, style:'cursor: pointer;', action:'resolve', initComponent:function() {
  var actions = collaboration.Module.getAssetForType('tasks', 'actions');
  this.html = '\x3cspan class\x3d"' + actions[this.action].iconCls + '" style\x3d"' + actions[this.action].style + '"\x3e ' + actions[this.action].text + '\x3c/span\x3e';
  this.callParent(arguments);
}, listeners:{element:'el', click:'onResolveTaskClick', afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, prefix:cmp.prefix, listeners:{beforeshow:function(tt) {
    tt.update(collaboration.Support.TR('COLLABORATION_TASK_RESOLVE'));
  }}});
}}});
Ext.define('collaboration.support.StateSelectCombo', {extend:'Ext.form.ComboBox', alias:['widget.stateselectcombo'], editable:false, fieldLabel:'Status:', displayField:'translated', valueField:'state', queryMode:'local', itemId:'taskState', listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{[this.returnRenderer(values)]}"\x3e{[this.returnLabel(values)]}\x3c/div\x3e\x3c/tpl\x3e', {returnLabel:function(recordData) {
  return collaboration.Support.renderState(recordData.state, undefined, true);
}, returnRenderer:function(recordData) {
  return collaboration.Support.TR(recordData.text);
}})}, initComponent:function() {
  this.store = Ext.create('Ext.data.Store', {data:collaboration.Support.getStatesAsArray()});
  this.callParent(arguments);
}});
Ext.define('collaboration.support.TakeTaskBox', {extend:'collaboration.support.generics.Box', alias:['widget.collabTakeTaskBox'], vmHolderComponent:undefined, ttPrefix:undefined, style:'cursor: pointer;', collabType:'tasks', action:'take', initComponent:function() {
  this.html = this.createHtml(this.action);
  this.callParent(arguments);
}, createHtml:function(action) {
  var actions = collaboration.Module.getAssetForType(this.collabType, 'actions');
  var iconCls = actions[action].iconCls;
  var style = actions[action].style;
  var text = actions[action].text;
  return '\x3cspan class\x3d"' + iconCls + '" style\x3d"' + style + '"\x3e ' + text + '\x3c/span\x3e';
}, listeners:{element:'el', click:'onTakeTaskClick', afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, prefix:cmp.prefix, listeners:{beforeshow:function(tt) {
    tt.update(collaboration.Support.TR('TASK_TAKE'));
  }}});
}}});
Ext.define('collaboration.support.Timebox', {extend:'collaboration.support.generics.Box', alias:['widget.collabTimebox', 'widget.collabTimeBox'], style:{whiteSpace:'nowrap', overflow:'hidden', textOverflow:'ellipsis'}, bind:{html:'{formattedDateString}'}, vmHolderComponent:undefined, ttPrefix:undefined, listeners:{afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, prefix:cmp.prefix, listeners:{beforeshow:function(tt) {
    var date = tt.targetCard.up(tt.targetCard.vmHolderComponent).getViewModel().get('formattedDate');
    var prefix = tt.targetCard.ttPrefix;
    if (!Ext.isDefined(prefix)) {
      prefix = collaboration.Support.TR('CREATED') + ':';
    }
    tt.update(prefix + date);
  }}});
}}});
Ext.define('collaboration.support.UserBox', {extend:'collaboration.support.generics.Box', alias:['widget.collabUserbox', 'widget.collabUserBox'], bind:{html:'{formattedUserString}'}, vmHolderComponent:undefined, ttPrefix:undefined});
Ext.define('collaboration.support.UserSelectCombo', {extend:'Ext.form.field.Tag', alias:['widget.userselcetcombo', 'widget.userSelectCombo'], viewModel:{data:{to:null, userList:[]}, stores:{userListStore:{data:'{userList}'}}}, emptyText:collaboration.Support.TR('COLLABORATION_USER_GROUP_SELECT'), fieldLabel:collaboration.Support.TR('TO') + ':', displayField:'userName', valueField:'userName', itemId:'to', queryMode:'local', bind:{store:'{userListStore}'}, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{[this.returnLabel(values)]}"\x3e{[this.returnRenderer(values)]}\x3c/div\x3e\x3c/tpl\x3e', 
{returnRenderer:function(recordData) {
  var color = collaboration.assets.Artefacts.userColor;
  var iconCls = collaboration.assets.Artefacts.userIcon;
  var userName = recordData.userName;
  userName = collaboration.Module.getRenderedUsername(userName);
  if (recordData.type === 'group') {
    color = collaboration.assets.Artefacts.groupColor;
    iconCls = collaboration.assets.Artefacts.groupIcon;
  }
  if (recordData.type === 'role') {
    color = collaboration.assets.Artefacts.roleColor;
    iconCls = collaboration.assets.Artefacts.roleIcon;
  }
  return collaboration.Module.getIconClsSpan(iconCls, color) + Ext.util.Format.htmlEncode2(userName);
}, returnLabel:function(recordData) {
  return Ext.util.Format.htmlDoubleEncode(recordData.userName);
}})}, listeners:{change:function(tagField, newVal, oldVal, eOpts) {
  tagField.collapse();
}}, initComponent:function() {
  this.callParent(arguments);
  this.getViewModel().set('userList', collaboration.Module.users);
}, doQuery:function(queryString, forceAll) {
  this.expand();
  this.store.clearFilter(!forceAll);
  var customFilter = new Ext.util.Filter({filterFn:function(item) {
    var firstName = item.get('firstName') === null || !item.get('firstName') ? '' : item.get('firstName');
    var lastName = item.get('lastName') === null || !item.get('lastName') ? '' : item.get('lastName');
    var userName = item.get('userName') === null || !item.get('userName') ? '' : item.get('userName');
    var firstNameMatch = firstName.match(new RegExp(queryString, 'ig')) || [];
    var lastNameMatch = lastName.match(new RegExp(queryString, 'ig')) || [];
    var userNameMatch = userName.match(new RegExp(queryString, 'ig')) || [];
    return firstNameMatch.length > 0 || lastNameMatch.length > 0 || userNameMatch.length > 0;
  }});
  if (!forceAll) {
    this.store.filter(customFilter);
  }
}});
Ext.define('collaboration.view.Main', {extend:'Ext.container.Container', alias:'collab.view.main', layout:'fit', bpcSkipStateInjection:true, collabInitError:false, moduleCfg:undefined, moduleId:undefined, type:undefined, initComponent:function() {
  if (Ext.isDefined(this.moduleId)) {
    var type = collaboration.Support.getSetting(this.moduleId, 'instance_channelUI');
    this.moduleTypeCmp = Ext.create({xtype:'collab' + type, type:type, moduleId:this.moduleId});
    this.type = type.toLowerCase();
    this.items = [this.moduleTypeCmp];
  } else {
    this.items = [{xtype:'box', html:collaboration.Support.TR('COLLABORATION_COLLAB_LOAD_ERROR_1') + '.\x3cbr\x3e\x3cbr\x3e' + collaboration.Support.TR('COLLABORATION_COLLAB_LOAD_ERROR_2') + '\x3cbr\x3e\x3cbr\x3e' + collaboration.Support.TR('COLLABORATION_COLLAB_LOAD_ERROR_3')}];
    this.collabInitError = true;
  }
  this.callParent(arguments);
  if (!this.collabInitError && this.moduleCfg) {
    this.updateInstance(this.moduleCfg);
  }
}, updateInstance:function(configObject) {
  try {
    if (configObject) {
      if (configObject.tab && Ext.isString(configObject.tab) && this.type === 'inbox') {
        this.moduleTypeCmp.switchActiveTab(configObject.tab);
      }
      if (configObject.uiType && configObject.uiType === 'journal') {
        this.moduleTypeCmp.selectTopic(configObject.channelId, configObject.topicId, configObject.uiType);
      }
      if (configObject.uiType && configObject.uiType === 'forum' && this.type === 'forum') {
        this.moduleTypeCmp.updateChannel(configObject.channelId, configObject.topicId, configObject.uiType);
      }
    }
  } catch (e) {
  }
}});
Ext.define('collaboration.view.Feed', {extend:'Ext.panel.Panel', alias:['widget.colFeed'], layout:{type:'vbox', align:'stretch'}, instanceId:undefined, referenceId:undefined, referenceType:undefined, editorPanel:undefined, filesSupported:false, dateAbsolute:false, referenceData:{}, hideArtefactDetails:true, hideStreamInfo:false, initComponent:function() {
  var $jscomp$this = this;
  this.header = false;
  this.moduleId = BpcCommon.Api.getBaseModuleIdById(this.instanceId);
  var editorTopWrapper = Ext.create({xtype:'container', layout:'fit', items:[]});
  var displayField = Ext.create({xtype:'displayfield', width:200});
  this.displayField = displayField;
  var infoField = Ext.create({xtype:'displayfield', width:300});
  var html = '\x3cdiv\x3e\x3cspan style\x3d"display: inline-block; text-align: right; width: 30%; margin-right: 5px;"\x3e ' + collaboration.Support.TR('REFERENCE') + ' \x3c/span\x3e\x3cspan style\x3d"display: inline-block; width: 60%; text-align: left;"\x3e' + this.referenceId + '\x3c/span\x3e\x3c/div\x3e';
  var html1 = '\x3cdiv\x3e\x3cspan style\x3d"display: inline-block; text-align: right; width: 30%; margin-right: 5px;"\x3e' + collaboration.Support.TR('MODULE') + ' \x3c/span\x3e\x3cspan style\x3d"display: inline-block; width: 60%; text-align: left; "\x3e' + this.instanceId + '\x3c/span\x3e\x3c/div\x3e';
  infoField.setValue(html + html1);
  var summaryField = Ext.create({xtype:'displayfield', width:120, hidden:this.hideStreamInfo, updateDisplayData:function(aggregation) {
    aggregation = aggregation === undefined ? [] : aggregation;
    var data = [];
    collaboration.Api.getArtefactTypes().reverse().map(function(artefactType) {
      var matches = aggregation.filter(function(item) {
        return item.value === artefactType;
      });
      var value = matches.length ? matches[0].count : '-1';
      data.push('\x3cdiv\x3e\x3cspan style\x3d"display: inline-block; text-align: right; width: 30%; margin-right: 5px;"\x3e  ' + value + ' \x3c/span\x3e\x3cspan style\x3d"display: inline-block; width: 60%; text-align: left;"\x3e' + collaboration.Support.TR(artefactType.toUpperCase()) + '\x3c/span\x3e\x3c/div\x3e');
    });
    this.setValue(data.join(''));
  }});
  var editorComponent = Ext.create({xtype:'panel', layout:'center', items:[editorTopWrapper]});
  this.feedComponent = this.getFeedComponent(this.instanceId, this.referenceId, this.referenceType);
  this.feedComponent.store.on('prefetch', function(s, r, success, op) {
    if (success) {
      var jsonResponse = collaboration.Support.getResponseObject(op.getResponse()) || {};
      var $jscomp$destructuring$var11 = jsonResponse;
      var aggregations = $jscomp$destructuring$var11.aggregations;
      var $jscomp$destructuring$var12 = aggregations || {};
      var overview = $jscomp$destructuring$var12.overview;
      summaryField.updateDisplayData(overview);
    }
  });
  var conf = {xtype:'container', flex:1, layout:{type:'hbox', align:'stretch'}, items:[this.feedComponent, {xtype:'splitter', hidden:this.hideArtefactDetails}, {xtype:'container', layout:'fit', items:[], flex:1, hidden:this.hideArtefactDetails}]};
  this.items = [editorComponent, conf];
  this.editorPlaceHolder = Ext.create({xtype:'container', layout:'fit', flex:1, items:[]});
  this.resetButton = Ext.create({xtype:'button', iconCls:collaboration.Support.getBpcIconCls('fa-ban'), scale:'large', ui:'secondary', iconAlign:'top', disabled:true, margin:'2 2 2 2', text:collaboration.Support.getTranslation('RESET'), handler:function(btn) {
    $jscomp$this.resetEditor();
  }});
  this.dockedItems = [{xtype:'toolbar', dock:'top', padding:'3 3 3 3', style:'background-color: #eee;', items:[{iconCls:collaboration.Support.getBpcIconCls('fa-info'), scale:'large', ui:'primary', iconAlign:'top', text:collaboration.Support.TR('INFO'), margin:'2 2 2 2'}, infoField, {xtype:'tbfill'}, summaryField, {iconCls:collaboration.Support.getBpcIconCls('fa-sync'), scale:'large', ui:'secondary', iconAlign:'top', text:collaboration.Support.TR('REFRESH'), margin:'2 2 2 2', handler:function() {
    $jscomp$this.feedComponent.store.load();
  }}]}, {xtype:'toolbar', dock:'bottom', padding:'3 3 3 3', style:'background-color: #eee;', items:[].concat(collaboration.Api.getArtefactTypes().map(function(knaId) {
    return {iconCls:collaboration.assets.Artefacts.collabTypes[knaId].iconCls, text:collaboration.assets.Artefacts.collabTypes[knaId].text, collabItem:collaboration.assets.Artefacts.collabTypes[knaId], type:knaId, scale:'large', ui:'primary', iconAlign:'top', padding:'9 0 9 0', margin:'2 2 2 2', knaButtons:true, collabEditor:undefined, handler:function(b) {
      b.setDisabled(true);
      $jscomp$this.resetButton.setDisabled(false);
      displayField.setVisible(false);
      $jscomp$this.editorPlaceHolder.removeAll();
      $jscomp$this.editorPanel = undefined;
      $jscomp$this.editorPlaceHolder.add($jscomp$this.getEditorPanel(b.type));
    }, listeners:{mouseover:function(b) {
      displayField.setValue(b.collabItem.create);
    }, mouseout:function() {
      displayField.setValue('');
    }}};
  })).concat([displayField, this.editorPlaceHolder, {iconCls:collaboration.Support.getBpcIconCls('fa-upload'), text:collaboration.Support.TR('UPLOAD'), iconAlign:'top', scale:'large', margin:'2 2 2 2', ui:'primary', hidden:!this.filesSupported, handler:function() {
    Ext.toast('Wiring needed...');
  }}, this.resetButton, {iconCls:collaboration.assets.Artefacts.replyOnCancelIcon, scale:'large', ui:'secondary', margin:'2 2 2 2', iconAlign:'top', text:collaboration.Support.getTranslation('CLOSE'), handler:function(btn) {
    var w = $jscomp$this.up('window');
    if (w) {
      w.close();
    }
  }}])}];
  this.callParent(arguments);
}, resetEditor:function() {
  this.resetButton.setDisabled(true);
  this.query('[knaButtons]').map(function(b) {
    return b.setDisabled(false);
  });
  this.displayField.setVisible(true);
  this.editorPlaceHolder.removeAll();
  this.editorPanel = undefined;
}, getEditorPanel:function(type) {
  var $jscomp$this = this;
  if (!this.editorPanel) {
    var subjectField = Ext.create({xtype:'textfield', emptyText:'Subject', hidden:type === 'comments'});
    var toField = Ext.create({xtype:'userSelectCombo', width:150, emptyText:'To', fieldLabel:undefined, hidden:type === 'comments'});
    var states = collaboration.Support.getSetting(collaboration.Module.getDefaultCollabInstance(), 'default_taskStates');
    var storeData = Object.keys(states).map(function(key) {
      return {key:key, value:collaboration.Support.TR(states[key].languageKey || 'NOT_FOUND') || key};
    });
    var stateField = Ext.create('Ext.form.ComboBox', {queryMode:'local', emptyText:'State', displayField:'value', valueField:'key', hidden:type !== 'tasks', store:Ext.create('Ext.data.Store', {data:storeData})});
    if (type === 'tasks') {
      stateField.setValue('new');
    }
    var dueDateField = Ext.create({xtype:'datefield', emptyText:'DueDate', hidden:type !== 'tasks'});
    var messageField = Ext.create({xtype:'textfield', emptyText:'Message', flex:1});
    this.editorPanel = Ext.create({xtype:'container', layout:{type:'vbox', align:'stretch'}, margin:'2 2 2 2', listeners:{afterrender:function() {
      if (type === 'comments') {
        messageField.focus();
      } else {
        subjectField.focus();
      }
    }}, items:[{xtype:'toolbar', padding:'1 1 1 1', items:[subjectField, toField, stateField, dueDateField]}, {xtype:'toolbar', padding:'1 1 1 1', items:[messageField, {iconCls:collaboration.Support.getBpcIconCls('fa-play'), ui:'primary', handler:function(b) {
      var artefact = Ext.apply({}, {type:type, msg:messageField.getValue()}, $jscomp$this.referenceData);
      if (type !== 'comments') {
        artefact.header = subjectField.getValue();
        artefact.recipients = toField.getValue().map(function(userName) {
          return {name:userName, type:'user'};
        });
        if (type === 'tasks') {
          artefact.state = stateField.getValue();
          artefact.duedate = (dueDateField.getValue() || new Date).toISOString();
        }
      }
      collaboration.Api.createArtefact(collaboration.Api.createArtefactUrlFromType(type), JSON.stringify(artefact)).then(function(success) {
        $jscomp$this.resetEditor();
        $jscomp$this.feedComponent.store.load();
      }, function(error) {
      });
    }}]}]});
  }
  return this.editorPanel;
}, getArtefactDisplaySettings:function() {
  var $jscomp$destructuring$var13 = this;
  var filesSupported = $jscomp$destructuring$var13.filesSupported;
  var dateAbsolute = $jscomp$destructuring$var13.dateAbsolute;
  return {filesSupported:filesSupported, dateAbsolute:dateAbsolute};
}, getFeedComponent:function(instanceId, reference, referenceType) {
  this.referenceData = {reference:reference, src_moduleId:this.moduleId, src_instanceId:instanceId};
  return Ext.create({xtype:'collabReferenceFeed', feedPanel:this, referenceId:reference, src_moduleId:this.moduleId, src_instanceId:instanceId, referenceType:referenceType, displaySettings:this.getArtefactDisplaySettings(), flex:4});
}});
Ext.define('collaboration.view.uitypes.Inbox', {extend:'Ext.container.Container', alias:['widget.collabInbox', 'widget.collabinbox'], stateful:true, stateId:'collabInboxState', layout:'border', initComponent:function() {
  var supportedTypes = collaboration.Support.getSupportedTypes();
  var cmpDefinitions = {comments:{}, messages:{}, tasks:{}};
  var tabItems = supportedTypes.map(function(item) {
    var cmpCfg = cmpDefinitions[item.id];
    cmpCfg = Ext.applyIf(cmpCfg, {title:collaboration.assets.Artefacts[item.id + 'Text']});
    cmpCfg.collabType = item.id;
    return cmpCfg;
  });
  this.tabPanel = Ext.create({xtype:'tabpanel', tabPosition:'left', region:'center', activeTab:0, stateEvents:['tabchange'], stateful:true, stateId:'collabInboxTabState', tabRotation:0, flex:3, defaults:{textAlign:'left', moduleId:this.moduleId, xtype:'collabTypewrapper'}, items:tabItems, applyState:function(state) {
    if (state) {
      if (state.activeTab) {
        this.setActiveTab(state.activeTab);
      }
      Ext.apply(this, state);
    }
  }, listeners:{beforestatesave:function(tab, state) {
    tab.addPropertyToState(state, 'activeTab', tab.items.indexOf(tab.getActiveTab()));
  }}});
  this.items = [this.tabPanel];
  if (!collaboration.Support.getSetting(collaboration.Module.getDefaultCollabInstance(), 'instance_inboxHideActivities')) {
    this.tabPanel.add({xtype:'collabActivityGrid', collabType:'activities', iconCls:collaboration.assets.Artefacts.activitiesIcon, title:collaboration.assets.Artefacts.activitiesText});
    this.items.push({xtype:'collabStreamGrid', region:'east', collapsible:true, collapsed:true, iconCls:collaboration.assets.Artefacts.activitiesIcon, title:collaboration.Support.TR('COLLABORATION_ACTIVITIES'), moduleId:this.moduleId, collabType:'activities', flex:1});
  }
  this.callParent(arguments);
}, switchActiveTab:function(collabType) {
  var arr = this.tabPanel.query('collabTypewrapper[collabType\x3d' + collabType + ']');
  this.tabPanel.setActiveTab(arr.length ? arr[0] : 0);
}});
Ext.define('collaboration.view.uitypes.streams.BaseStreamView', {extend:'Ext.container.Container', alias:'widget.collabStreamBase', moduleCfg:undefined, collabInitError:false, moduleId:undefined, channelPrefix:undefined, layout:'fit', streamHolderCmp:undefined, defaults:{}, lastTopic:{}, initComponent:function() {
  this.prepareDefaults();
  if (Ext.isDefined(this.moduleId)) {
    this.bootStrapStreamView(this.moduleId);
  } else {
    collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_MODULEID_NOT_CORRECT_CHECK_CONFIG'));
  }
  this.callParent(arguments);
}, prepareDefaults:function() {
  Ext.log('This function should be overwritten by the component which extends this baseComponent');
}, bootStrapStreamView:function(moduleId) {
  var me = this;
  if (Ext.isDefined(this.channelPrefix)) {
    collaboration.Support.initBlogJournal(moduleId, this.channelPrefix).then(function(channelId) {
      me.channelId = channelId;
      me.initStreamView(channelId);
    }, function(errorMsg) {
      collaboration.Support.notify(errorMsg);
    });
  }
}, initStreamView:function(channelId) {
  Ext.log('This function should be overwritten by the component which extends this baseComponent');
}, selectTopic:function(channelId, topicId, uiType) {
  var topicHolder = this.down('collabTopicList');
  if (Ext.isDefined(topicHolder) && Ext.isDefined(uiType)) {
    Ext.defer(function() {
      topicHolder.selectTopic(channelId, topicId, uiType);
    }, 100);
  } else {
    collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_INSTANCE_NOT_TYPE_STREAM'));
  }
}, updateStream:function(channelId, topicId, moduleId, topicRecord) {
  this.lastTopic.channelId = channelId;
  this.lastTopic.topicId = topicId;
  this.lastTopic.moduleId = moduleId;
  var item = {xtype:'collabTopic', channelId:channelId, moduleId:moduleId, showNewButton:true, topicId:topicId, header:false, flex:1, topicRecord:topicRecord};
  var streamHolder = this.down('#' + this.streamHolderCmp);
  var currentTopic;
  var currentlyOpenedTopics = streamHolder.getLayout().getLayoutItems();
  if (currentlyOpenedTopics && currentlyOpenedTopics.length > 0) {
    Ext.each(currentlyOpenedTopics, function(topic) {
      if (topic.topicId === topicId) {
        currentTopic = topic;
      }
    });
  }
  if (!currentTopic && streamHolder && streamHolder.isComponent) {
    currentTopic = streamHolder.add(item);
  }
  streamHolder.getLayout().setActiveItem(currentTopic);
}, refreshStream:function() {
  var channelView = this.down('collabChannelView');
  if (channelView !== null && Ext.isDefined(channelView) && channelView.isComponent) {
    channelView.getStore().load();
  } else {
    collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_CHANNELVIEW_NOT_FOUND') + '.');
  }
}, getTopicUrl:function() {
  var obj = null;
  if (this.lastTopic.channelId && this.lastTopic.topicId) {
    obj = this.lastTopic;
  }
  return obj;
}});
Ext.define('collaboration.view.uitypes.streams.Blog', {extend:'collaboration.view.uitypes.streams.BaseStreamView', alias:['widget.collabBlog', 'widget.collabblog'], instanceMode:undefined, channelId:undefined, channelPrefix:'blog', collabStreamMode:'blog', moduleId:undefined, defaults:{}, prepareDefaults:function() {
  this.defaults = {moduleId:this.moduleId, instanceMode:this.instanceMode};
}, initStreamView:function(channelId) {
  if (Ext.isDefined(channelId)) {
    var defaults = this.defaults;
    defaults['channelId'] = channelId;
    defaults['header'] = false;
    this.add([{xtype:'collabTopicChannelBase', defaults:defaults}]);
  } else {
    collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_CHANNELID_NOT_CORRECT_CHECK_CONFIG'));
  }
}, enableGrouping:function(bool, groupKey) {
  this.removeAll();
  var defaults = this.defaults;
  defaults['channelId'] = this.channelId;
  defaults['header'] = false;
  defaults['groupingEnabled'] = bool;
  defaults['groupKey'] = groupKey;
  this.add([{xtype:'collabTopicChannelBase', defaults:defaults}]);
}});
Ext.define('collaboration.view.uitypes.streams.CollabForumChannelEditor', {extend:'Ext.form.Panel', alias:'widget.collabForumChannelEditor', layout:{type:'vbox', align:'stretch'}, viewModel:{data:{allChannelFieldsValid:false}}, padding:'5 5 5 5', record:undefined, moduleId:undefined, gridReference:undefined, listeners:{fieldvaliditychange:function(fieldAncestor, field, isValid, eOpts) {
  var form = this;
  var fields = form.getForm().getFields();
  if (isValid) {
    var allFieldsValid = true;
    fields.each(function(field, index, len) {
      if (!field.isValid()) {
        allFieldsValid = false;
        return false;
      }
    });
    form.getViewModel().set('allChannelFieldsValid', allFieldsValid);
  } else {
    form.getViewModel().set('allChannelFieldsValid', false);
  }
}}, initComponent:function() {
  var me = this;
  me.setViewModelData();
  me.items = [{xtype:'bpcTextField', allowBlank:false, fieldLabel:collaboration.Support.TR('CHANNEL_NAME'), labelWidth:50, name:'channel', bind:'{channel}', listeners:{specialkey:me.onSpecialKey.bind(me)}}, {xtype:'colorfield', allowBlank:false, fieldLabel:collaboration.Support.TR('COLOR'), labelWidth:50, name:'color', bind:'{color}'}];
  me.buttons = [{xtype:'tbfill'}, {xtype:'button', bind:{disabled:'{!allChannelFieldsValid}'}, padding:'5 5 5 5', itemId:'channelSubmitBtn', iconCls:me.record ? collaboration.assets.Artefacts.refreshIcon : collaboration.assets.Artefacts.createIcon, text:me.record ? collaboration.Support.TR('REFRESH') : collaboration.Support.TR('COLLABORATION_GENERATE'), handler:me.createModifyChannel.bind(this)}, {iconCls:collaboration.assets.Artefacts.replyOnCancelIcon, localized:{text:'COLLABORATION_CANCEL'}, padding:'5 5 5 5', 
  handler:function(btn) {
    btn.up('window').close();
  }}];
  this.callParent(arguments);
}, onSpecialKey:function(field, e) {
  try {
    var me = this;
    if (e.getKey() == e.ENTER) {
      me.down('#channelSubmitBtn').click();
    }
  } catch (e$3) {
  }
}, setViewModelData:function() {
  if (this.record) {
    var data = this.record.getData();
    if (data['id']) {
      delete data['id'];
    }
    if (!data['color']) {
      data['color'] = '#' + (Math.random() * 16777215 << 0).toString(16);
    }
    data = collaboration.Module.cleanRecord(data);
    this.getViewModel().set(data);
  } else {
    this.getViewModel().set('color', '#' + (Math.random() * 16777215 << 0).toString(16));
  }
}, createModifyChannel:function() {
  var me = this;
  var payload = {lastupdate:new Date, instanceId:me.moduleId.toString(), instanceType:'forum'};
  me.getForm().getFields().each(function(field) {
    payload[field.getName()] = field.getValue();
  });
  if (payload.color.indexOf('#') === -1) {
    payload.color = '#' + payload.color;
  }
  if (me.record) {
    var channelId = me.record.getData().artefactId;
    payload.active = me.record.get('active');
    collaboration.Api.modifyChannel(me.moduleId, channelId, payload).then(function() {
      var panels = me.gridReference.up('tabpanel').items.items;
      Ext.Array.each(panels, function(panel) {
        panel.getStore().load();
      });
      me.up('window').close();
    });
  } else {
    var url = collaboration.Module.getBaseUrl() + me.moduleId + '/channels/' + payload.channel;
    payload.active = true;
    collaboration.Api.createChannel(me.moduleId, payload.channel, payload).then(function(obj) {
      if (obj !== null && obj.id) {
        me.gridReference.up('collab' + me.gridReference.collabStreamMode).updateChannel(obj.id, me.moduleId);
        Ext.Function.defer(function() {
          me.gridReference.getStore().load();
        }, 500);
        me.up('window').close();
      }
    });
  }
}});
Ext.define('collaboration.view.uitypes.streams.Forum', {extend:'collaboration.view.uitypes.streams.BaseStreamView', alias:['widget.collabForum', 'widget.collabforum'], instanceMode:undefined, moduleCfg:undefined, layout:{type:'hbox', align:'stretch'}, streamHolderCmp:'forumTopicHolderCmp', initComponent:function() {
  this.defaults = {moduleId:this.moduleId, instanceMode:this.instanceMode, header:false, collabStreamMode:'forum'};
  this.items = [{xtype:'tabpanel', defaults:this.defaults, flex:1, items:[{xtype:'collabForumChannelView', collabTitlePrefix:collaboration.assets.Artefacts.topicStateActive}, {xtype:'collabForumChannelView', collabTitlePrefix:collaboration.assets.Artefacts.topicStateArchieved}], listeners:{tabchange:function(tabPanel, newCard, oldCard, eOpts) {
    oldCard.setTitle('( ' + oldCard.totalCount + ' )');
    newCard.setTitle(newCard.collabTitlePrefix + ' ( ' + newCard.totalCount + ' )');
  }}}, {xtype:'container', flex:5, layout:{type:'hbox', align:'stretch'}, items:[{xtype:'collabTopicChannelBase', flex:2, itemId:'forumChannelHolderCmp', defaults:this.defaults}, {xtype:'container', flex:5, layout:'card', itemId:this.streamHolderCmp, items:[{xtype:'component', html:collaboration.Support.TR('COLLABORATION_NO_TOPIC_SELECTED_NO_ENTRIES')}]}]}];
  this.callParent(arguments);
}, updateChannel:function(channelId, topicId, uiType) {
  var channelHolder = this.down('collabTopicChannelBase');
  var topicListGrids = channelHolder.query('collabTopicList');
  Ext.Array.each(topicListGrids, function(grid) {
    grid.updateStoreWithChannelId(channelId);
  });
  this.selectTopic(channelId, topicId, uiType);
}, refreshTopics:function() {
  var channelHolder = this.down('collabTopicChannelBase');
  channelHolder.refreshTopics();
}});
Ext.define('collaboration.view.uitypes.streams.grid.ChannelView', {extend:'Ext.grid.Panel', alias:'widget.collabChannelView', title:false, sortableColumns:false, controller:'collabChannelViewController', hideHeaders:true, overflowY:'auto', trackMouseOver:false, collabType:undefined, instanceMode:'popup', moduleId:undefined, channelId:undefined, topicId:undefined, src_moduleId:undefined, src_instanceId:undefined, feedType:'standardStream', feedTypeConfig:{standardStream:{viewConfig:{emptyText:collaboration.Support.TR('COLLABORATION_NO_ENTRIES_AVAILABLE'), 
deferEmptyText:false}, url:function() {
  return collaboration.Module.getBaseUrl() + this.moduleId + '/stream/' + this.channelId + '/' + this.topicId;
}}}, viewConfig:{loadMask:false}, feedPanel:undefined, initComponent:function() {
  var $jscomp$this = this;
  if (Ext.isDefined(this.moduleId) && Ext.isDefined(this.channelId) && Ext.isDefined(this.topicId)) {
  } else {
  }
  if (this.feedTypeConfig[this.feedType]) {
    this.viewConfig = this.feedTypeConfig[this.feedType].viewConfig;
  }
  if (this.feedTypeConfig[this.feedType].url) {
    this.url = this.feedTypeConfig[this.feedType].url.call(this);
  }
  var flip = 1;
  this.columns = [{text:'Widget', flex:1, xtype:'widgetcolumn', dataIndex:'id', padding:'5 5 5 5', widget:{xtype:'colArtefactCard', header:false, instanceMode:this.instanceMode, moduleId:this.moduleId, feedType:this.feedType, channelView:this}, onWidgetAttach:function(col, widget, rec) {
    try {
      var obj = {};
      widget.feedPanel = $jscomp$this.feedPanel;
      widget.displaySettings = $jscomp$this.displaySettings || {};
      obj.moduleId = widget.moduleId;
      obj = Ext.applyIf(obj, rec.getData());
      if (widget.channelView.feedType === 'referenceFeed' && !widget.channelView.referenceData) {
        if (widget.channelView.src_moduleId === 'vam') {
          widget.channelView.referenceData = collaboration.Module.loadVamReferenceData(rec);
        } else {
          if (widget.channelView.src_moduleId === 'monitor') {
            widget.channelView.referenceData = collaboration.Module.loadMonitorReferenceData(rec);
          }
        }
      }
      widget.setStyle({backgroundColor:flip === 1 ? 'bisque' : 'darkgrey'});
      flip = flip === 2 ? 1 : 2;
      if (widget.channelView.referenceData) {
        widget.getViewModel().set('referenceData', widget.channelView.referenceData);
      }
      widget.applyArtefactData(rec);
      if (rec && !widget.channelView.referenceData && rec.get('reference') && rec.get('src_instanceId')) {
        widget.loadReferenceData(rec);
      }
    } catch (e) {
    }
  }}];
  this.store = Ext.create('Ext.data.BufferedStore', {model:'collaboration.model.CollabData', pageSize:100, proxy:{type:'ajax', url:this.url, disableCaching:false, useDefaultXhrHeader:false, reader:{type:'json', rootProperty:'items', totalProperty:'total'}}, autoLoad:false});
  this.callParent(arguments);
  this.store.load();
}, loadReferenceTypes:function(instance) {
  return collaboration.Api.module.instance.getReferenceTypes(instance);
}});
Ext.define('collaboration.view.uitypes.streams.GenericFeed', {extend:'collaboration.view.uitypes.streams.grid.ChannelView', alias:['widget.collabGenericFeed'], feedTypeConfig:{moduleStream:{viewConfig:{emptyText:collaboration.Support.TR('COLLABORATION_NO_ARTEFACTS_AT_MODULE') + '.', deferEmptyText:false}, url:function() {
  return collaboration.Module.getReferenceUrl(this.src_instanceId, null, null, 'module');
}}}, feedType:'moduleStream', src_moduleId:undefined, src_instanceId:undefined, initComponent:function() {
  var $jscomp$this = this;
  this.bbar = [{text:collaboration.Support.TR('COLLABORATION_ARTEFACTS_ALL'), handler:function() {
    $jscomp$this.store.getProxy().url = $jscomp$this.url;
    $jscomp$this.store.load();
  }}, {xtype:'button', text:collaboration.Support.TR('COLLABORATION_REF_TYPES'), iconCls:collaboration.assets.Icons.filter, disabled:true, listeners:{'afterrender':function(cmp) {
    var instance = $jscomp$this.src_instanceId;
    $jscomp$this.loadReferenceTypes(instance).then(function(response) {
      var tb = cmp.up('toolbar');
      if (Ext.isDefined(tb)) {
        if (response !== null && response.items) {
          cmp.setText(cmp.getText() + ' ( ' + response.total + ' )');
          Ext.Array.each(response.items, function(referenceType) {
            var $jscomp$this = this;
            var refType = referenceType['_source']['referenceType'];
            var btnTitle = collaboration.Support.TR(refType, collaboration.Api.getBaseModule(instance));
            tb.add({xtype:'button', flex:1, refType:refType, text:btnTitle, tooltip:collaboration.Support.TR('COLLABORATION_FOR_REF_TYPE') + ': ' + btnTitle + ' ' + collaboration.Support.TR('FILTER') + '.', handler:function(btn) {
              $jscomp$this.store.getProxy().url = collaboration.Module.getBaseUrl() + 'module/' + collaboration.Api.getBaseModule(instance) + '/' + instance + '/referenceTypes/' + btn.refType + '?data\x3dtrue';
              $jscomp$this.store.load();
            }});
          });
        }
      }
    });
  }}}, {xtype:'collabGridInput', flex:1, emptyText:' ' + collaboration.Support.TR('COLLABORATION_SEARCH_FEED'), gridReference:'collabGenericFeed'}, {xtype:'collabDateRangeField', gridReference:'collabGenericFeed'}];
  this.callParent(arguments);
}, onStoreLoad:function() {
  if (this.isComponent) {
    this.setLoading(false);
  }
}});
Ext.define('collaboration.view.uitypes.streams.GeneticTypeCombo', {extend:'Ext.form.field.ComboBox', alias:'widget.genericTypeCombo', valueList:undefined, viewModel:{data:{values:[]}, stores:{valueStore:{data:'{values}', sorters:[{property:'position', direction:'ASC'}]}}}, bind:{store:'{valueStore}'}, valueField:'value', displayField:'value', queryMode:'local', editable:false, listConfig:{itemTpl:new Ext.XTemplate('\x3ctpl\x3e\x3cdiv data-qtip\x3d"{[this.returnRenderer(values)]}"\x3e{[this.returnLabel(values)]}\x3c/div\x3e\x3c/tpl\x3e', 
{returnLabel:function(recordData) {
  var color = 'black';
  var label = recordData.value;
  if ('color' in recordData) {
    color = recordData.color;
  }
  if ('label' in recordData) {
    label = recordData.label;
  }
  if ('iconCls' in recordData) {
    label = collaboration.Module.getIconClsSpan(recordData.iconCls, color) + '  ' + Ext.util.Format.htmlEncode2(label);
  }
  return label;
}, returnRenderer:function(recordData) {
  return Ext.util.Format.htmlDoubleEncode(recordData.value);
}})}, initComponent:function() {
  var me = this;
  try {
    if (me.valueList.length > 0) {
      me.getViewModel().set('values', me.valueList);
      Ext.Array.each(me.valueList, function(value, index, values) {
        if ('label' in value) {
          me.displayField = 'label';
          return false;
        }
      });
    }
    me.callParent(arguments);
  } catch (e) {
  }
}});
Ext.define('collaboration.view.uitypes.streams.Journal', {extend:'collaboration.view.uitypes.streams.BaseStreamView', alias:['widget.collabJournal', 'widget.collabjournal'], layout:{type:'hbox', align:'stretch'}, instanceMode:undefined, channelId:undefined, channelPrefix:'journal', collabStreamMode:'journal', moduleId:undefined, streamHolderCmp:'streamHolderCmp', defaults:{flex:1}, prepareDefaults:function() {
  var defaults = this.defaults;
  defaults.moduleId = this.moduleId;
  defaults.collabStreamMode = this.collabStreamMode;
  this.defaults = defaults;
}, initStreamView:function(channelId) {
  if (Ext.isDefined(channelId)) {
    this.channelId = channelId;
    var defaults = this.defaults;
    defaults['channelId'] = channelId;
    defaults['header'] = false;
    this.add([{xtype:'container', flex:1, layout:{type:'hbox', align:'stretch'}, items:[{xtype:'collabTopicChannelBase', channelId:channelId, defaults:defaults, flex:3}, {xtype:'container', flex:10, layout:'card', itemId:this.streamHolderCmp, items:[{xtype:'component', html:collaboration.Support.TR('COLLABORATION_NO_JOURNALTOPIC_SELECTED')}]}]}]);
  } else {
    collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_NO_CHANNELID_CHECK_CONFIG'));
  }
}});
Ext.define('collaboration.view.uitypes.streams.ProcessInfo', {extend:'Ext.Container', alias:'widget.collabProcessInfo', viewModel:{data:{collabData:undefined, processData:undefined}}, layout:'vbox', style:{backgroundColor:'#414144', color:'white', padding:'1px 15px', borderRadius:'10px'}, items:[{xtype:'box', html:collaboration.Support.TR('COLLABORATION_NO_INFORMATION_AVAILABLE') + '.'}], recData:undefined, initComponent:function() {
  var me = this;
  me.setVmData();
  var colArray = [];
  var items = [];
  if (me.recData && me.recData.src_instanceId && me.recData.reference) {
    var descriptor = collaboration.Module.getCollabIntegrationDescriptor(me.recData.src_instanceId);
    if (descriptor && Ext.isObject(descriptor) && descriptor.hasOwnProperty('plugins')) {
      colArray = descriptor.plugins.stream.reference;
    }
  }
  if (colArray && colArray.length > 0) {
    items = me.prepareItemsToAdd(colArray);
    me.items = [{xtype:'container', layout:'hbox', items:[{xtype:'box', html:'\x3cspan class\x3d"fa-stack fa-lg"\x3e\x3cspan class\x3d"fa fa-square fa-stack-2x" style\x3d"color: darkseagreen"\x3e\x3c/span\x3e\x3cspan class\x3d"fa fa-info fa-stack-1x"\x3e\x3c/span\x3e\x3c/span\x3e'}, {xtype:'container', layout:'vbox', items:[{xtype:'tbspacer', height:10}, {xtype:'box', html:'\x3cspan style\x3d"font-size: 150%;"\x3e' + collaboration.Support.TR('COLLABORATION_PROCESS_INFO') + '\x3c/span\x3e'}]}]}, {xtype:'container', 
    layout:'vbox', items:[{xtype:'container', layout:'vbox', items:items}]}];
  }
  me.callParent(arguments);
}, setVmData:function() {
  var recData = this.recData;
  var referenceData;
  if (recData) {
    var vm = this.getViewModel();
    vm.set('collabData', recData);
    if (this.parentCmp) {
      referenceData = this.parentCmp.getViewModel().get('referenceData');
      if (referenceData) {
        vm.set('processData', referenceData);
      }
    }
    if (!referenceData && this.sourceRecord) {
      var data = collaboration.Module.loadMonitorReferenceData(this.sourceRecord);
      if (data) {
        vm.set('processData', data);
      }
    }
  }
}, prepareItemsToAdd:function(colArray) {
  var me = this;
  var items = [];
  var processData = me.getViewModel().get('processData');
  if (processData && !Ext.Object.isEmpty(processData)) {
    Ext.each(colArray, function(item) {
      if (processData.hasOwnProperty(item)) {
        items.push(me.prepareItem(item));
      }
    });
  }
  return items;
}, prepareItem:function(item) {
  return {xtype:'box', padding:'2 2 2 2', bind:{html:'\x3cb\x3e' + item + '\x3c/b\x3e : {processData.' + item + '}'}};
}});
Ext.define('collaboration.view.uitypes.streams.ReferenceFeed', {extend:'collaboration.view.uitypes.streams.grid.ChannelView', alias:['widget.collabReferenceFeed'], feedType:'referenceFeed', referenceType:undefined, referenceId:undefined, src_moduleId:undefined, src_instanceId:undefined, referenceData:undefined, displaySettings:{}, initComponent:function() {
  var $jscomp$this = this;
  this.feedTypeConfig = {referenceFeed:{viewConfig:{emptyText:BpcCommon.Api.getTranslation('COLLABORATION_REFERENCE_WITHOUT_ARTEFACTS_PREFIX') + ' ' + this.referenceId + ' ' + BpcCommon.Api.getTranslation('COLLABORATION_REFERENCE_WITHOUT_ARTEFACTS_SUFFIX') + '.', deferEmptyText:false, reserveScrollbar:true}, url:function() {
    return collaboration.Module.getBaseUrl() + 'artefacts?' + Ext.Object.toQueryString({instance:$jscomp$this.src_instanceId, reference:$jscomp$this.referenceId, referenceType:$jscomp$this.referenceType, module:collaboration.Api.getBaseModule($jscomp$this.src_instanceId)});
  }}};
  this.callParent(arguments);
}, onStoreLoad:function(store, records, successful, operation, eOpts) {
  if (this.isComponent) {
    var win = this.up('window');
    if (win) {
      if (!win.tempTitle) {
        win.tempTitle = win.getTitle();
      }
      this.up('window').setTitle(win.tempTitle + '  (' + store.getTotalCount() + ' ' + collaboration.Support.TR('ARTEFACTS') + ')');
    }
  }
}});
Ext.define('collaboration.view.uitypes.streams.channelTopicSelectors.TopicChannelBase', {extend:'Ext.tab.Panel', alias:'widget.collabTopicChannelBase', items:[{xtype:'collabTopicList', collabTitlePrefix:collaboration.assets.Artefacts.topicStateActive, iconCls:collaboration.assets.Artefacts.topicActiveIcon, collabFilters:[{property:'active', value:true}]}, {xtype:'collabTopicList', collabTitlePrefix:collaboration.assets.Artefacts.topicStateInactive, iconCls:collaboration.assets.Artefacts.topicInactiveIcon, 
collabFilters:[{property:'active', value:false}]}, {xtype:'collabTopicList', collabTitlePrefix:collaboration.assets.Artefacts.topicStateArchieved, iconCls:collaboration.assets.Artefacts.archiveIcon, collabFilters:[{property:'active', value:null}]}], refreshTopics:function() {
  var topicListGrids = this.query('collabTopicList');
  Ext.Array.each(topicListGrids, function(grid) {
    grid.getStore().load();
  });
}, initComponent:function() {
  if (this.defaults) {
    if (this.defaults.collabStreamMode) {
      this.stateId = 'collab' + this.defaults.collabStreamMode + 'TabState' + this.defaults.moduleId;
    } else {
      this.stateId = 'collabBlogTabState' + this.defaults.moduleId;
    }
  }
  this.callParent(arguments);
}, listeners:{tabchange:function(tabPanel, newCard, oldCard, eOpts) {
  Ext.log(this.self.getName() + ': tabchange');
  var streamViewCmp = newCard.up('collab' + newCard.collabStreamMode);
  if (newCard.collabStreamMode && (newCard.collabStreamMode === 'journal' || newCard.collabStreamMode === 'forum')) {
    oldCard.setTitle('( ' + oldCard.totalCount + ' )');
    newCard.setTitle(newCard.collabTitlePrefix + ' ( ' + newCard.totalCount + ' )');
    var total = newCard.getStore().getData().length;
    if (total && total > 0) {
      var record = newCard.getStore().getData().getAt(0);
      var topicId = record.get('artefactId');
      var channelId;
      if (record['data']['channelId']) {
        channelId = record['data']['channelId'];
      } else {
        channelId = newCard.channelId;
      }
      var moduleId = newCard.moduleId;
      streamViewCmp.updateStream(channelId, topicId, moduleId, record);
    } else {
      var topicCmp = streamViewCmp.down('#' + streamViewCmp.streamHolderCmp);
      if (topicCmp && topicCmp.isComponent) {
        var placeholder;
        var currentlyOpenedTopics = topicCmp.getLayout().getLayoutItems();
        if (currentlyOpenedTopics && currentlyOpenedTopics.length > 0) {
          Ext.each(currentlyOpenedTopics, function(topic) {
            if (topic.topicId === 'placeholder') {
              placeholder = topic;
            }
          });
        }
        if (!placeholder) {
          placeholder = topicCmp.add({xtype:'component', style:{display:'flex', alignItems:'center', color:'lightgrey', justifyContent:'center', fontSize:'32px'}, topicId:'placeholder', html:collaboration.Support.TR('COLLABORATION_NO_ENTRIES_AVAILABLE')});
        }
        topicCmp.setActiveItem(placeholder);
      }
    }
  }
}}});
Ext.define('collaboration.view.uitypes.streams.TopicCreatePanel', {extend:'Ext.panel.Panel', alias:['widget.collabTopicCreatePanel'], layout:{type:'vbox', align:'stretch'}, viewModel:{data:{active:true, activeFrom:new Date, activeTo:null, creator:null, created:null, description:null, duedate:null, lastupdate:null, priority:null, team:null, topic:null}, formulas:{activeTominValue:function(get) {
  var currentFromDate = get('activeFrom');
  var now = new Date;
  return now > currentFromDate ? now : currentFromDate;
}}}, padding:'5 5 5 5', mode:undefined, parentCmp:undefined, moduleId:undefined, record:undefined, initComponent:function() {
  var me = this;
  var mode = me.mode;
  var buttonText = collaboration.Support.TR('COLLABORATION_GENERATE'), buttonIcon = collaboration.assets.Artefacts.createIcon;
  if (mode && Ext.isDefined(mode) && mode === 'edit') {
    buttonText = collaboration.Support.TR('REFRESH');
    buttonIcon = collaboration.assets.Artefacts.refreshIcon;
  }
  me.buttons = [{xtype:'tbfill'}, {xtype:'button', padding:'5 5 5 5', iconCls:buttonIcon, text:buttonText, itemId:'topicSubmitBtn', handler:function(btn) {
    var mainPanel = btn.up('collabTopicCreatePanel');
    var topicName = mainPanel.down('textfield').getValue();
    var topicId;
    var channelId = mainPanel.parentCmp.up('collabTopicChannelBase').channelId;
    if (!Ext.isDefined(channelId)) {
      channelId = mainPanel.parentCmp.channelId;
    }
    if (mainPanel.record && Ext.isDefined(mainPanel.record)) {
      topicId = mainPanel.record.get('topicId') || mainPanel.record.getId();
    }
    var moduleId = mainPanel.moduleId;
    var fields = mainPanel.query('[collabId]');
    var payload = {};
    var valid = true;
    if (fields.length > 0) {
      Ext.Array.each(fields, function(field) {
        if (field.isValid()) {
          if (mainPanel.mode && mainPanel.mode === 'new') {
            payload[field.collabId] = field.getValue();
          }
        } else {
          valid = false;
        }
      });
    }
    if (mainPanel.mode && mainPanel.mode === 'edit') {
      payload = mainPanel.getViewModel().getData();
      Ext.iterate(payload, function(k, v) {
        if (v === '' || v === null) {
          delete payload[k];
        }
      });
    }
    payload['lastupdate'] = new Date;
    payload['instanceId'] = moduleId.toString();
    if (channelId && !payload['channelId']) {
      payload['channelId'] = channelId;
    }
    if (valid) {
      if (mainPanel.mode && Ext.isDefined(mainPanel.mode) && mainPanel.mode === 'new') {
        collaboration.Api.createTopic(moduleId, topicName, payload).then(function() {
          btn.up('window').close();
        });
      } else {
        collaboration.Api.modifyTopic(moduleId, topicId, payload).then(function() {
          btn.up('window').close();
        });
      }
    }
  }}, {iconCls:collaboration.assets.Artefacts.replyOnCancelIcon, localized:{text:'COLLABORATION_CANCEL'}, padding:'5 5 5 5', handler:function(btn) {
    btn.up('window').close();
  }}];
  var obj = collaboration.Support.getSetting(this.moduleId, 'instance_channelTopicTemplate'), items = [{xtype:'bpcTextField', collabId:'topic', fieldLabel:'Name', emptyText:'Topic Name', allowBlank:false, bind:'{topic}', listeners:{specialkey:me.onSpecialKey.bind(me)}}];
  if (Ext.isObject(obj)) {
    items = items.concat(me.getItemsForOldConfig(obj, mode));
  } else {
    if (Ext.isArray(obj)) {
      items = items.concat(me.getTopicItems(obj, mode));
    }
  }
  this.items = items;
  if (this.mode && this.mode === 'edit' && Ext.isDefined(this.record)) {
    this.setViewModelData();
  }
  this.callParent(arguments);
}, onSpecialKey:function(field, e) {
  try {
    var me = this;
    if (e.getKey() == e.ENTER) {
      me.down('#topicSubmitBtn').click();
    }
  } catch (e$4) {
  }
}, setViewModelData:function() {
  var data = this.record.getData();
  data = collaboration.Module.cleanRecord(data);
  if (data['topicName']) {
    data['topic'] = data['topicName'];
    delete data['topicName'];
  }
  if (data.activeFrom && Ext.isDefined(data.activeFrom)) {
    var date = data.activeFrom;
    if (date.indexOf('T') !== -1) {
      data['activeFrom'] = date.substring(0, date.indexOf('T'));
    }
  }
  if (data.activeTo && Ext.isDefined(data.activeTo)) {
    var date = data.activeTo;
    if (date.indexOf('T') !== -1) {
      data['activeTo'] = date.substring(0, date.indexOf('T'));
    }
  }
  this.getViewModel().set(data);
}, getTopicItems:function(arr, mode) {
  var me = this;
  var items = [];
  arr.sort(function(a, b) {
    return a.position - b.position;
  });
  Ext.each(arr, function(item, idx) {
    var type = 'bpcTextField';
    var label = '\x3cspan class\x3d"' + item['iconCls'] + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + Ext.util.Format.htmlEncode2(item['defaultText']);
    if (item['type'] === 'date') {
      type = 'bpcDateField';
    } else {
      if (item['type'] === 'boolean') {
        type = 'checkbox';
      } else {
        if ((item['type'] === 'list' || item['type'] === 'combo') && 'values' in item && item.values.length > 0) {
          type = 'genericTypeCombo';
        }
      }
    }
    if (item.name === 'description') {
      type = 'bpcTextArea';
    }
    if ('defaultValue' in item && Ext.isDefined(item.defaultValue)) {
      if (type !== 'bpcDateField') {
        me.getViewModel().set(item.name, item.defaultValue);
      } else {
        var dateVal = BpcCommon.util.EsDateMath.getCalculatedDateFromOffsetMap(BpcCommon.util.EsDateMath.getOffsetMapFromEsDateMathString(item.defaultValue));
        if (!dateVal) {
          dateVal = new Date;
        }
        me.getViewModel().set(item.name, dateVal);
      }
    }
    var itemConfig = {xtype:type, collabId:item.name, bind:{value:'{' + item.name + '}'}, fieldLabel:label};
    if (item.name === 'activeFrom') {
      itemConfig.allowBlank = false;
      itemConfig.bind.maxValue = '{activeTo}';
    } else {
      if (item.name === 'activeTo') {
        itemConfig.minValue = new Date;
        itemConfig.bind.minValue = '{activeTominValue}';
      } else {
        if (type === 'genericTypeCombo') {
          itemConfig.valueList = item.values;
        }
      }
    }
    items.push(itemConfig);
  });
  return items;
}, getItemsForOldConfig:function(obj, mode) {
  var me = this;
  var items = [];
  Ext.iterate(obj, function(k, v) {
    var type = 'bpcTextField';
    var label = k;
    if (v && Ext.isDefined(v) && Ext.isObject(v)) {
      label = '\x3cspan class\x3d"' + v['iconCls'] + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + v['defaultText'];
      if (v['type'] === 'date') {
        type = 'bpcDateField';
      }
      if (v['type'] === 'boolean') {
        type = 'checkbox';
      }
      if (k === 'description') {
        type = 'bpcTextArea';
      }
    } else {
      if (v === 'date') {
        type = 'bpcDateField';
      }
      if (v === 'boolean') {
        type = 'checkbox';
      }
    }
    var itemConfig = {xtype:type, collabId:k, bind:{value:'{' + k + '}'}, fieldLabel:label};
    if (k === 'activeFrom') {
      itemConfig.allowBlank = false;
      itemConfig.bind.maxValue = '{activeTo}';
    } else {
      if (k === 'activeTo') {
        itemConfig.minValue = new Date;
        itemConfig.bind.minValue = '{activeTominValue}';
      }
    }
    items.push(itemConfig);
  });
  return items;
}});
Ext.define('collaboration.view.uitypes.streams.TopicCreateWindow', {extend:'collaboration.components.window.Window', alias:['widget.topicCreateWindow', 'widget.topiccreatewindow'], record:undefined, moduleId:undefined, parentCmp:undefined, mode:undefined, layout:'fit', modal:true, initComponent:function() {
  var me = this;
  this.items = [{xtype:'collabTopicCreatePanel', moduleId:me.moduleId, parentCmp:me.parentCmp, record:me.record, mode:me.mode}];
  me.callParent(arguments);
  var mode = me.mode;
  me.title = collaboration.Support.TR('COLLABORATION_TOPIC_CREATE');
  me.iconCls = collaboration.assets.Artefacts.createIcon;
  if (mode && Ext.isDefined(mode) && mode === 'edit') {
    me.title = collaboration.Support.TR('COLLABORATION_TOPIC_EDIT');
    me.iconCls = collaboration.assets.Artefacts.editIcon;
  }
}});
Ext.define('collaboration.view.uitypes.streams.contextMenu.BlogWidgetContextMenu', {extend:'Ext.menu.Menu', alias:'widget.blogwidgetmenu', moduleId:undefined, gridReference:undefined, selectedRecord:undefined, items:[{text:'Im Module öffnen', iconCls:collaboration.assets.Icons.externalLink, handler:function(menuItem) {
  var menu = menuItem.up('blogwidgetmenu');
  var record = menu.selectedRecord;
  var topicId = record.get('artefactId');
  var channelId = record.get('channelId');
  if (!Ext.isDefined(channelId)) {
    channelId = menu.gridReference.channelId;
  }
  var moduleId = menu.gridReference.moduleId;
  var instanceType = collaboration.Support.getSetting(moduleId, 'instance_channelUi');
  if (Ext.isDefined(instanceType) && instanceType === 'blog') {
    BpcCommon.Api.showModule(moduleId, false);
    Ext.Function.defer(function() {
      collaboration.Support.showSteamWindow(channelId, topicId, moduleId, record);
    }, 100);
  } else {
    if (Ext.isDefined(instanceType) && instanceType === 'journal') {
      BpcCommon.Api.showModule(moduleId, false, {uiType:instanceType, channelId:channelId, topicId:topicId});
    } else {
      BpcCommon.Api.showModule(moduleId, false, {uiType:instanceType, channelId:channelId, topicId:topicId});
    }
  }
}}, {iconCls:collaboration.assets.Icons.angleRight, text:'Zeige Topic', handler:function(menuItem) {
  var menu = menuItem.up('blogwidgetmenu');
  var widget = menu.gridReference.up('collabBlogWidget');
  if (menu.gridReference.getId().indexOf('Forum') > -1) {
    widget = menu.gridReference.up('collabForumWidget');
  }
  var topicId = menu.selectedRecord.get('artefactId');
  var channelId = menu.selectedRecord.get('channelId');
  if (!Ext.isDefined(channelId)) {
    channelId = menu.gridReference.channelId;
  }
  var activeItem = widget.getLayout().getActiveItem();
  var initTitle = widget.up('dbWidgetWrapper').getTitle();
  var moduleId = menu.gridReference.moduleId;
  var topicView = Ext.create({xtype:'collabTopic', moduleId:moduleId, topicId:topicId, channelId:channelId, overView:menu.gridReference.up('collabTopic'), instanceMode:'widget', holderCmp:widget, header:false, bbar:[{xtype:'tbfill'}, {xtype:'button', text:collaboration.Support.TR('BACK'), iconCls:collaboration.assets.Icons.angleDoubleLeft, handler:function(btn) {
    var cmp = btn.up('collabTopic');
    cmp.holderCmp.getLayout().setActiveItem(cmp.overView);
    cmp.up('dbWidgetWrapper').setTitle(initTitle);
    cmp.destroy();
    widget.getLayout().setActiveItem(activeItem);
  }}], listeners:{afterrender:function(cmp) {
    Ext.defer(function() {
      var title = cmp.getViewModel().get('topicTitle');
      if (menu.gridReference.getId().indexOf('Forum') > -1) {
        var topicMeta = collaboration.Api.getTopicById(topicId);
        if (!topicMeta) {
          return;
        }
        var channelMeta = collaboration.Api.getChannelById(topicMeta['channelId']);
        var channelTitle = '\x3cspan style\x3d"color:' + channelMeta['color'] + '"\x3e # \x3c/span\x3e' + channelMeta['channel'];
        cmp.up('dbWidgetWrapper').setTitle(initTitle + ' ( ' + channelTitle + ' - ' + title + ' )');
      } else {
        cmp.up('dbWidgetWrapper').setTitle(initTitle + ' - ' + title);
      }
    }, 1000);
  }}});
  widget.getLayout().setActiveItem(topicView);
}}, {iconCls:collaboration.assets.Icons.angleDoubleRight, text:'Setze Topic', disabled:!BpcCommon.Api.hasUserRole('bpcadmin'), handler:function(menuItem) {
  var menu = menuItem.up('blogwidgetmenu');
  var instanceType = collaboration.Support.getSetting(menu.moduleId, 'instance_channelUi');
  if (instanceType && Ext.isDefined(instanceType) && instanceType === 'forum') {
    menu.gridReference.up('collabForumWidget').updateTopicValue(menu.selectedRecord.id);
  } else {
    menu.gridReference.up('collabBlogWidget').updateTopicValue(menu.selectedRecord.id);
  }
}}]});
Ext.define('collaboration.view.uitypes.streams.contextMenu.ForumContextMenu', {extend:'Ext.menu.Menu', alias:'widget.forummenu', moduleId:undefined, gridReference:undefined, selectedRecord:undefined, initComponent:function() {
  var me = this;
  var data = me.selectedRecord.getData();
  var activeText, activeIcon;
  var hideDeactivieren = me.hideDeactive();
  if (data['active']) {
    activeIcon = collaboration.assets.Artefacts.topicInactiveIcon;
    activeText = collaboration.Support.TR('DEACTIVATE');
  } else {
    activeIcon = collaboration.assets.Artefacts.topicActiveIcon;
    activeText = collaboration.Support.TR('ACTIVATE');
  }
  var hideArchievieren = false;
  if (data['active'] === null || !Ext.isDefined(data['active'])) {
    hideArchievieren = true;
  }
  var disableEditBtn = true;
  if (me.selectedRecord.getData().creator === collaboration.Module.user) {
    disableEditBtn = false;
  } else {
    if (BpcCommon.Api.hasUserRole('bpcadmin')) {
      disableEditBtn = false;
    }
  }
  var hideDelete = true;
  if (me.moduleId && me.selectedRecord.get('artefactId')) {
    var url = collaboration.Module.getBaseUrl() + this.moduleId + '/topics?channelId\x3d' + me.selectedRecord.get('artefactId');
    var total = 0;
    Ext.Ajax.request({async:false, url:url, success:function(response, opts) {
      var obj = Ext.decode(response.responseText);
      total = obj.items.length;
    }, failure:function(response, opts) {
      console.log('server-side failure with status code ' + response.status);
    }});
    if (me.gridReference.collabStreamMode && total === 0) {
      hideDelete = false;
    }
  }
  this.items = [{iconCls:collaboration.assets.Artefacts.editIcon, text:collaboration.Support.TR('EDIT'), disabled:disableEditBtn, handler:function(menuItem, e) {
    var menu = menuItem.up('forummenu');
    Ext.create('collaboration.components.window.Window', {title:collaboration.Support.TR('CHANNEL_EDIT'), iconCls:collaboration.assets.Artefacts.editIcon, layout:'fit', modal:true, items:[{xtype:'collabForumChannelEditor', moduleId:menu.moduleId, record:menu.selectedRecord, gridReference:menu.gridReference}]}).show();
  }}, {iconCls:activeIcon, text:activeText, hidden:hideDeactivieren, handler:function(menuItem, e) {
    var menu = menuItem.up('forummenu');
    var record = menu.selectedRecord;
    var grid = menu.gridReference;
    var moduleId = menu.moduleId;
    var payload = record.getData();
    payload = collaboration.Module.cleanRecord(payload);
    if (payload['topicName']) {
      payload['topic'] = payload['topicName'];
      delete payload['topicName'];
    }
    payload['lastupdate'] = new Date;
    payload['instanceId'] = moduleId.toString();
    payload['active'] = payload['active'] !== true;
    if (payload['id']) {
      delete payload['id'];
    }
    var channelId = record.data.artefactId;
    collaboration.Api.modifyChannel(moduleId, channelId, payload).then(function() {
      var panels = grid.up('tabpanel').items.items;
      Ext.Array.each(panels, function(panel) {
        panel.getStore().load();
      });
    });
  }}, {iconCls:collaboration.assets.Artefacts.archiveIcon, text:collaboration.Support.TR('ARCHIVE'), hidden:hideArchievieren, handler:function(menuItem, e) {
    var menu = menuItem.up('forummenu');
    var record = menu.selectedRecord;
    var grid = menu.gridReference;
    var moduleId = menu.moduleId;
    var payload = record.getData();
    payload = collaboration.Module.cleanRecord(payload);
    if (payload['topicName']) {
      payload['topic'] = payload['topicName'];
      delete payload['topicName'];
    }
    payload['lastupdate'] = new Date;
    payload['instanceId'] = moduleId.toString();
    payload['active'] = null;
    if (payload['id']) {
      delete payload['id'];
    }
    var channelId = record.data.artefactId;
    collaboration.Api.modifyChannel(moduleId, channelId, payload).then(function() {
      var panels = grid.up('tabpanel').items.items;
      Ext.Array.each(panels, function(panel) {
        panel.getStore().load();
      });
    });
  }}, {iconCls:collaboration.assets.Icons.trash, text:collaboration.Support.TR('DELETE'), hidden:hideDelete, handler:function(menuItem, e) {
    var menu = menuItem.up('forummenu');
    var record = menu.selectedRecord;
    var grid = menu.gridReference;
    var channelId = record.get('artefactId');
    if (Ext.isDefined(channelId)) {
      Ext.Msg.confirm(collaboration.Support.TR('DELETE'), collaboration.Support.TR('COLLABORATION_DELETE_ENTRY_WARNING_Q'), function(btnText) {
        if (btnText === 'yes') {
          collaboration.Api.deleteChannel(channelId).then(function() {
            collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ENTRY_DELETED_SUCCESS'));
            Ext.defer(function() {
              grid.getStore().load();
            }, 100);
          }, function() {
            collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ENTRY_DELETED_ERROR'));
          });
        }
      });
    } else {
      collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_NO_ID_CANT_BE_DELETED'));
    }
  }}];
  me.callParent(arguments);
}, hideDeactive:function() {
  if (this.gridReference.collabStreamMode === 'forum') {
    var data = this.selectedRecord.getData();
    if (data['active']) {
      return true;
    }
  }
  return false;
}});
Ext.define('collaboration.view.uitypes.streams.contextMenu.JournalContextMenu', {extend:'Ext.menu.Menu', alias:'widget.journalmenu', moduleId:undefined, gridReference:undefined, selectedRecord:undefined, initComponent:function() {
  var me = this;
  var data = me.selectedRecord.getData();
  var activeText, activeIcon;
  if (data['active']) {
    activeIcon = collaboration.assets.Artefacts.topicInactiveIcon;
    activeText = collaboration.Support.TR('DEACTIVATE');
  } else {
    activeIcon = collaboration.assets.Artefacts.topicActiveIcon;
    activeText = collaboration.Support.TR('ACTIVATE');
  }
  var hideArchievieren = false;
  if (data['active'] === null) {
    hideArchievieren = true;
  }
  var channelsMenu = me.channelsMenu();
  var isJournal = false, color;
  if (me.gridReference.collabStreamMode === 'journal') {
    isJournal = true;
    color = me.gridReference.color;
  }
  var hideMoveTo = true;
  var topicId = me.selectedRecord.getId();
  if (data['channelId']) {
    var url = collaboration.Module.getBaseUrl() + me.moduleId + '/stream/' + data['channelId'] + '/' + topicId;
    var total = 0;
    Ext.Ajax.request({async:false, url:url, success:function(response, opts) {
      var obj = Ext.decode(response.responseText);
      total = obj.items.length;
    }, failure:function(response, opts) {
      console.log('server-side failure with status code ' + response.status);
    }});
    if (me.gridReference.collabStreamMode && total === 0) {
      hideMoveTo = false;
    }
  }
  this.items = [{iconCls:collaboration.assets.Artefacts.editIcon, text:collaboration.Support.TR('EDIT'), handler:function(menuItem, e) {
    var menu = menuItem.up('journalmenu');
    var topicEditWindow = Ext.createByAlias('widget.topicCreateWindow', {moduleId:menu.moduleId, record:menu.selectedRecord, parentCmp:menu.gridReference, mode:'edit'});
    topicEditWindow.show();
  }}, {iconCls:activeIcon, text:activeText, handler:function(menuItem, e) {
    var menu = menuItem.up('journalmenu');
    var record = menu.selectedRecord;
    var grid = menu.gridReference;
    var topicId = record.getId();
    var moduleId = menu.moduleId;
    var payload = record.getData();
    payload = collaboration.Module.cleanRecord(payload);
    if (payload['topicName']) {
      payload['topic'] = payload['topicName'];
      delete payload['topicName'];
    }
    payload['lastUpdate'] = Ext.Date.format(new Date, 'Y-m-d H:i:s');
    payload['instanceId'] = moduleId.toString();
    payload['active'] = payload['active'] !== true;
    if (payload['state']) {
      delete payload['state'];
    }
    collaboration.Api.modifyTopic(moduleId, topicId, payload).then(function() {
      var panels = grid.up('tabpanel').items.items;
      Ext.Array.each(panels, function(panel) {
        panel.getStore().load();
      });
    });
  }}, {iconCls:collaboration.assets.Artefacts.archiveIcon, text:collaboration.Support.TR('ARCHIVE'), hidden:hideArchievieren, handler:function(menuItem, e) {
    var menu = menuItem.up('journalmenu');
    var record = menu.selectedRecord;
    var grid = menu.gridReference;
    var topicId = record.getId();
    var moduleId = menu.moduleId;
    var payload = record.getData();
    payload = collaboration.Module.cleanRecord(payload);
    if (payload['topicName']) {
      payload['topic'] = payload['topicName'];
      delete payload['topicName'];
    }
    payload['lastUpdate'] = Ext.Date.format(new Date, 'Y-m-d H:i:s');
    payload['instanceId'] = moduleId.toString();
    payload['active'] = null;
    payload['state'] = 'archived';
    collaboration.Api.modifyTopic(moduleId, topicId, payload).then(function() {
      var panels = grid.up('tabpanel').items.items;
      Ext.Array.each(panels, function(panel) {
        panel.getStore().load();
      });
    });
  }}, {iconCls:collaboration.assets.Icons.bars, text:'Move to', hidden:hideMoveTo, menu:channelsMenu}, {iconCls:collaboration.assets.Icons.trash, text:collaboration.Support.TR('DELETE'), hidden:hideMoveTo, handler:function(menuItem, e) {
    var menu = menuItem.up('journalmenu');
    var record = menu.selectedRecord;
    var grid = menu.gridReference;
    var topicId = record.getId();
    if (Ext.isDefined(topicId)) {
      Ext.Msg.confirm(collaboration.Support.TR('DELETE'), collaboration.Support.TR('COLLABORATION_DELETE_ENTRY_WARNING_Q'), function(btnText) {
        if (btnText === 'yes') {
          collaboration.Api.deleteTopic(topicId).then(function() {
            collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ENTRY_DELETED_SUCCESS'));
            Ext.defer(function() {
              grid.getStore().load();
            }, 100);
          }, function() {
            collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ENTRY_DELETED_ERROR'));
          });
        }
      });
    } else {
      collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_NO_ID_CANT_BE_DELETED'));
    }
  }}, {iconCls:collaboration.assets.Icons.colorIcon, localized:{text:'COLLABORATION_CHANGE_COLOR'}, hidden:!isJournal, menu:[{xtype:'colorfield', value:color, initial:true, listeners:{change:function(field, color) {
    var menu = field.up('journalmenu');
    var record = menu.selectedRecord;
    var grid = menu.gridReference;
    var channelId = grid.up('collabJournal').channelId;
    var topicId = record.get('artefactId');
    if (field.initial) {
      field.initial = false;
    } else {
      if (channelId) {
        var channelMeta = collaboration.Api.getChannelById(channelId);
        channelMeta.color = '#' + color;
        collaboration.Api.modifyChannel(me.moduleId, channelId, channelMeta).then(function() {
          grid.getStore().load();
          menu.close();
        });
      }
    }
  }}}]}];
  me.callParent(arguments);
}, channelsMenu:function() {
  var me = this;
  var channels = collaboration.Api.streams.channels.getAll();
  var channesMenu = [];
  var channelClickHandler = function(menuEntry) {
    try {
      var menu = menuEntry.up('journalmenu');
      var record = menu.selectedRecord;
      var topicId = record.getId();
      var moduleId = menuEntry.moduleId;
      var payload = record.getData();
      payload = collaboration.Module.cleanRecord(payload);
      if (payload['topicName']) {
        payload['topic'] = payload['topicName'];
        delete payload['topicName'];
      }
      payload['channelId'] = menuEntry.channelId;
      payload['instanceId'] = moduleId;
      collaboration.Api.modifyTopic(moduleId, topicId, payload).then(function(successCode) {
        collaboration.Support.notify('Topic "' + payload['topic'] + '" moved to "' + menuEntry.name + '" successfully');
        Ext.Function.defer(function() {
          me.gridReference.getStore().load();
        }, 100);
      });
    } catch (e) {
      Ext.log({msg:': ERROR topicMoveTo handler', dump:e, stack:true, level:'error'});
    }
  };
  Ext.Array.each(channels, function(channel, index, channels) {
    var icon = '\x3cspan style\x3d"color:' + channel['color'] + ' ; font-size: 18px; margin-right: 8px; margin-left: -25px"\x3e#\x3c/span\x3e';
    var name = channel['channel'];
    if (channel['channel'].includes(channel['instanceId'])) {
      name = channel.instanceName;
    }
    channesMenu.push({name:name, channelId:channel['channelId'], text:icon + ' ' + Ext.util.Format.htmlEncode2(name), moduleId:channel['instanceId'], handler:channelClickHandler});
  });
  return channesMenu;
}});
Ext.define('collaboration.view.uitypes.streams.grid.ArtefactCard', {extend:'Ext.panel.Panel', alias:'widget.colArtefactCard', layout:'fit', instanceMode:undefined, displaySettings:{}, initComponent:function() {
  this.callParent(arguments);
}, getDisplayDate:function(date, absolute, format) {
  absolute = absolute === undefined ? false : absolute;
  format = format === undefined ? 'd.m.y H:i' : format;
  var displayDate = absolute ? Ext.Date.format(date, format) : collaboration.Module.formatDate(date);
  return absolute ? displayDate : '\x3cspan class\x3d"' + collaboration.assets.Icons.clock + '"\x3e\x3c/span\x3e' + ' ' + displayDate;
}, addArtefactComponents:function() {
  var $jscomp$this = this;
  this.typeBox = Ext.create({xtype:'box', style:{width:'100%', height:'100%'}});
  this.typeBoxWrapper = Ext.create({xtype:'container', flex:1, itemId:'collabDataType', items:[this.typeBox]});
  this.userBox = Ext.create({xtype:'box'});
  this.artefactCreationBox = Ext.create({xtype:'box', margin:'0 0 0 3', style:{fontSize:'12px'}});
  this.toBox = Ext.create({xtype:'box', hidden:true, itemId:'to', html:''});
  this.artefactMetaContainer = Ext.create({xtype:'container', flex:1, layout:{type:'vbox', align:'stretch'}, padding:'2 2 2 2', items:[this.typeBoxWrapper, this.userBox, this.toBox, this.artefactCreationBox]});
  this.duedateBox = Ext.create({xtype:'box', hidden:true, margin:'0 15 0 0'});
  this.headerBox = Ext.create({xtype:'box', hidden:true, width:200, margin:'0 15 0 0'});
  this.messageBox = Ext.create({xtype:'box', flex:1});
  this.detailWrapper = Ext.create({xtype:'container', hidden:true, layout:'fit', lastItemKey:''});
  this.add([{xtype:'container', layout:{type:'vbox', align:'stretch'}, items:[{xtype:'container', layout:{type:'hbox', align:'stretch'}, items:[this.artefactMetaContainer, {xtype:'container', flex:Ext.isDefined(this.instanceMode) && this.instanceMode === 'widget' ? 3 : 6, layout:{type:'fit'}, height:120, items:[{xtype:'container', layout:{type:'vbox', align:'stretch'}, items:[{xtype:'container', height:20, layout:{type:'hbox'}, items:[this.headerBox, this.duedateBox]}, this.messageBox]}]}, {xtype:'container', 
  layout:'center', items:[{xtype:'toolbar', padding:'5 5 5 5', defaults:{scale:'medium', ui:'secondary', margin:'2 2 2 2'}, items:[{iconCls:collaboration.Support.getBpcIconCls('fa-history'), handler:function(b) {
    var updateView = $jscomp$this.detailWrapper.lastItemKey !== 'history';
    if (updateView) {
      $jscomp$this.detailWrapper.removeAll();
      $jscomp$this.detailWrapper.setHeight(150);
      $jscomp$this.detailWrapper.add({xtype:'collabArtefactHistoryGrid', record:$jscomp$this.sourceRecord});
      $jscomp$this.detailWrapper.lastItemKey = 'history';
    }
    var visible = $jscomp$this.detailWrapper.isVisible(true);
    if (visible && updateView) {
    } else {
      $jscomp$this.detailWrapper.setVisible(!visible);
    }
  }}, {xtype:'tbseparator'}, {iconCls:collaboration.Support.getBpcIconCls('fa-external-link'), handler:function() {
    collaboration.Api.openReference($jscomp$this.sourceRecord);
  }}, {xtype:'tbseparator'}, {iconCls:collaboration.Support.getBpcIconCls('fa-comment-dots'), tooltip:collaboration.Support.TR('ADD_COMMENT'), handler:function(b) {
    var messageField = Ext.create({xtype:'textfield', emptyText:'Kommentar', flex:1});
    var updateView = $jscomp$this.detailWrapper.lastItemKey !== 'comment';
    if (updateView) {
      $jscomp$this.detailWrapper.removeAll();
      $jscomp$this.detailWrapper.setHeight(null);
      $jscomp$this.detailWrapper.add({xtype:'toolbar', padding:'1 1 1 1', items:[messageField, {iconCls:collaboration.Support.getBpcIconCls('fa-play'), ui:'primary', handler:function(b) {
        collaboration.Api.modifyArtefactByType('messages', $jscomp$this.sourceRecord.get('artefactId'), {}, 'artefactComment', messageField.getValue()).then(function(r) {
          $jscomp$this.detailWrapper.setVisible(false);
          $jscomp$this.detailWrapper.removeAll();
          $jscomp$this.feedPanel.feedComponent.store.load();
        });
      }}]});
      $jscomp$this.detailWrapper.lastItemKey = 'comment';
    }
    var visible = $jscomp$this.detailWrapper.isVisible(true);
    if (visible && updateView) {
      messageField.focus();
    } else {
      $jscomp$this.detailWrapper.setVisible(!visible);
    }
  }}, {iconCls:collaboration.Support.getBpcIconCls('fa-reply'), tooltip:collaboration.Support.TR('ANSWER'), handler:function(b, e) {
    var messageField = Ext.create({xtype:'textfield', emptyText:'Message', flex:1});
    var updateView = $jscomp$this.detailWrapper.lastItemKey !== 'answer';
    if (updateView) {
      $jscomp$this.detailWrapper.removeAll();
      $jscomp$this.detailWrapper.setHeight(null);
      $jscomp$this.detailWrapper.add({xtype:'toolbar', padding:'1 1 1 1', items:[messageField, {iconCls:collaboration.Support.getBpcIconCls('fa-play'), ui:'primary', handler:function(b) {
        var type = 'messages';
        $jscomp$this.referenceData = {reference:$jscomp$this.sourceRecord.get('reference'), src_moduleId:$jscomp$this.sourceRecord.get('src_moduleId'), src_instanceId:$jscomp$this.sourceRecord.get('src_instanceId')};
        var artefact = Ext.apply({}, {type:type, msg:messageField.getValue(), header:'Re: ' + $jscomp$this.sourceRecord.get('header'), recipients:[{name:$jscomp$this.sourceRecord.get('creator'), type:'user'}]}, $jscomp$this.referenceData);
        collaboration.Api.createArtefact(collaboration.Api.createArtefactUrlFromType(type), JSON.stringify(artefact)).then(function(success) {
          $jscomp$this.detailWrapper.setVisible(false);
          $jscomp$this.detailWrapper.removeAll();
          $jscomp$this.feedPanel.feedComponent.store.load();
        }, function(error) {
        });
      }}]});
      $jscomp$this.detailWrapper.lastItemKey = 'answer';
    }
    var visible = $jscomp$this.detailWrapper.isVisible(true);
    if (visible && updateView) {
      messageField.focus();
    } else {
      $jscomp$this.detailWrapper.setVisible(!visible);
    }
  }}, {iconCls:collaboration.Support.getBpcIconCls('fa-edit'), handler:function(v, rowIndex, colndex, item, e, r) {
    var win = Ext.create({xtype:'collabEditorWindow', collabType:$jscomp$this.sourceRecord.get('artefactType'), items:[Ext.apply({xtype:'collabEditorPanel', collabType:$jscomp$this.sourceRecord.get('artefactType'), record:$jscomp$this.sourceRecord}, {})]});
    win.show();
  }}, {iconCls:collaboration.Support.getBpcIconCls('fa-trash'), handler:function() {
    collaboration.Support.promiseDialog().then(function(success) {
      if (success) {
        collaboration.Api.deleteArtefacts([$jscomp$this.sourceRecord.get('artefactId')]).then(function(statusCode) {
          collaboration.Support.notify(BpcCommon.Api.getTranslation('COLLABORATION_FILE_DELETE_SUCCESS'));
        });
      }
    });
  }}]}]}]}, this.detailWrapper]}]);
}, applyArtefactData:function(rec) {
  this.removeAll();
  this.addArtefactComponents();
  this.sourceRecord = rec;
  this.artefactType = this.sourceRecord.get('artefactType');
  this.setCardIconCls(this.artefactType);
  var creator = this.sourceRecord.get('creator');
  this.userBox.update(collaboration.Module.getRenderedUsername(creator));
  this.artefactCreationBox.update(this.getDisplayDate(this.sourceRecord.get('created'), this.displaySettings.dateAbsolute));
  this.messageBox.update(this.sourceRecord.get('msg'));
  if (this.artefactType !== 'comments') {
    var recipients = this.sourceRecord.get('recipients') || [];
    this.toBox.update(collaboration.Support.TR('TO') + ': ' + recipients.map(function(recipient) {
      return recipient.name;
    }).join(', '));
    this.toBox.setVisible(true);
    this.headerBox.update('Betreff: ' + '\x3cspan style\x3d"font-weight: bold;"\x3e' + this.sourceRecord.get('header') + '\x3c/span\x3e');
    this.headerBox.setVisible(true);
    if (this.artefactType === 'tasks') {
      this.duedateBox.update(collaboration.Support.TR('UNTIL') + ':' + Ext.Date.format(this.sourceRecord.get('duedate'), 'd.m.Y'));
      this.duedateBox.setVisible(true);
    }
  }
}, setCardIconCls:function(type) {
  var iconMap = {comments:'fa-comment-alt', messages:'fa-envelope', tasks:'fa-tasks'};
  var iconCls = iconMap[type] || 'fa-warning';
  this.typeBox.update('\x3cdiv style\x3d"width: 100%; height: 100%;display: flex;align-items: center; margin-left: 5px;"\x3e\x3ci class\x3d"' + BpcCommon.util.Icons.getBpcIconCls(iconCls) + '" style\x3d"display:inline-block;border-radius:60px;font-style: unset;box-shadow: 0px 0px 2px #888; padding: 0.5em 0.6em; background-color: ghostwhite;"\x3e\x3c/i\x3e\x3c/div\x3e');
}, loadReferenceData:function(rec) {
  var data;
  if (rec.get('src_moduleId') === 'vam') {
    data = collaboration.Module.loadVamReferenceData(rec);
  } else {
    if (rec.get('src_moduleId') === 'monitor') {
      data = collaboration.Module.loadMonitorReferenceData(rec);
    }
  }
  if (data) {
  }
}, vamSettingValue:function(store, key, defaultValue) {
  var index = store.findExact('name', key);
  return index >= 0 ? store.getAt(index).data.value : defaultValue;
}});
Ext.define('collaboration.view.grid.ChannelViewController', {extend:'Ext.app.ViewController', alias:'controller.collabChannelViewController', requires:[], listen:{global:{newActivity:'onNewActivity', newMessage:'onNewData', newTask:'onNewData', newComment:'onNewData', itemDeleted:'onNewData', commentUpdated:'onNewData', messageUpdated:'onNewData', taskUpdated:'onNewData'}}, onNewData:function(obj) {
  this.getView().getStore().load();
}, onNewComment:function(btn) {
  var v = this.getView();
  collaboration.Api.getEditorWindow(null, 'comments', v.moduleId, undefined, {topicId:v.topicId, channelId:v.channelId});
}, onNewActivity:function() {
  try {
    var v = this.getView();
    var collabTopic = v.up('collabTopic');
    if (collabTopic) {
      collabTopic.recreateView();
    } else {
      Ext.Function.defer(function() {
        v.store.load();
      }.bind(this), 300);
    }
  } catch (e) {
  }
}});
Ext.define('collaboration.view.uitypes.streams.grid.CollabForumChannelView', {extend:'Ext.grid.Panel', alias:['widget.collabForumChannelView'], stateful:true, stateEvents:['rowclick'], viewModel:{data:{totalCount:null, validations:{validChannel:false}}, formulas:{isEditorValid:function(get) {
  var is = false;
  if (get('validations.validChannel') === true) {
    is = true;
  }
  return is;
}}}, totalCount:0, hideHeaders:true, forceFit:true, viewConfig:{emptyText:collaboration.Support.TR('COLLABORATION_NO_ENTRIES_AVAILABLE'), deferEmptyText:false}, selModel:{mode:'SINGLE'}, plugins:{ptype:'cellediting', clicksToEdit:2, listeners:{beforeedit:function(editor, context, eOpts) {
  var record = context.record;
  if (record.getData().creator !== collaboration.Module.user) {
    if (BpcCommon.Api.hasUserRole('bpcadmin')) {
      return true;
    }
    return false;
  }
}}}, applyState:function(state) {
  var me = this;
  if (state && state.infoStr) {
    var valuesArr = Ext.String.splitWords(state.infoStr);
    if (valuesArr.length === 2) {
      Ext.defer(function() {
        var channelMeta = collaboration.Api.getChannelById(valuesArr[0]);
        var valid = false;
        if (channelMeta) {
          if (me.collabTitlePrefix.toLowerCase() === 'aktiv' && channelMeta.active === true) {
            valid = true;
          } else {
            if (me.collabTitlePrefix.toLowerCase() === 'archiv' && channelMeta.active === null) {
              valid = true;
            }
          }
        }
        if (valid === true) {
          var activeTab = me.up('tabpanel').getActiveTab();
          if (activeTab && activeTab.collabTitlePrefix === me.collabTitlePrefix) {
            me.up('collab' + me.collabStreamMode).updateChannel(valuesArr[0], valuesArr[1]);
          }
        }
      }, 1000);
    }
  }
  Ext.apply(this, state);
}, listeners:{beforestatesave:function(grid, state) {
  if (grid.stateId) {
    if (Ext.util.Cookies.get(grid.stateId)) {
      grid.addPropertyToState(state, 'infoStr', Ext.util.Cookies.get(grid.stateId));
    }
  }
}, rowclick:function(view, record, element, rowIndex, e, eOpts) {
  var gridView = view.up('collabForumChannelView');
  if (gridView.collabStreamMode && gridView.collabStreamMode === 'forum') {
    var channelId = record.get('artefactId');
    var moduleId = gridView.moduleId;
    gridView.up('collab' + gridView.collabStreamMode).updateChannel(channelId, moduleId, record);
    if (gridView.stateId) {
      var infoStr = channelId + ' ' + moduleId;
      Ext.util.Cookies.set(gridView.stateId, infoStr, null);
    }
  }
}, cellcontextmenu:function(tableView, td, cellIndex, record, tr, rowIndex, e, eOpts) {
  e.preventDefault();
  var mainView = tableView.up('collabForumChannelView');
  if (mainView.collabStreamMode && mainView.collabStreamMode === 'forum') {
    var contextMenu = Ext.create('collaboration.view.uitypes.streams.contextMenu.ForumContextMenu', {gridReference:mainView, moduleId:mainView.moduleId, selectedRecord:record});
    var xy = e.getXY();
    contextMenu.showAt(xy);
  }
}}, initComponent:function() {
  var me = this;
  me.stateId = 'collabforumChannel' + me.collabTitlePrefix + 'Grid' + me.moduleId;
  if (me.collabTitlePrefix.toLowerCase() === 'aktiv') {
    me.iconCls = collaboration.assets.Artefacts.topicActiveIcon;
  }
  if (me.collabTitlePrefix.toLowerCase() === 'archiv') {
    me.iconCls = collaboration.assets.Artefacts.archiveIcon;
  }
  me.store = me.createStore(me.moduleId);
  me.dockedItems = [{xtype:'toolbar', dock:'bottom', itemId:'channelToolbar', padding:'5 5 5 5', ui:'footer', defaults:{padding:'5 5 5 5'}, items:['-\x3e', {tooltip:'Channel anlegen', localized:{text:'COLLABORATION_NEW_CHANNEL'}, iconCls:collaboration.assets.Icons.plus, handler:function(btn) {
    var parentCmp = btn.up('collabForumChannelView');
    var moduleId = parentCmp.moduleId;
    Ext.create('collaboration.components.window.Window', {title:collaboration.Support.TR('CHANNEL_NEW'), iconCls:collaboration.assets.Artefacts.createIcon, layout:'fit', modal:true, items:[{xtype:'collabForumChannelEditor', moduleId:moduleId, gridReference:me}]}).show();
  }}]}];
  this.callParent(arguments);
}, createStore:function(instanceId) {
  var me = this;
  var url = collaboration.Module.getBaseUrl() + instanceId + '/channels';
  this.columns = [{dataIndex:'channel', renderer:function(value, metaData, record) {
    var data = record.getData();
    if (!data['color']) {
      data['color'] = '#' + (Math.random() * 16777215 << 0).toString(16);
    }
    return '\x3cspan style\x3d"font-size: 18px; color:' + data['color'] + '"\x3e\x3cb\x3e # \x3c/b\x3e\x3c/span\x3e' + ' ' + '\x3cspan style\x3d"font-size: 18px;"\x3e' + Ext.util.Format.htmlEncode2(value) + '\x3c/span\x3e';
  }}];
  var newModel = Ext.define(null, {extend:'collaboration.model.CollabDataChannels'});
  var filters = [];
  if (this.collabTitlePrefix.toLowerCase() === 'aktiv') {
    filters.push({property:'active', value:true});
  }
  if (this.collabTitlePrefix.toLowerCase() === 'archiv') {
    filters.push({property:'active', value:null});
  }
  var storeConfig = {model:newModel, proxy:{type:'ajax', url:url, reader:{type:'json', rootProperty:'items', totalProperty:'total'}}, filters:filters, emptyText:collaboration.Support.TR('COLLABORATION_NO_ENTRIES_AVAILABLE'), autoLoad:true, collabFirstLoaded:false, listeners:{load:function(store, records, successful, operation, eOpts) {
    var total = store.getTotalCount();
    me.updateTotalCount(total);
    if (store.collabFirstLoaded === false) {
      if (total > 0) {
        var activeTab = me.up('tabpanel').getActiveTab();
        if (!me.loadChannelInfoFromCookie(activeTab)) {
          me.autoLoadTopic(records);
        }
      }
      store.collabFirstLoaded = true;
    }
  }}};
  return Ext.create('Ext.data.Store', storeConfig);
}, updateTotalCount:function(totalCount) {
  this.totalCount = this.getStore().getCount();
  var tabpanel = this.up('tabpanel');
  if (tabpanel && tabpanel.isComponent && this.getId() === tabpanel.getActiveTab().getId()) {
    this.setTitle(this.collabTitlePrefix + ' ( ' + this.totalCount + ' )');
  } else {
    this.setTitle(' ( ' + this.totalCount + ' )');
  }
}, autoLoadTopic:function(records) {
  var me = this;
  if (this.collabTitlePrefix === collaboration.assets.Artefacts.topicStateActive) {
    Ext.defer(function() {
      Ext.Array.each(records, function(record) {
        var active = record.get('active');
        if (Ext.isDefined(active) && active === true) {
          var channelId = record.get('artefactId');
          if (Ext.isDefined(channelId)) {
            var forum = me.up('collabforum');
            if (Ext.isDefined(forum) && forum !== null && forum.isComponent) {
              forum.updateChannel(channelId);
            }
            return false;
          }
        }
      });
    }, 800);
  }
}, loadChannelInfoFromCookie:function(forumGrid) {
  if (forumGrid.stateId && Ext.util.Cookies.get(forumGrid.stateId)) {
    var valuesArr = Ext.String.splitWords(Ext.util.Cookies.get(forumGrid.stateId));
    if (valuesArr.length === 2) {
      var channelMeta = collaboration.Api.getChannelById(valuesArr[0]);
      var valid = false;
      if (channelMeta) {
        if (forumGrid.collabTitlePrefix.toLowerCase() === 'aktiv' && channelMeta.active === true) {
          valid = true;
        } else {
          if (forumGrid.collabTitlePrefix.toLowerCase() === 'archiv' && channelMeta.active === null) {
            valid = true;
          }
        }
      }
      if (forumGrid.channelId && channelMeta.channelId) {
        if (forumGrid.channelId !== channelMeta.channelId) {
          valid = false;
        }
      }
      if (valid === true) {
        forumGrid.up('collab' + forumGrid.collabStreamMode).updateChannel(valuesArr[0], valuesArr[1]);
        return true;
      }
    }
  }
  return false;
}, hideArchiv:function() {
  return !BpcCommon.Api.hasUserRole('bpcadmin');
}});
Ext.define('collaboration.view.uitypes.streams.grid.CollabStreamEditor', {extend:'Ext.panel.Panel', alias:'widget.collabStreamEditor', header:false, topicId:undefined, channelId:undefined, viewModel:{data:{message:'', emptyText:collaboration.Support.TR('COLLABORATION_PLEASE_ENTER_YOUR_COMMENT')}}, items:[], bbar:[{xtype:'htmleditor', bind:{value:'{message}'}, flex:2}, {xtype:'button', iconCls:collaboration.assets.Icons.submit, text:collaboration.Support.TR('SUBMIT'), handler:function(btn) {
  btn.up('collabStreamEditor').commitArtefact();
}}, {xtype:'tbfill'}], commitArtefact:function() {
  var artefact = {topicId:this.topicId, channelId:this.channelId, msg:this.getViewModel().get('message')};
  Ext.Ajax.request({url:collaboration.Module.getBaseUrl() + 'artefacts?type\x3dcomments', method:'POST', jsonData:artefact}).then(function(response, opts) {
    this.getViewModel().set('message', '');
  }.bind(this), function(response, opts) {
    this.getViewModel().set('message', '');
  }.bind(this));
}});
Ext.define('collaboration.view.uitypes.streams.grid.CollabTopicCard', {extend:'Ext.Container', alias:'widget.collabTopicCard', viewModel:{data:{artifact:undefined, channelInfo:undefined}}, data:undefined, channelInfo:undefined, defaults:{xtype:'container', defaults:{xtype:'box', padding:'2 2 2 2'}, padding:'2 2 2 2', layout:{type:'hbox', align:'stretch'}}, layout:{type:'vbox', align:'stretch'}, initComponent:function() {
  var me = this;
  me.items = [{items:[{html:'\x3cspan  class\x3d"' + collaboration.assets.Artefacts.betreffIcon + ' "style\x3d" font-size:14px; color:black"\x3e\x3c/span\x3e'}, {style:{color:'black', fontSize:'18px', fontWeight:'bold'}, bind:{html:'{artifact.topicName}'}}]}, {items:[{style:{color:'grey', overflow:'hidden', textOverflow:'ellipsis'}, bind:{html:'{artifact.description}'}, flex:2}, {style:{color:'grey', marginRight:'5px'}, bind:{html:'\x3cspan class\x3d"' + collaboration.assets.Artefacts.refreshIcon + 
  '" style\x3d"color:' + collaboration.assets.Artefacts.greenColor + '"\x3e\x3c/span\x3e\x26nbsp; {artifact.lastupdate}'}, flex:1}]}, {items:[{xtype:'tbfill'}, {xtype:'label', bind:{style:{backgroundColor:'{channelInfo.color}', color:'white', padding:'1px 15px', borderRadius:'10px'}, text:'# {channelInfo.channel}'}}]}];
  me.callParent(arguments);
}, setVM:function() {
  if (this.data && this.channelInfo) {
    this.getViewModel().set('artifact', this.data);
    this.getViewModel().set('channelInfo', this.channelInfo);
  }
}});
Ext.define('collaboration.view.uitypes.streams.grid.CollabTopicListView', {extend:'Ext.grid.Panel', alias:'widget.collabTopicList', viewModel:{data:{totalCount:null}}, controller:'collabtopiclistviewcontroller', forceFit:true, features:[{ftype:'grouping'}], mnuContext:null, totalCount:0, viewConfig:{emptyText:collaboration.Support.TR('NO_ENTRIES_AVAILABLE'), deferEmptyText:false}, listeners:{rowdblclick:function(view, record, element, rowIndex, e, eOpts) {
  var gridView = view.up('collabTopicList');
  var topicId = record.get('artefactId');
  var channelId;
  if (record['data']['channelId']) {
    channelId = record['data']['channelId'];
  } else {
    channelId = gridView.channelId;
  }
  var moduleId = gridView.moduleId;
  if (gridView.instanceMode && gridView.instanceMode === 'widget') {
    var widget = gridView.up('collabBlogWidget');
    var activeItem = widget.getLayout().getActiveItem();
    var initTitle = widget.up('dbWidgetWrapper').getTitle();
    if (!gridView.collabStreamMode) {
      var topicView = Ext.create({xtype:'collabTopic', moduleId:moduleId, topicId:topicId, channelId:channelId, overView:gridView.up('collabTopic'), instanceMode:'widget', holderCmp:widget, header:false, bbar:[{xtype:'tbfill'}, {xtype:'button', text:collaboration.Support.TR('BACK'), iconCls:collaboration.assets.Icons.angleDoubleLeft, handler:function(btn) {
        var cmp = btn.up('collabTopic');
        cmp.holderCmp.getLayout().setActiveItem(cmp.overView);
        cmp.up('dbWidgetWrapper').setTitle(initTitle);
        cmp.destroy();
        widget.getLayout().setActiveItem(activeItem);
      }}], listeners:{afterrender:function(cmp) {
        Ext.defer(function() {
          var title = cmp.getViewModel().get('topicTitle');
          cmp.up('dbWidgetWrapper').setTitle(initTitle + ' - ' + title);
        }, 1000);
      }}});
      widget.getLayout().setActiveItem(topicView);
    }
  } else {
    if (gridView.collabStreamMode && (gridView.collabStreamMode === 'journal' || gridView.collabStreamMode === 'forum')) {
      gridView.up('collab' + gridView.collabStreamMode).updateStream(channelId, topicId, moduleId, record);
    } else {
      if (Ext.isDefined(record)) {
        channelId = record.get('channelId');
        topicId = record.get('topicId');
        if (Ext.isDefined(channelId) && Ext.isDefined(topicId)) {
          collaboration.Support.showSteamWindow(channelId, topicId, moduleId, record);
        }
      }
    }
  }
}, rowclick:function(view, record, element, rowIndex, e, eOpts) {
  var gridView = view.up('collabTopicList');
  if (gridView.collabStreamMode && (gridView.collabStreamMode === 'journal' || gridView.collabStreamMode === 'forum')) {
    var topicId = record.get('artefactId');
    var channelId;
    if (record['data']['channelId']) {
      channelId = record['data']['channelId'];
    } else {
      channelId = gridView.channelId;
    }
    var moduleId = gridView.moduleId;
    var collabStreamMode;
    if (gridView.collabStreamMode === ' ') {
      collabStreamMode = 'forum';
    } else {
      collabStreamMode = gridView.collabStreamMode;
    }
    gridView.up('collab' + collabStreamMode).updateStream(channelId, topicId, moduleId, record);
    if (gridView.stateId) {
      var infoStr = channelId + ' ' + topicId + ' ' + moduleId;
      Ext.util.Cookies.set(gridView.stateId, infoStr, null);
    }
  } else {
  }
}, cellcontextmenu:function(tableView, td, cellIndex, record, tr, rowIndex, e, eOpts) {
  e.preventDefault();
  var mainView = tableView.up('collabTopicList');
  if (mainView.collabStreamMode && (mainView.collabStreamMode === 'journal' || mainView.collabStreamMode === 'forum')) {
    var contextMenu = Ext.create('collaboration.view.uitypes.streams.contextMenu.JournalContextMenu', {gridReference:mainView, moduleId:mainView.moduleId, selectedRecord:record});
    contextMenu.showAt(e.getXY());
  } else {
    if (tableView.up('collabTopicList')) {
      var grid = tableView.up('collabTopicList');
      if (grid.instanceMode && grid.instanceMode === 'widget') {
        if (BpcCommon.Api.hasUserRight(dashboard.assets.ModuleRights.editDashbaord) === true) {
          var contextMenu = Ext.create('collaboration.view.uitypes.streams.contextMenu.BlogWidgetContextMenu', {gridReference:grid, moduleId:grid.moduleId, selectedRecord:record});
          contextMenu.showAt(e.getXY());
        }
      }
    }
  }
}, filterchange:function(store, filters, eOpts) {
  var me = this;
  var widget = me.up('collabBlogWidget');
  if (Ext.isDefined(widget)) {
    var clearFiltersBtn = widget.down('#clearFiltersBtn');
    if (Ext.isDefined(clearFiltersBtn)) {
      if (filters && Ext.isArray(filters) && filters.length > 0) {
        if (clearFiltersBtn) {
          clearFiltersBtn.setDisabled(false);
        }
      } else {
        if (clearFiltersBtn) {
          clearFiltersBtn.setDisabled(true);
        }
      }
    }
  }
}}, collabFilters:[], groupKey:undefined, groupingEnabled:false, collabStreamMode:undefined, initComponent:function() {
  var me = this;
  try {
    if (!this.channelId || Ext.isDefined(this.channelId)) {
      this.channelId = null;
    }
    if (this.collabStreamMode) {
      if (this.collabStreamMode === 'journal') {
        this.stateId = 'collab' + this.collabStreamMode + this.collabTitlePrefix + 'GridState' + this.moduleId;
      } else {
        me.stateId = 'collab' + me.collabStreamMode + me.collabTitlePrefix + 'GridState' + me.moduleId + me.channelId;
      }
    }
    this.mnuContext = Ext.create('collaboration.view.uitypes.streams.contextMenu.BlogWidgetContextMenu', {currentGrid:this});
    var instanceMode;
    if (this.up('collabBlog')) {
      instanceMode = this.up('collabBlog').instanceMode;
    }
    this.store = this.createStore(this.moduleId);
    this.title = this.collabTitlePrefix;
    if (instanceMode && instanceMode === 'widget') {
      var plugins = this.getPlugins();
      if (!plugins) {
        plugins = [];
      }
      this.plugins = plugins;
      this.stateful = true;
      this.stateId = 'collabBlogWidgetState';
      this.bbar = [{xtype:'collabGridInput', flex:3, collabType:this.collabType, gridReference:this.getXType()}, {xtype:'collabDateRangeField', gridReference:this.getXType()}];
      this.setHidden(true);
    } else {
      this.sortableColumns = false;
      var items = [];
      var dock = null;
      if (!this.collabStreamMode) {
        dock = 'top';
        var groupMenu = [];
        groupMenu.push({text:collaboration.Support.TR('NO_GROUPING'), groupKey:'off', handler:function(btn) {
          var blog = btn.up('collabblog'), groupingEnabled = true;
          if (btn.groupKey === 'off') {
            groupingEnabled = false;
          }
          blog.enableGrouping(groupingEnabled, btn.groupKey);
        }});
        if (Ext.isDefined(this.columns) && Ext.isArray(this.columns) && this.columns.length > 0) {
          Ext.Array.each(this.columns, function(columnConfig) {
            if ('dataIndex' in columnConfig && Ext.isDefined(columnConfig['dataIndex'])) {
              if ('text' in columnConfig && Ext.isDefined(columnConfig['text'])) {
                groupMenu.push({text:columnConfig['text'], groupKey:columnConfig['dataIndex'], handler:function(btn) {
                  var blog = btn.up('collabblog'), groupingEnabled = true;
                  if (btn.groupKey === 'off') {
                    groupingEnabled = false;
                  }
                  blog.enableGrouping(groupingEnabled, btn.groupKey);
                }});
              }
            }
          });
        }
        items = [{tooltip:collaboration.Support.TR('CREATE_BLOGENTRY'), text:collaboration.Support.TR('NEW_TOPIC'), iconCls:collaboration.assets.Artefacts.createIcon, handler:function(btn) {
          var parentCmp = btn.up('collabTopicList');
          var topicMetaWindow = Ext.createByAlias('widget.topicCreateWindow', {moduleId:parentCmp.moduleId, parentCmp:parentCmp, mode:'new'});
          topicMetaWindow.show();
        }}, {xtype:'bpcTextField', flex:1, emptyText:collaboration.Support.TR('LOOKUP_IN_TOPICS'), tooltip:collaboration.Support.TR('LOOKUP_IN_TOPICS_DESC'), triggers:{resetField:{cls:collaboration.assets.Icons.remove, hidden:true, handler:function() {
          this.setValue('');
        }}}, toggleBpcResetTrigger:function() {
          var trigger = this.getTrigger('resetField');
          if (this.getValue()) {
            trigger.show();
          } else {
            trigger.hide();
          }
        }, listeners:{change:{fn:function(tf) {
          var topicList = tf.up('collabTopicList');
          var store = topicList.getStore();
          var filterText = tf.getValue();
          tf.toggleBpcResetTrigger();
          if (Ext.isDefined(filterText) && filterText !== '') {
            store.clearFilter(true);
            store.addFilter([me.collabFilters[0], {id:'nameFilter', filterFn:function(item) {
              var matcher = Ext.String.createRegex(filterText, false, false, true);
              var match = false;
              Ext.Object.each(item.data, function(property, value) {
                if (BpcCommon.Util.isStringEqual(property, 'channelId') || BpcCommon.Util.isStringEqual(property, 'id') || BpcCommon.Util.isStringEqual(property, 'topicId') || BpcCommon.Util.isStringEqual(property, 'instanceId')) {
                  return;
                }
                if (Ext.isObject(value) || Ext.isArray(value)) {
                  value = JSON.stringify(value);
                }
                match = matcher.test(value);
                if (match === true) {
                  return false;
                }
              });
              return match;
            }}]);
          } else {
            store.clearFilter();
            store.addFilter(me.collabFilters);
          }
        }, buffer:500}}}, {iconCls:collaboration.assets.Icons.clone, tooltip:collaboration.Support.TR('GROUP'), menu:groupMenu}, {iconCls:collaboration.assets.Icons.refresh, handler:function(btn) {
          btn.up('collabTopicList').getStore().load();
        }}];
      } else {
        this.hideHeaders = true;
        dock = 'bottom';
        items = [{xtype:'tbfill', flex:1}, {tooltip:collaboration.Support.TR('CREATE_ENTRY'), text:collaboration.Support.TR('NEW_TOPIC'), iconCls:collaboration.assets.Icons.plus, handler:function(btn) {
          var parentCmp = btn.up('collabTopicList');
          var topicMetaWindow = Ext.createByAlias('widget.topicCreateWindow', {moduleId:parentCmp.moduleId, parentCmp:parentCmp, mode:'new'});
          topicMetaWindow.show();
        }}];
      }
      this.dockedItems = {xtype:'toolbar', dock:dock, padding:'5 5 5 5', ui:'footer', defaults:{padding:'5 5 5 5'}, items:items};
    }
  } catch (e) {
  }
  this.callParent(arguments);
  this.getStore().load();
}, createStoreUrl:function(instanceId) {
  var url = collaboration.Module.getBaseUrl() + instanceId + '/topics';
  if (this.collabStreamMode && this.collabStreamMode === 'forum') {
    url += '?channelId\x3d' + this.channelId;
  }
  return url;
}, updateStoreWithChannelId:function(channelId) {
  this.channelId = channelId;
  var url = this.createStoreUrl(this.moduleId);
  var store = this.getStore();
  store.getProxy().setUrl(url);
  store.reload();
}, getBlogColumns:function(instanceId) {
  return [{xtype:'collabWidgetColumn', text:this.getColumnHeaderBase('x-fal fa-list', '#4CAF50', collaboration.Support.TR('OVERVIEW')), items:[], instanceType:'blog', moduleId:this.moduleId, onWidgetAttach:function(col, widget, rec) {
    if (rec) {
      var topicOverview = rec.get('topicOverview');
      widget.initCollabData(topicOverview ? {details:topicOverview} : undefined);
      widget.record = rec;
      widget.instanceType = col.instanceType;
      widget.moduleId = col.moduleId;
    }
  }}, {xtype:'datecolumn', format:'d.m.Y H:i:s', dataIndex:'created', flex:1, text:this.getColumnHeaderBase(collaboration.assets.Icons.clock, '#4CAF50', collaboration.Support.TR('CREATED'))}, {dataIndex:'creator', flex:1, text:this.getColumnHeaderBase(collaboration.assets.Icons.user, '#4CAF50', collaboration.Support.TR('CREATOR2'))}, {dataIndex:'topicName', text:this.getColumnHeaderBase(collaboration.assets.Icons.bookmark, '#4CAF50', collaboration.Support.TR('COLLABORATION_BLOG_HEADER')), cellWrap:false, 
  flex:2, renderer:function(value, meta) {
    meta.tdStyle = 'text-overflow: clip ellipsis;';
    if (value && Ext.isDefined(value)) {
      meta.tdAttr = 'data-qtip\x3d"' + Ext.util.Format.htmlDoubleEncode(value) + '" data-qtitle\x3d"' + collaboration.Support.TR('HEADER') + '"';
    }
    return Ext.util.Format.htmlEncode2(value);
  }}];
}, getColumnHeader:function(v) {
  return this.getColumnHeaderBase(v['iconCls'], '#4CAF50', Ext.util.Format.htmlEncode2(v['defaultText']));
}, getColumnHeaderBase:function(cls, color, text) {
  return '\x3cspan class\x3d"' + cls + '" style\x3d"color:' + color + '"\x3e\x3c/span\x3e\x26nbsp;' + text;
}, createStore:function(instanceId) {
  try {
    var me = this, type = 'topics';
    var columns = this.getBlogColumns(instanceId);
    if (!this.collabStreamMode) {
      columns[3].dataIndex = 'topic';
    }
    var fields = [];
    var customFields = collaboration.Support.getSetting(this.moduleId, 'instance_channelTopicTemplate');
    if (Ext.isObject(customFields)) {
      columns = columns.concat(me.prepareColumnsForOldConfig(customFields, fields));
    } else {
      if (Ext.isArray(customFields)) {
        columns = columns.concat(me.prepareColumns(customFields, fields));
      }
    }
    var tabPrefix = this.collabTitlePrefix;
    var activeIcon = collaboration.assets.Artefacts.topicActiveIcon;
    var activeTooltip = collaboration.Support.TR('ACTIVATE');
    if (tabPrefix && tabPrefix === 'Aktiv') {
      activeIcon = collaboration.assets.Artefacts.topicInactiveIcon;
      activeTooltip = collaboration.Support.TR('DEACTIVATE');
    }
    var hideActionCol = false;
    if (!this.collabStreamMode) {
      if (this.up('collabBlog').instanceMode === 'widget') {
        hideActionCol = true;
      }
    }
    columns.push({xtype:'actioncolumn', text:this.getColumnHeaderBase(collaboration.assets.Icons.edit, '#4CAF50', 'Action'), width:50, flex:1, hidden:hideActionCol, menuDisabled:true, sortable:false, items:[{iconCls:collaboration.assets.Artefacts.editIcon, moduleId:me.moduleId, tooltip:collaboration.Support.TR('EDIT_TOPIC'), handler:'onEdit'}, {iconCls:activeIcon, moduleId:me.moduleId, handler:'onActiveChange', tooltip:activeTooltip}, {iconCls:collaboration.assets.Artefacts.archiveIcon, moduleId:me.moduleId, 
    handler:'onArchive', tooltip:collaboration.Support.TR('ARCHIVE'), isActionDisabled:function(view, rowIndex, colIndex, item, record) {
      if (record['data']['state'] && record['data']['state'] === 'archived') {
        return true;
      }
      return false;
    }}, {iconCls:collaboration.assets.Icons.bars, moduleId:me.moduleId, handler:'onMoveTo'}, {iconCls:collaboration.assets.Icons.trash, text:collaboration.Support.TR('DELETE'), handler:'onTopicDelete'}]});
    if (this.collabStreamMode) {
      if (this.collabStreamMode === 'forum' || this.collabStreamMode === 'journal') {
        columns = [this.loadForumOrJournalColumn()];
      }
    }
    if (this.instanceMode && this.instanceMode === 'widget') {
      Ext.Array.each(columns, function(column) {
        if (column.xtype && column.xtype === 'datecolumn') {
          column.filter = true;
        } else {
          column.filter = {type:'string', itemDefaults:{emptyText:collaboration.Support.TR('LOOKUP') + '...'}};
        }
      });
    }
    this.columns = columns;
    var newModel = Ext.define(null, {extend:'collaboration.model.CollabDataTopics', fields:fields});
    var url = this.createStoreUrl(instanceId);
    var storeConfig = {model:newModel, pageSize:0, proxy:{type:'ajax', url:url, reader:{type:'json', rootProperty:'items', totalProperty:'total'}}, filters:this.collabFilters, autoLoad:false, listeners:{load:function(store, records, successful, operation, eOpts) {
      var total = store.getData().length;
      me.updateTotalCount(total);
      var activeTab = me.up('tabpanel').getActiveTab();
      if (me.collabStreamMode && me.collabTitlePrefix === activeTab.collabTitlePrefix) {
        var collabStreamMode = me.collabStreamMode;
        if (total && total > 0) {
          var record = store.getData().getAt(0);
          var channelId;
          if (record['data']['channelId']) {
            channelId = record['data']['channelId'];
          } else {
            channelId = me.channelId;
          }
        } else {
          var topicCmp = me.up('collab' + collabStreamMode).down('#forumTopicHolderCmp');
          if (topicCmp && topicCmp.isComponent) {
            topicCmp.removeAll();
            topicCmp.add({xtype:'component', html:collaboration.Support.TR('NO_ENTRIES_AVAILABLE')});
          }
        }
      }
    }}};
    if (!this.collabStreamMode) {
      storeConfig = {pageSize:0, proxy:{type:'ajax', url:url, reader:{type:'json', rootProperty:'items', totalProperty:'total'}, extraParams:{includeCounts:true}}, filters:this.collabFilters, autoLoad:false, listeners:{load:function(store, records, successful, operation, eOpts) {
        var total = store.getData().length;
        me.updateTotalCount(total);
        var activeTab = me.up('tabpanel').getActiveTab();
        if (me.collabStreamMode && me.collabTitlePrefix === activeTab.collabTitlePrefix) {
          var collabStreamMode = me.collabStreamMode;
          if (total && total > 0) {
            var record = store.getData().getAt(0);
            var topicId = record.get('artefactId');
            var channelId;
            if (record['data']['channelId']) {
              channelId = record['data']['channelId'];
            } else {
              channelId = me.channelId;
            }
            var moduleId = me.moduleId;
          }
        }
      }}};
    }
    if (this.groupingEnabled === true && Ext.isDefined(this.groupKey)) {
      storeConfig.groupField = this.groupKey;
    }
    return Ext.create('Ext.data.Store', storeConfig);
  } catch (e) {
  }
}, prepareColumnsForOldConfig:function(obj, fields) {
  var me = this;
  var columns = [];
  Ext.iterate(obj, function(k, v) {
    var col = {};
    if (v && Ext.isDefined(v) && Ext.isObject(v)) {
      var text = me.getColumnHeader(v);
      fields.push({name:k, mapping:k});
      col = {dataIndex:k, text:text, xtype:'gridcolumn', flex:1};
      if (k === 'description') {
        col.flex = 3;
        col.renderer = function(value, meta) {
          meta.tdStyle = 'text-overflow: clip ellipsis;';
          meta.tdAttr = 'data-qtip\x3d"' + Ext.util.Format.htmlDoubleEncode(value) + '" data-qtitle\x3d"' + Ext.util.Format.htmlDoubleEncode(v['defaultText']) + '"';
          return Ext.util.Format.htmlEncode2(value);
        };
      }
      if (v['type'] === 'date') {
        col.xtype = 'datecolumn';
        col.format = 'd.m.Y';
      }
    } else {
      fields.push({name:k, mapping:k});
      col = {dataIndex:k, text:k, xtype:'gridcolumn', flex:1};
      if (k === 'description') {
        col.flex = 3;
        col.renderer = function(value, meta) {
          value = value || '';
          meta.tdStyle = 'text-overflow: clip ellipsis;';
          meta.tdAttr = 'data-qtip\x3d"' + Ext.util.Format.htmlDoubleEncode(value) + '" data-qtitle\x3d"' + Ext.util.Format.htmlDoubleEncode(k) + '"';
          return Ext.util.Format.htmlEncode2(value);
        };
      }
      if (v === 'date') {
        col.xtype = 'datecolumn';
        col.format = 'd.m.Y';
      }
    }
    if (k === 'active') {
      col.hidden = true;
    }
    columns.push(col);
  });
  return columns;
}, prepareColumns:function(arr, fields) {
  var me = this;
  var columns = [];
  Ext.each(arr, function(item, index) {
    var col = {};
    if (item && Ext.isDefined(item) && Ext.isObject(item)) {
      var text = me.getColumnHeader(item);
      fields.push({name:item.name, mapping:item.name});
      col = {dataIndex:item.name, text:text, xtype:'gridcolumn', flex:1};
      if (item.name === 'description') {
        col.flex = 3;
        col.renderer = function(value, meta) {
          value = value || '';
          meta.tdStyle = 'text-overflow: clip ellipsis;';
          meta.tdAttr = 'data-qtip\x3d"' + Ext.util.Format.htmlDoubleEncode(value) + '" data-qtitle\x3d"' + Ext.util.Format.htmlDoubleEncode(item['defaultText']) + '"';
          return Ext.util.Format.htmlEncode2(value);
        };
      }
      if ((item.type === 'combo' || item.type === 'list') && 'values' in item && item.values.length > 0) {
        col.renderer = function(value, meta, rec, rowIdx, colIdx, store, view) {
          var val = value;
          if ('values' in item) {
            var comboValueConfig = item.values;
            Ext.Array.each(comboValueConfig, function(valConfig, index, comboValueConfig) {
              if (valConfig.value === value) {
                var color = 'black';
                if ('color' in valConfig) {
                  color = valConfig.color;
                }
                if ('label' in valConfig) {
                  val = Ext.util.Format.htmlEncode2(valConfig.label);
                }
                if ('iconCls' in valConfig) {
                  val = collaboration.Module.getIconClsSpan(valConfig.iconCls, color) + '  ' + val;
                }
                return false;
              }
            });
          }
          return val;
        };
      }
      if (item['type'] === 'date') {
        col.xtype = 'datecolumn';
        col.format = 'd.m.Y';
      }
    }
    if (item.name === 'active') {
      col.hidden = true;
    }
    columns.push(col);
  });
  return columns;
}, selectTopic:function(channelId, topicId, uiType) {
  var me = this;
  if (Ext.isDefined(channelId) && Ext.isDefined(topicId) && Ext.isDefined(uiType)) {
    var baseCmp;
    if (uiType === 'journal') {
      baseCmp = 'collab' + uiType;
    } else {
      if (uiType === 'forum') {
        baseCmp = 'collab' + uiType;
      }
    }
    if (Ext.isDefined(uiType)) {
      me.up(baseCmp).updateStream(channelId, topicId, me.moduleId);
    }
  }
}, hideArchiv:function() {
  return !BpcCommon.Api.hasUserRole('bpcadmin');
}, updateTotalCount:function(totalCount) {
  this.totalCount = this.getStore().getCount();
  var tabpanel = this.up('tabpanel');
  var blog = tabpanel.up('collabBlog');
  if (blog) {
    this.setTitle(this.collabTitlePrefix + ' ( ' + this.totalCount + ' )');
  } else {
    if (tabpanel && tabpanel.isComponent && this.getId() === tabpanel.getActiveTab().getId()) {
      this.setTitle(this.collabTitlePrefix + ' ( ' + this.totalCount + ' )');
    } else {
      this.setTitle(' ( ' + this.totalCount + ' )');
    }
  }
}, loadForumOrJournalColumn:function() {
  var me = this;
  var isJournal = false;
  if (this.collabStreamMode && this.collabStreamMode === 'journal') {
    isJournal = true;
    var channelId = me.up('collabJournal').channelId;
    var channelMeta = collaboration.Api.getChannelById(channelId);
    if (!channelMeta || !channelMeta.color) {
      me.color = '#' + (Math.random() * 16777215 << 0).toString(16);
    }
  }
  var col = {xtype:'widgetcolumn', flex:1, widget:{xtype:'collabTopicCard'}, onWidgetAttach:function(col, widget, rec) {
    try {
      var data = rec.getData();
      var channelInfo;
      if (data['channelId']) {
        channelInfo = collaboration.Api.getChannelById(data['channelId']);
        if (!channelInfo.color) {
          channelInfo.color = me.color;
        }
      }
      if (data['lastupdate']) {
        data['lastupdate'] = Ext.Date.format(data['lastupdate'], 'Y-m-d H:i:s');
      }
      widget.data = data;
      widget.channelInfo = channelInfo;
      widget.setVM();
    } catch (e) {
    }
  }};
  if (Ext.isIE) {
    col = {dataIndex:'topicName', renderer:function(value, metaData, record) {
      var data = record.getData();
      if (!data['color']) {
        data['color'] = '#' + (Math.random() * 16777215 << 0).toString(16);
      }
      return '\x3cspan style\x3d"font-size: 18px; color:' + data['color'] + '"\x3e\x3cb\x3e # \x3c/b\x3e\x3c/span\x3e' + ' ' + '\x3cspan style\x3d" white-space:normal !important; font-size: 18px;"\x3e' + value + '\x3c/span\x3e';
    }};
  }
  return col;
}});
Ext.define('collaboration.view.uitypes.streams.grid.CollabTopicListViewController', {extend:'Ext.app.ViewController', alias:'controller.collabtopiclistviewcontroller', listen:{global:{newTopic:'onNewData', topicDeleted:'onNewData', topicUpdated:'onNewData'}}, onNewData:function(obj) {
  this.getView().getStore().reload();
}, onEdit:function(grid, rowIndex, colIndex, btn, event, record) {
  if (collaboration.Module.user === record['data']['creator'] || BpcCommon.Api.hasUserRole('bpcadmin')) {
    var topicEditWindow = Ext.createByAlias('widget.topicCreateWindow', {moduleId:btn.moduleId, record:record, parentCmp:grid, mode:'edit'});
    topicEditWindow.show();
  }
}, onActiveChange:function(grid, rowIndex, colIndex, btn, event, record) {
  var topicId = this.getTopicId(record);
  var moduleId = btn.moduleId;
  var payload = this.getTopicPayload(record, moduleId, 'archived');
  if ('state' in payload) {
    delete payload['state'];
  }
  if (!payload.activeTo || 'activeTo' in payload && payload.activeTo === null && payload.active === true) {
    payload.activeTo = Ext.Date.format(new Date, 'Y-m-d\\TH:m:s');
  }
  if ('active' in payload) {
    payload['active'] = !payload['active'];
  }
  collaboration.Api.modifyTopic(moduleId, topicId, payload).then(function() {
    var panels = grid.up('tabpanel').items.items;
    Ext.Array.each(panels, function(panel) {
      panel.getStore().load();
    });
  });
}, getTopicId:function(record) {
  var topicId = record.get('topicId');
  if (!Ext.isDefined(topicId)) {
    topicId = record.getId();
  }
  return topicId;
}, getTopicPayload:function(record, moduleId, state) {
  var payload = record.getData();
  payload = collaboration.Module.cleanRecord(payload);
  if (payload['topicName']) {
    payload['topic'] = payload['topicName'];
    delete payload['topicName'];
  }
  payload['lastUpdate'] = Ext.Date.format(new Date, 'Y-m-d H:i:s');
  payload['instanceId'] = moduleId.toString();
  payload['state'] = state;
  return payload;
}, onArchive:function(grid, rowIndex, colIndex, btn, event, record) {
  var topicId = this.getTopicId(record);
  var moduleId = btn.moduleId;
  var payload = this.getTopicPayload(record, moduleId, 'archived');
  payload['active'] = null;
  if (!payload.activeTo || 'activeTo' in payload && payload.activeTo === null && payload.active === true) {
    payload.activeTo = Ext.Date.format(new Date, 'Y-m-d\\TH:m:s');
  }
  collaboration.Api.modifyTopic(moduleId, topicId, payload).then(function() {
    var panels = grid.up('tabpanel').items.items;
    Ext.Array.each(panels, function(panel) {
      panel.getStore().load();
    });
  });
}, onMoveTo:function(grid, rowIndex, colIndex, item, e, record) {
  var me = this;
  var topicId = record.get('topicId');
  var channelId = record.get('channelId');
  if (me.isTopicReferenced(topicId, channelId)) {
    BpcCommon.Api.showNotification({text:'COLLABORATION_ERROR_EXISTING_REFERENCE', type:'ERROR', popUp:true});
    return;
  }
  var menu_grid = new Ext.menu.Menu({items:[{text:'MoveTo', menu:this.channelsMenu(record)}]});
  var position = e.getXY();
  e.stopEvent();
  menu_grid.showAt(position);
}, onTopicDelete:function(grid, rowIndex, colIndex, item, e, record) {
  var me = this;
  var topicId = record.get('topicId');
  var channelId = record.get('channelId');
  if (Ext.isDefined(topicId)) {
    if (me.isTopicReferenced(topicId, channelId)) {
      BpcCommon.Api.showNotification({text:'COLLABORATION_ERROR_EXISTING_REFERENCE', type:'ERROR', popUp:true});
      return;
    }
    Ext.Msg.confirm(collaboration.Support.TR('DELETE'), collaboration.Support.TR('COLLABORATION_DELETE_ENTRY_WARNING_Q'), function(btnText) {
      if (btnText === 'yes') {
        collaboration.Api.deleteTopic(topicId).then(function() {
          collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ENTRY_DELETED_SUCCESS'));
          Ext.defer(function() {
            grid.getStore().load();
          }, 100);
        }, function() {
          collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ENTRY_DELETED_ERROR'));
        });
      }
    });
  } else {
    collaboration.Support.notify(collaboration.Support.TR('COLLABORATION_ERROR_NO_ID_CANT_BE_DELETED'));
  }
}, isTopicReferenced:function(topicId, channelId) {
  var me = this;
  var url = collaboration.Module.getBaseUrl() + me.getView().moduleId + '/stream/' + channelId + '/' + topicId + '?limit\x3d0';
  var referenceCount = 0;
  Ext.Ajax.request({async:false, url:url, success:function(response, opts) {
    var obj = Ext.decode(response.responseText);
    referenceCount = obj.total;
  }, failure:function(response, opts) {
    console.log('server-side failure with status code ' + response.status);
  }});
  return referenceCount !== 0;
}, channelsMenu:function(record) {
  var me = this;
  var channels = collaboration.Api.streams.channels.getAll();
  var channesMenu = [];
  var channelClickHandler = function(menuEntry) {
    try {
      var topicId = record.get('topicId');
      var moduleId = menuEntry.moduleId;
      var payload = record.getData();
      payload = collaboration.Module.cleanRecord(payload);
      if (payload['topicName']) {
        payload['topic'] = payload['topicName'];
        delete payload['topicName'];
      }
      payload['channelId'] = menuEntry.channelId;
      payload['instanceId'] = moduleId;
      collaboration.Api.modifyTopic(moduleId, topicId, payload).then(function(successCode) {
        collaboration.Support.notify('Topic "' + payload['topic'] + '" moved to "' + menuEntry.name + '" successfully');
        Ext.Function.defer(function() {
          me.getView().getStore().load();
        }, 100);
      });
    } catch (e) {
    }
  };
  Ext.Array.each(channels, function(channel, index, channels) {
    var icon = '\x3cspan style\x3d"color:' + channel['color'] + ' ; font-size: 18px; margin-right: 8px; margin-left: -25px"\x3e#\x3c/span\x3e';
    var name = channel['channel'];
    if (channel['channel'].includes(channel['instanceId'])) {
      name = channel.instanceName;
    }
    channesMenu.push({name:name, channelId:channel['channelId'], text:icon + ' ' + Ext.util.Format.htmlEncode2(name), moduleId:channel['instanceId'], handler:channelClickHandler});
  });
  return channesMenu;
}});
Ext.define('collaboration.view.uitypes.streams.topic.Topic', {extend:'Ext.panel.Panel', alias:['widget.collabTopic', 'widget.collabtopic'], viewModel:{type:'collabTopicViewModel'}, bind:{title:'{topicTitle}'}, layout:{type:'vbox', align:'stretch'}, moduleId:undefined, channelId:undefined, topicId:undefined, instanceMode:undefined, initComponent:function() {
  this.defaults = {moduleId:this.moduleId, channelId:this.channelId, topicId:this.topicId};
  var showNewButton = false;
  var showCloseButton = false;
  if (this.showNewButton === true) {
    showNewButton = true;
  }
  if (this.showCloseButton === true) {
    showCloseButton = true;
  }
  this.items = [{xtype:'collabTopicHeader'}, {xtype:'collabChannelView', flex:1, showNewButton:showNewButton, showCloseButton:showCloseButton}];
  if (Ext.isDefined(this.instanceMode) && this.instanceMode === 'widget') {
    this.defaults.instanceMode = this.instanceMode;
    this.items = [{xtype:'collabChannelView', flex:1}];
  }
  this.callParent(arguments);
  this.customizeWindow();
  this.getViewModel().set('defaultTitle', this.topicId);
  this.loadTopicMetaData();
}, recreateView:function() {
  this.removeAll();
  this.add([{xtype:'collabTopicHeader'}, {xtype:'collabChannelView', flex:1}, {xtype:'collabStreamEditorPanel', bind:{hidden:'{!isActive}'}}]);
}, loadTopicMetaData:function() {
  var url = collaboration.Module.getBaseUrl() + this.moduleId + '/topics', me = this;
  Ext.Ajax.request({method:'GET', url:url}).then(function(response, opts) {
    var obj = Ext.decode(response.responseText, true);
    if (obj && obj !== null) {
      me.processResponse(obj);
    }
  }, function(response, opts) {
  });
}, customizeWindow:function() {
  var win = this.up('window');
  if (Ext.isDefined(win) && win.isComponent) {
    if (this.moduleId && Ext.isDefined(this.moduleId)) {
      var moduleName = collaboration.Support.getSetting(this.moduleId, 'module_name');
      var iconCls = collaboration.Support.getSetting(this.moduleId, 'module_iconCls');
      win.setIconCls(iconCls);
      win.setTitle(moduleName);
    }
  }
}, processResponse:function(response) {
  var me = this;
  if (response.items && response.total && response.total > 0) {
    var rawEsArray = response.items;
    var vm = me.getViewModel();
    Ext.Array.each(rawEsArray, function(elem) {
      if (elem['artefactId'] === me.topicId) {
        var obj = elem;
        obj = Ext.apply(obj, {topicId:elem['artefactId']});
        if (vm) {
          vm.set('topicObj', obj);
          vm.set('iconCls', collaboration.assets.Icons.bookmark);
        }
      }
    });
  }
}});
Ext.define('collaboration.view.uitypes.streams.topic.TopicHeader', {extend:'Ext.container.Container', alias:['widget.collabTopicHeader', 'widget.collabtopicheader'], layout:{type:'vbox', align:'stretch'}, padding:'5 5 5 5', defaults:{padding:'2 2 2 2'}, initComponent:function() {
  this.items = [{xtype:'container', layout:'hbox', defaults:{xtype:'box', padding:'2 2 2 2'}, vmHolderComponent:'collabTopic', items:[{height:45, style:'font-size: 18px;line-height:40px;vertical-align:middle;', bind:{html:'\x3cspan class\x3d"{iconCls}"\x3e\x3c/span\x3e'}}, {height:45, style:'font-size: 24px;line-height:40px;vertical-align:middle;', bind:{html:'\x3cb\x3e{topicTitle}\x3c/b\x3e'}}]}, {xtype:'container', height:40, width:150, scrollable:true, bind:{style:{color:'grey'}, html:'{topicDescription}'}}, 
  {xtype:'container', layout:'hbox', defaults:{xtype:'box', padding:'2 2 2 2'}, items:[{xtype:'collabTimebox', vmHolderComponent:'collabTopic', ttPrefix:''}, {bind:{html:'\x3cspan class\x3d"{activeClass}" style\x3d"color:{activeColor}"\x3e\x3c/span\x3e {activeText}'}}, {xtype:'collabTopicHeaderBoxBase', boxTooltip:BpcCommon.Api.getTranslation('COLLABORATION_CREATOR2'), bind:{html:'\x3cspan class\x3d"' + collaboration.assets.Artefacts.creatorIcon + '" style\x3d"color:' + collaboration.assets.Artefacts.greenColor + 
  '"\x3e\x3c/span\x3e {topicObj.creator}'}}, {xtype:'collabTopicHeaderBoxBase', boxTooltip:collaboration.Support.TR('ACTIVE_TO'), bind:{html:'{activeFromHtml}'}}, {xtype:'collabTopicHeaderBoxBase', boxTooltip:BpcCommon.Api.getTranslation('COLLABORATION_UNTIL'), bind:{html:'{activeToHtml}'}}, {xtype:'collabTopicHeaderBoxBase', boxTooltip:collaboration.Support.TR('TEAM'), bind:{html:'{teamHtml}'}}, {xtype:'tbfill'}, {xtype:'collabTopicHeaderBoxBase', boxTooltip:collaboration.Support.TR('LAST_UPDATE'), 
  bind:{html:'{lastUpdateToHtml}'}}]}];
  this.callParent(arguments);
}});
Ext.define('collaboration.view.uitypes.streams.topic.TopicHeaderBoxBase', {extend:'collaboration.support.generics.Box', alias:['widget.collabTopicHeaderBoxBase'], boxTooltip:'not set properly', listeners:{afterrender:function(cmp) {
  var tip = Ext.create('Ext.tip.ToolTip', {target:cmp.id, targetCard:cmp, listeners:{beforeshow:function(tt) {
    tt.update(cmp.boxTooltip);
  }}});
}}});
Ext.define('collaboration.view.uitypes.streams.topic.TopicViewModel', {extend:'Ext.app.ViewModel', alias:'viewmodel.collabTopicViewModel', data:{defaultTitle:'default', defaultDescription:collaboration.Support.TR('COLLABORATION_NO_DESCRIPTION_AVAILABLE'), topicObj:null, iconCls:collaboration.assets.Icons.fire}, formulas:{topicTitle:function(get) {
  var title = get('defaultTitle');
  var obj = get('topicObj');
  if (obj !== null && Ext.isObject(obj) && obj.topic) {
    title = obj.topic;
  }
  return Ext.util.Format.htmlEncode2(title);
}, topicDescription:function(get) {
  var desc = get('defaultDescription');
  var obj = get('topicObj');
  if (obj !== null && Ext.isObject(obj) && obj.topic) {
    desc = obj.description;
  }
  return Ext.util.Format.htmlEncode2(desc);
}, created:function(get) {
  return new Date(get('topicObj.created'));
}, formattedDate:function(get) {
  var created = get('created'), now = new Date, fromDate = parseInt(created.getTime() / 1000), toDate = parseInt(now.getTime() / 1000), timeDiff = (toDate - fromDate) / 3600, timeDiff = Math.round(timeDiff), dateString = collaboration.Support.TR('BEFORE') + ' ' + +timeDiff + ' ' + collaboration.Support.TR('HOURS');
  if (timeDiff === 0) {
    dateString = collaboration.Support.TR('NOW');
  }
  if (timeDiff === 1) {
    dateString = collaboration.Support.TR('BEFORE') + ' ' + timeDiff + ' ' + collaboration.Support.TR('HOUR');
  }
  if (timeDiff > 24) {
    timeDiff = timeDiff / 24;
    timeDiff = Math.round(timeDiff);
    dateString = collaboration.Support.TR('BEFORE') + ' ' + timeDiff + ' ' + collaboration.Support.TR('DAYS');
  }
  dateString = '\x3ci\x3e' + dateString + '\x3c/i\x3e';
  return dateString;
}, formattedDateString:function(get) {
  var created = get('created');
  return '\x3cspan class\x3d"' + collaboration.assets.Icons.clock + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;\x26nbsp;' + Ext.Date.format(created, 'd.m.Y H:i:s');
}, activeColor:function(get) {
  var color = '#4CAF50';
  var obj = get('topicObj');
  if (obj !== null && Ext.isObject(obj) && !obj.active) {
    color = '#FF0000';
  }
  return color;
}, activeText:function(get) {
  var text = 'Aktiv';
  var obj = get('topicObj');
  if (obj !== null && Ext.isObject(obj) && !obj.active) {
    if (obj.active === null) {
      text = 'Archiv';
    } else {
      text = 'InAktiv';
    }
  }
  return text;
}, activeClass:function(get) {
  var iconClass = collaboration.assets.Artefacts.topicActiveIcon;
  var obj = get('topicObj');
  if (obj !== null && Ext.isObject(obj) && !obj.active) {
    if (obj.active === null) {
      iconClass = collaboration.assets.Artefacts.archiveIcon;
    } else {
      iconClass = collaboration.assets.Artefacts.topicInactiveIcon;
    }
  }
  return iconClass;
}, teamHtml:function(get) {
  var html = '';
  var obj = get('topicObj');
  if (obj !== null && Ext.isObject(obj) && obj.team) {
    html = '\x3cspan class\x3d"' + collaboration.assets.Artefacts.teamIcon + '" style\x3d"color:' + collaboration.assets.Artefacts.greenColor + '"\x3e\x3c/span\x3e\x26nbsp; ' + obj.team;
  }
  return html;
}, activeFromHtml:function(get) {
  var html = '';
  var obj = get('topicObj');
  if (obj !== null && Ext.isObject(obj) && obj.activeFrom) {
    html = '\x3cspan class\x3d"' + collaboration.assets.Artefacts.activeFromIcon + '" style\x3d"color:' + collaboration.assets.Artefacts.greenColor + '"\x3e\x3c/span\x3e\x26nbsp; ' + obj.activeFrom;
  }
  return html;
}, activeToHtml:function(get) {
  var html = '';
  var obj = get('topicObj');
  if (obj !== null && Ext.isObject(obj) && obj.activeTo) {
    html = '\x3cspan class\x3d"' + collaboration.assets.Artefacts.activeToIcon + '" style\x3d"color:' + collaboration.assets.Artefacts.greenColor + '"\x3e\x3c/span\x3e\x26nbsp; ' + obj.activeTo;
  }
  return html;
}, lastUpdateToHtml:function(get) {
  var html = '';
  var obj = get('topicObj');
  if (obj !== null && Ext.isObject(obj) && obj.lastupdate) {
    html = '\x3cspan class\x3d"' + collaboration.assets.Artefacts.refreshIcon + '" style\x3d"color:' + collaboration.assets.Artefacts.greenColor + '"\x3e\x3c/span\x3e\x26nbsp; ' + obj.lastupdate.replace('T', ' ');
  }
  return html;
}, isActive:function(get) {
  var active = get('topicObj.active');
  return Ext.isDefined(active) && active === true;
}}});
Ext.define('collaboration.widgets.ForumWidgetGrid', {extend:'Ext.grid.Panel', alias:'widget.collabForumWidgetGrid', stateful:true, stateId:'collabForumWidgetState', title:false, forceFit:true, features:[{ftype:'grouping', enableGroupingMenu:false, startCollapsed:true, groupHeaderTpl:['', '\x3cdiv\x3e{name:this.formatName}\x3c/div\x3e', {formatName:function(channelId) {
  var channelMeta = collaboration.Api.getChannelById(channelId);
  if (Ext.isDefined(channelMeta) && !Ext.Object.isEmpty(channelMeta)) {
    var icon = '\x3cspan style\x3d"font-size: 18px; color:' + channelMeta['color'] + '"\x3e # \x3c/span\x3e';
    return icon + '\x3cspan style\x3d"font-size: 18px;"\x3e' + Ext.util.Format.htmlEncode2(channelMeta['channel']) + '\x3c/span\x3e';
  } else {
    return channelId;
  }
}}]}], instanceMode:'widget', collabStreamMode:'forum', initComponent:function() {
  var me = this;
  this.store = this.createStore(this.moduleId);
  this.bbar = [{xtype:'collabGridInput', flex:3, collabType:this.collabType, gridReference:this.getXType()}, {xtype:'collabDateRangeField', gridReference:this.getXType()}];
  Ext.defer(function() {
    me.getStore().filterBy(function(record) {
      var isVisible = false;
      var recActive = record.get('active');
      var channelMeta = collaboration.Api.getChannelById(record.get('channelId'));
      if (Ext.isDefined(recActive) && recActive === true) {
        if (channelMeta && channelMeta.active === true) {
          isVisible = !isVisible;
        }
      }
      return isVisible;
    });
  }, 1000);
  this.callParent(arguments);
}, createStore:function(instanceId) {
  Ext.log(this.self.getName() + ': createStore');
  try {
    var url = collaboration.Module.getBaseUrl() + instanceId + '/' + 'topics';
    var me = this;
    var columns = [{xtype:'datecolumn', format:'d.m.Y H:i:s', dataIndex:'created', text:'\x3cspan class\x3d"' + collaboration.assets.Icons.clock + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + collaboration.Support.TR('CREATED')}, {dataIndex:'creator', text:'\x3cspan class\x3d"' + collaboration.assets.Icons.user + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + collaboration.Support.TR('CREATOR2')}, {dataIndex:'topicName', text:'\x3cspan class\x3d"' + collaboration.assets.Icons.bookmark + 
    '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + collaboration.Support.TR('HEADER'), cellWrap:true}];
    var fields = [];
    var customFields = collaboration.Support.getSetting(this.moduleId, 'instance_channelTopicTemplate');
    if (Ext.isObject(customFields)) {
      columns = columns.concat(me.prepareColumnsForOldConfig(customFields, fields));
    } else {
      if (Ext.isArray(customFields)) {
        columns = columns.concat(me.prepareColumns(customFields, fields));
      }
    }
    Ext.Array.each(columns, function(column) {
      if (column.xtype && column.xtype === 'datecolumn') {
        column.filter = true;
      } else {
        column.filter = {type:'string', itemDefaults:{emptyText:'Suche nach...'}};
      }
    });
    me.columns = columns;
    var newModel = Ext.define(null, {extend:'collaboration.model.CollabDataTopics', fields:fields});
    var storeConfig = {model:newModel, proxy:{type:'ajax', url:url, reader:{type:'json', rootProperty:'items', totalProperty:'total'}}, groupField:'channelId', autoLoad:true, emptyText:collaboration.Support.TR('COLLABORATION_NO_ENTRIES_AVAILABLE')};
    return Ext.create('Ext.data.Store', storeConfig);
  } catch (e) {
    Ext.log({msg:this.self.getName() + ': ERROR createStore', dump:e, stack:true, level:'error'});
  }
}, listeners:{rowdblclick:function(view, record, element, rowIndex, e, eOpts) {
  var gridView = view.up('collabForumWidgetGrid');
  var topicId = record.get('artefactId');
  var channelId;
  if (record['data']['channelId']) {
    channelId = record['data']['channelId'];
  }
  var moduleId = gridView.moduleId;
  if (gridView.instanceMode && gridView.instanceMode === 'widget') {
    var widget = gridView.up('collabForumWidget');
    var activeItem = widget.getLayout().getActiveItem();
    var activeTitle = widget.up('dbWidgetWrapper').getTitle();
    var topicView = Ext.create({xtype:'collabTopic', moduleId:moduleId, topicId:topicId, overView:gridView.up('collabTopic'), instanceMode:'widget', channelId:channelId, collabStreamMode:'forum', holderCmp:widget, header:false, bbar:[{xtype:'tbfill'}, {xtype:'button', text:'Zurück', iconCls:collaboration.assets.Icons.angleDoubleLeft, handler:function(btn) {
      var cmp = btn.up('collabTopic');
      cmp.holderCmp.getLayout().setActiveItem(cmp.overView);
      cmp.up('dbWidgetWrapper').setTitle('Forum');
      cmp.destroy();
      widget.getLayout().setActiveItem(activeItem);
      widget.up('dbWidgetWrapper').setTitle(activeTitle);
    }}], listeners:{afterrender:function(cmp) {
      Ext.defer(function() {
        var title = cmp.getViewModel().get('topicTitle');
        var initTitle = cmp.up('dbWidgetWrapper').getTitle();
        var channelMeta = collaboration.Api.getChannelById(channelId);
        var channelTitle;
        if (channelMeta['channel']) {
          channelTitle = '\x3cspan style\x3d"color:' + channelMeta['color'] + '"\x3e # \x3c/span\x3e' + channelMeta['channel'];
        }
        if (channelTitle) {
          cmp.up('dbWidgetWrapper').setTitle(initTitle + ' (' + channelTitle + ' - ' + title + ' )');
        }
      }, 500);
    }}});
    widget.getLayout().setActiveItem(topicView);
  }
}, cellcontextmenu:function(tableView, td, cellIndex, record, tr, rowIndex, e, eOpts) {
  e.preventDefault();
  var mainView = tableView.up('collabForumWidgetGrid');
  var contextMenu = Ext.create('collaboration.view.uitypes.streams.contextMenu.BlogWidgetContextMenu', {gridReference:mainView, moduleId:mainView.moduleId, selectedRecord:record});
  contextMenu.showAt(e.getXY());
}, groupcontextmenu:function(tableView, node, groupName, e, eOpts) {
  e.preventDefault();
  var mainView = tableView.up('collabForumWidgetGrid');
  var contextMenu = Ext.create('collaboration.widgets.components.contextMenu.ForumWidgetGroupMenu', {gridReference:tableView, moduleId:mainView.moduleId, groupName:groupName});
  contextMenu.showAt(e.getXY());
}, filterchange:function(store, filters, eOpts) {
  var me = this;
  var widget = me.up('collabForumWidget');
  var clearFiltersBtn = widget.down('#clearFiltersBtn');
  if (filters && Ext.isArray(filters) && filters.length > 0) {
    if (clearFiltersBtn) {
      clearFiltersBtn.setDisabled(false);
    }
  } else {
    if (clearFiltersBtn) {
      clearFiltersBtn.setDisabled(true);
    }
  }
}}, prepareColumnsForOldConfig:function(obj, fields) {
  var columns = [];
  Ext.iterate(obj, function(k, v) {
    var col = {};
    if (v && Ext.isDefined(v) && Ext.isObject(v)) {
      var text = '\x3cspan class\x3d"' + v['iconCls'] + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + v['defaultText'];
      fields.push({name:k, mapping:k});
      col = {dataIndex:k, text:text, xtype:'gridcolumn', cellWrap:true};
      if (v['type'] === 'date') {
        col.xtype = 'datecolumn';
        col.format = 'd.m.Y H:i:s';
      }
    } else {
      fields.push({name:k, mapping:k});
      col = {dataIndex:k, text:k, xtype:'gridcolumn', cellWrap:true};
      if (v === 'date') {
        col.xtype = 'datecolumn';
        col.format = 'd.m.Y H:i:s';
      }
    }
    if (k === 'active') {
      col.hidden = true;
    }
    columns.push(col);
  });
  return columns;
}, prepareColumns:function(arr, fields) {
  var columns = [];
  Ext.each(arr, function(item, idx) {
    var col = {};
    if (item && Ext.isDefined(item) && Ext.isObject(item)) {
      var text = '\x3cspan class\x3d"' + item['iconCls'] + '" style\x3d"color:#4CAF50"\x3e\x3c/span\x3e\x26nbsp;' + item['defaultText'];
      fields.push({name:item.name, mapping:item.name});
      col = {dataIndex:item.name, text:text, xtype:'gridcolumn', cellWrap:true};
      if (item['type'] === 'date') {
        col.xtype = 'datecolumn';
        col.format = 'd.m.Y H:i:s';
      }
    }
    if (item.name === 'active') {
      col.hidden = true;
    }
    columns.push(col);
  });
  return columns;
}});
Ext.define('collaboration.widgets.WidgetGridWrapper', {extend:'Ext.panel.Panel', alias:['widget.widgetGridWrapper', 'widget.widgetgridwrapper'], layout:'fit', header:false, bbar:[], parentWidgetCmp:undefined, childCmp:undefined, gridCmp:undefined, moduleId:undefined, filterId:undefined, collabMethod:undefined, initComponent:function() {
  this.items = [{xtype:this.childCmp, header:false, moduleId:this.moduleId, instanceMode:'widget'}];
  if (this.header) {
    this.iconCls = collaboration.Module.getAssetForType(this.collabType, 'iconCls');
    this.title = collaboration.Module.getAssetForType(this.collabType, 'plural');
    var tooltip = collaboration.Module.getAssetForType(this.collabType, 'plural');
    if (this.tabConfig) {
      this.tabConfig.tooltip = tooltip;
    } else {
      this.tabConfig = {tooltip:tooltip};
    }
  }
  this.callParent(arguments);
}, updateComponentTitle:function(text, count) {
  this.setTitle((!Ext.isDefined(text) ? this.getTitle() : text) + ' ( ' + count + ' ) ');
}});
